#!/bin/ksh
# Script to run the TSTRAD code in Forward mode (direct mode)
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# This script runs only ONE test
# 
# P. Brunel Dec 2002
# M. Matricardi, ECMWF, December 2006
#

usage(){
  echo
  echo " run one RTTOV forward test"; 
  echo
  echo "usage  $0 Test_number platform instrument sat_id file_channels file_profile zenangle azangle \
        sunzenangle sunazangle surface_type water_type latitude elevation refraction ";
  echo "  test_number            the reference test number (see test plan)";
  echo "  platform	platform name (noaa, goes...)";
  echo "  instrunemt    instrument name (hirs amsua ..)   ";
  echo "  sat_id        satellite number (xx for NOAAxx)   ";
  echo "  file_channels file containg the list of channels/emissivity  ";
  echo "  file_profile  file containg the input profile   ";
  echo "  zenangle      zenithal angle (deg)   ";
  echo "  azangle       azimuthal angle (deg)   ";
  echo "  sunzenangle   Solar Zenith angle (degrees) ";
  echo "  sunazangle    Solar Azimuth angle (degrees) ";
  echo "  surface_type  surface type   (0=land, 1=sea, 2=ice/snow) ";
  echo "  water_type    Water type (0=fresh water, 1=ocean water ";
  echo "  latitude      Latitude of sounding (degrees) ";
  echo "  elevation     Elevation of sounding (km) ";
  echo "  refraction    0=No , 1=Yes "; 
  echo "     ";
  echo "environment variables:"
  echo "  DATAIN  	directory for file channels and profiles "
  echo "  DATART	directory for coefficient files"
  echo "  REF_TEST	directory for reference test results"
  echo "  TEST		directory for test results "
  echo
  echo "environment variables default values:"
  echo "  DATAIN  	../data "
  echo "  DATART	../rtcoef"
  echo "  REF_TEST	../new_reftest"
  echo "  TEST		../new_test "
  echo
}
if [ $# -lt 14 ]
then
	usage
	exit 1
fi

# use function for rttov ids
. ./rttov_id.ksh

# Get arguments
TEST_NUMBER=$1
PLATFORM=$2
INST=$3
SATID=$4
FILE_CHANNELS=$5
FILE_PROFILE=$6
FILE_AEROSOLS=$7    
FILE_CLOUDS=$8
FILE_CLOUDS_FRAC=$9
ZENANGLE=${10}
AZANGLE=${11}
SUNRAD=${12}
SUNZENANGLE=${13}
SUNAZANGLE=${14}
AEROSOLS=${15}
DEFPAR=${16}
CLIMAT=${17}
CLOUDS=${18}
EFFDIAM=${19}
SHAPE=${20}
SURF_TYPE=${21}
WATER_TYPE=${22}
LATITUDE=${23}
ELEVATION=${24}
REFRACTION=${25}



# Following variables should have been initialised and exported
# in the calling script, else default values are appling
DATAIN=${DATAIN:=../data}  
DATART=${DATART:=../rtcoef} 
REF_TEST=${REF_TEST:=../new_reftest}
TEST=${TEST:=../new_test}

# Create directory for test results
if [ ! -d ${TEST} ]; then
  mkdir ${TEST}
fi
#
###################################################
# The followind sed command file is to change all occurence of
# .xxx or -.yyy into 0.xxx and -0.yyy to allow a good comparison
# between the refernce test and fortran output (especially for HP)
cat <<EOF > sed.cmd
s/ \./0\./g
s/ -\./-0\./g
EOF
#

CUR_DIR=`pwd`


echo " "
echo " "
echo " Test forward  ${INST} number ${TEST_NUMBER}"
echo " "

echo  "Platform   ${PLATFORM}"
echo  "Instrument ${INST}"
echo  "Satellite  ${SATID}"
echo  "File for channels ${FILE_CHANNELS}"
echo  "File for profile  ${FILE_PROFILE}"
echo  "File for aerosols ${FILE_AEROSOLS}"
echo  "File for clouds   ${FILE_CLOUDS}"
echo  "File for cloud fraction ${FILE_CLOUDS_FRAC}"
echo  "Zenith angle ${ZENANGLE}"
echo  "Azimuth angle ${AZANGLE}"
echo  "Solar radiation ${SUNRAD}"
echo  "Solar Zenith angle ${SUNZENANGLE}"
echo  "Solar Azimuth angle ${SUNAZANGLE}"
echo  "Aerosols ${AEROSOLS}"
echo  "Clouds ${CLOUDS}"
echo  "Surface type ${SURF_TYPE}"
echo  "Water type ${WATER_TYPE}"
echo  "Latitude ${LATITUDE}"
echo  "Elevation ${ELEVATION}"
echo  "Refraction ${REFRACTION}"


# call to function rttov_id
rttov_id ${PLATFORM} ${INST}

# diff command ignoring case of letters and spaces
DIFF='diff -biw'

rm -f input.dat prof.dat
if [ ! -s ${DATAIN}/${FILE_CHANNELS} ] ; then
  echo "file ${FILE_CHANNELS}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_PROFILE} ] ; then
  echo "file ${FILE_PROFILE}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_AEROSOLS} ] ; then
  echo "file ${FILE_AEROSOLS}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_CLOUDS} ] ; then
  echo "file ${FILE_CLOUDS}  does not exist on directory  ${DATAIN}"
  exit
fi
if [ ! -s ${DATAIN}/${FILE_CLOUDS_FRAC} ] ; then
  echo "file ${FILE_CLOUDS_FRAC}  does not exist on directory  ${DATAIN}"
  exit
fi
cp ${DATAIN}/${FILE_CHANNELS} input.dat
cp ${DATAIN}/${FILE_PROFILE}  prof.dat
cp ${DATAIN}/${FILE_AEROSOLS} prof_aerosl_cl.dat
cp ${DATAIN}/${FILE_CLOUDS} prof_cloud.dat
cp ${DATAIN}/${FILE_CLOUDS_FRAC} prof_cfrac.dat

COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_NAME_BIN=rtcoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_NAME}
rm -f ${COEF_NAME_BIN}
if [ -s  $DATART/${COEF_NAME} ] ; then
  ln -s $DATART/${COEF_NAME} ${COEF_NAME}
fi
if [ -s  $DATART/${COEF_NAME_BIN} ] ; then
  ln -s $DATART/${COEF_NAME_BIN} ${COEF_NAME_BIN}
fi

COEF_AER_NAME=scaercoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_AER_NAME_BIN=scaercoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_AER_NAME}
rm -f ${COEF_AER_NAME_BIN}
if [ -s  $DATART/${COEF_AER_NAME} ] ; then
  ln -s $DATART/${COEF_AER_NAME} ${COEF_AER_NAME}
fi
if [ -s  $DATART/${COEF_AER_NAME_BIN} ] ; then
  ln -s $DATART/${COEF_AER_NAME_BIN} ${COEF_AER_NAME_BIN}
fi

COEF_CLD_NAME=sccldcoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_CLD_NAME_BIN=sccldcoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_CLD_NAME}
rm -f ${COEF_CLD_NAME_BIN}
if [ -s  $DATART/${COEF_CLD_NAME} ] ; then
  ln -s $DATART/${COEF_CLD_NAME} ${COEF_CLD_NAME}
fi
if [ -s  $DATART/${COEF_CLD_NAME_BIN} ] ; then
  ln -s $DATART/${COEF_CLD_NAME_BIN} ${COEF_CLD_NAME_BIN}
fi
echo
echo
echo "FWD test ${PLATFORM} ${SATID} ${INST}"

../bin/tstrad.exe <<EOF
${PLATFORM_N}   , platform name
${SATID}        , satellite number (NOAAxx = xx)
${INST_N}       , instrument number
0               , forward model only == 0, full gradient==1, radiance output ==2
1               , number of profiles
${SURF_TYPE}    , Surface type (0=land, 1=sea, 2=ice/snow)
${WATER_TYPE}   , Water type (0=fresh,1=ocean)
${AEROSOLS}     , Aerosols
${DEFPAR}       , Aerosols default parameters 
${CLIMAT}       , Aerosols climatological profile
${CLOUDS}       , Clouds
${EFFDIAM}      , Ice Clouds effective diameter
${SHAPE}        , Ice Clouds particle shape
${ZENANGLE}     , Zenith angle
${AZANGLE}      , Azimuth angle
${SUNRAD}       , Solar radiation
${SUNZENANGLE}  , Solar Zenith angle
${SUNAZANGLE}   , Solar Azimuth angle
${LATITUDE}     , Latitude of sounding
${ELEVATION}    , Elevation of terrain
${REFRACTION}   , Atmospheric refraction (0=no, 1=yes)
EOF

rm -f ${COEF_NAME} ${COEF_NAME_BIN}

sed -f sed.cmd print.dat > ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst
${DIFF} ${REF_TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.lst > ${TEST}/tstrad_fwd_${INST}_${TEST_NUMBER}.diff

rm -f print.dat sed.cmd input.dat prof.dat

date
exit

