
Subroutine rttov_ad( &
       & errorstatus,     &! out
       & nprofiles,       &! in
       & nchannels,       &! in
       & channels,        &! in
       & lprofiles,       &! in
       & addinterp,       &! in
       & profiles,        &! in
       & profiles_ad,     &! inout  adjoint
       & coef,            &! in
       & coef_scatt_ir,   &! in
       & optp,            &! in
       & addsun,          &! in
       & addaerosl,       &! in
       & addclouds,       &! in
       & switchrad,       &! in
       & calcemis,        &! in
       & emissivity,      &! inout  direct model
       & emissivity_ad,   &! inout  adjoint
       & transmission,    &! inout  direct model
       & transmission_ad, &! inout  adjoint input
       & radiancedata,    &! inout  direct model   (input due to pointers alloc)
       & radiancedata_ad, &! inout  adjoint input  (output if converstion Bt -> rad)  
       & temp,            &
       & temp_ad ) 
  !
  ! Description:
  ! Adjoint of rttov_direct
  ! to compute multi-channel level to space transmittances,
  ! top of atmosphere and level to space radiances and brightness
  ! temperatures and optionally surface emissivities, for many
  ! profiles in a single call, for satellite
  ! infrared or microwave sensors. The code requires a coefficient file
  ! for each sensor for which simulated radiances are requested.
  !
  ! Note that radiancedata_ad can be used for all its structure elements
  ! In normal case the element total or bt is the only one initialised but
  ! for some particular cases like for rttov_cld_ad some other elements
  ! have been already init.
  ! According to the argument switchrad the main input total or bt is used
  ! switchrad == true    bt is the input, brightness temperature
  ! switchrad == false   total is the input, radiance
  !
  ! The AD outputs profiles_ad and emissivity_ad should be allocated and
  ! initialised before calling the subroutine
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is described in the following:
  !
  ! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
  ! in the microwave region: a fast model. Applied Optics 27  3244-3249
  !
  ! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
  ! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
  ! librarian at ECMWF).
  !
  ! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
  ! Transfer Model for Assimilation of Satellite Radiance Observations.
  ! QJRMS, 125, 1407-1425.
  !
  ! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
  ! fast radiative transfer model for the assimilation of radiance
  ! observations. ECMWF Research Dept. Tech. Memo. 345
  ! (available from the librarian at ECMWF).
  !
  ! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
  ! transfer model for the infrared atmospheric sounding interferometer.
  ! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.1   01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.2   02/01/2003  More comments added (R Saunders)
  !  1.3   24/01/2003  Error return code by input profile (P Brunel)
  !  1.4               Add WV Continuum and CO2 capability
  !  1.5   04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.6   02/06/2004  Change tests on id_comp_lvl == 7 by tests on fmv_model_ver (P. Brunel)
  !  1.7   08/12/2005  Added surface humidity to lowest mean layer q (R Saunders) 
  !  1.8   01/06/2005  Marco Matricardi (ECMWF):
  !              --    IASI capability added.
  !              --    Variable trace gases added for IASI and AIRS.
  !              --    Solar radiation added for IASI and AIRS
  !              --    Altitude dependent local zenith angle added.
  !              --    Linear in tau approximation for RT equation added.
  !  1.9    06/12/2005 Minor mods to initialise of sunglint_ad (R Saunders)
  !  1.10   08/02/2007 Removed polarisation indexing (R Saunders)
  !         29/06/2007 Introduce interpolator (P.J.Rayer)
  !              --    Profiles on USER levs -> COEF levs
  !              --    Duplicate profaux and setgeometry (USER and COEF levs)
  !              --    Predict atm/sol optical depths on COEF levs -> USER levs
  !              --    Calculate transmittance and integrate RT on USER levs
  !  1.11   11/10/2007 Move iaernum & iaertyp profile members to aux_prof P.Marguinaud
  !  1.12   16/01/2008 Facility to apply regression limits (N. Bormann)
  !  1.13   26/09/2008 Disable vectorization of the pointer association
  !                    loops and nullify temp pointers; this is for the NEC 
  !                    under -Chopt. P. Marguinaud
  !
  !  1.13   03/10/2008 Fix bug in checking addsolar (A. Geer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  ! A user guide and technical documentation is available at
  ! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :       &
         & errorstatus_success ,&
         & errorstatus_warning ,&
         & errorstatus_fatal   ,&
         & max_optical_depth   ,&
         & sensor_id_mw        ,&
         & sensor_id_ir        ,&
         & sensor_id_hi        ,&  
         & sensor_id_po        ,& 
         & ncldtyp             ,&
         & ngases_max          ,&
         & apply_reg_limits

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :              &
         & rttov_coef                 ,&
         & rttov_coef_scatt_ir        ,&
         & transmission_scatt_ir_type ,&
         & rttov_optpar_ir            ,&
         & profile_Type               ,&
         & geometry_Type              ,&
         & predictors_Type            ,&
         & profile_aux                ,&
         & opdp_path_type             ,&
         & transmission_Type          ,&
         & radiance_Type              ,&
         & radiance_aux               ,&
         & raytracing_type            ,&
         & sunglint_type              ,&
         & ircld_type                 ,&
         & blob_type                  ,&
         & rttov_temp

  Use rttov_global, Only: &
        & verbose_checkinput_warnings

  Use parkind1, Only : jpim     ,jprb

  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),      Intent(in)    :: nprofiles                    ! Number of profiles
  Integer(Kind=jpim),      Intent(in)    :: nchannels                    ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),      Intent(in)    :: channels      (nchannels)
  Integer(Kind=jpim),      Intent(in)    :: lprofiles     (nchannels)
  Logical,                 Intent(in)    :: addsun
  Logical,                 Intent(in)    :: addaerosl
  Logical,                 Intent(in)    :: addclouds
  Logical,                 Intent(in)    :: addinterp  ! switch for User-Coef-User interpolations
  Logical,                 Intent(in)    :: switchrad  ! true if input is BT

  Type(profile_Type),      Intent(in)    :: profiles      (nprofiles)
  Type(profile_Type),      Intent(inout) :: profiles_ad   (nprofiles)

  Type(rttov_coef),        Intent(in)    :: coef
  Type(rttov_coef_scatt_ir), Intent(in)  :: coef_scatt_ir
  Type(rttov_optpar_ir),   Intent(in)    :: optp

  Logical,                 Intent(in)    :: calcemis      (nchannels)

  Integer(Kind=jpim),      Intent(out)   :: errorstatus   (nprofiles)

  Real(Kind=jprb),         Intent(inout) :: emissivity    (nchannels)
  Real(Kind=jprb),         Intent(inout) :: emissivity_ad (nchannels)

  Type(transmission_Type), Intent(inout) :: transmission    ! in because of meme allocation
  Type(transmission_Type), Intent(inout) :: transmission_ad ! in because of meme allocation

  Type(radiance_Type),     Intent(inout) :: radiancedata    ! in because of meme allocation
  Type(radiance_Type),     Intent(inout) :: radiancedata_ad ! in because of meme allocation
  Type(rttov_temp),  Intent(inout), Optional, Target :: temp, temp_ad

!INTF



#include "rttov_checkinput.interface"
#include "rttov_checkinput_ad.interface"
#include "rttov_errorreport.interface"
#include "rttov_profaux.interface"
#include "rttov_setgeometry.interface"
#include "rttov_setgeometry_ad.interface"
#include "rttov_setpredictors_7.interface"
#include "rttov_setpredictors_8.interface"
#include "rttov_setpredictors_9.interface"
#include "rttov_opdep.interface"
#include "rttov_opdep_9.interface"
#include "rttov_opdep_9_solar.interface"
#include "rttov_transmit.interface"
#include "rttov_transmit_9_solar.interface"
#include "rttov_calcemis_ir.interface"
#include "rttov_calcemis_mw.interface"
#include "rttov_integrate.interface"
#include "rttov_profaux_ad.interface"
#include "rttov_setpredictors_7_ad.interface"
#include "rttov_setpredictors_8_ad.interface"
#include "rttov_setpredictors_9_ad.interface"
#include "rttov_opdep_ad.interface"
#include "rttov_opdep_9_ad.interface"
#include "rttov_opdep_9_solar_ad.interface"
#include "rttov_transmit_ad.interface"
#include "rttov_transmit_9_solar_ad.interface"
#include "rttov_calcemis_mw_ad.interface"
#include "rttov_integrate_ad.interface"
#include "rttov_intavg_prof.interface"
#include "rttov_intavg_chan.interface"
#include "rttov_intavg_prof_ad.interface"
#include "rttov_intavg_chan_ad.interface"
#include "rttov_alloc_trans_scatt_ir.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_refsun.interface"
#include "rttov_refsun_ad.interface"
#include "rttov_init_profiles.interface"
#include "rttov_check_temp.interface"
#include "rttov_init_predictor.interface"
#include "rttov_init_raytracing.interface"
#include "rttov_checkargs.interface"
#include "rttov_setpredictors_9_solar.interface"
#include "rttov_setpredictors_9_solar_ad.interface"
#include "rttov_cldstr.interface"
#include "rttov_cldstr_ad.interface"
#include "rttov_opdpscattir.interface"
#include "rttov_opdpscattir_ad.interface"
#include "rttov_fresnel.interface"
#include "rttov_fresnel_ad.interface"

  !local variables:


  Integer(Kind=jpim) :: i, j, prof, chan           ! loop index
  Integer(Kind=jpim) :: nstreams
  Logical            :: addcosmic         ! switch for adding temp of cosmic background
  Integer(Kind=jpim) :: alloc_status(111)
  Integer(Kind=jpim) :: callnum           ! which interpolator call: 1=USER->COEF, 2=COEF->USER 
  Integer(Kind=jpim) :: nvar, nc1, nc2,igas    ! no of profile elements to interpolator (USER->COEF)
  Logical            :: intvar(ngases_max)
  Logical            :: sun(nchannels)                   ! solar switch located in channel sequence

  Real(Kind=jprb)    :: reflectivity(nchannels)      ! surface reflectivity
  Real(Kind=jprb)    :: reflectivity_ad(nchannels)   ! AD surface reflectivity
 
  Real(Kind=jprb)    :: fresnrefl(nchannels)
  Real(Kind=jprb)    :: fresnrefl_ad(nchannels)
 
  Real(Kind=jprb)    :: tau_ref          (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tau_ref_surf     (nchannels)
  Real(Kind=jprb)    :: tau_surf         (nchannels)
 
  Real(Kind=jprb)    :: tausun_ref       (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_ref_surf  (nchannels)
  Real(Kind=jprb)    :: tausun_layer     (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_surf      (nchannels)
 
  Real(Kind=jprb)    :: opdp_ref         (profiles(1)%nlevels,nchannels) ! layer optical depth before threshold
  Real(Kind=jprb)    :: opdp_ref_COEF    (coef%nlevels,nchannels)        ! layer optical depth before threshold
  Real(Kind=jprb)    :: od_layer         (profiles(1)%nlevels,nchannels) ! layer optical depth
  Real(Kind=jprb)    :: od_layer_COEF    (coef%nlevels,nchannels) ! layer optical depth
  Real(Kind=jprb)    :: tau_layer        (profiles(1)%nlevels,nchannels) ! layer transmittance

  Real(Kind=jprb)    :: opdpsun_ref      (profiles(1)%nlevels,nchannels)              ! layer optical depth
  Real(Kind=jprb)    :: opdpsun_ref_COEF (coef%nlevels,nchannels)        ! layer optical depth
  Real(Kind=jprb)    :: odsun_layer      (profiles(1)%nlevels,nchannels)        ! sat to layer optical depth
  Real(Kind=jprb)    :: odsun_singlelayer(profiles(1)%nlevels,nchannels)        ! sat to layer optical depth
  Real(Kind=jprb)    :: od_frac          (nchannels) 

  Real(Kind=jprb)    :: sum4
  Integer(Kind=jpim) :: iprof,ilev

  Character (len=80) :: errMessage
  Character (len=8)  :: NameOfRoutine = 'rttov_ad'

  Type(profile_Type)    :: profiles_COEF_ref (nprofiles)   
  Type(blob_type)       :: profiles_COEF_blob_ref

  Type(geometry_Type)   :: angles            (nprofiles)   ! geometry angles
  Type(geometry_Type)   :: angles_COEF       (nprofiles)   ! geometry angles

  Type(profile_aux)     :: aux_prof          (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_ad       (nprofiles)   ! AD of above aux_prof

  Type(profile_aux)     :: aux_prof_COEF     (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_COEF_ad  (nprofiles)   ! AD of above aux_prof

  Type(radiance_aux)    :: auxrad

  Type(radiance_aux)    :: auxrad_stream
  Type(radiance_aux)    :: auxrad_stream_ad

  Type(sunglint_type)   :: sunglint          (nprofiles)
  Type(sunglint_type)   :: sunglint_ad       (nprofiles)


  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream_ad

  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_ad

  Real(Kind=jprb),  target :: zdg              (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac1_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac2_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac3_dg         (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zdg_ad           (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac1_dg_ad      (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac2_dg_ad      (profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zfac3_dg_ad      (profiles(1)%nlevels,nprofiles)

  Real(Kind=jprb),  target :: zdeb        (5,profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zdeb_COEF   (5,coef%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zdeb_ad     (5,profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zdeb_COEF_ad(5,coef%nlevels,nprofiles)



  Real(Kind=jprb),  target :: surfair(nchannels)
  Real(Kind=jprb),  target :: skin   (nchannels)
  Real(Kind=jprb),  target :: cosmic (nchannels)
  Real(Kind=jprb),  target :: layer     (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb),  target :: up        (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb),  target :: down      (profiles(1)%nlevels,nchannels)
  Real(Kind=jprb),  target :: down_cloud(profiles(1)%nlevels,nchannels)


  Real(Kind=jprb), target ::  zbeta        (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zbeta_ad     (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zpsi         (coef%ws_nomega,nprofiles)
  Real(Kind=jprb), target ::  zpsi_ad      (coef%ws_nomega,nprofiles)
  Integer(Kind=jpim) :: jn,jl,ncha,nchb

  Type(rttov_temp), Target  :: temp1
  Type(rttov_temp), Pointer :: temp0
  Type(rttov_temp), Target  :: temp1_ad
  Type(rttov_temp), Pointer :: temp0_ad
  Integer(Kind=jpim)        :: temp_status, checkargs_status

  REAL(KIND=JPRB) :: ZHOOK_HANDLE


  !- End of header --------------------------------------------------------

  !-------------
  !0. initialize
  !-------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_AD',0_jpim,ZHOOK_HANDLE)
  errorstatus(:)  = errorstatus_success
  alloc_status(:) = 0


  Call rttov_checkargs(  &
      & addinterp,       &
      & addsun,          &
      & addaerosl,       &
      & addclouds,       &
      & checkargs_status &
  )
  
  If( checkargs_status .eq. errorstatus_fatal ) Then
     Write( errMessage, '( "checkargs fatal error")' )
     Call Rttov_ErrorReport( errorstatus_fatal, errMessage,NameOfRoutine )
     errorstatus(:)  = errorstatus_fatal    
     IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
     Return
  EndIf


  Nullify( temp0 )
  Nullify( temp0_ad )

  !-----------------------------------------------------------------------
  !1. interpolator first call - input profiles from USER levs to COEF levs
  !-----------------------------------------------------------------------
  ! done immediately in direct code - for quick escape if target values are out of bounds



  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 1_jpim,                                  &
    & temp0 = temp0, temp0_ad = temp0_ad,      &   
    & temp1 = temp1, temp1_ad = temp1_ad,      &   
    & temp2 = temp,  temp2_ad = temp_ad )  

  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  

  Call rttov_init_profiles( nprofiles, temp0%profiles_COEF )

  Do i = 1 , nprofiles
     temp0%profiles_COEF(i) % nlevels =  coef % nlevels
     temp0%profiles_COEF(i) % p(:) = coef % ref_prfl_p(:)
  Enddo


  If (addinterp) Then
     Call rttov_intavg_prof( &
            & nprofiles,                       &
            & profiles(1)%nlevels,             & !in
            & temp0%profiles_COEF(1)%nlevels,  & !in
            & profiles,                        & !in     source variables
            & temp0%profiles_COEF,             & !inout  target variables
            & errorstatus)                       !out
     If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
     End If
  Else 
! COEF levs same as USER levs
     Do i = 1, nprofiles
        temp0%profiles_COEF(i)%t(:)   =    profiles(i)%t(:)
        temp0%profiles_COEF(i)%q(:)   =    profiles(i)%q(:)
        if(profiles(i) % clw_data)   temp0%profiles_COEF(i)%clw(:) =  profiles(i)%clw(:)
        if(profiles(i) % ozone_data) temp0%profiles_COEF(i)%o3(:)  =  profiles(i)%o3(:)
        if(profiles(i) % co2_data)   temp0%profiles_COEF(i)%co2(:) =  profiles(i)%co2(:)
        if(profiles(i) % n2o_data)   temp0%profiles_COEF(i)%n2o(:) =  profiles(i)%n2o(:)   
        if(profiles(i) % co_data)    temp0%profiles_COEF(i)%co(:)  =  profiles(i)%co(:)    
        if(profiles(i) % ch4_data)   temp0%profiles_COEF(i)%ch4(:) =  profiles(i)%ch4(:)  
     Enddo
  EndIf

    
! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
  Do i = 1, nprofiles
    temp0%profiles_COEF(i) % ozone_data = profiles(i) % ozone_data
    temp0%profiles_COEF(i) % co2_data   = profiles(i) % co2_data
    temp0%profiles_COEF(i) % clw_data   = profiles(i) % clw_data

    temp0%profiles_COEF(i) % n2o_data   = profiles(i) % n2o_data
    temp0%profiles_COEF(i) % co_data    = profiles(i) % co_data
    temp0%profiles_COEF(i) % ch4_data   = profiles(i) % ch4_data
    temp0%profiles_COEF(i) % aer_data   = profiles(i) % aer_data
    temp0%profiles_COEF(i) % cld_data   = profiles(i) % cld_data

    temp0%profiles_COEF(i) % S2m        = profiles(i) % S2m
    temp0%profiles_COEF(i) % skin       = profiles(i) % skin
    temp0%profiles_COEF(i) % ctp        = profiles(i) % ctp
    temp0%profiles_COEF(i) % cfraction  = profiles(i) % cfraction

    temp0%profiles_COEF(i) % idg    = profiles(i) % idg
    temp0%profiles_COEF(i) % ish    = profiles(i) % ish

    temp0%profiles_COEF(i) % zenangle   = profiles(i) % zenangle
    temp0%profiles_COEF(i) % azangle    = profiles(i) % azangle

    temp0%profiles_COEF(i) % sunzenangle = profiles(i) % sunzenangle
    temp0%profiles_COEF(i) % sunazangle  = profiles(i) % sunazangle
    temp0%profiles_COEF(i) % latitude    = profiles(i) % latitude
    temp0%profiles_COEF(i) % elevation   = profiles(i) % elevation
    temp0%profiles_COEF(i) % addsolar    = profiles(i) % addsolar
    temp0%profiles_COEF(i) % addrefrac   = profiles(i) % addrefrac  

    temp0%profiles_COEF(i) % Be     = profiles(i) % Be  
    temp0%profiles_COEF(i) % cosbk  = profiles(i) % cosbk  
!

  End Do      ! over profiles
  
  Call rttov_init_raytracing( temp0%raytracing )

  !------------------------------------------------------
  !2. check input data is within suitable physical limits
  !------------------------------------------------------
  If ( apply_reg_limits ) Then

    Call rttov_alloc_prof( alloc_status(1), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 1_jpim, blob = profiles_COEF_blob_ref )
    If( Any(alloc_status /= 0) ) Then
      errorstatus = errorstatus_fatal
      Write( errMessage, '( "allocation of profiles_COEF_ref")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    End If

    profiles_coef_ref = temp0%profiles_coef

  End If
  Do i = 1, nprofiles

     Call rttov_checkinput(           &
           & temp0%profiles_COEF( i ),&!inout
           & coef,                    &!in
           & errorstatus(i)    )       !out 

  End Do

  ! test check input return code

  If ( any( errorstatus(:) == errorstatus_warning)  .and. verbose_checkinput_warnings ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_warning ) Then
           Write( errMessage, '( "checkinput warning error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_warning, errMessage, NameOfRoutine)
        End If
     End Do
  End If

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_fatal ) Then
           ! Some unphysical values; Do not run RTTOV
           Write( errMessage, '( "checkinput fatal error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        End If
     End Do
     ! nothing processed so all profiles get the fatal error code
     ! user will know which profile
     errorstatus(:) = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  !------------------------------------------------------------
  !3. determine cloud top, surface levels, ice cloud parameters
  !------------------------------------------------------------

  !3.1 USER levs
  !-------------
  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
     jn=profiles(1)%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof(i) % debye_prof => zdeb(1:5,1:jn,i)
     End Do
  Endif
  If( addclouds ) Then
     jn=profiles(1)%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof(i) % dg => zdg(1:jn,i)
       aux_prof(i) % fac1_dg => zfac1_dg(1:jn,i)
       aux_prof(i) % fac2_dg => zfac2_dg(1:jn,i)
       aux_prof(i) % fac3_dg => zfac3_dg(1:jn,i)
     End Do
  Endif
  Do i = 1, nprofiles
     Call rttov_profaux( &
           & addclouds,     &! in
           & profiles(i),   &! in
           & coef,          &! in
           & aux_prof(i))    ! inout 
  End Do

   !3.2 COEF levs
  !-------------

  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
     jn=coef%nlevels
!CDIR NOVECTOR
     Do i = 1, nprofiles
       aux_prof_COEF(i) % debye_prof => zdeb_COEF(1:5,1:jn,i)
     End Do
  Endif
  Do i = 1, nprofiles
    If(addinterp) then
      Call rttov_profaux(            &
           & .false.  ,              &! no cloud computations on coef levels
           & temp0%profiles_COEF(i), &! in
           & coef,                   &! in
           & aux_prof_COEF(i))        ! inout 
    Else
      aux_prof_COEF(i) % nearestlev_surf = aux_prof(i) % nearestlev_surf
      aux_prof_COEF(i) % pfraction_surf  = aux_prof(i) % pfraction_surf
      aux_prof_COEF(i) % nearestlev_ctp  = aux_prof(i) % nearestlev_ctp 
      aux_prof_COEF(i) % pfraction_ctp   = aux_prof(i) % pfraction_ctp
      aux_prof_COEF(i) % cfraction       = aux_prof(i) % cfraction 
      If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
        aux_prof_COEF(i) % debye_prof(:,:) = aux_prof(i) % debye_prof(:,:)
      Endif
    Endif
  End Do


  !------------------------------------------------------------------
  !4. set up common geometric variables for transmittance calculation
  !------------------------------------------------------------------

  !4.1 USER levs
  !-------------


  Call rttov_setgeometry(       &
            & nprofiles,        &
            & profiles,         &
            & aux_prof,         &! in
            & coef,             &! in
            & angles,           &
            & temp0%raytracing    )     ! out  

  !4.2 COEF levs
  !-------------

   
  If(addinterp) then
    Call rttov_setgeometry(         &
            & nprofiles,            &
            & temp0%profiles_COEF,  &
            & aux_prof_COEF,        &! in
            & coef,                 &! in
            & angles_COEF,          &
            & temp0%raytracing_COEF) ! out  
   Else
     Do i = 1, nprofiles
       angles_COEF(i)      = angles(i)
       temp0%raytracing_COEF % HGPL(:,i)     = temp0%raytracing % HGPL(:,i)
       temp0%raytracing_COEF % DAIR(:,i)     = temp0%raytracing % DAIR(:,i)
       temp0%raytracing_COEF % DMAIR(:,i)    = temp0%raytracing % DMAIR(:,i)
       temp0%raytracing_COEF % R(:,i)        = temp0%raytracing % R(:,i) 
       temp0%raytracing_COEF % RATOESUN(:,i) = temp0%raytracing % RATOESUN(:,i)
       temp0%raytracing_COEF % RATOESAT(:,i) = temp0%raytracing % RATOESAT(:,i)
       temp0%raytracing_COEF % ZASUN(:,i)    = temp0%raytracing % ZASUN(:,i)
       temp0%raytracing_COEF % ZASAT(:,i)    = temp0%raytracing % ZASAT(:,i)
       temp0%raytracing_COEF % INT(:,i)      = temp0%raytracing % INT(:,i)
       temp0%raytracing_COEF % HL(:,i)       = temp0%raytracing % HL(:,i)
       temp0%raytracing_COEF % PPW(:,i)      = temp0%raytracing % PPW(:,i)
       temp0%raytracing_COEF % DISPCO2(:,i)  = temp0%raytracing % DISPCO2(:,i)
       temp0%raytracing_COEF % PATHSAT(:,i)  = temp0%raytracing % PATHSAT(:,i)
       temp0%raytracing_COEF % PATHSUN(:,i)  = temp0%raytracing % PATHSUN(:,i)
       temp0%raytracing_COEF % PATHEFF(:,i)  = temp0%raytracing % PATHEFF(:,i)
       temp0%raytracing_COEF % LTICK(:,i)    = temp0%raytracing % LTICK(:,i)
     enddo
   Endif

  !---------------------------------------------------
  !5. calculate atm/sol predictors - atm/sol COEF levs
  !---------------------------------------------------


  Do i = 1, nprofiles
    temp0%predictors%clw(:,i) = 0._jprb
  EndDo  



  If (coef%fmv_model_ver == 7) Then
     Call rttov_setpredictors_7( &
           & nprofiles ,                  &
           & temp0%profiles_COEF,         &! in
           & angles_COEF,                 &! in
           & coef,                        &! in
           & aux_prof_COEF,               &! in
           & temp0%predictors,            &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF )       ! inout    

  Else If (coef%fmv_model_ver == 8) Then
     Call rttov_setpredictors_8(          &
           & nprofiles,                   &
           & temp0%profiles_COEF,         &! in
           & angles_COEF,                 &! in
           & coef,                        &! in
           & aux_prof_COEF,               &! in
           & temp0%predictors,            &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF )       ! inout    
  Else If (coef%fmv_model_ver == 9) Then
     Call rttov_setpredictors_9(          &
            & nprofiles,                  &
            & temp0%profiles_COEF,        &! in
            & angles_COEF,                &! in
            & coef,                       &! in
            & temp0%predictors,           &
            & temp0%raytracing_COEF ) 

     Call rttov_setpredictors_9_solar(    &
          & nprofiles,                    &
          & temp0%profiles_COEF,          &! in
          & angles_COEF,                  &! in
          & coef,                         &! in
          & temp0%predictors,             &
          & temp0%raytracing_COEF ) 
  Else
     errorstatus(:) = errorstatus_fatal
     Write( errMessage,&
            & '( "Unexpected RTTOV compatibility version number" )' ) 
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  !--------------------------------------------------------------------
  !6. predict atmospheric and solar optical depths - atm/sol COEF levs 
  !--------------------------------------------------------------------


  temp0_ad%opdp_path%atm_layer = 0._jprb
  temp0_ad%opdp_path%sun_layer = 0._jprb
  temp0_ad%opdp_path_COEF%atm_layer = 0._jprb
  temp0_ad%opdp_path_COEF%sun_layer = 0._jprb
 
  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9(              &
       & nchannels,                  &! in
       & nprofiles,                  &! in
       & coef%nlevels,               &! in
       & channels,                   &! in
       & lprofiles,                  &! in
       & temp0%predictors,           &! in
       & aux_prof_COEF,              &! in
       & coef,                       &! in
       & temp0%opdp_path_COEF,       &! inout
       & opdp_ref_COEF)               ! inout
    Call rttov_opdep_9_solar  (   &
       & nchannels,               &! in
       & nprofiles,               &! in
       & coef%nlevels,            &! in
       & channels,                &! in
       & lprofiles,               &! in
       & profiles,                &! in    ! for solar switch 
       & temp0%predictors,        &! in
       & aux_prof_COEF,           &! in
       & coef,                    &! in
       & temp0%opdp_path_COEF,    &! inout
       & opdpsun_ref_COEF)        ! inout
   else
     Call rttov_opdep(            &
       & nchannels,               &! in
       & nprofiles,               &! in
       & coef%nlevels,            &! in
       & channels,                &! in
       & lprofiles,               &! in
       & temp0%predictors,        &! in
       & aux_prof_COEF,           &! in
       & coef,                    &! in
       & temp0%opdp_path_COEF,    &! inout
       & opdp_ref_COEF)        ! inout
   endif
   
  !--------------------------------------------------------------------------
  !7. interpolator second  call - optical depths from COEF levs to USER levs
  !--------------------------------------------------------------------------

  If (addinterp) Then
! locate any profile flags in the nchannels sequence 
! at present, only solar switch
    do j=1,nchannels
      prof=lprofiles(j)
      chan=channels(j)
      If(profiles(prof)%addsolar)Then
        If(coef%ss_val_chn(chan) .ne. 0_jpim)Then
          sun(j)=.true.     
        Else
          sun(j)=.false.   
        Endif
      Else
        sun(j)=.false.     
      Endif
    enddo
        Call rttov_intavg_chan(               &
           & nchannels,                       & !in
           & sun,                             & !in
           & temp0%profiles_COEF(1)%nlevels,  & !in
           & profiles(1)%nlevels,             & !in
           & nprofiles,                       &
           & lprofiles,                       &
           & temp0%profiles_COEF,             & !inout
           & profiles,                        & !inout  only p and addsolar needed
           & temp0%opdp_path_COEF,            & !inout  source variable
           & temp0%opdp_path,                 & !inout  target variable
           & errorstatus)                    !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else 
    temp0%opdp_path % atm_layer(:,:) = temp0%opdp_path_COEF % atm_layer(:,:)
    temp0%opdp_path % sun_layer(:,:) = temp0%opdp_path_COEF % sun_layer(:,:)
  EndIf

!! Call to testop - for validation of interpolator only, otherwise comment out
!  If(addinterp) then
!! may be many channels -- sequence for tabulation is set in rttov_testop
!    nvar=nchannels
!    varnam='tran'
!! easier to validate transmittances (range 0->1) than opdeps, so pass in
!    tau_testop_COEF(:,:) = exp(temp0%opdp_path_COEF % atm_layer(:,:))
!    tau_testop(:,:)      = exp(temp0%opdp_path % atm_layer(:,:))
!    Call rttov_testop(                            &
!             & callnum,                           &! in
!             & nchannels,                         &! in
!             & varnam,                            &! in
!             & temp0%profiles_COEF(1) % nlevels,  &! in
!             & profiles(1) % nlevels,             &! in
!             & temp0%profiles_COEF(1) %p,         &! in
!             & profiles(1) %p,                    &! in
!             & tau_testop_COEF,                   &! in
!             & tau_testop)                         ! in
!    EndIf

  !---------------------------------------------------------------------------------------
  !8. If clouds are present,calculate the number of streams and 
  !   the cloud distribution in each stream - remains on USER levs
  !---------------------------------------------------------------------------------------

  nstreams=0_jpim

  
  If( addaerosl ) Then
    Do i = 1, nprofiles
      temp0_ad%ircld % tave(:,i)    =0._jprb
      temp0_ad%ircld % wmixave(:,i) =0._jprb
      temp0_ad%ircld % esw(:,i)     =0._jprb
      temp0_ad%ircld % esi(:,i)     =0._jprb
      temp0_ad%ircld % ppv(:,i)     =0._jprb
    End Do
  End If


  temp0%ircld%NSTREAM =0_jpim
  temp0%ircld%XSTRCLR =1._jprb

  if(addclouds)then

    Call rttov_cldstr(&
      & nprofiles,    &! in
      & profiles,     &! in
      & coef,         &! in
      & temp0%ircld,  &! out
      & addclouds,    &! in
      & nstreams)      ! out 

  endif


  !---------------------------------------------------------------------------------------
  !9. Calculate optical depths of aerosols and/or clouds - USER levs
  !---------------------------------------------------------------------------------------

  if(addaerosl)then
    do i=1,nprofiles

      Allocate( aux_prof(i)%iaernum     (profiles(1)%nlevels),stat= alloc_status(7))
      Allocate( aux_prof(i)%iaertyp  (11,profiles(1)%nlevels),stat= alloc_status(8))
      Allocate( aux_prof(i)%relhum      (profiles(1)%nlevels),stat= alloc_status(9))
      Allocate( aux_prof(i)%relhumref   (profiles(1)%nlevels),stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of aux_prof")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      End If


      Allocate( aux_prof_ad(i)%relhum      (profiles(1)%nlevels),stat= alloc_status(9))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of aux_prof_ad")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      End If

      aux_prof_ad(i)%relhum =0._jprb

    enddo
  End if

  if(addaerosl.or.addclouds)then


    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 1_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_ad, &
      nchannels, profiles(1) %nlevels, 1_jpim, init = .true. )
    Call rttov_alloc_trans_scatt_ir( alloc_status(3), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, .true. )
    Call rttov_alloc_trans_scatt_ir( alloc_status(4), transmission_scatt_ir_stream_ad, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, .true., init = .true. )

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "allocation of trans_scatt_ir ")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    End If




    transmission_scatt_ir_stream%SSA = 0._jprb





  Call rttov_opdpscattir(              &
      & nchannels,                     &
      & nprofiles,                     &
      & profiles(1)%nlevels,           &
      & channels,                      &
      & lprofiles,                     &
      & aux_prof,                      &
      & profiles,                      &
      & coef,                          &
      & coef_scatt_ir,                 &
      & temp0%raytracing,              &
      & transmission_scatt_ir,         &
      & transmission_scatt_ir_stream,  &
      & optp,                          &
      & temp0%ircld,                   &
      & addaerosl,                     &
      & addclouds) 
  Endif


  !----------------------------------------
  !10. calculate transmittances - USER levs
  !----------------------------------------

  Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, calc = .true. )
  Call rttov_alloc_transmission( alloc_status(2), transmission_ad, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, init = .true., calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
    Return
  End If


    Call rttov_transmit(                 &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & od_layer,                     &! out
         & transmission,                 &! inout
         & transmission_scatt_ir_stream, &
         & tau_ref,                      &! out 
         & tau_ref_surf,                 &! out 
         & tau_surf,                     &! out 
         & tau_layer)

  If (coef%fmv_model_ver == 9) Then
    Call rttov_transmit_9_solar(         &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & profiles,                     &! in   ! for solar switch 
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%raytracing,             &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & odsun_layer,                  & ! out
         & odsun_singlelayer,            & ! out
         & od_frac,                      &! out
         & transmission,                 &! out
         & transmission_scatt_ir_stream, &! out
         & tausun_ref,                   & ! out
         & tausun_ref_surf,              & ! out
         & tausun_surf,                  & ! out
         & tausun_layer)                   ! out
 endif

  !-------------------------------------------------
  !11. calculate channel emissivity values - SURFACE
  !-------------------------------------------------

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
           !Microwave
        Call rttov_calcemis_mw ( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & transmission,  &! in
              & calcemis,      &! in
              & emissivity,    &! inout
              & reflectivity,  &! out
              & errorstatus   ) ! out 
        If ( Any( errorstatus == errorstatus_fatal ) ) Then
           errorstatus(:) = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Else
        ! Hires
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        reflectivity(:) = 1 - emissivity(:)
     End Where

  Else
     ! reflectivity for all channels
     reflectivity(:) = 1 - emissivity(:)
  End If



  !-------------------------------------------------------
  !12. Compute Fresnel reflectance and sun glint - SURFACE
  !-------------------------------------------------------

     If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then

!CDIR NOVECTOR
         do i=1,nprofiles
           sunglint  (i) % beta     => zbeta     (1:coef%ws_nomega,i)
           sunglint  (i) % psi      => zpsi      (1:coef%ws_nomega,i)
         enddo

         call rttov_refsun(     &
         & nprofiles,           &
         & profiles,            &
         & coef,                &
         & aux_prof,            &
         & sunglint,            &
         & temp0%raytracing) 

         call rttov_fresnel(    &
         & nchannels,           &
         & channels,            &
         & nprofiles,           &
         & lprofiles,           &
         & profiles,            &
         & coef,                &
         & sunglint,            &
         & fresnrefl) 

       endif
     endif

  !---------------------------------------------------------
  !13. integrate the radiative transfer equation - USER levs
  !---------------------------------------------------------

  auxrad % layer   => layer(:,:)
  auxrad % surfair => surfair(:)
  auxrad % skin    => skin(:)
  auxrad % cosmic  => cosmic(:)

  Allocate(auxrad_stream % up      (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(1))
  Allocate(auxrad_stream % down    (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(2))
  Allocate(auxrad_stream % down_ref(0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(3))
  Allocate(auxrad_stream % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(13))

  if(addsun)then
    if(addaerosl.or.addclouds)then
      Allocate(auxrad_stream % FAC1_1    (0:nstreams,nchannels) ,stat= alloc_status(14))
      Allocate(auxrad_stream % FAC2_1    (0:nstreams,nchannels) ,stat= alloc_status(15))
      Allocate(auxrad_stream % FAC3_1    (0:nstreams,nchannels) ,stat= alloc_status(16))
      Allocate(auxrad_stream % FAC4_1    (0:nstreams,nchannels ) ,stat= alloc_status(17))
      Allocate(auxrad_stream % FAC5_1    (0:nstreams,nchannels ) ,stat= alloc_status(18))
      Allocate(auxrad_stream % FAC6_1    (0:nstreams,nchannels ) ,stat= alloc_status(19))
      Allocate(auxrad_stream % FAC7_1    (0:nstreams,nchannels) ,stat= alloc_status(20))
      Allocate(auxrad_stream % FAC1_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(21))
      Allocate(auxrad_stream % FAC2_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC3_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC4_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(24))
      Allocate(auxrad_stream % FAC5_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(25))
      Allocate(auxrad_stream % FAC6_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(26))
      Allocate(auxrad_stream % FAC7_2    (profiles(1) % nlevels,0:nstreams,nchannels ) ,stat= alloc_status(27))
      Allocate(auxrad_stream % FAC1_3    (0:nstreams,nchannels) ,stat= alloc_status(28))
      Allocate(auxrad_stream % FAC2_3    (0:nstreams,nchannels) ,stat= alloc_status(29))
      Allocate(auxrad_stream % FAC3_3    (0:nstreams,nchannels) ,stat= alloc_status(30))
      Allocate(auxrad_stream % FAC4_3    (0:nstreams,nchannels ),stat= alloc_status(31))
      Allocate(auxrad_stream % FAC5_3    (0:nstreams,nchannels ),stat= alloc_status(32))
      Allocate(auxrad_stream % FAC6_3    (0:nstreams,nchannels ),stat= alloc_status(33))
      Allocate(auxrad_stream % FAC7_3    (0:nstreams,nchannels) ,stat= alloc_status(34))
    endif
  endif

  addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)

  Call rttov_integrate(                 &
         & addcosmic,                   &! in
         & addaerosl,                   &! in
         & addclouds,                   &! in
         & nchannels,                   &! in
         & nprofiles,                   &! in
         & nstreams,                    &! in
         & angles,                      &! in
         & channels,                    &! in
         & lprofiles,                   &! in
         & emissivity,                  &! in
         & reflectivity,                &! in
         & fresnrefl,                   &! in
         & sunglint,                    &! in
         & transmission,                &! in
         & transmission_scatt_ir_stream,&
         & profiles,                    &! in
         & aux_prof,                    &! in
         & coef,                        &! in
         & temp0%raytracing,            &! in
         & temp0%ircld,                 &! in
         & radiancedata,                &! inout
         & auxrad,                      &
         & auxrad_stream  ) ! inout  

  !
  ! Adjoint
  !----------------
  !
  !---------------------------------------------
  !0. allocate and initialize local AD variables
  !---------------------------------------------

  !0.1 profile AD-variables on COEF levels (USER levels done in tstrad_ad)
  !------------------------------------------------------------------------

  Call rttov_init_profiles( nprofiles, temp0_ad%profiles_COEF )

  !0.2 auxillary profile AD-variables
  !----------------------------------

  ! on USER levels
  jn=profiles(1)%nlevels

  Do i = 1, nprofiles
     aux_prof_ad(i) % nearestlev_surf = 0        ! no meaning
     aux_prof_ad(i) % pfraction_surf  = 0._JPRB  ! calculated
     aux_prof_ad(i) % nearestlev_ctp  = 0        ! no meaning
     aux_prof_ad(i) % pfraction_ctp   = 0._JPRB  ! calculated
     aux_prof_ad(i) % cfraction       = 0._JPRB  ! calculated
  EndDo

  
  If( coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po ) Then
!CDIR NOVECTOR
    Do i = 1, nprofiles
      aux_prof_ad(i) % debye_prof => zdeb_ad(1:5,1:jn,i)
    EndDo
  Endif

  if(addclouds)then
!CDIR NOVECTOR
    Do i = 1, nprofiles
      aux_prof_ad(i) % dg => zdg_ad(1:jn,i)
      aux_prof_ad(i) % fac1_dg => zfac1_dg_ad(1:jn,i)
      aux_prof_ad(i) % fac2_dg => zfac2_dg_ad(1:jn,i)
      aux_prof_ad(i) % fac3_dg => zfac3_dg_ad(1:jn,i)
    End Do
  endif

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        aux_prof_ad(i) % debye_prof(:,:)  = 0._JPRB
     Endif
     if(addclouds)then
       aux_prof_ad(i) % dg     =0._jprb
       aux_prof_ad(i) % fac1_dg=0._jprb
       aux_prof_ad(i) % fac2_dg=0._jprb
       aux_prof_ad(i) % fac3_dg=0._jprb
     endif
  End Do


  ! on COEF levels
  jn=coef%nlevels

  Do i = 1, nprofiles
     aux_prof_COEF_ad(i) % nearestlev_surf = 0   ! no meaning
     aux_prof_COEF_ad(i) % pfraction_surf  = 0._JPRB  ! calculated
     aux_prof_COEF_ad(i) % nearestlev_ctp  = 0   ! no meaning
     aux_prof_COEF_ad(i) % pfraction_ctp   = 0._JPRB  ! calculated
     aux_prof_COEF_ad(i) % cfraction       = 0._JPRB  ! calculated
  End Do
  If( coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po ) Then
!CDIR NOVECTOR
    Do i = 1, nprofiles
      aux_prof_COEF_ad(i) % debye_prof => zdeb_COEF_ad(1:5,1:jn,i)
    EndDo
  Endif

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        aux_prof_COEF_ad(i) % debye_prof(:,:)  = 0._JPRB
     Endif
  End Do

  !0.3 predictor AD-variables on COEF levels
  !------------------------------------------

  Call rttov_init_predictor( temp0_ad%predictors )


  !0.4 raytracing AD-variables
  !---------------------------

  Call rttov_init_raytracing( temp0_ad%raytracing )
  Call rttov_init_raytracing( temp0_ad%raytracing_COEF )
  
   Do i = 1, nprofiles

  ! other 
     sunglint_ad  (i) % GLINT     =0._JPRB
     sunglint_ad  (i) % OMEGA     =0._JPRB
     temp0_ad%ircld % XSTRCLR(i)   =0._JPRB
     if(addclouds)then
       temp0_ad%ircld% xstr  (:,i)  =0._JPRB
     endif
      
   enddo


  !0.5 path opdep AD-variables on COEF levels
  !--------------------------------------------------
  ! allocated with the direct arrays
  ! on USER levels

  !0.6 transmission AD-variables on USER levels
  !--------------------------------------------
  ! allocated with the direct arrays

  !0.7 emissivity and reflectivity AD-variables - SURFACE
  !-------------------------------------------------------
   fresnrefl_ad(:)=0._JPRB
  ! emissivity_ad is init before calling
  reflectivity_ad(:) = 0._JPRB

  !--------------------------------------------------
  !1. AD of radiative transfer integration - USER levs
  !--------------------------------------------------

!!$    Allocate(auxrad_stream_ad % up      (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(5))
!!$    Allocate(auxrad_stream_ad % down    (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(6))
    Allocate(auxrad_stream_ad % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(16))
    Allocate(auxrad_stream    % rad_tmp          (0:nstreams,nchannels) ,stat= alloc_status(5))
    Allocate(auxrad_stream    % meanrad_up       (0:nstreams,nchannels) ,stat= alloc_status(6))
    Allocate(auxrad_stream    % meanrad_up_2d    (0:nstreams,nchannels,profiles(1) % nlevels) ,stat= alloc_status(7))
    Allocate(auxrad_stream    % meanrad_down     (0:nstreams,nchannels) ,stat= alloc_status(8))
    Allocate(auxrad_stream    % meanrad_down_ref (0:nstreams,nchannels) ,stat= alloc_status(9))

    If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of auxrad_stream_ad")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
    endif

    auxrad_stream_ad % cloudy =0._JPRB

  addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)

  Call rttov_integrate_ad(                   &
         & addcosmic,                        &! in
         & addaerosl,                        &
         & addclouds,                        &
         & switchrad,                        &! in
         & nchannels,                        &! in
         & nprofiles,                        &! in
         & nstreams,                         &! in
         & angles,                           &! in
         & channels,                         &! in
         & lprofiles,                        &! in
         & emissivity,                       &! in
         & emissivity_ad,                    &! inout
         & reflectivity,                     &! in
         & reflectivity_ad,                  &! inout
         & fresnrefl,                        &
         & fresnrefl_ad,                     &
         & sunglint,                         &!  in
         & sunglint_ad,                      &
         & transmission,                     &! in
         & transmission_ad,                  &! inout
         & transmission_scatt_ir_stream,     &
         & transmission_scatt_ir_stream_ad,  &
         & profiles,                         &! in
         & profiles_ad,                      &! inout  (input only due to mem alloc)
         & aux_prof,                         &! in
         & aux_prof_ad,                      &! inout
         & coef,                             &! in
         & temp0%raytracing,                 &! in
         & temp0_ad%raytracing,              &! in
         & temp0%ircld,                      &! in
         & temp0_ad%ircld,                   &! in
         & radiancedata,                     &! in
         & auxrad ,                          &! in
         & auxrad_stream,                    &! in
         & auxrad_stream_ad,                 &! in
         & radiancedata_ad  ) ! inout  (output if converstion Bt -> rad)  

  !---------------------------------------
  !2. AD of channel emissivities - SURFACE
  !---------------------------------------

 If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then

         call rttov_fresnel_ad(&
          & nchannels,         &
          & channels,          &
          & nprofiles,         &
          & lprofiles,         &
          & profiles,          &
          & coef,              &
          & sunglint,          &
          & sunglint_ad,       &
          & fresnrefl,         &
          & fresnrefl_ad)  

!CDIR NOVECTOR
         Do i = 1, nprofiles
           sunglint_ad  (i) % beta     => zbeta_ad     (1:coef%ws_nomega,i)
           sunglint_ad  (i) % psi      => zpsi_ad      (1:coef%ws_nomega,i)
         End Do

         call rttov_refsun_ad(     &
          & nprofiles,             &
          & profiles,              &
          & profiles_ad,           &
          & coef,                  &
          & aux_prof,              &
          & sunglint,              &
          & sunglint_ad,           &
          & temp0%raytracing,      &
          & temp0_ad%raytracing)  
       endif
     endif

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        emissivity_ad(:) = -reflectivity_ad(:) + emissivity_ad(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
           !Microwave
        Call rttov_calcemis_mw_ad ( &
               & profiles,         &! in
               & profiles_ad,      &! inout
               & angles,           &! in
               & coef,             &! in
               & nchannels,        &! in
               & nprofiles,        &! in
               & channels,         &! in
               & lprofiles,        &! in
               & transmission,     &! in
               & transmission_ad,  &! inout
               & calcemis,         &! in
               & emissivity_ad,    &! inout
               & reflectivity_ad  ) ! inout  
     Else
        ! Hires
        emissivity_ad(:) = -reflectivity_ad(:) + emissivity_ad(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        emissivity_ad(:) = -reflectivity_ad(:) + emissivity_ad(:)
     End Where

  Else
     ! reflectivity for all channels
     emissivity_ad(:) = -reflectivity_ad(:) + emissivity_ad(:)
  End If

  !------------------------------------
  !3. AD of transmittances - USER levs
  !------------------------------------

  If (coef%fmv_model_ver == 9) Then

  Call rttov_transmit_9_solar_ad( &
         & addaerosl,                       &! in
         & addclouds,                       &! in
         & nchannels,                       &! in
         & nprofiles,                       &! in
         & profiles(1)%nlevels,             &! in
         & channels,                        &! in
         & lprofiles,                       &! in
         & profiles,                        &! in
         & aux_prof,                        &! in
         & aux_prof_ad,                     &! inout
         & coef,                            &! in
         & temp0%raytracing,                &! in
         & temp0_ad%raytracing,             &! in
         & temp0%ircld,                     &! in
         & temp0%opdp_path,                 &! in
         & temp0_ad%opdp_path,              &! in
         & odsun_layer,                     &! out
         & odsun_singlelayer,               &! out
         & od_frac,                         &! out
         & transmission,                    &! in
         & transmission_ad,                 &
         & transmission_scatt_ir_stream,    &
         & transmission_scatt_ir_stream_ad, &! inout  
         & tausun_ref,                      &! in
         & tausun_ref_surf,                 &! in
         & tausun_layer,                    &! in
         & tausun_surf)                      ! in
              
  endif
  Call rttov_transmit_ad(                    &
         & addaerosl,                        &! in
         & addclouds,                        &! in
         & nchannels,                        &! in
         & nprofiles,                        &! in
         & profiles(1)%nlevels,              &! in
         & channels,                         &! in
         & lprofiles,                        &! in
         & aux_prof,                         &! in
         & aux_prof_ad,                      &! inout
         & coef,                             &! in
         & temp0%ircld,                      &! in
         & temp0%opdp_path,                  &! inout
         & temp0_ad%opdp_path,               &! inout
         & od_layer,                         &! in
         & transmission,                     &! in
         & transmission_ad,                  &
         & transmission_scatt_ir_stream,     &! in
         & transmission_scatt_ir_stream_ad,  & ! inout   
         & tau_ref,                          &! in         
         & tau_ref_surf,                     &! in 
         & tau_surf,                         &! in 
         & tau_layer)                         ! in 
 
  !-------------------------------------------------------------
  !4. AD of optical depths of aerosols and/or clouds - USER levs
  !-------------------------------------------------------------

 If(addaerosl.or.addclouds)Then 
    Call rttov_opdpscattir_ad(           &
       & nchannels,                      &
       & nprofiles,                      &
       & profiles(1)%nlevels,            &
       & channels,                       &
       & lprofiles,                      &
       & aux_prof,                       &
       & aux_prof_ad,                    &
       & profiles,                       &
       & profiles_ad,                    &
       & coef,                           &
       & coef_scatt_ir,                  &
       & temp0%raytracing,               &
       & temp0_ad%raytracing,            &
       & transmission_scatt_ir,          &
       & transmission_scatt_ir_ad,       &
       & transmission_scatt_ir_stream,   &
       & transmission_scatt_ir_stream_ad,&
       & optp,                           &
       & temp0%ircld,                    &
       & temp0_ad%ircld,                 &
       & addaerosl,                      &
       & addclouds)  
  Endif


  !----------------------------------------------------
  !5. AD of cloud streams and distributions - USER levs
  !----------------------------------------------------

  if(addclouds)then
  
    Call rttov_cldstr_ad(&
       & nprofiles,      &! in
       & profiles,       &! in
       & profiles_ad,    &
       & coef,           &! in
       & temp0%ircld,    &
       & temp0_ad%ircld, &
       & addclouds)       ! out  
       
     do i=1,nprofiles
       temp0_ad%ircld%XSTRCLR(i)=0._jprb
     enddo
  endif

  !------------------------------------
  !6. AD of optical depth interpolation 
  !------------------------------------

    If (addinterp) Then
      Call rttov_intavg_chan_ad(                 &
	   & nchannels,                          & !in 
           & sun,                                & !in
           & temp0%profiles_COEF(1)%nlevels,     & !in
           & profiles(1)%nlevels,                & !in
           & nprofiles,                          &! in
           & lprofiles,                          &! in
           & temp0%profiles_COEF,                & !in     only p and addsolar used this call 
           & profiles,                           & !in     only p and addsolar used this call 
           & profiles_ad,                        & !inout  only p  used this call 
           & temp0%opdp_path_COEF,               & !in  
           & temp0_ad%opdp_path_COEF,            & !inout
           & temp0%opdp_path,                    & !in  
           & temp0_ad%opdp_path,                 & !in
           & errorstatus)                          !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else
     temp0_ad%opdp_path_COEF%atm_layer(:,:) = temp0_ad%opdp_path_COEF%atm_layer(:,:) + temp0_ad%opdp_path%atm_layer(:,:)
     temp0_ad%opdp_path_COEF%sun_layer(:,:) = temp0_ad%opdp_path_COEF%sun_layer(:,:) + temp0_ad%opdp_path%sun_layer(:,:)
  EndIf

!! testop omitted

  !---------------------------------------------------------
  !7. AD of atmospheric and solar optical depths - COEF levs 
  !---------------------------------------------------------

  ! optical depth arrays allocated at start of AD-code


  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9_solar_ad  (  &
       & nchannels,                 &! in
       & nprofiles,                 &! in
       & coef%nlevels,              &! in
       & channels,                  &! in
       & lprofiles,                 &! in
       & profiles,                  &! in    ! for solar switch 
       & temp0%predictors,          &! in
       & temp0_ad%predictors,       &! inout
       & aux_prof_COEF,             &! in
       & aux_prof_COEF_ad,          &! inout
       & coef,                      &! in
       & temp0%opdp_path_COEF,      &! in
       & temp0_ad%opdp_path_COEF,   &! inout
       & opdpsun_ref_COEF)           ! in
    Call rttov_opdep_9_ad(         &
       & nchannels,                &! in
       & nprofiles,                &! in
       & coef%nlevels,             &! in
       & channels,                 &! in
       & lprofiles,                &! in
       & temp0%predictors,         &! in
       & temp0_ad%predictors,      &! inout
       & aux_prof_COEF,            &! in
       & aux_prof_COEF_ad,         &! inout
       & coef,                     &! in
       & temp0%opdp_path_COEF,     &! in
       & temp0_ad%opdp_path_COEF,  &! inout
       & opdp_ref_COEF)             ! in
   else
     Call rttov_opdep_ad(         &
       & nchannels,               &! in
       & nprofiles,               &! in
       & coef%nlevels,            &! in
       & channels,                &! in
       & lprofiles,               &! in
       & temp0%predictors,        &! in
       & temp0_ad%predictors,     &! inout
       & aux_prof_COEF,           &! in
       & aux_prof_COEF_ad,        &! inout
       & coef,                    &! in
       & temp0%opdp_path_COEF,    &! in
       & temp0_ad%opdp_path_COEF, &! inout
       & opdp_ref_COEF)        ! in
   endif

  !-------------------------------------------------------
  !8. AD of RTTOV-7 RTTOV-8 RTTOV-9 predictors - COEF levs 
  !-------------------------------------------------------
  If (coef%fmv_model_ver == 7) Then
        Call rttov_setpredictors_7_ad(        &
               & nprofiles,                   &
               & temp0%profiles_COEF,         &! in
               & temp0_ad%profiles_COEF,      &! inout
               & angles_COEF,                 &! in
               & coef,                        &! in
               & aux_prof_COEF,               &! in
               & temp0%predictors,            &! in
               & temp0_ad%predictors,         &! in
               & temp0%raytracing_COEF,       &!
               & temp0_ad%raytracing_COEF )    ! in   
  Else If (coef%fmv_model_ver == 8) Then
    Call rttov_setpredictors_8_ad(        &
           & nprofiles,                   &
           & temp0%profiles_COEF,         &! in
           & temp0_ad%profiles_COEF,      &! inout
           & angles_COEF,                 &! in
           & coef,                        &! in
           & aux_prof_COEF,               &! in
           & temp0%predictors,            &! in
           & temp0_ad%predictors,         &! in
           & temp0%raytracing_COEF,       &
           & temp0_ad%raytracing_COEF )    ! in   
  Else If (coef%fmv_model_ver == 9) Then

         Call rttov_setpredictors_9_solar_ad( &
               & nprofiles,                   &
               & temp0%profiles_COEF,         &! in
               & temp0_ad%profiles_COEF,      &! inout
               & angles_COEF,                 &! in
               & coef,                        &! in
               & temp0%predictors,            &! in
               & temp0_ad%predictors,         &
               & temp0%raytracing_COEF,       &
               & temp0_ad%raytracing_COEF )  

        Call rttov_setpredictors_9_ad(        &
               & nprofiles,                   &
               & temp0%profiles_COEF,         &! in
               & temp0_ad%profiles_COEF,      &! inout
               & angles_COEF,                 &! in
               & coef,                        &! in
               & temp0%predictors,            &! in
               & temp0_ad%predictors,         &
               & temp0%raytracing_COEF,       &
               & temp0_ad%raytracing_COEF )    ! in  

  End If

  !-------------------------------------------------------
  !9. AD of common geometric set-up for the RT integration
  !-------------------------------------------------------
 
  !9.1 COEF levs
  !-------------
   If(addinterp) then
     Call rttov_setgeometry_ad(          &
             & nprofiles,                &
             & temp0%profiles_COEF,      &
             & temp0_ad%profiles_COEF,   &
             & aux_prof_COEF,            &! in
             & coef,                     &! in
             & angles_COEF,              &
             & temp0%raytracing_COEF,    &
             & temp0_ad%raytracing_COEF   )     ! out  
   Else
     Do i = 1, nprofiles
       angles_COEF(i) = angles(i)
       temp0_ad%raytracing % HGPL(:,i)     = temp0_ad%raytracing % HGPL(:,i)     + temp0_ad%raytracing_COEF % HGPL(:,i)
       temp0_ad%raytracing % DAIR(:,i)     = temp0_ad%raytracing % DAIR(:,i)     + temp0_ad%raytracing_COEF % DAIR(:,i)
       temp0_ad%raytracing % DMAIR(:,i)    = temp0_ad%raytracing % DMAIR(:,i)    + temp0_ad%raytracing_COEF % DMAIR(:,i)
       temp0_ad%raytracing % R(:,i)        = temp0_ad%raytracing % R(:,i)        + temp0_ad%raytracing_COEF % R(:,i)  
       temp0_ad%raytracing % RATOESUN(:,i) = temp0_ad%raytracing % RATOESUN(:,i) + temp0_ad%raytracing_COEF % RATOESUN(:,i)
       temp0_ad%raytracing % RATOESAT(:,i) = temp0_ad%raytracing % RATOESAT(:,i) + temp0_ad%raytracing_COEF % RATOESAT(:,i)
       temp0_ad%raytracing % ZASUN(:,i)    = temp0_ad%raytracing % ZASUN(:,i)    + temp0_ad%raytracing_COEF % ZASUN(:,i) 
       temp0_ad%raytracing % ZASAT(:,i)    = temp0_ad%raytracing % ZASAT(:,i)    + temp0_ad%raytracing_COEF % ZASAT(:,i)
       temp0_ad%raytracing % INT(:,i)      = temp0_ad%raytracing % INT(:,i)      + temp0_ad%raytracing_COEF % INT(:,i)
       temp0_ad%raytracing % HL(:,i)       = temp0_ad%raytracing % HL(:,i)       + temp0_ad%raytracing_COEF % HL(:,i)
       temp0_ad%raytracing % PPW(:,i)      = temp0_ad%raytracing % PPW(:,i)      + temp0_ad%raytracing_COEF % PPW(:,i)
       temp0_ad%raytracing % DISPCO2(:,i)  = temp0_ad%raytracing % DISPCO2(:,i)  + temp0_ad%raytracing_COEF % DISPCO2(:,i)
       temp0_ad%raytracing % PATHSAT(:,i)  = temp0_ad%raytracing % PATHSAT(:,i)  + temp0_ad%raytracing_COEF % PATHSAT(:,i)
       temp0_ad%raytracing % PATHSUN(:,i)  = temp0_ad%raytracing % PATHSUN(:,i)  + temp0_ad%raytracing_COEF % PATHSUN(:,i)
       temp0_ad%raytracing % PATHEFF(:,i)  = temp0_ad%raytracing % PATHEFF(:,i)  + temp0_ad%raytracing_COEF % PATHEFF(:,i)
       temp0_ad%raytracing % LTICK(:,i)    = temp0_ad%raytracing % LTICK(:,i)    + temp0_ad%raytracing_COEF % LTICK(:,i)
    enddo
  Endif
 
  !9.1 USER levs
  !-------------
    Call rttov_setgeometry_ad(      &
             & nprofiles,           &
             & profiles,            &
             & profiles_ad,         &
             & aux_prof,            &! in
             & coef,                &! in
             & angles,              &
             & temp0%raytracing,    &
             & temp0_ad%raytracing   )     ! out  

  !---------------------------------------------------------
  !10. AD of cloud top, surface levels, ice cloud parameters
  !---------------------------------------------------------

  !10.1 COEF levs
  !-------------
 Do i = 1, nprofiles
     If(addinterp) then
       Call rttov_profaux_ad(            &
           & .false.,                    &! no cloud computations on coef levels
           & temp0%profiles_COEF(i),     &! in
           & temp0_ad%profiles_COEF(i),  &! in
           & coef,                       &! in
           & aux_prof_COEF(i),           &! in
           & aux_prof_COEF_ad(i))         ! inout 
     Else 
       aux_prof_ad(i) % nearestlev_surf = aux_prof_ad(i) % nearestlev_surf &
                                           & + aux_prof_COEF_ad(i) % nearestlev_surf
       aux_prof_ad(i) % pfraction_surf  = aux_prof_ad(i) % pfraction_surf &
                                           & + aux_prof_COEF_ad(i) % pfraction_surf
       aux_prof_ad(i) % nearestlev_ctp  = aux_prof_ad(i) % nearestlev_ctp &
                                           & + aux_prof_COEF_ad(i) % nearestlev_ctp 
       aux_prof_ad(i) % pfraction_ctp   = aux_prof_ad(i) % pfraction_ctp &
                                           & + aux_prof_COEF_ad(i) % pfraction_ctp
       aux_prof_ad(i) % cfraction       = aux_prof_ad(i) % cfraction &
                                           & + aux_prof_COEF_ad(i) % cfraction 
       If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
         aux_prof_ad(i) % debye_prof(:,:) = aux_prof_ad(i) % debye_prof(:,:) &
	                                   & + aux_prof_COEF_ad(i) % debye_prof(:,:)
       Endif
     Endif
  End Do

  !10.2 USER levs
  !-------------
  Do i = 1, nprofiles
     Call rttov_profaux_ad( &
            & addclouds,       &! in
            & profiles(i),     &! in
            & profiles_ad(i),  &! inout
            & coef,            &! in
            & aux_prof(i),     &! in
            & aux_prof_ad(i))   ! inout  
  End Do
  !------------------------------------------------------------------
  !11 AD of check input data is within suitable physical limits - COEF levs
  !------------------------------------------------------------------

  If ( apply_reg_limits ) Then
    Do i = 1, nprofiles
 
      Call rttov_checkinput_ad(           &
           & profiles_COEF_ref( i ),      &!in
           & temp0_ad%profiles_COEF( i ), &!inout
           & coef) !in 

    End Do

  End If 
  !-----------------------------------------------------------------
  !12. AD of profile variable interpolation - COEF levs to USER levs
  !-----------------------------------------------------------------

! intvar logical array set by direct code
  If (addinterp) Then
     Call rttov_intavg_prof_ad( &
            & nprofiles,                         &! in
            & profiles(1)%nlevels,               & !in
            & temp0%profiles_COEF(1)%nlevels,    & !in
            & profiles,                          & !in     source variables
            & profiles_ad,                       & !in     source variables
            & temp0%profiles_COEF,               & !inout  target variables
            & temp0_ad%profiles_COEF,            & !inout  target variables
            & errorstatus)                     !out
     If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
     End If
  Else 
! COEF levs same as USER levs
     Do i = 1, nprofiles
        profiles_ad(i)%t(:)   =  profiles_ad(i)%t(:) + temp0_ad%profiles_COEF(i)%t(:) 
        profiles_ad(i)%q(:)   =  profiles_ad(i)%q(:) + temp0_ad%profiles_COEF(i)%q(:)
        if(profiles(i) % clw_data)   profiles_ad(i)%clw(:) =  profiles_ad(i)%clw(:) + temp0_ad%profiles_COEF(i)%clw(:)
        if(profiles(i) % ozone_data) profiles_ad(i)%o3(:)  =  profiles_ad(i)%o3(:)  + temp0_ad%profiles_COEF(i)%o3(:)
        if(profiles(i) % co2_data)   profiles_ad(i)%co2(:) =  profiles_ad(i)%co2(:) + temp0_ad%profiles_COEF(i)%co2(:)
        if(profiles(i) % n2o_data)   profiles_ad(i)%n2o(:) =  profiles_ad(i)%n2o(:) + temp0_ad%profiles_COEF(i)%n2o(:)   
        if(profiles(i) % co_data)    profiles_ad(i)%co(:)  =  profiles_ad(i)%co(:)  + temp0_ad%profiles_COEF(i)%co(:)    
        if(profiles(i) % ch4_data)   profiles_ad(i)%ch4(:) =  profiles_ad(i)%ch4(:) + temp0_ad%profiles_COEF(i)%ch4(:)  
     End Do
  EndIf
  

! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
    Do i = 1, nprofiles
      profiles_ad(i) % ozone_Data = .False.  ! no meaning
      profiles_ad(i) % co2_Data   = .False.  ! no meaning
      profiles_ad(i) % n2o_Data   = .False.  ! no meaning
      profiles_ad(i) % co_Data    = .False.  ! no meaning
      profiles_ad(i) % ch4_Data   = .False.  ! no meaning
      profiles_ad(i) % clw_Data   = .False.  ! no meaning

     profiles_ad(i) % S2m % t      = profiles_ad(i) % S2m % t + temp0_ad%profiles_COEF(i) % S2m % t
     profiles_ad(i) % S2m % q      = profiles_ad(i) % S2m % q + temp0_ad%profiles_COEF(i) % S2m % q
     profiles_ad(i) % S2m % o      = profiles_ad(i) % S2m % o + temp0_ad%profiles_COEF(i) % S2m % o
     profiles_ad(i) % S2m % p      = profiles_ad(i) % S2m % p + temp0_ad%profiles_COEF(i) % S2m % p
     profiles_ad(i) % S2m % u      = profiles_ad(i) % S2m % u + temp0_ad%profiles_COEF(i) % S2m % u
     profiles_ad(i) % S2m % v      = profiles_ad(i) % S2m % v + temp0_ad%profiles_COEF(i) % S2m % v
     profiles_ad(i) % S2m % wfetc  = profiles_ad(i) % S2m % wfetc + temp0_ad%profiles_COEF(i) % S2m % wfetc

     profiles_ad(i) % skin % surftype   = -1       ! no meaning
     profiles_ad(i) % skin % watertype  = -1       ! no meaning
     profiles_ad(i) % skin % t          = profiles_ad(i) % skin % t         + temp0_ad%profiles_COEF(i) % skin % t
     profiles_ad(i) % skin % fastem(:)  = profiles_ad(i) % skin % fastem(:) + temp0_ad%profiles_COEF(i) % skin % fastem(:)

     profiles_ad(i) % ctp        = profiles_ad(i) % ctp       + temp0_ad%profiles_COEF(i) % ctp 
     profiles_ad(i) % cfraction  = profiles_ad(i) % cfraction + temp0_ad%profiles_COEF(i) % cfraction

    profiles_ad(i) % idg    = profiles_ad(i) % idg + temp0_ad%profiles_COEF(i) % idg
    profiles_ad(i) % ish    = profiles_ad(i) % ish + temp0_ad%profiles_COEF(i) % ish

    profiles_ad(i) % zenangle   = profiles_ad(i) % zenangle + temp0_ad%profiles_COEF(i) % zenangle
    profiles_ad(i) % azangle    = profiles_ad(i) % azangle  + temp0_ad%profiles_COEF(i) % azangle

    profiles_ad(i) % sunzenangle = profiles_ad(i) % sunzenangle + temp0_ad%profiles_COEF(i) % sunzenangle
    profiles_ad(i) % sunazangle  = profiles_ad(i) % sunazangle  + temp0_ad%profiles_COEF(i) % sunazangle
    profiles_ad(i) % latitude    = profiles_ad(i) % latitude    + temp0_ad%profiles_COEF(i) % latitude
    profiles_ad(i) % elevation   = profiles_ad(i) % elevation   + temp0_ad%profiles_COEF(i) % elevation
!    profiles_ad(i) % addsolar    = profiles_ad(i) % addsolar    + temp0_ad%profiles_COEF(i) % addsolar
!    profiles_ad(i) % addrefrac   = profiles_ad(i) % addrefrac   + temp0_ad%profiles_COEF(i) % addrefrac
!
  End Do      ! over profiles

  !-----------------------------------------
  !13. deallocate memory for local variables
  !-----------------------------------------



  if(addaerosl)then
    Do i=1,nprofiles

      DeAllocate( aux_prof_ad(i)%relhum   ,stat= alloc_status(7))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "deallocation of aux_prof_ad" )' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      endif


      DeAllocate( aux_prof(i)%iaernum  ,stat= alloc_status(7))
      DeAllocate( aux_prof(i)%iaertyp  ,stat= alloc_status(8))
      DeAllocate( aux_prof(i)%relhum   ,stat= alloc_status(9))
      DeAllocate( aux_prof(i)%relhumref,stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "deallocation of aux_prof" )' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
        Return
      endif

    enddo
  endif

  if(addaerosl.or.addclouds)then

      Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
        nchannels, profiles(1) %nlevels, 0_jpim )
      Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_ad, &
        nchannels, profiles(1) %nlevels, 0_jpim ) 
      Call rttov_alloc_trans_scatt_ir( alloc_status(3), transmission_scatt_ir_stream, &
        nchannels, profiles(1) %nlevels, 0_jpim )
      Call rttov_alloc_trans_scatt_ir( alloc_status(4), transmission_scatt_ir_stream_ad, &
        nchannels, profiles(1) %nlevels, 0_jpim )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of trans_scatt_ir" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    endif

  endif




  ! Deallocate model profiles atmospheric arrays with model levels dimension



    Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
    & 0_jpim, nstreams = nstreams, calc = .true. )
    Call rttov_alloc_transmission( alloc_status(2), transmission_ad, profiles(1) % nlevels, nchannels, &
    & 0_jpim, nstreams = nstreams, calc = .true. )

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of transmission_ad" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    endif


    DeAllocate(auxrad_stream_ad % cloudy ,stat= alloc_status(12))


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream_ad" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    endif


  
    DeAllocate(auxrad_stream % up     ,stat= alloc_status(1))
    DeAllocate(auxrad_stream % down   ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % down_ref   ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % cloudy ,stat= alloc_status(12))
    DeAllocate(auxrad_stream % rad_tmp            ,stat= alloc_status(1))
    DeAllocate(auxrad_stream % meanrad_up         ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % meanrad_up_2d      ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % meanrad_down       ,stat= alloc_status(3))
    DeAllocate(auxrad_stream % meanrad_down_ref   ,stat= alloc_status(4))

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    endif

     if(addsun)then
      if(addaerosl.or.addclouds)then 
        DeAllocate(auxrad_stream % FAC1_1    ,stat= alloc_status(91))
        DeAllocate(auxrad_stream % FAC2_1    ,stat= alloc_status(92))
        DeAllocate(auxrad_stream % FAC3_1    ,stat= alloc_status(93))
        DeAllocate(auxrad_stream % FAC4_1    ,stat= alloc_status(94))
        DeAllocate(auxrad_stream % FAC5_1    ,stat= alloc_status(95))
        DeAllocate(auxrad_stream % FAC6_1    ,stat= alloc_status(96))
        DeAllocate(auxrad_stream % FAC7_1    ,stat= alloc_status(97))
        DeAllocate(auxrad_stream % FAC1_2    ,stat= alloc_status(98))
        DeAllocate(auxrad_stream % FAC2_2    ,stat= alloc_status(99))
        DeAllocate(auxrad_stream % FAC3_2    ,stat= alloc_status(100))
        DeAllocate(auxrad_stream % FAC4_2    ,stat= alloc_status(101))
        DeAllocate(auxrad_stream % FAC5_2    ,stat= alloc_status(102))
        DeAllocate(auxrad_stream % FAC6_2    ,stat= alloc_status(103))
        DeAllocate(auxrad_stream % FAC7_2    ,stat= alloc_status(104))
        DeAllocate(auxrad_stream % FAC1_3    ,stat= alloc_status(105))
        DeAllocate(auxrad_stream % FAC2_3    ,stat= alloc_status(106))
        DeAllocate(auxrad_stream % FAC3_3    ,stat= alloc_status(107))
        DeAllocate(auxrad_stream % FAC4_3    ,stat= alloc_status(108))
        DeAllocate(auxrad_stream % FAC5_3    ,stat= alloc_status(109))
        DeAllocate(auxrad_stream % FAC6_3    ,stat= alloc_status(110))
        DeAllocate(auxrad_stream % FAC7_3    ,stat= alloc_status(111))
      endif
    endif


  If ( apply_reg_limits ) Then
    Call rttov_alloc_prof( alloc_status(3), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 0_jpim, blob = profiles_COEF_blob_ref )
  End If 

  If( Any(alloc_status /= 0) ) Then
    errorstatus = errorstatus_fatal
    Write( errMessage, '( "deallocation")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
    Return
  End If


  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 0_jpim,                                  &
    & temp0 = temp0, temp0_ad = temp0_ad,      &   
    & temp1 = temp1, temp1_ad = temp1_ad,      &   
    & temp2 = temp,  temp2_ad = temp_ad )  

  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  


  IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_ad
