!
Subroutine rttov_direct( &
      & errorstatus,     &! out
      & nprofiles,       &! in
      & nchannels,       &! in
      & channels,        &! in
      & lprofiles,       &! in
      & addinterp,       &! in
      & profiles,        &! in 
      & coef,            &! in
      & coef_scatt_ir,   &
      & optp,            &
      & addsun,          &! in
      & addaerosl,       &! in
      & addclouds,       &! in
      & calcemis,        &! in
      & emissivity,      &! inout
      & transmission,    &! inout
      & radiancedata,    &! inout 
      & temp )    
  !
  ! Description:
  ! to compute multi-channel level to space transmittances,
  ! top of atmosphere and level to space radiances and brightness
  ! temperatures and optionally surface emissivities, for many
  ! profiles in a single call, for satellite
  ! infrared or microwave sensors. The code requires a coefficient file
  ! for each sensor for which simulated radiances are requested.
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is described in the following:
  !
  ! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
  ! in the microwave region: a fast model. Applied Optics 27  3244-3249
  !
  ! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
  ! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
  ! librarian at ECMWF).
  !
  ! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
  ! Transfer Model for Assimilation of Satellite Radiance Observations.
  ! QJRMS, 125, 1407-1425.
  !
  ! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
  ! fast radiative transfer model for the assimilation of radiance
  ! observations. ECMWF Research Dept. Tech. Memo. 345
  ! (available from the librarian at ECMWF).
  !
  ! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
  ! transfer model for the infrared atmospheric sounding interferometer.
  ! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
  !
  ! Rochon Y.J., L. Garand, D.S. Turner and S. Polavarapu Jacobian mapping 
  ! between  vertical coordinate systems in data assimilation (submitted QJRMS
  ! June 2006)
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          13/8/92.  For version 2.
  !                    ksat added to argument list; ssu included;
  !                    internal changes to move big arrays from commons to
  !                    arguments and to introduce taskcommons
  !           8/7/97   added ozone and extended water vapour in control vector
  !        01/05/2000  F90 code
  !        21/08/2000  Interface to rtint changed to include pref (surface reflectivity).
  !                    (Stephen English)
  !        31/01/2001  More cloud computations. stored in radov (F. Chevallier)
  !        6/2/2001    pgrody and knav etc arrays removed from call (R Saunders)
  !        18/01/2002  Thread safe (D.Salmond)
  !        01/12/2002  New F90 code with structures (P Brunel A Smith)
  !        02/01/2003  More comments added (R Saunders)
  !        24/01/2003  Error return code by input profile (P Brunel)
  !                    Add WV Continuum and CO2 capability
  !        02/06/2004  Change tests on id_comp_lvl == 7 by tests on fmv_model_ver (P. Brunel)
  !        08/12/2005  Added surface humidity to lowest mean layer q (R Saunders) 
  !        16/01/2006  Marco Matricardi (ECMWF):
  !              --    IASI capability added.
  !              --    Variable trace gases added for IASI and AIRS.
  !              --    Solar radiation added for IASI and AIRS
  !              --    Altitude dependent local zenith angle added.
  !              --    Linear in tau approximation for RT equation added.
  !         1/07/2006  Marco Matricardi (ECMWF):
  !              --    Parameterization of multiple scattering for aerosols
  !                    and clouds added 
  !        22/01/2007  Removed polarisation indexing R. Saunders.
  !        29/06/2007  Introduce interpolator (P.J.Rayer)
  !              --    Profiles on USER levs -> COEF levs
  !              --    Duplicate profaux and setgeometry (USER and COEF levs)
  !              --    Predict atm/sol optical depths on COEF levs -> USER levs
  !              --    Calculate transmittance and integrate RT on USER levs
  !        22/08/2007  Remove Allocates (D Salmond)
  !        11/10/2007  Move iaernum and iaertyp to profile_aux P.Marguinaud
  !        26/09/2008  Nullify temp pointers; this is for the NEC 
  !                    under -Chopt. P. Marguinaud
  !        03/10/2008  Fix bug in checking addsolar
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  ! A user guide and technical documentation is available at
  ! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :     &
        & errorstatus_success ,&
        & errorstatus_warning ,&
        & errorstatus_fatal   ,&
        & max_optical_depth   ,&
        & sensor_id_mw        ,&
        & sensor_id_ir        ,&
        & sensor_id_hi        ,&         
        & sensor_id_po        ,&
        & ncldtyp             ,&
        & ngases_max
	
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

 ! Imported Type Definitions:
  Use rttov_types, Only :            &
        & rttov_coef                 ,&
        & rttov_coef_scatt_ir        ,&
        & transmission_scatt_ir_type ,&
        & rttov_optpar_ir            ,&
        & profile_Type               ,&
        & geometry_Type              ,&
        & predictors_Type            ,&
        & profile_aux                ,&
        & opdp_path_type             ,&
        & transmission_type          ,&
        & radiance_Type              ,&
        & radiance_aux               ,&
        & raytracing_type            ,&
        & sunglint_type              ,&
        & ircld_type                 ,&
        & blob_type                  ,&
        & rttov_temp

  Use rttov_global, Only: &
        & verbose_checkinput_warnings

  Use parkind1, Only : jpim,jprb
  Implicit None


  !subroutine arguments:

  Integer(Kind=jpim),       Intent(in)    :: nprofiles                    ! Number of profiles
  Integer(Kind=jpim),       Intent(in)    :: nchannels                    ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),       Intent(in)    :: channels       (nchannels)   ! Channel indices
  Integer(Kind=jpim),       Intent(in)    :: lprofiles      (nchannels)   ! Profiles indices
  Logical,                  Intent(in)    :: addsun
  Logical,                  Intent(in)    :: addaerosl                    ! switch for aerosol computations
  Logical,                  Intent(in)    :: addclouds                    ! switch for  cloud computations
  Logical,                  Intent(in)    :: addinterp                    ! switch for User-Coef-User interpolations

  Type(profile_Type),       Intent(in)    :: profiles       (nprofiles)     ! Atmospheric profiles (supplied on User levels)
  Type(rttov_coef),         Intent(in)    :: coef                         ! RT Coefficients
  Type(rttov_coef_scatt_ir),Intent(in)    :: coef_scatt_ir                ! subrtn opdpscattir  
  Type(rttov_optpar_ir),    Intent(in)    :: optp                         ! subrtn opdpscattir

  Logical,                  Intent(in)    :: calcemis       (nchannels)   ! switch for emmissivity calc.
  Real(Kind=jprb),          Intent(inout) :: emissivity     (nchannels)   ! surface emmissivity

  Type(transmission_type),  Intent(inout) :: transmission                 ! transmittances and singlelayer optical depths (on User levels)
  Type(radiance_Type),      Intent(inout) :: radiancedata                 ! radiances (mw/cm-1/ster/sq.m) and degK

  Integer(Kind=jpim),       Intent(out)   :: errorstatus    (nprofiles)   ! return flag
  Type(rttov_temp),  Intent(inout), Optional, Target :: temp

!INTF


#include "rttov_errorreport.interface"
#include "rttov_cldstr.interface" 
#include "rttov_checkinput.interface"
#include "rttov_profaux.interface"
#include "rttov_setgeometry.interface"
#include "rttov_setpredictors_7.interface"
#include "rttov_setpredictors_8.interface"
#include "rttov_setpredictors_9.interface"
#include "rttov_setpredictors_9_solar.interface" 
#include "rttov_opdpscattir.interface" 
#include "rttov_fresnel.interface" 
#include "rttov_opdep.interface"
#include "rttov_opdep_9.interface"
#include "rttov_opdep_9_solar.interface"
#include "rttov_transmit.interface"
#include "rttov_transmit_9_solar.interface"
#include "rttov_calcemis_ir.interface"
#include "rttov_calcemis_mw.interface"
#include "rttov_integrate.interface"
#include "rttov_intavg_chan.interface"
#include "rttov_intavg_prof.interface"
#include "rttov_alloc_trans_scatt_ir.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_refsun.interface"
#include "rttov_check_temp.interface"
#include "rttov_init_profiles.interface"
#include "rttov_checkargs.interface"
#include "rttov_init_raytracing.interface"

  !local variables:

  Integer(Kind=jpim) :: i, j, prof, chan                                                 ! loop index
  Integer(Kind=jpim) :: nstreams          
  Integer(Kind=jpim) :: alloc_status(120)                                 ! memory allocation status
  Integer(Kind=jpim) :: callnum                          ! which interpolator call: 1=USER->COEF, 2=COEF->USER 
  Logical            :: sun(nchannels)                   ! solar switch located in channel sequence

  Integer(Kind=jpim) :: nvar, nc1, nc2 ! to determine number of variables for interpolation

  Logical            :: addcosmic                                         ! switch for adding temp of cosmic background

  Real(Kind=jprb)    :: reflectivity(nchannels)                           ! surface reflectivity
  Real(Kind=jprb)    :: fresnrefl         (nchannels)

  Real(Kind=jprb)    :: tau_ref           (profiles(1)%nlevels,nchannels) ! subrtn transmission_9
  Real(Kind=jprb)    :: tau_ref_surf      (nchannels)                     ! subrtn transmission_9
  Real(Kind=jprb)    :: tau_surf          (nchannels)                     ! subrtn transmission_9

  Real(Kind=jprb)    :: tausun_ref        (profiles(1)%nlevels,nchannels) ! subrtn transmission_9_solar
  Real(Kind=jprb)    :: tausun_ref_surf   (nchannels)                     ! subrtn transmission_9_solar
  Real(Kind=jprb)    :: tausun_layer      (profiles(1)%nlevels,nchannels) ! subrtn transmission_9_solar
  Real(Kind=jprb)    :: tausun_surf       (nchannels)                     ! subrtn transmission_9_solar

  Real(Kind=jprb)    :: opdp_ref          (profiles(1)%nlevels,nchannels) ! neg singlelayer atm optical depths before min threshold applied
  Real(Kind=jprb)    :: opdp_ref_COEF     (coef%nlevels,nchannels)        ! on CoefLevs
!
  Real(Kind=jprb)    :: od_layer          (profiles(1)%nlevels,nchannels) ! neg sat to layer atm optical depths
  Real(Kind=jprb)    :: od_layer_COEF     (coef%nlevels,nchannels)        ! on CoefLevs
  Real(Kind=jprb)    :: tau_layer         (profiles(1)%nlevels,nchannels) ! sat to layer transmittance

  Real(Kind=jprb)    :: opdpsun_ref       (profiles(1)%nlevels,nchannels) ! neg singlelayer sun optical depth before min threshold applied
  Real(Kind=jprb)    :: opdpsun_ref_COEF  (coef%nlevels,nchannels)        ! on CoefLevs
  Real(Kind=jprb)    :: odsun_layer       (profiles(1)%nlevels,nchannels) ! neg sat to layer sun optical depths
  Real(Kind=jprb)    :: odsun_singlelayer (profiles(1)%nlevels,nchannels) ! on CoefLevs
  Real(Kind=jprb)    :: od_frac           (nchannels)                     !

  Character (len=80) :: errMessage  
  Character (len=12) :: NameOfRoutine = 'rttov_direct'
!
  Type(geometry_Type)               :: angles        (nprofiles)          ! geometry angles
  Type(geometry_Type)               :: angles_COEF   (nprofiles)          ! geometry angles
  Type(profile_aux)                 :: aux_prof_COEF (nprofiles)          ! auxillary profiles information (COEF levs)
  Type(profile_aux)                 :: aux_prof      (nprofiles)          ! auxillary profiles information (USER levs)
  Type(radiance_aux)                :: auxrad                             ! auxillary radiances
  Type(radiance_aux)                :: auxrad_stream 
  Type(sunglint_type)               :: sunglint      (nprofiles)
!
  Type(transmission_scatt_ir_type)  :: transmission_scatt_ir_stream 
  Type(transmission_scatt_ir_type)  :: transmission_scatt_ir
  Type(rttov_temp), Target   :: temp1
  Type(rttov_temp), Pointer  :: temp0
  Integer(Kind=jpim)         :: temp_status, checkargs_status
  
  
  
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  
  !- End of header -----------------------------------------------------------------------

  !-------------
  !0. initialize
  !-------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',0_jpim,ZHOOK_HANDLE)
  errorstatus(:)  = errorstatus_success
  alloc_status(:) = 0


  Call rttov_checkargs(  &
      & addinterp,       &
      & addsun,          &
      & addaerosl,       &
      & addclouds,       &
      & checkargs_status &
  )
  
  If( checkargs_status .eq. errorstatus_fatal ) Then
     Write( errMessage, '( "checkargs fatal error")' )
     Call Rttov_ErrorReport( errorstatus_fatal, errMessage,NameOfRoutine )
     errorstatus(:)  = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
  EndIf

  Nullify( temp0 )


  !-----------------------------------------------------------------------
  !1. interpolator first call - input profiles from USER levs to COEF levs
  !-----------------------------------------------------------------------
  ! do immediately - for quick escape if target values are out of bounds

  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 1_jpim,                                  &
    & temp0 = temp0,                           &
    & temp1 = temp1,                           &
    & temp2 = temp )
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  

  Call rttov_init_profiles( nprofiles, temp0%profiles_COEF )

  Do i = 1 , nprofiles
    temp0%profiles_COEF(i) % nlevels = coef % nlevels
    temp0%profiles_COEF(i) % p(:) = coef % ref_prfl_p(:)
  Enddo


     
    If (addinterp) Then
! set up profile variables for interpolation
! easy extension to more trace following gas id codes in rttov_const
!    nvar=ngases_max 
! add more trace gases here if appropriate
    Call rttov_intavg_prof( &
      & nprofiles,                       &
      & profiles(1)%nlevels,             & !in
      & temp0%profiles_COEF(1)%nlevels,  & !in
      & profiles,                        & !in     source variables
      & temp0%profiles_COEF,             & !inout  target variables
      & errorstatus)                    !out errorstatus
    If ( Any(errorstatus == errorstatus_fatal) ) Then
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
    End If
!
    Else 
! COEF levs same as USER levs
  Do i = 1, nprofiles
      temp0%profiles_COEF(i)%t(:)   =    profiles(i)%t(:)
      temp0%profiles_COEF(i)%q(:)   =    profiles(i)%q(:)
      if(profiles(i) % clw_data)   temp0%profiles_COEF(i)%clw(:) =  profiles(i)%clw(:)
      if(profiles(i) % ozone_data) temp0%profiles_COEF(i)%o3(:)  =  profiles(i)%o3(:)
      if(profiles(i) % co2_data)   temp0%profiles_COEF(i)%co2(:) =  profiles(i)%co2(:)
      if(profiles(i) % n2o_data)   temp0%profiles_COEF(i)%n2o(:) =  profiles(i)%n2o(:)   
      if(profiles(i) % co_data)    temp0%profiles_COEF(i)%co(:)  =  profiles(i)%co(:)    
      if(profiles(i) % ch4_data)   temp0%profiles_COEF(i)%ch4(:) =  profiles(i)%ch4(:)  
  EndDo
    EndIf
! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
  Do i = 1, nprofiles
    temp0%profiles_COEF(i) % ozone_data = profiles(i) % ozone_data
    temp0%profiles_COEF(i) % co2_data   = profiles(i) % co2_data
    temp0%profiles_COEF(i) % clw_data   = profiles(i) % clw_data

    temp0%profiles_COEF(i) % n2o_data    = profiles(i) % n2o_data
    temp0%profiles_COEF(i) % co_data     = profiles(i) % co_data
    temp0%profiles_COEF(i) % ch4_data    = profiles(i) % ch4_data
    temp0%profiles_COEF(i) % aer_data    = profiles(i) % aer_data
    temp0%profiles_COEF(i) % cld_data    = profiles(i) % cld_data

    temp0%profiles_COEF(i) % S2m        = profiles(i) % S2m
    temp0%profiles_COEF(i) % skin       = profiles(i) % skin
    temp0%profiles_COEF(i) % ctp        = profiles(i) % ctp
    temp0%profiles_COEF(i) % cfraction  = profiles(i) % cfraction

    temp0%profiles_COEF(i) % idg    = profiles(i) % idg
    temp0%profiles_COEF(i) % ish    = profiles(i) % ish

    temp0%profiles_COEF(i) % zenangle   = profiles(i) % zenangle
    temp0%profiles_COEF(i) % azangle    = profiles(i) % azangle

    temp0%profiles_COEF(i) % sunzenangle = profiles(i) % sunzenangle
    temp0%profiles_COEF(i) % sunazangle  = profiles(i) % sunazangle
    temp0%profiles_COEF(i) % latitude    = profiles(i) % latitude
    temp0%profiles_COEF(i) % elevation   = profiles(i) % elevation
    temp0%profiles_COEF(i) % addsolar    = profiles(i) % addsolar
    temp0%profiles_COEF(i) % addrefrac   = profiles(i) % addrefrac  

    temp0%profiles_COEF(i) % Be     = profiles(i) % Be  
    temp0%profiles_COEF(i) % cosbk  = profiles(i) % cosbk  
!

  End Do      ! over profiles

  Call rttov_init_raytracing( temp0%raytracing )

  !------------------------------------------------------------------
  !2. check input data is within suitable physical limits - COEF levs
  !------------------------------------------------------------------

  Do i = 1, nprofiles

     Call rttov_checkinput( &
           & temp0%profiles_COEF( i ),  &!inout
           & coef,                      &!in
           & errorstatus(i)    ) !out 

  End Do


  ! test check input return code
  If ( any( errorstatus(:) == errorstatus_warning)  .and. verbose_checkinput_warnings ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_warning ) Then
           Write( errMessage, '( "checkinput warning error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_warning, errMessage, NameOfRoutine)
        End If
     End Do
  End If

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_fatal ) Then
           ! Some unphysical values; Do not run RTTOV
           Write( errMessage, '( "checkinput fatal error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        End If
     End Do
     ! nothing processed so all profiles get the fatal error code
     ! user will know which profile
     errorstatus(:) = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
  End If



  !------------------------------------------------------------------------
  !3. determine cloud top, surface levels, ice cloud parameters
  !------------------------------------------------------------------------

  !3.1 USER levs
  !-------------
  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
        Allocate( aux_prof(i) % debye_prof( 5 , profiles(1)%nlevels ), stat= alloc_status(1)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof(i) % debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           Return
        End If
     Endif

     if(addclouds)then
       Allocate( aux_prof(i) % dg(profiles(1)%nlevels ), stat= alloc_status(1)  )
       Allocate( aux_prof(i) % fac1_dg(profiles(1)%nlevels ), stat= alloc_status(2)  )
       Allocate( aux_prof(i) % fac2_dg(profiles(1)%nlevels ), stat= alloc_status(3)  )
       Allocate( aux_prof(i) % fac3_dg(profiles(1)%nlevels ), stat= alloc_status(4)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof(i) % dg")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     endif

       Call rttov_profaux( &
            & addclouds,     &! in
            & profiles(i),   &! in
            & coef,          &! in
            & aux_prof(i))    ! inout
  End Do

  !3.2 COEF levs
  !-------------
  
  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
        Allocate( aux_prof_COEF(i) % debye_prof( 5 , coef%nlevels ), stat= alloc_status(1)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof_COEF(i) % debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Endif

     If(addinterp) then
       Call rttov_profaux(             &
           & .false.,                  &
           & temp0%profiles_COEF(i),   &! in
           & coef,                     &! in
           & aux_prof_COEF(i))    ! inout 
     Else
       aux_prof_COEF(i) % nearestlev_surf = aux_prof(i) % nearestlev_surf
       aux_prof_COEF(i) % pfraction_surf  = aux_prof(i) % pfraction_surf
       aux_prof_COEF(i) % nearestlev_ctp  = aux_prof(i) % nearestlev_ctp 
       aux_prof_COEF(i) % pfraction_ctp   = aux_prof(i) % pfraction_ctp
       aux_prof_COEF(i) % cfraction       = aux_prof(i) % cfraction 
       If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
         aux_prof_COEF(i) % debye_prof(:,:) = aux_prof(i) % debye_prof(:,:)
       Endif
     Endif
  End Do

  !------------------------------------------------------------------------
  !4. set up common geometric variables for the RT integration
  !------------------------------------------------------------------------

  !4.1 USER levs
  !-------------



  Call rttov_setgeometry(&
            & nprofiles,        &! in
            & profiles,         &! in
            & aux_prof,         &! in
            & coef,             &! in
            & angles,           &! out
            & temp0%raytracing )       ! out  

  !4.2 COEF levs
  !-------------


  If(addinterp) then
    Call rttov_setgeometry(&
            & nprofiles,                &! in
            & temp0%profiles_COEF,      &! in
            & aux_prof_COEF,            &! in
            & coef,                     &! in
            & angles_COEF,              &! out
            & temp0%raytracing_COEF )    ! out  
   Else

   Do i = 1, nprofiles
    angles_COEF(i)      = angles(i)
    temp0%raytracing_COEF % HGPL(:,i)     = temp0%raytracing % HGPL(:,i)
    temp0%raytracing_COEF % DAIR(:,i)     = temp0%raytracing % DAIR(:,i)
    temp0%raytracing_COEF % DMAIR(:,i)    = temp0%raytracing % DMAIR(:,i)
    temp0%raytracing_COEF % R(:,i)        = temp0%raytracing % R(:,i) 
    temp0%raytracing_COEF % RATOESUN(:,i) = temp0%raytracing % RATOESUN(:,i)
    temp0%raytracing_COEF % RATOESAT(:,i) = temp0%raytracing % RATOESAT(:,i)
    temp0%raytracing_COEF % ZASUN(:,i)    = temp0%raytracing % ZASUN(:,i)
    temp0%raytracing_COEF % ZASAT(:,i)    = temp0%raytracing % ZASAT(:,i)
    temp0%raytracing_COEF % INT(:,i)      = temp0%raytracing % INT(:,i)
    temp0%raytracing_COEF % HL(:,i)       = temp0%raytracing % HL(:,i)
    temp0%raytracing_COEF % PPW(:,i)      = temp0%raytracing % PPW(:,i)
    temp0%raytracing_COEF % DISPCO2(:,i)  = temp0%raytracing % DISPCO2(:,i)
    temp0%raytracing_COEF % PATHSAT(:,i)  = temp0%raytracing % PATHSAT(:,i)
    temp0%raytracing_COEF % PATHSUN(:,i)  = temp0%raytracing % PATHSUN(:,i)
    temp0%raytracing_COEF % PATHEFF(:,i)  = temp0%raytracing % PATHEFF(:,i)
    temp0%raytracing_COEF % LTICK(:,i)    = temp0%raytracing % LTICK(:,i)
  enddo
  Endif

  !---------------------------------------------------
  !5. calculate atm/sol predictors - atm/sol COEF levs
  !---------------------------------------------------


  Do i = 1, nprofiles
    temp0%predictors%clw(:,i) = 0._jprb
  EndDo  



  If (coef%fmv_model_ver == 7) Then
     Call rttov_setpredictors_7(    &
           & nprofiles,             &
           & temp0%profiles_COEF,   &! in
           & angles_COEF,           &! in
           & coef,                  &! in
           & aux_prof_COEF,         &! in
           & temp0%predictors,      &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF ) ! inout  (in because of mem allocation)  

  Else If (coef%fmv_model_ver == 8) Then
     Call rttov_setpredictors_8(      &
           & nprofiles,               &
           & temp0%profiles_COEF,     &! in
           & angles_COEF,             &! in
           & coef,                    &! in
           & aux_prof_COEF,           &! in
           & temp0%predictors,        &! inout  (in because of mem allocation)
           & temp0%raytracing_COEF )   ! inout  (in because of mem allocation)  

  Else If (coef%fmv_model_ver == 9) Then
     Call rttov_setpredictors_9(     &
            & nprofiles,             &
            & temp0%profiles_COEF,   &! in
            & angles_COEF,           &! in
            & coef,                  &! in
            & temp0%predictors,      &
            & temp0%raytracing_COEF ) ! inout  (in because of mem allocation) 


     Call rttov_setpredictors_9_solar( &
          & nprofiles,                 &
          & temp0%profiles_COEF,       &! in
          & angles_COEF,               &! in
          & coef,                      &! in
          & temp0%predictors,          &
          & temp0%raytracing_COEF ) 


  Else
     errorstatus(:) = errorstatus_fatal
     Write( errMessage,&
            & '( "Unexpected RTTOV compatibility version number" )' ) 
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
  End If

!  End Do ! Profile loop

  !--------------------------------------------------------------------
  !6. predict atmospheric and solar optical depths - atm/sol COEF levs 
  !--------------------------------------------------------------------
 

 
  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9(         &
       & nchannels,            &! in
       & nprofiles,            &! in
       & coef%nlayers,         &! in
       & channels,             &! in
       & lprofiles,            &! in
       & temp0%predictors,     &! in
       & aux_prof_COEF,        &! in
       & coef,                 &! in
       & temp0%opdp_path_COEF, &! inout
       & opdp_ref_COEF)         ! inout
    Call rttov_opdep_9_solar  ( &
       & nchannels,             &! in
       & nprofiles,             &! in
       & coef%nlayers,          &! in
       & channels,              &! in
       & lprofiles,             &! in
       & profiles,              &! in ! for solar switch only, but all profiles
       & temp0%predictors,      &! in
       & aux_prof_COEF,         &! in
       & coef,                  &! in
       & temp0%opdp_path_COEF,  &! inout
       & opdpsun_ref_COEF)       ! inout
   else
     Call rttov_opdep(         &
       & nchannels,            &! in
       & nprofiles,            &! in
       & coef%nlayers,         &! in
       & channels,             &! in
       & lprofiles,            &! in
       & temp0%predictors,     &! in
       & aux_prof_COEF,        &! in
       & coef,                 &! in
       & temp0%opdp_path_COEF, &! inout
       & opdp_ref_COEF)         ! inout
   endif
   
  !--------------------------------------------------------------------------
  !7. interpolator second  call - optical depths from COEF levs to USER levs
  !--------------------------------------------------------------------------

   If (addinterp) Then
! locate any profile flags in the nchannels sequence 
! at present, only solar switch
    do j=1,nchannels
      prof=lprofiles(j)
      chan=channels(j)
      If (profiles(prof)%addsolar) Then
        If (coef%ss_val_chn(chan) .ne. 0_jpim) Then
        sun(j)=.true.     
        Else
          sun(j)=.false.  
        Endif
      Else
        sun(j)=.false.     
      Endif
    enddo
        Call rttov_intavg_chan( &
	   & nchannels,                          & !in 
	   & sun,                                & !in 
           & temp0%profiles_COEF(1)%nlevels,     & !in
           & profiles(1)%nlevels,                & !in
           & nprofiles,                          &
           & lprofiles,                          &
           & temp0%profiles_COEF,                & !inout
           & profiles,                           & !inout  only p and addsolar needed 
	   & temp0%opdp_path_COEF,               & !inout  source variable
	   & temp0%opdp_path,                    & !inout  target variable
           & errorstatus)                          !out
       If ( Any(errorstatus == errorstatus_fatal) ) Then
         IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
         Return
       End If
!
   Else
     temp0%opdp_path % atm_layer(:,:) = temp0%opdp_path_COEF % atm_layer(:,:)
     temp0%opdp_path % sun_layer(:,:) = temp0%opdp_path_COEF % sun_layer(:,:)
   EndIf

  !--------------------------------------------------------------------
  !8. If clouds are present,calculate the number of streams and
  !   the cloud distribution  in each stream - remains on USER levs
  !--------------------------------------------------------------------

  nstreams=0_jpim


  if(addclouds)then

    Call rttov_cldstr(&
      & nprofiles,    &! in
      & profiles,     &! in
      & coef,         &! in
      & temp0%ircld,  &! out
      & addclouds,    &! in 
      & nstreams)      ! out 

  endif

  !----------------------------------------------------------------------------
  !9. Calculate optical depths of aerosols and/or clouds - USER levs
  !----------------------------------------------------------------------------

  if(addaerosl)then
    do i=1,nprofiles

      Allocate( aux_prof(i)%iaernum     (profiles(1)%nlevels),stat= alloc_status(7))
      Allocate( aux_prof(i)%iaertyp  (11,profiles(1)%nlevels),stat= alloc_status(8))
      Allocate( aux_prof(i)%relhum      (profiles(1)%nlevels),stat= alloc_status(9))
      Allocate( aux_prof(i)%relhumref   (profiles(1)%nlevels),stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of ircld")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
        Return
      End If
    enddo
  Endif


  if(addaerosl.or.addclouds)then


    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 1_jpim )

    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, stream = .true. )

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "allocation of trans_scatt_ir ")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
      Return
    End If


  transmission_scatt_ir_stream%SSA = 0._jprb 

  Call rttov_opdpscattir(&
      & nchannels                   , &
      & nprofiles                   , &
      & profiles(1)%nlevels         , &
      & channels                    , &
      & lprofiles                   , &
      & aux_prof                    , &
      & profiles                    , &
      & coef                        , &
      & coef_scatt_ir               , &
      & temp0%raytracing            , &
      & transmission_scatt_ir       , &
      & transmission_scatt_ir_stream, &
      & optp                        , &
      & temp0%ircld                 , &
      & addaerosl                   , &
      & addclouds) 
  Endif


  !----------------------------------------
  !10. calculate transmittances - USER levs
  !----------------------------------------

  Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
    Return
  End If


    Call rttov_transmit( &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & od_layer,                     &! out
         & transmission,                 &! out
         & transmission_scatt_ir_stream, & ! out
         & tau_ref,                      &! out  
         & tau_ref_surf,                 &! out  
         & tau_surf,                     &! out  
         & tau_layer)

  If (coef%fmv_model_ver == 9) Then
     Call rttov_transmit_9_solar(        &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & profiles,                     &! in  ! for solar switch only, but all profiles
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%raytracing,             &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & odsun_layer,                  &! out
         & odsun_singlelayer,            & ! out
         & od_frac,                      &! out
         & transmission,                 &! out
         & transmission_scatt_ir_stream, &! out
         & tausun_ref,                   & ! out
         & tausun_ref_surf,              & ! out
         & tausun_surf,                  & ! out
         & tausun_layer)                   ! out
   Endif

  !-------------------------------------------------
  !11. calculate channel emissivity values - SURFACE
  !-------------------------------------------------

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        Call rttov_calcemis_ir( &
              & profiles,         &! in
              & angles,           &! in
              & coef,             &! in
              & nchannels,        &! in
              & nprofiles,        &! in
              & channels,         &! in
              & lprofiles,        &! in
              & calcemis,         &! in
              & emissivity  )      ! inout 
        reflectivity(:) = 1 - emissivity(:)

     Elseif ( coef%id_sensor==sensor_id_mw .or. coef%id_sensor==sensor_id_po ) Then
        !Microwave
        Call rttov_calcemis_mw ( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & transmission,  &! in
              & calcemis,      &! in
              & emissivity,    &! inout
              & reflectivity,  &! out
              & errorstatus   ) ! inout 
        If ( Any( errorstatus == errorstatus_fatal ) ) Then
           errorstatus(:) = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Else
        ! Hires
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        reflectivity(:) = 1 - emissivity(:)
     End Where

  Else
     ! reflectivity for all channels
     reflectivity(:) = 1 - emissivity(:)
  End If

  !-------------------------------------------------------
  !12. Compute Fresnel reflectance and sun glint - SURFACE
  !-------------------------------------------------------
  
     If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then

         Do i = 1, nprofiles
           Allocate( sunglint(i) % beta     (coef%ws_nomega),stat= alloc_status(1))
           Allocate( sunglint(i) % psi      (coef%ws_nomega),stat= alloc_status(2))

           If( Any(alloc_status /= 0) ) Then
             errorstatus(:) = errorstatus_fatal
             Write( errMessage, '( "allocation of sunglint")' )
             Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
             IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
             Return
           End If
         Enddo

         call rttov_refsun( &
         & nprofiles,         &
         & profiles,          &
         & coef,              &
         & aux_prof,          &
         & sunglint,          &
         & temp0%raytracing) 

         call rttov_fresnel(&
         & nchannels,         &
         & channels,          &
         & nprofiles,         &
         & lprofiles,         &
         & profiles,          &
         & coef,              &
         & sunglint,          &
         & fresnrefl) 

       endif
     endif

  !---------------------------------------------------------
  !13. integrate the radiative transfer equation - USER levs
  !---------------------------------------------------------

  Allocate(auxrad % layer   (profiles(1) % nlevels, nchannels) ,stat= alloc_status(1))
  Allocate(auxrad % surfair (nchannels) ,stat= alloc_status(2))
  Allocate(auxrad % skin    (nchannels) ,stat= alloc_status(3))
  Allocate(auxrad % cosmic  (nchannels) ,stat= alloc_status(4))
  Allocate(auxrad_stream % up      (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(1))
  Allocate(auxrad_stream % down    (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(2))
  Allocate(auxrad_stream % down_ref(0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(3))
  Allocate(auxrad_stream % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(13))

  if(addsun)then
    if(addaerosl.or.addclouds)then
      Allocate(auxrad_stream % FAC1_1    (0:nstreams,nchannels) ,stat= alloc_status(14))
      Allocate(auxrad_stream % FAC2_1    (0:nstreams,nchannels) ,stat= alloc_status(15))
      Allocate(auxrad_stream % FAC3_1    (0:nstreams,nchannels) ,stat= alloc_status(16))
      Allocate(auxrad_stream % FAC4_1    (0:nstreams,nchannels ) ,stat= alloc_status(17))
      Allocate(auxrad_stream % FAC5_1    (0:nstreams,nchannels ) ,stat= alloc_status(18))
      Allocate(auxrad_stream % FAC6_1    (0:nstreams,nchannels ) ,stat= alloc_status(19))
      Allocate(auxrad_stream % FAC7_1    (0:nstreams,nchannels) ,stat= alloc_status(20))
      Allocate(auxrad_stream % FAC1_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(21))
      Allocate(auxrad_stream % FAC2_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC3_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC4_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(24))
      Allocate(auxrad_stream % FAC5_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(25))
      Allocate(auxrad_stream % FAC6_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(26))
      Allocate(auxrad_stream % FAC7_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(27))
      Allocate(auxrad_stream % FAC1_3    (0:nstreams,nchannels) ,stat= alloc_status(28))
      Allocate(auxrad_stream % FAC2_3    (0:nstreams,nchannels) ,stat= alloc_status(29))
      Allocate(auxrad_stream % FAC3_3    (0:nstreams,nchannels) ,stat= alloc_status(30))
      Allocate(auxrad_stream % FAC4_3    (0:nstreams,nchannels ),stat= alloc_status(31))
      Allocate(auxrad_stream % FAC5_3    (0:nstreams,nchannels ),stat= alloc_status(32))
      Allocate(auxrad_stream % FAC6_3    (0:nstreams,nchannels ),stat= alloc_status(33))
      Allocate(auxrad_stream % FAC7_3    (0:nstreams,nchannels) ,stat= alloc_status(34))
    endif
  endif

  If( Any(alloc_status /= 0) ) Then
     errorstatus(:) = errorstatus_fatal
     Write( errMessage, '( "allocation of auxrad_stream")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  addcosmic = ( coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po )
  Call rttov_integrate(                &
         & addcosmic,                   &! in
         & addaerosl,                   &! in
         & addclouds,                   &! in
         & nchannels,                   &! in
         & nprofiles,                   &! in
         & nstreams,                    &! in
         & angles,                      &! in
         & channels,                    &! in
         & lprofiles,                   &! in
         & emissivity,                  &! in
         & reflectivity,                &! in
         & fresnrefl,                   &! in
         & sunglint,                    &! in
         & transmission,                &! in
         & transmission_scatt_ir_stream,&
         & profiles,                    &! in
         & aux_prof,                    &! in
         & coef,                        &! in
         & temp0%raytracing,            &! in
         & temp0%ircld,                 &! in
         & radiancedata,                &! inout
         & auxrad      ,                &
         & auxrad_stream  ) ! inout  

  !---------------------
  !14. deallocate memory
  !---------------------



  Do i = 1, nprofiles



    if(addaerosl)then

      Deallocate( aux_prof(i)%iaernum  ,stat= alloc_status(7))
      Deallocate( aux_prof(i)%iaertyp  ,stat= alloc_status(8))
      Deallocate( aux_prof(i)%relhum   ,stat= alloc_status(9))
      Deallocate( aux_prof(i)%relhumref,stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "deallocation of ircld")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
        Return
      endif
    endif

    If ( coef % id_sensor == sensor_id_hi ) Then
      If (coef%fmv_model_ver == 9) Then
        Deallocate( sunglint  (i) % beta     ,stat= alloc_status(1))
        Deallocate( sunglint  (i) % psi      ,stat= alloc_status(2))
      End if
    End if

    If( Any(alloc_status /= 0) ) Then 
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of sunglint")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
      Return
    endif

  enddo

  if(addaerosl.or.addclouds)then
    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 0_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 0_jpim )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of trans_scatt_ir")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
      Return
    endif
  endif

  Deallocate(auxrad_stream % up      ,stat=alloc_status(1))
  Deallocate(auxrad_stream % down    ,stat=alloc_status(2))
  Deallocate(auxrad_stream % down_ref,stat=alloc_status(3))
  Deallocate(auxrad_stream % cloudy  ,stat=alloc_status(4))

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "deallocation of auxrad_stream")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
    Return
  endif


  if(addsun)then
    if(addaerosl.or.addclouds)then
      DeAllocate(auxrad_stream % FAC1_1    ,stat= alloc_status(1))
      DeAllocate(auxrad_stream % FAC2_1    ,stat= alloc_status(2))
      DeAllocate(auxrad_stream % FAC3_1    ,stat= alloc_status(3))
      DeAllocate(auxrad_stream % FAC4_1    ,stat= alloc_status(4))
      DeAllocate(auxrad_stream % FAC5_1    ,stat= alloc_status(5))
      DeAllocate(auxrad_stream % FAC6_1    ,stat= alloc_status(6))
      DeAllocate(auxrad_stream % FAC7_1    ,stat= alloc_status(7))
      DeAllocate(auxrad_stream % FAC1_2    ,stat= alloc_status(8))
      DeAllocate(auxrad_stream % FAC2_2    ,stat= alloc_status(9))
      DeAllocate(auxrad_stream % FAC3_2    ,stat= alloc_status(10))
      DeAllocate(auxrad_stream % FAC4_2    ,stat= alloc_status(11))
      DeAllocate(auxrad_stream % FAC5_2    ,stat= alloc_status(12))
      DeAllocate(auxrad_stream % FAC6_2    ,stat= alloc_status(13))
      DeAllocate(auxrad_stream % FAC7_2    ,stat= alloc_status(14))
      DeAllocate(auxrad_stream % FAC1_3    ,stat= alloc_status(15))
      DeAllocate(auxrad_stream % FAC2_3    ,stat= alloc_status(16))
      DeAllocate(auxrad_stream % FAC3_3    ,stat= alloc_status(17))
      DeAllocate(auxrad_stream % FAC4_3    ,stat= alloc_status(18))
      DeAllocate(auxrad_stream % FAC5_3    ,stat= alloc_status(19))
      DeAllocate(auxrad_stream % FAC6_3    ,stat= alloc_status(20))
      DeAllocate(auxrad_stream % FAC7_3    ,stat= alloc_status(21))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "deallocation of auxrad_stream")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
        Return
      endif
    endif
  endif

  Deallocate(auxrad % layer   ,stat= alloc_status(1))
  Deallocate(auxrad % surfair ,stat= alloc_status(2))
  Deallocate(auxrad % skin    ,stat= alloc_status(3))
  Deallocate(auxrad % cosmic  ,stat= alloc_status(4))

  If( Any(alloc_status /= 0) ) Then
     errorstatus(:) = errorstatus_fatal
     Write( errMessage, '( "deallocation of aux radiances")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
     Do i = 1, nprofiles
        If( Associated( aux_prof(i) % debye_prof) ) Then
           Deallocate( aux_prof(i) % debye_prof ,stat= alloc_status(1))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of aux_prof(i) % debye_prof")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
              IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
              Return
           End If
        End If
     End Do
  Endif
  If ( addclouds ) Then
    Do i = 1, nprofiles
      Deallocate(aux_prof(i) % dg ,stat= alloc_status(1))
      Deallocate(aux_prof(i) % fac1_dg ,stat= alloc_status(1))
      Deallocate(aux_prof(i) % fac2_dg ,stat= alloc_status(2))
      Deallocate(aux_prof(i) % fac3_dg ,stat= alloc_status(3))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of aux_prof(i) % dg profile")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
              IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
              Return
           End If
    enddo
  End If

    ! Deallocate profiles on COEF levels
  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
        DeAllocate( aux_prof_COEF(i) % debye_prof, stat= alloc_status(1)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "deallocation of aux_prof_COEF(i) % debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Endif
  End Do




   ! Deallocate transmittances

  Call rttov_alloc_transmission( alloc_status(4), transmission, profiles(1) % nlevels, nchannels, &
  & 0_jpim, nstreams = nstreams, calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus = errorstatus_fatal
    Write( errMessage, '( "deallocation of profiles_COEF")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)
    Return
  End If


  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 0_jpim,                                  &
    & temp0 = temp0,                           &
    & temp1 = temp1,                           &
    & temp2 = temp )    
  If( temp_status .eq. errorstatus_fatal ) Then
    Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf


  IF (LHOOK) CALL DR_HOOK('RTTOV_DIRECT',1_jpim,ZHOOK_HANDLE)


End Subroutine rttov_direct
