!
Subroutine rttov_integrate( &
 & addcosmic,     &! in
 & addaerosl,     &! in
 & addclouds,     &! in
 & nchannels,     &! in
 & nprofiles,     &! in
 & nstreams ,     &! in
 & angles,        &! in
 & channels,      &! in
 & lprofiles,     &! in
 & emissivity,    &! in
 & reflectivity,  &! in
 & fresnrefl,     &! in
 & sunglint,      &! in
 & transmission,  &! in
 & transmission_scatt_ir_stream,&
 & profiles,      &! in
 & aux_prof,      &! in
 & coef,          &! in
 & raytracing  ,  &! in
 & ircld,         &! in
 & rad,           &! inout
 & auxrad,        &
 & auxrad_stream) ! inout 
!
! Description:
! To perform integration of radiative transfer equation
! in rttov suite, calculating radiances and brightness temperature.
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Method:
! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
! librarian at ECMWF).
!
! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
! Transfer Model for Assimilation of Satellite Radiance Observations.
! QJRMS, 125, 1407-1425.
!
! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
! transfer model for the infrared atmospheric sounding interferometer.
! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF).
! 
! Matricardi, M. 2005 The inclusion of aerosols and clouds in RTIASI,the
! ECMWF radiative transfer model for the infrared atmospheric sounding
! interferometer.
! ECMWF Research Dept. Tech. Memo. 474 (available from the librarian at ECMWF).
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
!          25/06/91.    Original code.  J.R.EYRE    *ECMWF*
!          21/08/00.    Emissivity and reflectivity handled separately. Steve English
!          31/01/01.    More cloud computations. F. Chevallier
!          23/03/01     New coef. format, new channel numbers (P. Brunel)
!          31/01/01.    More cloud computations. F. Chevallier
!          28/09/01     Cosmic background temp added G.Deblonde
!          18/01/2002   Thread safe (D.Salmond)
!  1.0     01/12/2002   New F90 code with structures (P Brunel A Smith)
!  1.1     02/01/2003   Added comments (R Saunders)
!  1.2     06/05/2003   Init rad%downcld to 0 in section 1 (P  Brunel)
!  1.3     26/09/2003   Modified to allow for multiple polarisations (S English)
!  1.4     03/09/2004   Mods. for Vectorisation (D Salmond ECMWF & BCarruthers, Cray)
!  1.5     28/02/2005   Further mods to vectorisation (D Dent)
!  1.6     01/06/2005   Marco Matricardi (ECMWF):
!             --        IASI capability added.
!             --        Linear in tau approximation for RT equation introduced.
!             --        Solar radiation introduced for IASI and AIRS.
!  1.7     01/06/2006   Marco Matricardi (ECMWF):
!                       Multiple scattering in the infrared introduced for
!                       water clouds,cirrus clouds and aerosols.
!  1.8     26/01/2007   Removed polarisation (R Saunders)
!  1.9     08/03/2007   Reintroduced overcast cloud (R Saunders)
!  1.10    13/07/2007   Added extra variables requested by P. Watts(RSaunders)
!  1.11    15/11/2007   Changed skin to surfair for overcast radiance calc (R Saunders) 
!  1.12    27/11/2007   Optimised for NEC/IBM (D. Salmond)
!  1.13    21/12/2007   Added polarimetric option (R. Saunders)
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!
! Declarations:
! Modules used:
! Imported Type Definitions:
Use rttov_const, only :   &
 & sensor_id_po   ,   &
 & surftype_land  ,   &
 & surftype_sea   ,   &
 & deg2rad        ,   &
 & pi             ,   &
 & rt8_mode       ,   &
 & min_tau        ,   &
 & min_od
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

Use rttov_types, Only : &
 & rttov_coef        ,&
 & geometry_Type     ,&
 & profile_Type      ,&
 & profile_aux       ,&
 & transmission_Type ,&
 & transmission_scatt_ir_type       ,   &
 & sunglint_type     ,&
 & radiance_Type     ,&
 & ircld_type        ,&
 & raytracing_type   ,&
 & radiance_aux 

Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
Implicit None


!subroutine arguments:

Logical,             Intent(in)    :: addcosmic   ! switch for adding cosmic background
Logical,             Intent(in)    :: addaerosl
Logical,             Intent(in)    :: addclouds
Integer(Kind=jpim),  Intent(in)    :: nchannels               ! Number of output radiances
Integer(Kind=jpim),  Intent(in)    :: nprofiles               ! Number of profiles
Integer(Kind=jpim),  Intent(in)    :: nstreams                ! 
Integer(Kind=jpim),  Intent(in)    :: channels(nchannels)     ! Channel indices
Integer(Kind=jpim),  Intent(in)    :: lprofiles(nchannels)    ! Profiles indices
Real(Kind=jprb),     Intent(in)    :: emissivity(nchannels)   ! surface emmissivity
Real(Kind=jprb),     Intent(in)    :: reflectivity(nchannels) ! surface reflectivity
Real(Kind=jprb),     Intent(in)    :: fresnrefl(nchannels)
Type(geometry_Type), Intent(in)    :: angles(nprofiles)       ! geometry angles
Type(ircld_type)                   :: ircld
Type(raytracing_type),intent(in)   :: raytracing
Type(sunglint_Type), Intent(in)    :: sunglint(nprofiles)
Type(transmission_Type), Intent(in):: transmission            ! transmittances and single-layer od
type(transmission_scatt_ir_type) , intent(inout)  :: transmission_scatt_ir_stream
Type(radiance_aux),  Intent(inout) :: auxrad_stream
Type(profile_Type),  Intent(in)    :: profiles(nprofiles) ! Profiles
Type(profile_aux) ,  Intent(in)    :: aux_prof(nprofiles) ! auxillary profiles info.
Type(rttov_coef),    Intent(in)    :: coef
Type(radiance_Type), Intent(inout) :: rad    ! radiances (mw/cm-1/ster/sq.m) and BTs
Type(radiance_aux),  Intent(inout) :: auxrad ! auxillary radiances


!INTF

#include "rttov_calcbt.interface"
#include "rttov_calcrad.interface"


!
!local variables:
Real(Kind=jprb) :: rad_tmp,dtau_lay,daux_lay
Real(Kind=jprb) :: b1_1,b2_1,b3_1
Real(Kind=jprb) :: b1_2,b2_2,b3_2
Real(Kind=jprb) :: b1_3,b2_3,b3_3
Real(Kind=jprb) :: meanrad_up
Real(Kind=jprb) :: meanrad_down
Real(Kind=jprb) :: z4pi_r, pi_r
Real(Kind=jprb) :: rad_up(profiles(1) % nlevels,0:nstreams)
Real(Kind=jprb) :: rad_dwn(profiles(1) % nlevels,0:nstreams)

Integer(Kind=jpim) :: i,lev,lev1,chan,prof,ist
Integer(Kind=jpim) :: pol_id       ! polarisation indice
Real(Kind=jprb) :: cfraction(nchannels)        ! cloud fraction
Real(Kind=jprb) :: pfraction(nchannels)        !

! Type(profile_aux),   Pointer   :: aux ! pointer on auxillary radiances

integer(Kind=jpim) :: iv1(nchannels), iv2(nchannels), iv3(nchannels)
Real(Kind=jprb) :: tau_layer_r
Real(Kind=jprb) :: tau_layer_rm1
Real(Kind=jprb) :: od_singlelayer_r
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!- End of header ------------------------------------------------------
! initialise radiance structure cloud flag

IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE',0_jpim,ZHOOK_HANDLE)

! rad % lcloud = addcloud
z4pi_r = 1.0_JPRB/(4._jprb*pi)
pi_r = 1.0_JPRB/pi

!----------------------------
!1. calculate layer radiances
!----------------------------

meanrad_up=0.0_JPRB
Call rttov_calcrad( &
 & addcosmic,    &!in
 & nchannels,    &!in
 & nprofiles,    &!in
 & channels,     &!in
 & lprofiles,    &!in
 & profiles,     &!in
 & coef,         &!in
 & auxrad%cosmic,   &!out
 & auxrad%skin,     &!out
 & auxrad%surfair,  &!out
 & auxrad%layer    ) !out 

rad%clear(:)=0._jprb
rad%cloudy(:)=0._jprb
rad%up(:,:)=0._jprb   ! Extra Phil Watts params
rad%down(:,:)=0._jprb !
rad%surf(:,:)=0._jprb !
auxrad_stream%cloudy(:,:)=0._jprb

!-------------------------------------
!2. calculate atmospheric contribution
!-------------------------------------

!2.1 layer above top pressure level
!----------------------------------

ist=0



!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)

  !---Thermal radiation: new code---------------------------------------------------

  If (.not. rt8_mode)Then  

    if(transmission %od_singlelayer(1,ist,i) < min_od .OR.             &
     & ((1-transmission % tau_layer(1,ist,i))<min_od))then 
      auxrad_stream%up(ist,1,i)   =0._JPRB
      auxrad_stream%down(ist,1,i) =0._JPRB
    else
      tau_layer_r=1.0_JPRB/transmission % tau_layer(1,ist,i)
      B1_1 = auxrad%layer(1,i)                   * &
       & (1.-transmission % tau_layer(1,ist,i)) 
      B2_1 = 0.
      B3_1 = 0.
      auxrad_stream%up(ist,1,i)   = B1_1-B2_1+B3_1
      auxrad_stream%down(ist,1,i) = (B1_1+B2_1-B3_1) * tau_layer_r
    endif

    !---Scattering of the solar beam--------------------------------------------------

    if(addaerosl.or.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then

          !-------------Upward single scattering of the solar beam--------------------------

          auxrad_stream%fac1_1(ist,i)=coef % ss_solar_spectrum(chan) * &
           & transmission_scatt_ir_stream%azphacup(ist,i,1)*z4pi_r 

          auxrad_stream%fac2_1(ist,I)=                                 &
           & transmission_scatt_ir_stream%ssa(ist,i,1) 

          auxrad_stream%fac3_1(ist,i)=raytracing%pathsat(1,prof)    / &
           & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof)) 

          auxrad_stream%fac4_1(ist,I)=                                 &
           & exp(-transmission % odsun_singlelayer(1,ist,i)) 

          auxrad_stream%fac5_1(ist,i)=                                 &
           & exp(-transmission % od_singlelayer(1,ist,i)) 

          auxrad_stream%fac6_1(ist,I)=coef % ss_solar_spectrum(chan) * &
           & transmission_scatt_ir_stream%azphacdo(ist,i,1)*z4pi_r 

          auxrad_stream%up(ist,1,i)=auxrad_stream%up(ist,1,i)  + &
           & auxrad_stream%FAC1_1(IST,i)                           * &
           & auxrad_stream%FAC2_1(IST,i)                           * &
           & auxrad_stream%FAC3_1(IST,i)                           * &
           & (1-transmission % tausun_layer(1,ist,i)) 

          !-------------Downward single scattering of the solar beam------------------------

          if(raytracing%pathsat(1,prof)/=raytracing%pathsun(1,prof))then

            auxrad_stream%fac7_1(ist,i)=raytracing%pathsat(1,prof)  / &
             & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof)) 

            auxrad_stream%down(ist,1,i)=                               &
             & auxrad_stream%down(ist,1,i)                           + &
             & auxrad_stream%FAC6_1(IST,i)                           * &
             & auxrad_stream%FAC2_1(IST,i)                           * &
             & auxrad_stream%FAC7_1(IST,i)                           * &
             & (auxrad_stream%fac5_1(ist,i)                          - &
             & auxrad_stream%fac4_1(ist,i))                          / &
             & transmission %tau_layer(1,ist,i) 

            auxrad_stream%down_ref(ist,1,i)=                           &
             & auxrad_stream%down(ist,1,i) 

            if(auxrad_stream%down(ist,1,i)<0._jprb)then
              auxrad_stream%down(ist,1,i)=0._jprb
            endif
          else
            auxrad_stream%down(ist,1,i)=                               &
             & auxrad_stream%down(ist,1,i)                           + &
             & auxrad_stream%FAC6_1(IST,i)                           * &
             & auxrad_stream%FAC2_1(IST,i)                           * &
             & auxrad_stream%FAC4_1(IST,i)                           * &
             & transmission % odsun_singlelayer(1,ist,i)             / &
             & transmission %tau_layer(1,ist,i) 

            auxrad_stream%down_ref(ist,1,i)=                           &
             & auxrad_stream%down(ist,1,i) 

            if(auxrad_stream%down(ist,1,i)<0._jprb)then
              auxrad_stream%down(ist,1,i)=0._jprb
            endif
          endif

        endif
      endif
    endif

  Else  
    tau_layer_r=1.0_JPRB/transmission % tau_layer(1,ist,i)

    !-------Thermal radiation:old code------------------------------------------------
    auxrad_stream%up(ist,1,i)   = auxrad%layer(1,i)                       * &
     & ( 1.0_JPRB-transmission % tau_layer(1,ist,i) ) 
    auxrad_stream%down(ist,1,i) = auxrad_stream%up(ist,1,i)* tau_layer_r
  Endif
Enddo



Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)

  do ist=1,ircld%nstream(prof)

    !---Thermal radiation: new code---------------------------------------------------

    If (.not.rt8_mode)Then  

      if(transmission %od_singlelayer(1,ist,i) < min_od .OR.             &
       & ((1-transmission % tau_layer(1,ist,i))<min_od))then 
        auxrad_stream%up(ist,1,i)   =0._JPRB
        auxrad_stream%down(ist,1,i) =0._JPRB
      else
        tau_layer_r=1.0_JPRB/transmission % tau_layer(1,ist,i)
        B1_1 = auxrad%layer(1,i)                   * &
         & (1.-transmission % tau_layer(1,ist,i)) 
        B2_1 = 0.
        B3_1 = 0.
        auxrad_stream%up(ist,1,i)   = B1_1-B2_1+B3_1
        auxrad_stream%down(ist,1,i) = (B1_1+B2_1-B3_1) * tau_layer_r
      endif

      !---Scattering of the solar beam--------------------------------------------------

      if(addaerosl.or.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then

            !-------------Upward single scattering of the solar beam--------------------------

            auxrad_stream%fac1_1(ist,i)=coef % ss_solar_spectrum(chan) * &
             & transmission_scatt_ir_stream%azphacup(ist,i,1)*z4pi_r 

            auxrad_stream%fac2_1(ist,I)=                                 &
             & transmission_scatt_ir_stream%ssa(ist,i,1) 

            auxrad_stream%fac3_1(ist,i)=raytracing%pathsat(1,prof)    / &
             & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof)) 

            auxrad_stream%fac4_1(ist,I)=                                 &
             & exp(-transmission % odsun_singlelayer(1,ist,i)) 

            auxrad_stream%fac5_1(ist,i)=                                 &
             & exp(-transmission % od_singlelayer(1,ist,i)) 

            auxrad_stream%fac6_1(ist,I)=coef % ss_solar_spectrum(chan) * &
             & transmission_scatt_ir_stream%azphacdo(ist,i,1)*z4pi_r 

            auxrad_stream%up(ist,1,i)=auxrad_stream%up(ist,1,i)  + &
             & auxrad_stream%FAC1_1(IST,i)                           * &
             & auxrad_stream%FAC2_1(IST,i)                           * &
             & auxrad_stream%FAC3_1(IST,i)                           * &
             & (1-transmission % tausun_layer(1,ist,i)) 

            !-------------Downward single scattering of the solar beam------------------------

            if(raytracing%pathsat(1,prof)/=raytracing%pathsun(1,prof))then

              auxrad_stream%fac7_1(ist,i)=raytracing%pathsat(1,prof)  / &
               & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof)) 

              auxrad_stream%down(ist,1,i)=                               &
               & auxrad_stream%down(ist,1,i)                           + &
               & auxrad_stream%FAC6_1(IST,i)                           * &
               & auxrad_stream%FAC2_1(IST,i)                           * &
               & auxrad_stream%FAC7_1(IST,i)                           * &
               & (auxrad_stream%fac5_1(ist,i)                          - &
               & auxrad_stream%fac4_1(ist,i))                          / &
               & transmission %tau_layer(1,ist,i) 

              auxrad_stream%down_ref(ist,1,i)=                           &
               & auxrad_stream%down(ist,1,i) 

              if(auxrad_stream%down(ist,1,i)<0._jprb)then
                auxrad_stream%down(ist,1,i)=0._jprb
              endif
            else
              auxrad_stream%down(ist,1,i)=                               &
               & auxrad_stream%down(ist,1,i)                           + &
               & auxrad_stream%FAC6_1(IST,i)                           * &
               & auxrad_stream%FAC2_1(IST,i)                           * &
               & auxrad_stream%FAC4_1(IST,i)                           * &
               & transmission % odsun_singlelayer(1,ist,i)             / &
               & transmission %tau_layer(1,ist,i) 

              auxrad_stream%down_ref(ist,1,i)=                           &
               & auxrad_stream%down(ist,1,i) 

              if(auxrad_stream%down(ist,1,i)<0._jprb)then
                auxrad_stream%down(ist,1,i)=0._jprb
              endif
            endif

          endif
        endif
      endif

    Else  
      tau_layer_r=1.0_JPRB/transmission % tau_layer(1,ist,i)

      !-------Thermal radiation:old code------------------------------------------------
      auxrad_stream%up(ist,1,i)   = auxrad%layer(1,i)                       * &
       & ( 1.0_JPRB-transmission % tau_layer(1,ist,i) ) 
      auxrad_stream%down(ist,1,i) = auxrad_stream%up(ist,1,i)* tau_layer_r
    Endif
  Enddo
Enddo


!-------------------------------------------------------------------------------
!2.2 layers between standard pressure levels
!-------------------------------------------------------------------------------

! loops switched for vectorisation on NEC - switch back for scalar machines

ist=0

#ifdef RTTOV_ARCH_VECTOR
Do lev = 2, profiles(1) % nlevels
!cdir nodep
  Do i = 1, nchannels
    prof=lprofiles(i)
    chan=channels(i)
#else
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  Do lev = 2, profiles(1) % nlevels
#endif

    IF (.not. rt8_mode)Then

      !---The optical depth cannot be computed for this layer so that the average-------  
      !   source function is used for the layer                                  -------

      if(transmission % tau_layer(lev-1,ist,i)>=0.0_JPRB .and.   &
       & transmission % tau_layer(lev,ist,i)<0.0_JPRB )then 

        rad_up(lev,ist)=0.5_JPRB*(auxrad%layer(lev,i)+auxrad%layer(lev-1,i) )  * &
         & ( transmission % tau_layer(lev-1,ist,i)            - &
         & transmission % tau_layer(lev,ist,i) ) 
        rad_dwn(lev,ist) = 0.0_JPRB
      else

        if(transmission %od_singlelayer(lev,ist,i)<min_od .OR.        &
         & (transmission % tau_layer(lev-1,ist,i)                - &
         & transmission % tau_layer(lev,ist,i))<min_od )THEN 

          rad_up(lev,ist) = 0._JPRB

          rad_dwn(lev,ist)=0._JPRB

        else 
          od_singlelayer_r=1.0_JPRB/transmission %od_singlelayer(lev,ist,i)
          dtau_lay = &
           & (transmission % tau_layer(lev-1,ist,i)               - &
           &  transmission % tau_layer(lev  ,ist,i) ) 
          daux_lay = (auxrad%layer(lev,i)-auxrad%layer(lev-1,i))
          B1_2 = auxrad%layer(lev-1,i)     * &
           & dtau_lay
          B2_2 =                             &
           & daux_lay * &
           & transmission % tau_layer(lev,ist,i) 

          B3_2 =                            &
           & daux_lay     * &
           & dtau_lay     * &
           & od_singlelayer_r

          rad_up(lev,ist)      = B1_2-B2_2+B3_2

          If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
            tau_layer_rm1=1.0_JPRB/transmission % tau_layer(lev-1,ist,i)
            tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
            !                rad_dwn(lev,ist) =(B1_2-B3_2 ) * &
            !                 & ( tau_layer_rm1 * tau_layer_r )               + &
            !                 & B2_2 * (tau_layer_r)**2 
            rad_dwn(lev,ist) = ((B1_2-B3_2 )* tau_layer_rm1  + &
             & daux_lay ) * tau_layer_r
          Else
            rad_dwn(lev,ist) = 0.0_JPRB
          End If

        endif
      endif

      !---Scattering of the solar beam--------------------------------------------------

      if(addaerosl.OR.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then

            !----------------Upward single scattering of the solar beam-----------------------
            !
            auxrad_stream%fac1_2(lev,ist,i)=                          &
             & coef % ss_solar_spectrum(chan)                      * &
             & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r 

            auxrad_stream%fac2_2(lev,ist,i)=                          &
             & transmission_scatt_ir_stream%ssa(ist,i,lev) 

            auxrad_stream%fac3_2(lev,ist,i)=                          &
             & raytracing%pathsat(lev,prof)                       / &
             & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

            auxrad_stream%fac4_2(lev,ist,i)=                          &
             & exp(-transmission % odsun_singlelayer(lev,ist,i)) 


            auxrad_stream%fac5_2(lev,ist,i)=                          &
             & exp(-transmission % od_singlelayer(lev,ist,i)) 


            auxrad_stream%fac6_2(lev,ist,i)=                          &
             & coef % ss_solar_spectrum(chan)                      * &
             & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

            rad_up(lev,ist)    =   rad_up(lev,ist)+                     &
             & auxrad_stream%fac1_2(lev,ist,i)                       * &
             & auxrad_stream%fac2_2(lev,ist,i)                       * &
             & auxrad_stream%fac3_2(lev,ist,i)                       * &
             & (transmission % tausun_layer(lev-1,ist,i)             - &
             & transmission % tausun_layer(lev,ist,i)) 


            !-------------------Downward single scattering of the solar beam------------------

            if(raytracing%pathsat(lev,prof) /=                             &
             & raytracing%pathsun(lev,prof))then 

              auxrad_stream%fac7_2(lev,ist,i) =                       &
               & raytracing%pathsat(lev,prof)                        / &
               & (raytracing%pathsun(lev,prof)                       - &
               & raytracing%pathsat(lev,prof)) 
              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                rad_dwn(lev,ist)  = rad_dwn(lev,ist)+                     &
                 & auxrad_stream%fac6_2(lev,ist,i)               * &
                 & auxrad_stream%fac2_2(lev,ist,i)               * &
                 & auxrad_stream%fac7_2(lev,ist,i)               * &
                 & (auxrad_stream%fac5_2(lev,ist,i)              - &
                 & auxrad_stream%fac4_2(lev,ist,i))              * &
                 & transmission %tausun_layer(lev-1,ist,i)       / &
                 & (transmission %tau_layer(lev-1,ist,i)         * &
                 & transmission %tau_layer(lev,ist,i)) 
              else
                 rad_dwn(lev,ist)=0._jprb
              endif
            else
              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                rad_dwn(lev,ist)  =   rad_dwn(lev,ist)           + &
                 & auxrad_stream%fac6_2(lev,ist,i)               * &
                 & auxrad_stream%fac2_2(lev,ist,i)               * &
                 & auxrad_stream%fac4_2(lev,ist,i)               * &
                 & transmission %tausun_layer(lev-1,ist,i)       * &
                 & transmission % odsun_singlelayer(lev,ist,i)   / &
                 & (transmission %tau_layer(lev-1,ist,i)         * &
                 & transmission %tau_layer(lev,ist,i)) 
              else
                rad_dwn(lev,ist)=0.
              endif
            endif
          endif
        endif
      endif
    ELSE
      !--Old code layer mean------------------------------------------------------------
      rad_up(lev,ist) = 0.5_JPRB * (auxrad%layer(lev,i)+auxrad%layer(lev-1,i) )   * &
       & ( transmission % tau_layer(lev-1,ist,i)                    - &
       & transmission % tau_layer(lev,ist,i) ) 


      If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
        tau_layer_rm1=1.0_JPRB/transmission % tau_layer(lev-1,ist,i)
        tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
        rad_dwn(lev,ist) = rad_up(lev,ist) * ( tau_layer_r * tau_layer_rm1 ) 
      Else
        rad_dwn(lev,ist) = 0.0_JPRB
      End If

      !---------------------------------------------------------------------------------
    ENDIF

    auxrad_stream%up(ist,lev,i) =                                 &
     & auxrad_stream%up(ist,lev-1,i) + rad_up(lev,ist)
    auxrad_stream%down(ist,lev,i) =                               &
     & auxrad_stream%down(ist,lev-1,i) + rad_dwn(lev,ist)
    if(addaerosl.OR.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then
          auxrad_stream%down_ref(ist,lev,i)  =                    &
           & auxrad_stream%down(ist,lev,i)

          if(auxrad_stream%down(ist,lev,i)<0._jprb)then
            auxrad_stream%down(ist,lev,i)=0._jprb
          endif
        endif
      endif
    endif


  end do


End Do



Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  do ist=1,ircld%nstream(prof)
    Do lev = 2, profiles(1) % nlevels
      IF (.not. rt8_mode)Then

        !---The optical depth cannot be computed for this layer so that the average-------  
        !   source function is used for the layer                                  -------

        if(transmission % tau_layer(lev-1,ist,i)>=0.0_JPRB .and.   &
         & transmission % tau_layer(lev,ist,i)<0.0_JPRB )then 

          rad_up(lev,ist)=0.5_JPRB*(auxrad%layer(lev,i)+auxrad%layer(lev-1,i) )  * &
           & ( transmission % tau_layer(lev-1,ist,i)            - &
           & transmission % tau_layer(lev,ist,i) ) 
          rad_dwn(lev,ist) = 0.0_JPRB
        else

          if(transmission %od_singlelayer(lev,ist,i)<min_od .OR.        &
           & (transmission % tau_layer(lev-1,ist,i)                - &
           & transmission % tau_layer(lev,ist,i))<min_od )THEN 

            rad_up(lev,ist) = 0._JPRB

            rad_dwn(lev,ist)=0._JPRB

          else 
            od_singlelayer_r=1.0_JPRB/transmission %od_singlelayer(lev,ist,i)
            dtau_lay = &
             & (transmission % tau_layer(lev-1,ist,i)               - &
             &  transmission % tau_layer(lev  ,ist,i) ) 
            daux_lay = (auxrad%layer(lev,i)-auxrad%layer(lev-1,i))
            B1_2 = auxrad%layer(lev-1,i)     * &
             & dtau_lay
            B2_2 =                             &
             & daux_lay * &
             & transmission % tau_layer(lev,ist,i) 

            B3_2 =                            &
             & daux_lay     * &
             & dtau_lay     * &
             & od_singlelayer_r

            rad_up(lev,ist)      = B1_2-B2_2+B3_2

            If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
              tau_layer_rm1=1.0_JPRB/transmission % tau_layer(lev-1,ist,i)
              tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
              !                rad_dwn(lev,ist) =(B1_2-B3_2 ) * &
              !                 & ( tau_layer_rm1 * tau_layer_r )               + &
              !                 & B2_2 * (tau_layer_r)**2 
              rad_dwn(lev,ist) = ((B1_2-B3_2 )* tau_layer_rm1  + &
               & daux_lay ) * tau_layer_r
            Else
              rad_dwn(lev,ist) = 0.0_JPRB
            End If

          endif
        endif

        !---Scattering of the solar beam--------------------------------------------------

        if(addaerosl.OR.addclouds)then
          if(profiles(prof)%addsolar)then
            if(coef%ss_val_chn(chan)==1)then

              !----------------Upward single scattering of the solar beam-----------------------
              !
              auxrad_stream%fac1_2(lev,ist,i)=                          &
               & coef % ss_solar_spectrum(chan)                      * &
               & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r 

              auxrad_stream%fac2_2(lev,ist,i)=                          &
               & transmission_scatt_ir_stream%ssa(ist,i,lev) 

              auxrad_stream%fac3_2(lev,ist,i)=                          &
               & raytracing%pathsat(lev,prof)                       / &
               & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

              auxrad_stream%fac4_2(lev,ist,i)=                          &
               & exp(-transmission % odsun_singlelayer(lev,ist,i)) 


              auxrad_stream%fac5_2(lev,ist,i)=                          &
               & exp(-transmission % od_singlelayer(lev,ist,i)) 


              auxrad_stream%fac6_2(lev,ist,i)=                          &
               & coef % ss_solar_spectrum(chan)                      * &
               & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

              rad_up(lev,ist)    =   rad_up(lev,ist)+                     &
               & auxrad_stream%fac1_2(lev,ist,i)                       * &
               & auxrad_stream%fac2_2(lev,ist,i)                       * &
               & auxrad_stream%fac3_2(lev,ist,i)                       * &
               & (transmission % tausun_layer(lev-1,ist,i)             - &
               & transmission % tausun_layer(lev,ist,i)) 


              !-------------------Downward single scattering of the solar beam------------------

              if(raytracing%pathsat(lev,prof) /=                             &
               & raytracing%pathsun(lev,prof))then 

                auxrad_stream%fac7_2(lev,ist,i) =                       &
                 & raytracing%pathsat(lev,prof)                        / &
                 & (raytracing%pathsun(lev,prof)                       - &
                 & raytracing%pathsat(lev,prof)) 

                if(transmission %tau_layer(lev,ist,i)>min_tau)then
                 rad_dwn(lev,ist)  = rad_dwn(lev,ist)+                     &
                  & auxrad_stream%fac6_2(lev,ist,i)               * &
                  & auxrad_stream%fac2_2(lev,ist,i)               * &
                  & auxrad_stream%fac7_2(lev,ist,i)               * &
                  & (auxrad_stream%fac5_2(lev,ist,i)              - &
                  & auxrad_stream%fac4_2(lev,ist,i))              * &
                  & transmission %tausun_layer(lev-1,ist,i)       / &
                  & (transmission %tau_layer(lev-1,ist,i)         * &
                  & transmission %tau_layer(lev,ist,i)) 
                else
                  rad_dwn(lev,ist)=0._jprb
                endif
               else
                if(transmission %tau_layer(lev,ist,i)>min_tau)then
                  rad_dwn(lev,ist)  =   rad_dwn(lev,ist)                 + &
                  & auxrad_stream%fac6_2(lev,ist,i)               * &
                  & auxrad_stream%fac2_2(lev,ist,i)               * &
                  & auxrad_stream%fac4_2(lev,ist,i)               * &
                  & transmission %tausun_layer(lev-1,ist,i)       * &
                  & transmission % odsun_singlelayer(lev,ist,i)   / &
                  & (transmission %tau_layer(lev-1,ist,i)         * &
                  & transmission %tau_layer(lev,ist,i)) 
                else
                  rad_dwn(lev,ist)=0._jprb
                endif
              endif
            endiF
          endif
        endif
      ELSE
        !--Old code layer mean------------------------------------------------------------
        rad_up(lev,ist) = 0.5_JPRB * (auxrad%layer(lev,i)+auxrad%layer(lev-1,i) )   * &
         & ( transmission % tau_layer(lev-1,ist,i)                    - &
         & transmission % tau_layer(lev,ist,i) ) 


        If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
          tau_layer_rm1=1.0_JPRB/transmission % tau_layer(lev-1,ist,i)
          tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
          rad_dwn(lev,ist) = rad_up(lev,ist) * ( tau_layer_r * tau_layer_rm1 ) 
        Else
          rad_dwn(lev,ist) = 0.0_JPRB
        End If

        !---------------------------------------------------------------------------------
      ENDIF

    end do
  end do


  do ist=1,ircld%nstream(prof)
    Do lev = 2, profiles(1) % nlevels
      auxrad_stream%up(ist,lev,i) =                                 &
       & auxrad_stream%up(ist,lev-1,i) + rad_up(lev,ist)
      auxrad_stream%down(ist,lev,i) =                               &
       & auxrad_stream%down(ist,lev-1,i) + rad_dwn(lev,ist)
      if(addaerosl.OR.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            auxrad_stream%down_ref(ist,lev,i)  =                    &
             & auxrad_stream%down(ist,lev,i)

            if(auxrad_stream%down(ist,lev,i)<0._jprb)then
              auxrad_stream%down(ist,lev,i)=0._jprb
            endif
          endif
        endif
      endif
    end do
  end do
End Do


! Fill additional output arrays for Phil Watts--------------
#ifdef RTTOV_ARCH_VECTOR
Do lev = 2 , profiles(1) % nlevels 
!cdir nodep
  Do i = 1, nchannels
#else
Do i = 1, nchannels
  Do lev = 2 , profiles(1) % nlevels 
#endif
    If ( transmission % tau_layer(lev,0,i) > min_tau ) Then
      rad%down(lev,i) = auxrad_stream%down(0,lev,i) * transmission % tau_layer(lev,0,i)
    Else
      rad%down(lev,i) = rad%down(lev-1,i) 
    Endif
    rad%surf(lev,i) = auxrad%layer(lev,i)
  End Do
enddo
!-----------------------------------------------------------
!
Do i = 1, nchannels
  prof=lprofiles(i)
  !    aux  => aux_prof( lprofiles(i) )
  cfraction(i) = aux_prof( prof )%cfraction
  pfraction(i) = aux_prof( prof )%pfraction_surf
  iv3(i) = aux_prof( prof ) % nearestlev_surf - 1
  iv2(i) = aux_prof( prof ) % nearestlev_surf
  iv1(i) = min( profiles(1) % nlevels, iv3(i) )

  If ( pfraction(i) < 0.0_JPRB ) iv3(i) = iv3(i) + 1



End Do


!-------------------------------------------------------------------------------
!2.3 near-surface layer
!-------------------------------------------------------------------------------


ist=0

!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  If(coef % id_sensor == sensor_id_po)Then
    pol_id = coef % fastem_polar(chan) + 1_jpim
  Else
    pol_id = 0_jpim
  Endif
  ! add upward and downward parts

  IF (.not. rt8_mode)Then

    lev = iv3(i)

    if(pfraction(i) < 0.0_JPRB )then
      lev1=iv3(i)
    else
      lev1=iv3(i)+1
    endif

    if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.              &
     & transmission % tau_surf(ist,i)<0._JPRB)then 

      meanrad_up= 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )  * &
       & (transmission % tau_layer(lev,ist,i)                   - &
       & transmission % tau_surf(ist,i)) 

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
        rad_tmp=meanrad_up * (tau_layer_r * transmission % tau_surf(ist,i) ) 
      Else
        rad_tmp = 0.0_JPRB
      End If

      meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

      meanrad_up = auxrad_stream%up(ist,lev,i) + meanrad_up
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up = 0.0_jprb

    else

      if(transmission%od_sfrac(ist,i) < min_od .OR.                   &
       & ((transmission % tau_layer(lev,ist,i)                       - &
       & transmission % tau_surf(ist,i))<min_od))THEN 

        ! small optical depth or optical depth change set radiance to zero
        meanrad_up=0.0_JPRB
        rad_tmp=0.0_JPRB
        meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp
        meanrad_up   = auxrad_stream%up(ist,lev,i) + meanrad_up
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
        if (pol_id >= 6_jpim ) meanrad_up = 0.0_jprb

      else
        ! compute linear in tau radiances

        B1_3=                       auxrad%layer(lev,i)               * &
         & (transmission % tau_layer(lev,ist,i)  - &
         & transmission % tau_surf(ist,i)) 

        B2_3=                             (auxrad%surfair(i)                - &
         & auxrad%layer(lev,i))                        * &
         & transmission % tau_surf(ist,i) 

        B3_3=                             (auxrad%surfair(i)                - &
         & auxrad%layer(lev,i))             * &
         & (transmission % tau_layer(lev,ist,i)                   - &
         & transmission % tau_surf(ist,i))                       / &
         & (transmission %od_sfrac (ist,i)) 


        meanrad_up=B1_3-B2_3+B3_3 

!        write(*,'(4f10.5)')transmission % tau_surf(ist,i),transmission % tau_layer(lev,ist,i),transmission %od_sfrac (ist,i)
!        print*,meanrad_up

        IF (transmission % tau_surf(ist,i) > min_tau) THEN
          rad_tmp= (B1_3-B3_3)                                        / &
           & (transmission % tau_layer(lev,ist,i)                    * &
           & transmission % tau_surf(ist,i))                          + &
           & B2_3                                / &
           & (transmission % tau_surf(ist,i))**2 
        ELSE
          rad_tmp=0.0_JPRB
        ENDIF

        meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

        meanrad_up   = auxrad_stream%up(ist,lev,i) + meanrad_up
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
        if (pol_id >= 6_jpim ) meanrad_up = 0.0_jprb

      endif
    endif

    !---Scattering of the solar beam--------------------------------------------------

    if(addaerosl.or.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then

          !--------------Upward single scattering of the solar beam-------------------------

          auxrad_stream%fac1_3(ist,i)=                                &
           & coef % ss_solar_spectrum(chan)                               * &
           & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r  

          auxrad_stream%fac2_3(ist,i)=                                &
           & transmission_scatt_ir_stream%ssa(ist,i,lev) 

          auxrad_stream%fac3_3(ist,i)=                                &
           & raytracing%pathsat(lev,prof)                                / &
           & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

          auxrad_stream%fac4_3(ist,i)=                                &
           & exp(-transmission % odsun_sfrac(ist,i)) 

          auxrad_stream%fac5_3(ist,i)=                                &
           & exp(-transmission % od_sfrac(ist,i)) 

          auxrad_stream%fac6_3(ist,i)=                                &
           & coef % ss_solar_spectrum(chan)                              * &
           & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

          meanrad_up   = meanrad_up + auxrad_stream%fac1_3(ist,i)   * &
           & auxrad_stream%fac2_3(ist,i)                           * &
           & auxrad_stream%fac3_3(ist,i)                           * &
           & (transmission % tausun_layer(lev,ist,i)               - &
           & transmission % tausun_surf(ist,i)) 
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
          if (pol_id >= 6_jpim ) meanrad_up = 0.0_jprb

          !--------------Downward single scattering of the solar beam-----------------------

          if(raytracing%pathsat(lev1,prof) /=                             &
           & raytracing%pathsun(lev1,prof))then 

            auxrad_stream%fac7_3(ist,i)=                               &
             & raytracing%pathsat(lev1,prof)                         / &
             & (raytracing%pathsun(lev1,prof)                        - &
             & raytracing%pathsat(lev1,prof)) 

            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              meanrad_down=meanrad_down+auxrad_stream%fac6_3(ist,i)    * &
               & auxrad_stream%fac2_3(ist,i)                       * &
               & auxrad_stream%fac7_3(ist,i)                       * &
               & (auxrad_stream%fac5_3(ist,i)                      - &
               & auxrad_stream%fac4_3(ist,i))                      * &
               & transmission %tausun_layer(lev,ist,i)             / &
               & (transmission %tau_layer(lev,ist,i)               * &
               & transmission %tau_surf(ist,i)) 
            endif

            if(meanrad_down<0._jprb)then
              meanrad_down=0._jprb
            endif
          else
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              meanrad_down=meanrad_down+  auxrad_stream%fac6_3(ist,i)  * &
               & auxrad_stream%fac2_3(ist,i)                       * &
               & auxrad_stream%fac4_3(ist,i)                       * &
               & transmission %tausun_layer(lev,ist,i)             * &
               & transmission % odsun_sfrac(ist,i)                 / &
               & (transmission %tau_layer(lev,ist,i)               * &
               & transmission %tau_surf(ist,i)) 
            endif
            if(meanrad_down<0._jprb)then
              meanrad_down=0._jprb
            endif
          endif
        endif
      endif
    endif


  ELSE 

    !--old mean layer code for backward compatibility---------------------------------

    lev = iv3(i)
    meanrad_up = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )   * &
     & ( transmission % tau_layer(lev,ist,i)                      - &
     & transmission % tau_surf(ist,i) ) 

    If ( transmission % tau_surf(ist,i) > min_tau ) Then
      rad_tmp = meanrad_up                                         / &
       & ( transmission % tau_layer(lev,ist,i)         * &
       & transmission % tau_surf(ist,i) ) 
    Else
      rad_tmp = 0.0_JPRB
    End If

    meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

    meanrad_up = auxrad_stream%up(ist,lev,i) + meanrad_up
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up = 0.0_jprb
    !-------------------------------------------------------------------------------
  ENDIF

  rad % clear(i) = meanrad_up +meanrad_down * reflectivity(i)           * &
   & transmission % tau_surf(ist,i)                                * &
   & transmission % tau_surf(ist,i)  

  ! clear sky radiance without reflection term
  ! without surface contribution at this line

  rad % upclear(i) = meanrad_up
  !  Add extra output variables for Phil Watts----------------------
  If ( transmission % tau_surf(0,i) > min_tau ) Then
    rad%down(iv2(i),i) = meanrad_down*transmission%tau_surf(0,i)
  Else
    rad%down(iv2(i),i) = rad%down(iv3(i),i)
  Endif
  rad%surf(iv2(i),i) = auxrad%skin(i)
  !-----------------------------------------------------------------
  ! clear sky downwelling radiance
  rad % dnclear(i) = &
   & meanrad_down * transmission % tau_surf(ist,i) 

  ! reflected clear sky downwelling radiance
  rad % reflclear(i) = &
   & rad % dnclear(i) * ( reflectivity(i) )                             * &
   & transmission % tau_surf(ist,i) 
  auxrad_stream%up(ist,iv2(i),i) = meanrad_up

Enddo



Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)

  do ist=1,ircld%nstream(prof)

    ! add upward and downward parts

    IF (.not. rt8_mode)Then

      lev = iv3(i)
      if(pfraction(i) < 0.0_JPRB )then
        lev1=iv3(i)
      else
        lev1=iv3(i)+1
      endif

      if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.              &
       & transmission % tau_surf(ist,i)<0._JPRB)then 

        meanrad_up= 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )  * &
         & (transmission % tau_layer(lev,ist,i)                   - &
         & transmission % tau_surf(ist,i)) 

        If ( transmission % tau_surf(ist,i) > min_tau ) Then
          tau_layer_r=1.0_JPRB/transmission % tau_layer(lev,ist,i)
          rad_tmp=meanrad_up * (tau_layer_r * transmission % tau_surf(ist,i) ) 
        Else
          rad_tmp = 0.0_JPRB
        End If

        meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

        meanrad_up = auxrad_stream%up(ist,lev,i) + meanrad_up

      else

        if(transmission%od_sfrac(ist,i) < min_od .OR.                   &
         & ((transmission % tau_layer(lev,ist,i)                       - &
         & transmission % tau_surf(ist,i))<min_od))THEN 

          ! small optical depth or optical depth change set radiance to zero
          meanrad_up=0.0_JPRB
          rad_tmp=0.0_JPRB
          meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp
          meanrad_up   = auxrad_stream%up(ist,lev,i) + meanrad_up

        else
          ! compute linear in tau radiances

          B1_3=                       auxrad%layer(lev,i)               * &
           & (transmission % tau_layer(lev,ist,i)  - &
           & transmission % tau_surf(ist,i)) 

          B2_3=                             (auxrad%surfair(i)                - &
           & auxrad%layer(lev,i))                        * &
           & transmission % tau_surf(ist,i) 

          B3_3=                             (auxrad%surfair(i)                - &
           & auxrad%layer(lev,i))             * &
           & (transmission % tau_layer(lev,ist,i)                   - &
           & transmission % tau_surf(ist,i))                       / &
           & (transmission %od_sfrac (ist,i)) 

          meanrad_up=B1_3-B2_3+B3_3 

          IF (transmission % tau_surf(ist,i) > min_tau) THEN
            rad_tmp= (B1_3-B3_3)                                        / &
             & (transmission % tau_layer(lev,ist,i)                    * &
             & transmission % tau_surf(ist,i))                          + &
             & B2_3                                / &
             & (transmission % tau_surf(ist,i))**2 
          ELSE
            rad_tmp=0.0_JPRB
          ENDIF

          meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

          meanrad_up   = auxrad_stream%up(ist,lev,i) + meanrad_up


        endif
      endif

      !---Scattering of the solar beam--------------------------------------------------

      if(addaerosl.or.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then

            !--------------Upward single scattering of the solar beam-------------------------

            auxrad_stream%fac1_3(ist,i)=                                &
             & coef % ss_solar_spectrum(chan)                               * &
             & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r  

            auxrad_stream%fac2_3(ist,i)=                                &
             & transmission_scatt_ir_stream%ssa(ist,i,lev) 

            auxrad_stream%fac3_3(ist,i)=                                &
             & raytracing%pathsat(lev,prof)                                / &
             & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

            auxrad_stream%fac4_3(ist,i)=                                &
             & exp(-transmission % odsun_sfrac(ist,i)) 

            auxrad_stream%fac5_3(ist,i)=                                &
             & exp(-transmission % od_sfrac(ist,i)) 

            auxrad_stream%fac6_3(ist,i)=                                &
             & coef % ss_solar_spectrum(chan)                              * &
             & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

            meanrad_up   = meanrad_up + auxrad_stream%fac1_3(ist,i)   * &
             & auxrad_stream%fac2_3(ist,i)                           * &
             & auxrad_stream%fac3_3(ist,i)                           * &
             & (transmission % tausun_layer(lev,ist,i)               - &
             & transmission % tausun_surf(ist,i)) 

            !--------------Downward single scattering of the solar beam-----------------------

            if(raytracing%pathsat(lev1,prof) /=                             &
             & raytracing%pathsun(lev1,prof))then 

              auxrad_stream%fac7_3(ist,i)=                               &
               & raytracing%pathsat(lev1,prof)                         / &
               & (raytracing%pathsun(lev1,prof)                        - &
               & raytracing%pathsat(lev1,prof)) 

              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                meanrad_down=meanrad_down+auxrad_stream%fac6_3(ist,i)    * &
                 & auxrad_stream%fac2_3(ist,i)                       * &
                 & auxrad_stream%fac7_3(ist,i)                       * &
                 & (auxrad_stream%fac5_3(ist,i)                      - &
                 & auxrad_stream%fac4_3(ist,i))                      * &
                 & transmission %tausun_layer(lev,ist,i)             / &
                 & (transmission %tau_layer(lev,ist,i)               * &
                 & transmission %tau_surf(ist,i)) 
              endif

              if(meanrad_down<0._jprb)then
                meanrad_down=0._jprb
              endif
            else
              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                meanrad_down=meanrad_down+  auxrad_stream%fac6_3(ist,i)  * &
                 & auxrad_stream%fac2_3(ist,i)                       * &
                 & auxrad_stream%fac4_3(ist,i)                       * &
                 & transmission %tausun_layer(lev,ist,i)             * &
                 & transmission % odsun_sfrac(ist,i)                 / &
                 & (transmission %tau_layer(lev,ist,i)               * &
                 & transmission %tau_surf(ist,i)) 
              endif
              if(meanrad_down<0._jprb)then
                meanrad_down=0._jprb
              endif
            endif
          endif
        endif
      endif

    ELSE 

      !--old mean layer code for backward compatibility---------------------------------

      lev = iv3(i)
      meanrad_up = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )   * &
       & ( transmission % tau_layer(lev,ist,i)                      - &
       & transmission % tau_surf(ist,i) ) 

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        rad_tmp = meanrad_up                                         / &
         & ( transmission % tau_layer(lev,ist,i)         * &
         & transmission % tau_surf(ist,i) ) 
      Else
        rad_tmp = 0.0_JPRB
      End If

      meanrad_down = auxrad_stream%down(ist,lev,i) + rad_tmp

      meanrad_up = auxrad_stream%up(ist,lev,i) + meanrad_up
      !-------------------------------------------------------------------------------
    ENDIF

    auxrad_stream%cloudy(ist,i)=meanrad_up                          + &
     & meanrad_down*reflectivity(i)*transmission % tau_surf(ist,i)    * &
     & transmission % tau_surf(ist,i) 
    auxrad_stream%up(ist,iv2(i),i) = meanrad_up

  End Do
Enddo


!-----------------------
!3. surface contribution
!-----------------------

Do i = 1, nchannels
!cdir nodep
  prof=lprofiles(i)
  do ist=1,ircld%nstream(prof)
    !     if(ist==0)then
    !       rad % clear(i) = rad % clear(i)                                      + &
    !       & emissivity(i) * transmission % tau_surf(ist,i) * auxrad%skin(i) 
    !     else
    auxrad_stream%cloudy(ist,i)=auxrad_stream%cloudy(ist,i)  + &
     & emissivity(i) * transmission % tau_surf(ist,i) * auxrad%skin(i) 
    !     endif
  End Do
End Do

! clear sky radiance without reflection term
!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  rad % clear(i) = rad % clear(i)                                      + &
   & emissivity(i) * transmission % tau_surf(0,i) * auxrad%skin(i) 
  rad % upclear(i) = rad % upclear(i) +&
   & emissivity(i) * transmission % tau_surf(0,i) * auxrad%skin(i) 
End Do


!--------------------------------
!4. solar contribution
!--------------------------------
if(coef%fmv_model_ver == 9)then    
  ist=0
!cdir nodep
  Do i = 1, nchannels
    prof=lprofiles(i)
    chan=channels(i)
    if(profiles(prof)%addsolar)then
      if(coef%ss_val_chn(chan)==1)then
        if(profiles(prof)%skin%surftype==surftype_sea)then
          if(sunglint(prof)%windsp==0._jprb)then
            rad % clear(i) = rad % clear(i)                        + &
             & coef % ss_solar_spectrum(chan)           * &
             & sunglint(prof)%glint                     *  &
             & transmission % tausun_surf(ist,i)  * &
             & fresnrefl(i)                          * &
             & cos(profiles(prof)%sunzenangle           * &
             & deg2rad)*pi_r
          else
            rad % clear(i) = rad % clear(i)                        + &
             & coef % ss_solar_spectrum(chan)           * &
             & sunglint(prof)%glint                     *  &
             & transmission % tausun_surf(ist,i)  * &
             & fresnrefl(i) 
          endif
        else if (profiles(prof)%skin%surftype==surftype_land)then
          rad % clear(i) = rad % clear(i) + reflectivity(i)        *  &
           & transmission % tausun_surf(ist,i)  * &
           & coef % ss_solar_spectrum(chan)           * &
           & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
        endif
      endif
    endif
  enddo
  Do i = 1, nchannels
    prof=lprofiles(i)
    chan=channels(i)
    do ist=1,ircld%nstream(prof)
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then
          if(profiles(prof)%skin%surftype==surftype_sea)then
            if(sunglint(prof)%windsp==0._jprb)then
              auxrad_stream%cloudy(ist,i)=                       &
               & auxrad_stream%cloudy(ist,i)        + &
               & coef % ss_solar_spectrum(chan)           * &
               & sunglint(prof)%glint                     * &
               & transmission % tausun_surf(ist,i)  * &
               & fresnrefl(i)                          * &
               & cos(profiles(prof)%sunzenangle           * &
               & deg2rad)*pi_r
            else
              auxrad_stream%cloudy(ist,i)=                       &
               & auxrad_stream%cloudy(ist,i)        + &
               & coef % ss_solar_spectrum(chan)           * &
               & sunglint(prof)%glint                     *  &
               & transmission % tausun_surf(ist,i)  * &
               & fresnrefl(i) 
            endif
          else if (profiles(prof)%skin%surftype==surftype_land)then
            auxrad_stream%cloudy(ist,i)=                         &
             & auxrad_stream%cloudy(ist,i)        + &
             & reflectivity(i)                          *  &
             & transmission % tausun_surf(ist,i)  * &
             & coef % ss_solar_spectrum(chan)           * &
             & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
          endif
        endif
      endif
    enddo
  enddo
endif

!--------------------------------
!5. cosmic temperature correction
!--------------------------------

!calculate planck function corresponding to tcosmic=2.7k
!deblonde tcosmic for microwave sensors only

If ( addcosmic ) Then
  ist=0
!cdir nodep
  Do i = 1, nchannels
    prof=lprofiles(i)
    rad % clear(i) = rad % clear(i) +reflectivity(i) * auxrad%cosmic(i)* &
     & transmission % tau_surf(ist,i)              * &
     & transmission % tau_surf(ist,i) 
  enddo
  Do i = 1, nchannels
    prof=lprofiles(i)
    do ist=1,ircld%nstream(prof)
      auxrad_stream%cloudy(ist,i)=auxrad_stream%cloudy(ist,i)+ &
       & reflectivity(i) * auxrad%cosmic(i)                * &
       & transmission % tau_surf(ist,i)              * &
       & transmission % tau_surf(ist,i)
    enddo
  enddo
Endif


!---------------------------------------------------
!6. calculate overcast and "simple" cloudy radiances
!---------------------------------------------------
!---------------
!6.1 Upward part
!---------------
! (levels, channels)
! overcast radiance at given cloud top
! rad % overcast(:,:) = auxrad_stream%up(0,:,:) + auxrad%layer(:,:) * transmission % tau_layer(:,0,:)
rad%up(:,1:nchannels)= auxrad_stream%up(0,:,1:nchannels)  ! Phil Watts parameter

Do i = 1, nchannels
!cdir nodep
  Do lev=1,profiles(1)%nlevels
    rad % overcast(lev,i) = auxrad_stream%up(0,lev,i) + auxrad%layer(lev,i) * transmission % tau_layer(lev,0,i)
  Enddo
Enddo

!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  !    aux  => aux_prof( lprofiles(i) )
  lev = aux_prof( prof) % nearestlev_surf
  rad % overcast(lev,i) = auxrad_stream%up(0,lev,i) + transmission % tau_surf(0,i) * auxrad%surfair(i)

  !--------------------------------------------
  !6.2 Interpolate to given cloud-top pressures
  !--------------------------------------------

  lev = aux_prof( prof) % nearestlev_ctp
  rad%cloudy(i) = rad % overcast(lev,i) * (1.0_JPRB-aux_prof( prof) % pfraction_ctp) + &
   & rad % overcast(lev-1,i) * (aux_prof( prof) % pfraction_ctp)
End Do

!----------------------------------------
!7. calculate complex cloudy radiances 
!----------------------------------------
If(addaerosl.OR.addclouds)then
  rad % cloudy=0.
  Do i = 1, nchannels
    prof=lprofiles(i)
    do ist=1,ircld%nstream(prof)
      if(ist==1)then
        rad % cloudy(i)=auxrad_stream % cloudy(ist,i)                  * &
         & (ircld%xstr(ist+1,prof)-ircld%xstr(ist,prof))
      else
        rad % cloudy(i)=rad % cloudy(i)                                      + &
         & auxrad_stream % cloudy(ist,i)                  * &
         & (ircld%xstr(ist+1,prof)-ircld%xstr(ist,prof))
      endif
    enddo
  End do
!cdir nodep
  Do i = 1, nchannels
    prof=lprofiles(i)
    rad % cloudy(i)=rad % cloudy(i)+rad % clear(i)*ircld%XSTRCLR(prof)
  End do

Endif

!---------------------------
!8. calculate total radiance
!---------------------------

If(addaerosl.OR.addclouds)then
  rad % total(:) = rad%cloudy(:)
Else
  rad % total(1:nchannels) = rad%clear(1:nchannels) + cfraction(:) * &
  & ( rad%cloudy(1:nchannels) - rad%clear(1:nchannels) )
Endif

!-----------------------------------------------
!9. convert radiances to brightness temperatures
!-----------------------------------------------

Call rttov_calcbt( &
 & nchannels,  &! in
 & channels,   &! in
 & coef,       &! in
 & rad        ) ! inout 


IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_integrate
