Subroutine rttov_integrate_k( &
 & addcosmic,                      &! in
 & addaerosl,                      &
 & addclouds,                      &
 & switchrad,                      &! in
 & nchannels,                      &! in
 & nprofiles,                      &! in
 & nstreams ,                      &! in
 & angles,                         &! in
 & channels,                       &! in
 & lprofiles,                      &! in
 & emissivity,                     &! in
 & emissivity_k,                   &! inout
 & reflectivity,                   &! in
 & reflectivity_k,                 &! inout
 & fresnrefl   ,                   &! in
 & fresnrefl_k   ,                 &! inout
 & sunglint,                       &
 & sunglint_k,                     &
 & transmission,                   &! in
 & transmission_k,                 &! inout
 & transmission_scatt_ir_stream,   &
 & transmission_scatt_ir_stream_k, &
 & profiles,                       &! in
 & profiles_k,                     &! inout
 & aux_prof,                       &! in
 & aux_prof_k,                     &! inout
 & coef,                           &! in
 & raytracing,                     &! i
 & raytracing_k,                   &! in
 & ircld,                          &! in
 & ircld_k,                        &! in
 & rad ,                           &! in
 & auxrad ,                        &! in
 & auxrad_stream,                  &! in
 & auxrad_stream_k,                &! in
 & rad_k           ) ! inout 
!
! Description:
! To perform K of integration of radiative transfer equation
! in rttov suite, calculating radiances and brightness temperature.
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Method:
! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
! librarian at ECMWF).
!
! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
! Transfer Model for Assimilation of Satellite Radiance Observations.
! QJRMS, 125, 1407-1425.
!   
! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
! transfer model for the infrared atmospheric sounding interferometer.
! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)  !
!
! Matricardi, M. 2005 The inclusion of aerosols and clouds in RTIASI, the ECMWF
! fast radiative transfer model for the infrared atmospheric sounding interferometer.
! ECMWF Research Dept. Tech. Memo. 474 (available from the librarian at ECMWF)  !
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
!          25/06/91.    Original code.  J.R.EYRE    *ECMWF*
!          21/08/00.    Emissivity and reflectivity handled separately. Steve English
!          31/01/01.    More cloud computations. F. Chevallier
!          23/03/01     New coef. format, new channel numbers (P. Brunel)
!          31/01/01.    More cloud computations. F. Chevallier
!          28/09/01     Cosmic background temp added G.Deblonde
!          18/01/2002   Thread safe (D.Salmond)
!  1.0     01/12/2002   New F90 code with structures (P Brunel A Smith)
!  1.1     02/01/2003   Added comments (R Saunders)
!  1.2     06/05/2003   Init rad%downcld to 0 in section 1 (P  Brunel)
!  1.3     26/09/2003   Modified to allow for multiple polarisations (S English)
!  1.4     10/05/2004   Fixed bug in call to rttov_calcpolarisation (D.Salmond)
!  1.5     06/09/2004   Mods. for Vectorisation (D Salmond ECMWF & B  Carruthers, Cray)
!  1.6     28/02/2005   More improvements to vectorisation (D Dent)
!  1.7     29/03/2005   Add end of header comment (J. Cameron)
!  1.8     03/03/2006   Marco Matricardi (ECMWF):
!             --        IASI capability added.
!             --        Linear in tau approximation for RT equation introduced.
!             --        Solar radiation introduced for IASI and AIRS.
!  1.9     01.07/2006   Marco Matricardi (ECMWF):
!             --        A parameterization of multiple scattering for aerosols
!                       and clouds has been included in RTTOV.
!             --        The "stream method" has been introduced to compute
!                       radiances in presence of partially cloudy layers
!             --        The single scattering of the solar beam has been
!                       introduced.
!          30/08/2006   New version of this routine created based on AD  
!  1.10    08/01/2007   Corrected some bugs in the 'old' code (R Saunders)
!  1.11    09/02/2007   Removed polarisation index
!  1.12    13/03/2007   Reintroduced overcast radiance (R Saunders)
!  1.13    27/11/2007   Optimised for IBM/NEC (D Salmond)
!  1.14    17/11/2007   Allowed adjoint/K sensitivity to clear BT/radiance (A Geer)
!  1.15    21/12/2007   Added polarimetric option (R. Saunders)
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!
! Declarations:
! Modules used:
! Imported Type Definitions:

! Adjoint Variables
! transmission_k % tau_surf        |
! transmission_k % tau_layer       |
! emissivity_k                     |
! reflectivity_k                   | initialised before calling
! profiles_k                       |
! aux_prof_k                       |

Use rttov_const, only :   &
 & sensor_id_po,      &
 & surftype_land  ,   &
 & surftype_sea   ,   &
 & deg2rad        ,   &
 & pi             ,   &
 & rt8_mode       ,   &
 & min_tau        ,   &
 & min_od
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

Use rttov_types, Only : &
 & geometry_Type  ,&
 & rttov_coef     ,&
 & profile_Type   ,&
 & profile_aux    ,&
 & transmission_type, &
 & transmission_scatt_ir_type , &
 & sunglint_type   , &
 & radiance_Type   , &
 & ircld_type      , &
 & raytracing_type , &
 & radiance_aux 

Use parkind1, Only : jpim     ,jprb
Implicit None

!subroutine arguments:
Logical,             Intent(in)    :: addcosmic
Logical,             Intent(in)    :: addaerosl
Logical,             Intent(in)    :: addclouds
Logical,             Intent(in)    :: switchrad  ! true if input is BT
Integer(Kind=jpim),  Intent(in)    :: nchannels
Integer(Kind=jpim),  Intent(in)    :: nprofiles
Integer(Kind=jpim),  Intent(in)    :: nstreams                ! 
Integer(Kind=jpim),  Intent(in)    :: channels(nchannels)
Integer(Kind=jpim),  Intent(in)    :: lprofiles(nchannels)
Real(Kind=jprb),     Intent(in)    :: emissivity(nchannels)
Real(Kind=jprb),     Intent(in)    :: reflectivity(nchannels)
Real(Kind=jprb),     Intent(in)    :: fresnrefl(nchannels)
Type(geometry_Type), Intent(in)    :: angles(nprofiles)  ! geometry angles
Type(rttov_coef),    Intent(in)    :: coef
Type(profile_Type),  Intent(in)    :: profiles(nprofiles)
Type(profile_aux) ,  Intent(in)    :: aux_prof(nprofiles)
Type(transmission_Type), Intent(in)         :: transmission
type(transmission_scatt_ir_type) ,intent(in):: transmission_scatt_ir_stream
Type(ircld_type)                 ,intent(in):: ircld
Type(raytracing_type)            ,intent(in):: raytracing
Type(sunglint_type),  Intent(in)            :: sunglint(nprofiles)
Type(sunglint_type),  Intent(inout)         :: sunglint_k(nchannels)
Type(radiance_Type), Intent(in)             :: rad
Type(radiance_aux),  Intent(inout)          :: auxrad
Type(radiance_aux),  Intent(inout)          :: auxrad_stream
Type(radiance_aux),  Intent(inout)          :: auxrad_stream_k

Real(Kind=jprb),             Intent(inout)  :: emissivity_k(nchannels)
Real(Kind=jprb),             Intent(inout)  :: reflectivity_k(nchannels)
Real(Kind=jprb),             Intent(inout)  :: fresnrefl_k(nchannels)

Type(profile_Type), Intent(inout)   :: profiles_k(nchannels)
Type(profile_aux) , Intent(inout)   :: aux_prof_k(nchannels)
Type(transmission_Type), Intent(inout)      :: transmission_k
type(transmission_scatt_ir_type) ,intent(inout)  :: transmission_scatt_ir_stream_k
Type(ircld_type)                 ,intent(inout)  :: ircld_k
Type(raytracing_type)            ,intent(inout)  :: raytracing_k
Type(radiance_Type), Intent(inout)  :: rad_k

!INTF

#include "rttov_calcbt_ad.interface"
#include "rttov_calcrad_k.interface"

!local variables:
Real(Kind=jprb) :: cfraction(nchannels)
Real(Kind=jprb) :: cfraction_k(nchannels)
Real(Kind=jprb) :: rad_layer_k(profiles(1) % nlevels,nchannels)
Real(Kind=jprb) :: rad_surfair_k(nchannels)
Real(Kind=jprb) :: rad_skin_k(nchannels)
!Real(Kind=jprb) :: rad_up_k(profiles(1) % nlevels, nchannels)
!Real(Kind=jprb) :: rad_down_k(profiles(1) % nlevels, nchannels)
!Real(Kind=jprb) :: rad_down_cloud_k(profiles(1) % nlevels, nchannels)
Real(Kind=jprb) :: zup_k(profiles(1) % nlevels,0:nstreams,nchannels)
Real(Kind=jprb) :: zdown_k(profiles(1) % nlevels,0:nstreams,nchannels)
Real(Kind=jprb) :: rad_tmp_k(nchannels)
!Real(Kind=jprb) :: rad_tmp_k_2d(nchannels, profiles(1) % nlevels)
Real(Kind=jprb) :: meanrad_up_k(nchannels)
!Real(Kind=jprb) :: meanrad_up_k_2d(nchannels, profiles(1) % nlevels)
Real(Kind=jprb) :: meanrad_down_k(nchannels)
Real(Kind=jprb) :: z4pi_r, pi_r
Real(Kind=jprb) :: tau_prod
Real(Kind=jprb) :: dtau_lay,daux_lay
Real(Kind=jprb) :: b1_1,b2_1,b3_1
Real(Kind=jprb) :: b1_2,b2_2,b3_2
Real(Kind=jprb) :: b1_3(0:nstreams,nchannels),b2_3(0:nstreams,nchannels),b3_3(0:nstreams,nchannels)
Real(Kind=jprb) :: b1_k
Real(Kind=jprb) :: b2_k
Real(Kind=jprb) :: b3_k

Integer(Kind=jpim) :: i,lev,lev1,lev2,prof,chan,ist
Integer(Kind=jpim) :: pol_id       ! polarisation indice
Integer(Kind=jpim) :: aux_levels(nchannels)
Integer(Kind=jpim) :: aux_nearestlev_surf(nchannels)
Real(Kind=jprb)    :: aux_pfraction(nchannels)
Real(Kind=jprb)    :: od_singlelayer_r

Real(Kind=jprb) :: rad_up_k_chan(nchannels,profiles(1) % nlevels,0:nstreams)
Real(Kind=jprb) :: rad_dwn_k_chan(nchannels,profiles(1) % nlevels,0:nstreams)
Real(Kind=jprb) :: fac1_1_k
Real(Kind=jprb) :: fac2_1_k
Real(Kind=jprb) :: fac3_1_k
Real(Kind=jprb) :: fac4_1_k
Real(Kind=jprb) :: fac5_1_k
Real(Kind=jprb) :: fac6_1_k
Real(Kind=jprb) :: fac7_1_k
Real(Kind=jprb) :: fac1_2_k
Real(Kind=jprb) :: fac2_2_k
Real(Kind=jprb) :: fac3_2_k
Real(Kind=jprb) :: fac4_2_k
Real(Kind=jprb) :: fac5_2_k
Real(Kind=jprb) :: fac6_2_k
Real(Kind=jprb) :: fac7_2_k
! Type(profile_aux),   Pointer   :: aux
! Type(profile_aux),   Pointer   :: aux_k
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!- End of header --------------------------------------------------------
!----------------------------------------------------------------------------
! initialise radiance structure cloud flag

IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_K',0_jpim,ZHOOK_HANDLE)
! rad_k % lcloud = addcloud
z4pi_r = 1.0_JPRB/(4._jprb*pi)
pi_r = 1.0_JPRB/pi

!----------------------------
!9. Convert radiances to BTs 
!----------------------------
! if input AD unit is temperature, convert it in radiance
if ( switchrad )  then
  Call rttov_calcbt_ad(  &
   & nchannels,      &! in
   & channels,       &! in
   & coef,           &! in
   & rad,            &! in
   & rad_k     )     ! inout   
endif

! initialisation of local variables
cfraction_k(:)   = 0._JPRB
rad_layer_k(:,:) = 0._JPRB
rad_surfair_k(:) = 0._JPRB
rad_skin_k(:)    = 0._JPRB
!rad_up_k(:,:)         = 0._JPRB
!rad_down_k(:,:)       = 0._JPRB
!rad_down_cloud_k(:,:) = 0._JPRB
b1_k                  = 0._JPRB
b2_k                  = 0._JPRB
b3_k                  = 0._JPRB
rad_k%upclear(:)      = 0._JPRB
rad_k%reflclear(:)    = 0._JPRB
meanrad_up_k(:)       = 0._JPRB
meanrad_down_k(:)     = 0._JPRB
zup_k(:,:,:)          = 0._jprb
zdown_k(:,:,:)        = 0._jprb

Do i = 1, nchannels
  cfraction(i) = aux_prof( lprofiles(i) )%cfraction
End Do
!---------------------------
!8. calculate total radiance
!---------------------------

If(addaerosl.OR.addclouds)then
  rad_k%cloudy(1:nchannels)=rad_k % total(1:nchannels)
Else
  rad_k%clear(1:nchannels)  = rad_k%clear(1:nchannels) + (1 - cfraction(1:nchannels)) * rad_k%total(1:nchannels)
  rad_k%cloudy(1:nchannels) = cfraction(1:nchannels) * rad_k%total(1:nchannels)
  cfraction_k(1:nchannels)  = ( rad%cloudy(1:nchannels) - rad%clear(1:nchannels) ) * rad_k%total(1:nchannels)
Endif

Do i = 1, nchannels
  aux_prof_k(i)%cfraction = aux_prof_k(i)%cfraction + cfraction_k(i) 
End Do

!----------------------------------------
!7. calculate complex cloudy radiances
!----------------------------------------

If(addaerosl.OR.addclouds)then
  Do i = 1, nchannels
    prof=lprofiles(i)
    rad_k % clear(i)     = rad_k % clear(i)+rad_k% cloudy(i)*ircld%XSTRCLR(prof)
    ircld_k%XSTRCLR(i)= ircld_k%XSTRCLR(i)+rad_k % cloudy(i)*rad % clear(i)
    do ist=ircld%nstream(prof),1,-1
      if(ist==1)then
        auxrad_stream_k % cloudy(ist,i)=auxrad_stream_k % cloudy(ist,i)  + &
         & rad_k % cloudy(i)*(ircld%xstr(ist+1,prof)-ircld%xstr(ist,prof)) 

        ircld_k%xstr(ist+1,i)=ircld_k%xstr(ist+1,i)+rad_k % cloudy(i)       * &
         & auxrad_stream % cloudy(ist,i) 

        ircld_k%xstr(ist,i)=ircld_k%xstr(ist,i)-rad_k % cloudy(i)           * &
         & auxrad_stream % cloudy(ist,i) 

      else
        auxrad_stream_k % cloudy(ist,i)=auxrad_stream_k % cloudy(ist,i)  + &
         & rad_k % cloudy(i)*(ircld%xstr(ist+1,prof)-ircld%xstr(ist,prof)) 

        ircld_k%xstr(ist+1,i)=ircld_k%xstr(ist+1,i)+rad_k % cloudy(i)       * &
         & auxrad_stream % cloudy(ist,i) 

        ircld_k%xstr(ist,i)=ircld_k%xstr(ist,i)-rad_k % cloudy(i)           * &
         & auxrad_stream % cloudy(ist,i) 
      endif
    enddo
  End do
  !
Else
  !-------------------------------------------------
  !6. calculate overcast and simple cloudy radiances
  !-------------------------------------------------
  Do i = 1, nchannels
    !    aux     => aux_prof(    lprofiles(i) )
    !    aux_k  => aux_prof_k( i )

    !--------------------------------------------
    !6.2 Interpolate to given cloud-top pressures
    !--------------------------------------------
    lev = aux_prof(    lprofiles(i) ) % nearestlev_ctp
    rad_k%overcast(lev,i)   = rad_k%overcast(lev,i) +&
     & (1 - aux_prof(    lprofiles(i) ) % pfraction_ctp) * rad_k%cloudy(i)

    rad_k%overcast(lev-1,i) = rad_k%overcast(lev-1,i) +&
     & aux_prof(    lprofiles(i) ) % pfraction_ctp * rad_k%cloudy(i)

    aux_prof_k( i ) % pfraction_ctp = aux_prof_k( i ) % pfraction_ctp +&
     & (rad%overcast(lev-1,i)  - rad%overcast(lev,i)) * rad_k%cloudy(i)
    lev = aux_prof(    lprofiles(i) ) % nearestlev_surf
    aux_nearestlev_surf(i) = lev
    aux_pfraction(i) = aux_prof(    lprofiles(i) ) % pfraction_surf
    lev2 = lev - 1
    If ( aux_pfraction(i) < 0.0_JPRB ) lev2 = lev2 + 1
    aux_levels(i) = lev2
  End Do

  !--------------------------------------------
  !6.1 Upwelling radiances
  !--------------------------------------------
  zup_k(:,0,1:nchannels)    = zup_k(:,0,1:nchannels)    + rad_k % overcast(:,1:nchannels)
  rad_layer_k(:,1:nchannels) = rad_layer_k(:,1:nchannels) + rad_k % overcast(:,1:nchannels)&
   & * transmission % tau_layer(:,0,1:nchannels)
  transmission_k % tau_layer(:,0,1:nchannels) = transmission_k % tau_layer(:,0,1:nchannels) + rad_k % overcast(:,1:nchannels) *&
   & auxrad%layer(:,1:nchannels)
  rad_k % overcast(:,1:nchannels) = 0._JPRB
Endif
rad_k % cloudy=0._jprb

!--------------------------------
!5. cosmic temperature correction
!--------------------------------

!calculate planck function corresponding to tcosmic=2.7k
!deblonde tcosmic for microwave sensors only

If ( addcosmic ) Then
  Do i = 1, nchannels
    prof=lprofiles(i)
    do ist=ircld%nstream(prof),0,-1
      if(ist==0)then
        reflectivity_k(i) = reflectivity_k(i) + rad_k % clear(i) *&
         & auxrad%cosmic(i) * transmission % tau_surf(ist,i) * transmission % tau_surf(ist,i) 

        transmission_k % tau_surf(ist,i) = transmission_k % tau_surf(ist,i) + rad_k % clear(i) *&
         & 2 * reflectivity(i) * auxrad%cosmic(i) * transmission % tau_surf(ist,i) 
      else
        reflectivity_k(i)=reflectivity_k(i)+auxrad_stream_k%cloudy(ist,i)   * &
         & auxrad%cosmic(i) * transmission % tau_surf(ist,i)                * &
         & transmission % tau_surf(ist,i) 

        transmission_k%tau_surf(ist,i)=transmission_k%tau_surf(ist,i)  + &
         & auxrad_stream_k%cloudy(ist,i)*2*reflectivity(i)*auxrad%cosmic(i)* &
         & transmission % tau_surf(ist,i) 
      endif
    enddo
  enddo
Endif

!----------------------------------------------------------------------------------------
! 4. solar contribution
!----------------------------------------------------------------------------------------
if(coef%fmv_model_ver == 9)then
  Do i=1,nchannels
    prof=lprofiles(i)
    chan=channels(i)
    do ist=ircld%nstream(prof),0,-1
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then
          if(profiles(prof)%skin%surftype==surftype_sea)then
            if(sunglint(prof)%windsp==0)then
              if(ist==0)then
                sunglint_k(i)%glint=sunglint_k(i)%glint                    + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & transmission % tausun_surf(ist,i)                     * &
                 & fresnrefl(i)*cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
                transmission_k % tausun_surf(ist,i)=                          &
                 & transmission_k % tausun_surf(ist,i)                  + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & sunglint(prof)%glint * fresnrefl(i)                      * &
                 & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
                fresnrefl_k(i)=fresnrefl_k(i)                              + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & sunglint(prof)%glint*transmission % tausun_surf(ist,i)* &
                 & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
              else
                sunglint_k(i)%glint=sunglint_k(i)%glint                    + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & transmission % tausun_surf(ist,i)                     * &
                 & fresnrefl(i)*cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
                transmission_k % tausun_surf(ist,i)=                          &
                 & transmission_k % tausun_surf(ist,i)                  + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & sunglint(prof)%glint * fresnrefl(i)                      * &
                 & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
                fresnrefl_k(i)=fresnrefl_k(i)                              + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & sunglint(prof)%glint*transmission % tausun_surf(ist,i)* &
                 & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r

              endif
            else
              if(ist==0)then
                sunglint_k(i)%glint=sunglint_k(i)%glint                    + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & transmission % tausun_surf(ist,i)*fresnrefl(i) 
                transmission_k % tausun_surf(ist,i)=                          &
                 & transmission_k % tausun_surf(ist,i)                  + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & sunglint(prof)%glint*fresnrefl(i) 
                fresnrefl_k(i)=fresnrefl_k(i)                              + &
                 & rad_k % clear(i)*coef % ss_solar_spectrum(chan)            * &
                 & sunglint(prof)%glint*transmission % tausun_surf(ist,i) 
              else
                sunglint_k(i)%glint=sunglint_k(i)%glint                    + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & transmission % tausun_surf(ist,i)*fresnrefl(i) 
                transmission_k % tausun_surf(ist,i)=                          &
                 & transmission_k % tausun_surf(ist,i)                  + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & sunglint(prof)%glint*fresnrefl(i) 
                fresnrefl_k(i)=fresnrefl_k(i)                              + &
                 & auxrad_stream_k%cloudy(ist,i)                        * &
                 & coef % ss_solar_spectrum(chan)                              * &
                 & sunglint(prof)%glint*transmission % tausun_surf(ist,i) 

              endif
            endif
          else if (profiles(prof)%skin%surftype==surftype_land)then
            if(ist==0)then
              reflectivity_k(i)=reflectivity_k(i)                                + &
               & rad_k % clear(i)*transmission % tausun_surf(ist,i)   * &
               & coef % ss_solar_spectrum(chan)                              * &
               & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
              transmission_k % tausun_surf(ist,i)=                            &
               & transmission_k % tausun_surf(ist,i)                  + &
               & rad_k % clear(i)*reflectivity(i)                           * &
               & coef % ss_solar_spectrum(chan)                              * &
               & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r

            else
              reflectivity_k(i)=reflectivity_k(i)                                + &
               & auxrad_stream_k%cloudy(ist,i)                        * &
               & transmission % tausun_surf(ist,i)                     * &
               & coef % ss_solar_spectrum(chan)                              * &
               & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r
              transmission_k % tausun_surf(ist,i)=                            &
               & transmission_k % tausun_surf(ist,i)                  + &
               & auxrad_stream_k%cloudy(ist,i)*reflectivity(i)        * &
               & coef % ss_solar_spectrum(chan)                              * &
               & cos(profiles(prof)%sunzenangle*deg2rad)*pi_r

            endif
          endif
        endif
      endif
    enddo
  enddo
endif

!---------------------------------------------------------------------------------------
! clear sky radiance without reflection term
!---------------------------------------------------------------------------------------

Do i = 1, nchannels
  prof=lprofiles(i)
  emissivity_k(i)  = emissivity_k(i) + rad_k % upclear(i)                         * &
   & auxrad%skin(i) * transmission % tau_surf(0,i) 
  transmission_k % tau_surf(0,i)    = transmission_k % tau_surf(0,i)   + &
   & rad_k % upclear(i) *auxrad%skin(i) * emissivity(i) 
  rad_skin_k(i)    = rad_skin_k(i)   + rad_k % upclear(i)                         * &
   & emissivity(i) * transmission % tau_surf(0,i) 
Enddo

!---------------------------------------------------------------------------------------
!3. surface contribution
!---------------------------------------------------------------------------------------
!rad_k % clear(:) = rad_k % clear(:)

Do i = 1, nchannels
  prof=lprofiles(i)
  do ist=ircld%nstream(prof),1,-1
    emissivity_k(i)  = emissivity_k(i) + auxrad_stream_k%cloudy(ist,i)    * &
     & auxrad%skin(i) * transmission % tau_surf(ist,i) 

    transmission_k % tau_surf(ist,i) = transmission_k%tau_surf(ist,i)+ &
     & auxrad_stream_k%cloudy(ist,i) *auxrad%skin(i) * emissivity(i) 

    rad_skin_k(i)    = rad_skin_k(i)   + auxrad_stream_k%cloudy(ist,i)    * &
     & emissivity(i) * transmission % tau_surf(ist,i) 
  enddo
Enddo
ist=0
Do i = 1, nchannels
  prof=lprofiles(i)
  emissivity_k(i)  = emissivity_k(i) + rad_k % clear(i)                       * &
   & auxrad%skin(i) * transmission % tau_surf(ist,i)

  transmission_k % tau_surf(ist,i)=transmission_k % tau_surf(ist,i)+ &
   & rad_k % clear(i) *auxrad%skin(i) * emissivity(i)

  rad_skin_k(i)    = rad_skin_k(i)   + rad_k % clear(i) *emissivity(i)        * &
   & transmission % tau_surf(ist,i)

Enddo


!---------------------------------------------------------------------------------------
!2.3 near-surface layer
!---------------------------------------------------------------------------------------
! add upward and downward parts

Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  !    aux     => aux_prof( lprofiles(i) )
  rad_tmp_k(i) = 0._JPRB

  lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
  If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1

  If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB )then
    lev1=lev
  else
    lev1=lev+1
  endif

  !---Repeat direct calculations------------------------------------------------------------

  do ist=1,ircld%nstream(prof)

    If (.not. rt8_mode)then  !----------New Code---------------------------------------------

      if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.                       &
       & transmission % tau_surf(ist,i)<0._JPRB)then 

        auxrad_stream%meanrad_up(ist,i) = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )   * &
         & ( transmission % tau_layer(lev,ist,i)                               -    &
         & transmission % tau_surf(ist,i) ) 

        If ( transmission % tau_surf(ist,i) > min_tau ) Then
          auxrad_stream%rad_tmp(ist,i) = auxrad_stream%meanrad_up(ist,i)           / &
           & ( transmission % tau_layer(lev,ist,i)               * &
           & transmission % tau_surf(ist,i) ) 
        Else
          auxrad_stream%rad_tmp(ist,i)    = 0.0_JPRB
        End If

        auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)

      else if(transmission%od_sfrac(ist,i)<min_od.OR.                            &
       & ((transmission % tau_layer(lev,ist,i)                                 - &
       & transmission % tau_surf(ist,i))<min_od))THEN 
        auxrad_stream%meanrad_up(ist,i)=0._JPRB
        auxrad_stream%rad_tmp(ist,i) = 0._JPRB
        auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
        auxrad_stream%meanrad_up(ist,i)   = auxrad_stream%up(ist,lev,i) + auxrad_stream%meanrad_up(ist,i)
      else
        B1_3(ist,i)=auxrad%layer(lev,i)                          * &
         & (transmission % tau_layer(lev,ist,i)                                 - &
         & transmission % tau_surf(ist,i)) 
        B2_3(ist,i)=(auxrad%surfair(i)-auxrad%layer(lev,i))      * &
         & transmission % tau_surf(ist,i) 
        B3_3(ist,i)=(auxrad%surfair(i)-auxrad%layer(lev,i))      * &
         & (transmission % tau_layer(lev,ist,i)                                 - &
         & transmission % tau_surf(ist,i))/(transmission %od_sfrac (ist,i)) 
        auxrad_stream%meanrad_up(ist,i)=B1_3(ist,i)                            - &
         & B2_3(ist,i)                                            + &
         & B3_3(ist,i) 

        IF (transmission % tau_surf(ist,i) > min_tau) THEN
          auxrad_stream%rad_tmp(ist,i) = &
           & (B1_3(ist,i)-B3_3(ist,i))          / &
           & (transmission % tau_layer(lev,ist,i)                                 * &
           & transmission % tau_surf(ist,i))                                      + &
           & B2_3(ist,i)/(transmission % tau_surf(ist,i))**2 
        ELSE
          auxrad_stream%rad_tmp(ist,i) = 0._JPRB
        ENDIF

        auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
        auxrad_stream%meanrad_up(ist,i)   = auxrad_stream%up(ist,lev,i) + auxrad_stream%meanrad_up(ist,i)
      endif

      !---Scattering of the solar beam----------------------------------------------------------

      if(addaerosl.or.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then

            !--------------Upward single scattering of the solar beam---------------------------------

            auxrad_stream%fac1_3(ist,i)=                                        &
             & coef % ss_solar_spectrum(chan)                                       * &
             & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r 

            auxrad_stream%fac2_3(ist,i)=                                        &
             & transmission_scatt_ir_stream%ssa(ist,i,lev) 

            auxrad_stream%fac3_3(ist,i)=                                        &
             & raytracing%pathsat(lev,prof)                                        / &
             & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

            auxrad_stream%fac4_3(ist,i)=                                        &
             & exp(-transmission % odsun_sfrac(ist,i)) 

            auxrad_stream%fac5_3(ist,i)=                                        &
             & exp(-transmission % od_sfrac(ist,i)) 

            auxrad_stream%fac6_3(ist,i)=                                        &
             & coef % ss_solar_spectrum(chan)                                      * &
             & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

            auxrad_stream%meanrad_up(ist,i)=auxrad_stream%meanrad_up(ist,i)+auxrad_stream%fac1_3(ist,i)* &
             & auxrad_stream%fac2_3(ist,i)                                   * &
             & auxrad_stream%fac3_3(ist,i)                                   * &
             & (transmission % tausun_layer(lev,ist,i)                       - &
             & transmission % tausun_surf(ist,i)) 

            !--------------Downward single scattering of the solar beam-------------------------------

            if(raytracing%pathsat(lev1,prof) /=                                     &
             & raytracing%pathsun(lev1,prof))then 
              auxrad_stream%fac7_3(ist,i)=                                      &
               & raytracing%pathsat(lev1,prof)                                / &
               & (raytracing%pathsun(lev1,prof)                               - &
               & raytracing%pathsat(lev1,prof)) 
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              auxrad_stream%meanrad_down(ist,i)=auxrad_stream%meanrad_down(ist,i)      + &
               & auxrad_stream%fac6_3(ist,i)                              * &
               & auxrad_stream%fac2_3(ist,i)                              * &
               & auxrad_stream%fac7_3(ist,i)                              * &
               & (auxrad_stream%fac5_3(ist,i)                             - &
               & auxrad_stream%fac4_3(ist,i))                             * &
               & transmission %tausun_layer(lev,ist,i)                    / &
               & (transmission %tau_layer(lev,ist,i)                      * &
               & transmission %tau_surf(ist,i)) 
             endif
             auxrad_stream%meanrad_down_ref(ist,i)=auxrad_stream%meanrad_down(ist,i)

              if(auxrad_stream%meanrad_down(ist,i)<0._jprb)then
                auxrad_stream%meanrad_down(ist,i)=0._jprb
              endif

            else
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              auxrad_stream%meanrad_down(ist,i)=auxrad_stream%meanrad_down(ist,i)      + &
               & auxrad_stream%fac6_3(ist,i)                              * &
               & auxrad_stream%fac2_3(ist,i)                              * &
               & auxrad_stream%fac4_3(ist,i)                              * &
               & transmission %tausun_layer(lev,ist,i)                    * &
               & transmission % odsun_sfrac(ist,i)                        / &
               & (transmission %tau_layer(lev,ist,i)                      * &
               & transmission %tau_surf(ist,i)) 
            endif
              auxrad_stream%meanrad_down_ref(ist,i)=auxrad_stream%meanrad_down(ist,i)

              if(auxrad_stream%meanrad_down(ist,i)<0._jprb)then
                auxrad_stream%meanrad_down(ist,i)=0._jprb
              endif

            endif
          endif
        endif
      endif
    Else  !---------------Old Code-------------------------------------------------------

      lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
      If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1

      auxrad_stream%meanrad_up(ist,i) = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )    * &
       & ( transmission % tau_layer(lev,ist,i)                               - &
       & transmission % tau_surf(ist,i) ) 

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        auxrad_stream%rad_tmp(ist,i) = auxrad_stream%meanrad_up(ist,i)          / &
         & ( transmission % tau_layer(lev,ist,i)                               * &
         & transmission % tau_surf(ist,i) ) 
      Else
        auxrad_stream%rad_tmp(ist,i)    = 0.0_JPRB
      End If
      auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
    Endif
  enddo
end do
ist=0
!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  If(coef % id_sensor == sensor_id_po)Then
    pol_id = coef % fastem_polar(chan) + 1_jpim
  Else
    pol_id = 0_jpim
  Endif
  !    aux     => aux_prof( lprofiles(i) )
  rad_tmp_k(i) = 0._JPRB

  !---Repeat direct calculations------------------------------------------------------------


  If (.not. rt8_mode)then  !----------New Code---------------------------------------------
    lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
    If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1
    If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB )then
      lev1=lev
    else
      lev1=lev+1
    endif
    if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.                       &
     & transmission % tau_surf(ist,i)<0._JPRB)then 

      auxrad_stream%meanrad_up(ist,i) = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )   * &
       & ( transmission % tau_layer(lev,ist,i)                               -    &
       & transmission % tau_surf(ist,i) ) 

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        auxrad_stream%rad_tmp(ist,i) = auxrad_stream%meanrad_up(ist,i)           / &
         & ( transmission % tau_layer(lev,ist,i)               * &
         & transmission % tau_surf(ist,i) ) 
      Else
        auxrad_stream%rad_tmp(ist,i)    = 0.0_JPRB
      End If

      auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) auxrad_stream%meanrad_up(ist,i) = 0.0_jprb
    else if(transmission%od_sfrac(ist,i)<min_od.OR.                            &
     & ((transmission % tau_layer(lev,ist,i)                                 - &
     & transmission % tau_surf(ist,i))<min_od))THEN 
      auxrad_stream%meanrad_up(ist,i)=0._JPRB
      auxrad_stream%rad_tmp(ist,i) = 0._JPRB
      auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
      auxrad_stream%meanrad_up(ist,i)   = auxrad_stream%up(ist,lev,i) + auxrad_stream%meanrad_up(ist,i)
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) auxrad_stream%meanrad_up(ist,i) = 0.0_jprb
    else
      B1_3(ist,i)=auxrad%layer(lev,i)                          * &
       & (transmission % tau_layer(lev,ist,i)                                 - &
       & transmission % tau_surf(ist,i)) 
      B2_3(ist,i)=(auxrad%surfair(i)-auxrad%layer(lev,i))      * &
       & transmission % tau_surf(ist,i) 
      B3_3(ist,i)=(auxrad%surfair(i)-auxrad%layer(lev,i))      * &
       & (transmission % tau_layer(lev,ist,i)                                 - &
       & transmission % tau_surf(ist,i))/(transmission %od_sfrac (ist,i)) 
      auxrad_stream%meanrad_up(ist,i)=B1_3(ist,i)                            - &
       & B2_3(ist,i)                                            + &
       & B3_3(ist,i) 

      IF (transmission % tau_surf(ist,i) > min_tau) THEN
        auxrad_stream%rad_tmp(ist,i) = &
         & (B1_3(ist,i)-B3_3(ist,i))          / &
         & (transmission % tau_layer(lev,ist,i)                                 * &
         & transmission % tau_surf(ist,i))                                      + &
         & B2_3(ist,i)/(transmission % tau_surf(ist,i))**2 
      ELSE
        auxrad_stream%rad_tmp(ist,i) = 0._JPRB
      ENDIF

      auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
      auxrad_stream%meanrad_up(ist,i)   = auxrad_stream%up(ist,lev,i) + auxrad_stream%meanrad_up(ist,i)
    endif

    !---Scattering of the solar beam----------------------------------------------------------

    if(addaerosl.or.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then

          !--------------Upward single scattering of the solar beam---------------------------------

          auxrad_stream%fac1_3(ist,i)=                                        &
           & coef % ss_solar_spectrum(chan)                                       * &
           & transmission_scatt_ir_stream%azphacup(ist,i,lev)*z4pi_r 

          auxrad_stream%fac2_3(ist,i)=                                        &
           & transmission_scatt_ir_stream%ssa(ist,i,lev) 

          auxrad_stream%fac3_3(ist,i)=                                        &
           & raytracing%pathsat(lev,prof)                                        / &
           & (raytracing%pathsun(lev,prof)+raytracing%pathsat(lev,prof)) 

          auxrad_stream%fac4_3(ist,i)=                                        &
           & exp(-transmission % odsun_sfrac(ist,i)) 

          auxrad_stream%fac5_3(ist,i)=                                        &
           & exp(-transmission % od_sfrac(ist,i)) 

          auxrad_stream%fac6_3(ist,i)=                                        &
           & coef % ss_solar_spectrum(chan)                                      * &
           & transmission_scatt_ir_stream%azphacdo(ist,i,lev)*z4pi_r 

          auxrad_stream%meanrad_up(ist,i)=auxrad_stream%meanrad_up(ist,i)+auxrad_stream%fac1_3(ist,i)* &
           & auxrad_stream%fac2_3(ist,i)                                   * &
           & auxrad_stream%fac3_3(ist,i)                                   * &
           & (transmission % tausun_layer(lev,ist,i)                       - &
           & transmission % tausun_surf(ist,i)) 
          !--------------Downward single scattering of the solar beam-------------------------------

          if(raytracing%pathsat(lev1,prof) /=                                     &
           & raytracing%pathsun(lev1,prof))then 
            auxrad_stream%fac7_3(ist,i)=                                      &
             & raytracing%pathsat(lev1,prof)                                / &
             & (raytracing%pathsun(lev1,prof)                               - &
             & raytracing%pathsat(lev1,prof)) 
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
            auxrad_stream%meanrad_down(ist,i)=auxrad_stream%meanrad_down(ist,i)      + &
             & auxrad_stream%fac6_3(ist,i)                              * &
             & auxrad_stream%fac2_3(ist,i)                              * &
             & auxrad_stream%fac7_3(ist,i)                              * &
             & (auxrad_stream%fac5_3(ist,i)                             - &
             & auxrad_stream%fac4_3(ist,i))                             * &
             & transmission %tausun_layer(lev,ist,i)                    / &
             & (transmission %tau_layer(lev,ist,i)                      * &
             & transmission %tau_surf(ist,i)) 
            endif
            auxrad_stream%meanrad_down_ref(ist,i)=auxrad_stream%meanrad_down(ist,i)

            if(auxrad_stream%meanrad_down(ist,i)<0._jprb)then
              auxrad_stream%meanrad_down(ist,i)=0._jprb
            endif

          else
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
            auxrad_stream%meanrad_down(ist,i)=auxrad_stream%meanrad_down(ist,i)      + &
             & auxrad_stream%fac6_3(ist,i)                              * &
             & auxrad_stream%fac2_3(ist,i)                              * &
             & auxrad_stream%fac4_3(ist,i)                              * &
             & transmission %tausun_layer(lev,ist,i)                    * &
             & transmission % odsun_sfrac(ist,i)                        / &
             & (transmission %tau_layer(lev,ist,i)                      * &
             & transmission %tau_surf(ist,i)) 
            endif
            auxrad_stream%meanrad_down_ref(ist,i)=auxrad_stream%meanrad_down(ist,i)

            if(auxrad_stream%meanrad_down(ist,i)<0._jprb)then
              auxrad_stream%meanrad_down(ist,i)=0._jprb
            endif

          endif
        endif
      endif
    endif
  Else  !---------------Old Code-------------------------------------------------------

    lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
    If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1

    auxrad_stream%meanrad_up(ist,i) = 0.5_JPRB * ( auxrad%surfair(i) + auxrad%layer(lev,i) )    * &
     & ( transmission % tau_layer(lev,ist,i)                               - &
     & transmission % tau_surf(ist,i) ) 

    If ( transmission % tau_surf(ist,i) > min_tau ) Then
      auxrad_stream%rad_tmp(ist,i) = auxrad_stream%meanrad_up(ist,i)          / &
       & ( transmission % tau_layer(lev,ist,i)                               * &
       & transmission % tau_surf(ist,i) ) 
    Else
      auxrad_stream%rad_tmp(ist,i)    = 0.0_JPRB
    End If

    auxrad_stream%meanrad_down(ist,i) = auxrad_stream%down(ist,lev,i) + auxrad_stream%rad_tmp(ist,i)
  Endif
end do

!-End of direct calculations--------------------------------------------------------------

Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  If(coef % id_sensor == sensor_id_po)Then
    pol_id = coef % fastem_polar(chan) + 1_jpim
  Else
    pol_id = 0_jpim
  Endif
  !   aux     => aux_prof(    lprofiles(i) )
  !   aux_k   => aux_prof_k ( lprofiles(i) )

  do ist=ircld%nstream(prof),1,-1 
    ! add upward and downward parts
    lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
    If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1
    If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB )then
      lev1=lev
    else
      lev1=lev+1
    endif

    meanrad_up_k(i) =meanrad_up_k(i)                                               + &
     & zup_k(aux_prof(    lprofiles(i) ) % nearestlev_surf,ist,i) 
      meanrad_up_k(i)=meanrad_up_k(i)+auxrad_stream_k%cloudy(ist,i)
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
      meanrad_down_k(i)=meanrad_down_k(i)+ auxrad_stream_k%cloudy(ist,i)* &
       & transmission % tau_surf(ist,i)*transmission % tau_surf(ist,i) * &
       & reflectivity(i) 

      transmission_k % tau_surf(ist,i)  =                                       &
       & transmission_k % tau_surf(ist,i)                                   + &
       & auxrad_stream_k%cloudy(ist,i)                                      * &
       & 2._JPRB*transmission % tau_surf(ist,i)*reflectivity(i)              * &
       & auxrad_stream%meanrad_down(ist,i) 

      reflectivity_k(i) = reflectivity_k(i) + auxrad_stream_k%cloudy(ist,i) * &
       & transmission % tau_surf(ist,i)*transmission % tau_surf(ist,i) * &
       & auxrad_stream%meanrad_down(ist,i) 

    If (.not. rt8_mode) Then !------------new code--------------------------------------------
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim )meanrad_up_k(i) = 0.0_jprb
      !-----Scattering of the solar beam----------------------------------------------------------

      if(addaerosl.or.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then

            !--------------Downward single scattering of the solar beam-------------------------------

            if(raytracing%pathsat(lev1,prof) /=                                      &
             & raytracing%pathsun(lev1,prof))then 

              if(auxrad_stream%meanrad_down_ref(ist,i)<0._jprb)then
                meanrad_down_k(i)=0._jprb
              endif
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              transmission_k %tau_surf(ist,i)=                                  &
               & transmission_k %tau_surf(ist,i)                               - &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)**2) 

              transmission_k %tau_layer(lev,ist,i)=                             &
               & transmission_k %tau_layer(lev,ist,i)                          - &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)**2                         * &
               & transmission %tau_surf(ist,i)) 

              transmission_k %tausun_layer(lev,ist,i)=                          &
               & transmission_k %tausun_layer(lev,ist,i)                       + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac4_3(ist,i)                                   = &
               & auxrad_stream_k%fac4_3(ist,i)                                 - &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac5_3(ist,i) =                                   &
               & auxrad_stream_k%fac5_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac7_3(ist,i)=                                    &
               & auxrad_stream_k%fac7_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac2_3(ist,i)=                                    &
               & auxrad_stream_k%fac2_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac6_3(ist,i)=                                    &
               & auxrad_stream_k%fac6_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac7_3(ist,i)                                    * &
               & (auxrad_stream%fac5_3(ist,i)                                   - &
               & auxrad_stream%fac4_3(ist,i))                                   * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              raytracing_k%pathsat(lev1,i)= raytracing_k%pathsat(lev1,i) + &
               & auxrad_stream_k%fac7_3(ist,i)                                 * &
               & raytracing%pathsun(lev1,prof)/ &
               & (raytracing%pathsun(lev1,prof)-raytracing%pathsat(lev1,prof))**2 

              raytracing_k%pathsun(lev1,i)=raytracing_k%pathsun(lev1,i)  - &
               & auxrad_stream_k%fac7_3(ist,i)                                 * &
               & raytracing%pathsat(lev1,prof)/ &
               & (raytracing%pathsun(lev1,prof)-raytracing%pathsat(lev1,prof))**2 
             endif
            else
              if(auxrad_stream%meanrad_down_ref(ist,i)<0._jprb)then
                meanrad_down_k(i)=0._jprb
              endif
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
              transmission_k %tau_surf(ist,i)=                                  &
               & transmission_k %tau_surf(ist,i)                               - &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)**2) 

              transmission_k %tau_layer(lev,ist,i)=                             &
               & transmission_k %tau_layer(lev,ist,i)                          - &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)**2                         * &
               & transmission %tau_surf(ist,i)) 

              transmission_k % odsun_sfrac(ist,i)=                              &
               & transmission_k % odsun_sfrac(ist,i)                           + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              transmission_k %tausun_layer(lev,ist,i)=                          &
               & transmission_k %tausun_layer(lev,ist,i)                       + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac4_3(ist,i)=                                    &
               & auxrad_stream_k%fac4_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac2_3(ist,i)=                                    &
               & auxrad_stream_k%fac2_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac6_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 

              auxrad_stream_k%fac6_3(ist,i)                                   = &
               & auxrad_stream_k%fac6_3(ist,i)                                 + &
               & meanrad_down_k(i)                                                   * &
               & auxrad_stream%fac2_3(ist,i)                                    * &
               & auxrad_stream%fac4_3(ist,i)                                    * &
               & transmission %tausun_layer(lev,ist,i)                          * &
               & transmission % odsun_sfrac(ist,i)                              / &
               & (transmission %tau_layer(lev,ist,i)                            * &
               & transmission %tau_surf(ist,i)) 
             endif
            endif

            !--------------Upward single scattering of the solar beam---------------------------------

            transmission_k % tausun_layer(lev,ist,i)=                          &
             & transmission_k % tausun_layer(lev,ist,i)+meanrad_up_k(i)     * &
             & auxrad_stream%fac1_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac3_3(ist,i) 

            transmission_k % tausun_surf(ist,i)=                               &
             & transmission_k % tausun_surf(ist,i)     -meanrad_up_k(i)     * &
             & auxrad_stream%fac1_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac3_3(ist,i) 

            auxrad_stream_k%fac3_3(ist,i)=                                     &
             & auxrad_stream_k%fac3_3(ist,i)                                 + &
             & meanrad_up_k(i)                                                     * &
             & auxrad_stream%fac1_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & (transmission % tausun_layer(lev,ist,i)                        - &
             & transmission % tausun_surf(ist,i)) 

            auxrad_stream_k%fac2_3(ist,i)=                                     &
             & auxrad_stream_k%fac2_3(ist,i)                                 + &
             & meanrad_up_k(i)                                                     * &
             & auxrad_stream%fac1_3(ist,i)                                    * &
             & auxrad_stream%fac3_3(ist,i)                                    * &
             & (transmission % tausun_layer(lev,ist,i)                        - &
             & transmission % tausun_surf(ist,i)) 

            auxrad_stream_k%fac1_3(ist,i)=                                     &
             & auxrad_stream_k%fac1_3(ist,i)                                 + &
             & meanrad_up_k(i)                                                     * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac3_3(ist,i)                                    * &
             & (transmission % tausun_layer(lev,ist,i)                        - &
             & transmission % tausun_surf(ist,i)) 


            transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)               = &
             & transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)            + &
             & auxrad_stream_k%fac6_3(ist,i)*coef % ss_solar_spectrum(chan)  * &
             & z4pi_r 

            transmission_k % od_sfrac(ist,i)                                 = &
             & transmission_k % od_sfrac(ist,i)                              - &
             & auxrad_stream_k%fac5_3(ist,i)                                 * &
             & exp(-transmission % od_sfrac(ist,i)) 

            transmission_k % odsun_sfrac(ist,i)=                               &
             & transmission_k % odsun_sfrac(ist,i)                           - &
             & auxrad_stream_k%fac4_3(ist,i)                                 * &
             & exp(-transmission % odsun_sfrac(ist,i)) 

            raytracing_k%pathsat(lev,i)                                        = &
             & raytracing_k%pathsat(lev,i)+auxrad_stream_k%fac3_3(ist,i)* &
             & raytracing%pathsun(lev,prof)                                        / &
             & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

            raytracing_k%pathsun(lev,i)                                        = &
             & raytracing_k%pathsun(lev,i)-auxrad_stream_k%fac3_3(ist,i)* &
             & raytracing%pathsat(lev,prof)                                        / &
             & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

            transmission_scatt_ir_stream_k%ssa(ist,i,lev)                    = &
             & transmission_scatt_ir_stream_k%ssa(ist,i,lev)                 + &
             & auxrad_stream_k%fac2_3(ist,i) 

            transmission_scatt_ir_stream_k%azphacup(ist,i,lev)               = &
             & transmission_scatt_ir_stream_k%azphacup(ist,i,lev)            + &
             & auxrad_stream_k%fac1_3(ist,i)*coef % ss_solar_spectrum(chan)  * &
             & z4pi_r
          endif
        endif
      endif

      if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.                       &
       & transmission % tau_surf(ist,i)<0._JPRB)then 

        zup_k(lev,ist,i)  =zup_k(lev,ist,i)  + &
         & meanrad_up_k(i) 
        zdown_k(lev,ist,i)=zdown_k(lev,ist,i)+ &
         & meanrad_down_k(i) 
        rad_tmp_k(i)      = rad_tmp_k(i) + meanrad_down_k(i)                  ! cor 28/11

        meanrad_down_k(i) = 0._JPRB

        ! on peut creer une variable intermediaire pour gagner
        ! du temps rad_tmp_k * auxrad_stream%meanrad_up / (tau_prod*tau_prod)

        If ( transmission % tau_surf(ist,i) > min_tau ) Then
          tau_prod = transmission % tau_layer(lev,ist,i)                       * &
           & transmission % tau_surf(ist,i) 
          meanrad_up_k(i) = meanrad_up_k(i) + rad_tmp_k(i)                        / &
           & tau_prod 
          transmission_k % tau_layer(lev,ist,i) =                               &
           & transmission_k % tau_layer(lev,ist,i) - rad_tmp_k(i)             * &
           & auxrad_stream%meanrad_up(ist,i) *  transmission % tau_surf(ist,i)                / &
           & (tau_prod*tau_prod) 
          transmission_k % tau_surf(ist,i)      =                               &
           & transmission_k % tau_surf(ist,i)      - rad_tmp_k(i)             * &
           & auxrad_stream%meanrad_up(ist,i)*transmission%tau_layer(lev,ist,i)                / &
           & (tau_prod*tau_prod) 
          rad_tmp_k(i) = 0.0_JPRB
        Else
          rad_tmp_k(i) = 0.0_JPRB
        End If

        ! on peut creer une variable intermediaire pour gagner
        ! du temps 0.5 * meanrad_up_k * ( auxrad%surfair(i) + auxrad%layer(lev,i)
        ! rad_surfair_k(i) est certainement inutile car plus utilise
        rad_surfair_k(i)   = rad_surfair_k(i)   + 0.5_JPRB * meanrad_up_k(i)      * &
         & ( transmission % tau_layer(lev,ist,i)                                - &
         & transmission % tau_surf(ist,i) ) 

        rad_layer_k(lev,i) = rad_layer_k(lev,i) + 0.5_JPRB * meanrad_up_k(i)      * &
         & ( transmission % tau_layer(lev,ist,i)                                - &
         & transmission % tau_surf(ist,i) ) 

        transmission_k % tau_layer(lev,ist,i) =                                 &
         & transmission_k % tau_layer(lev,ist,i) + 0.5_JPRB * meanrad_up_k(i) * &
         & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

        transmission_k % tau_surf(ist,i)      =                                 &
         & transmission_k % tau_surf(ist,i)     - 0.5_JPRB * meanrad_up_k(i)  * &
         & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

        meanrad_up_k(i) = 0._JPRB

      else
        if(transmission%od_sfrac(ist,i)<min_od.OR.                               &
         & ((transmission % tau_layer(lev,ist,i)                                 - &
         & transmission % tau_surf(ist,i))<min_od))THEN 
          zup_k(lev,ist,i)=zup_k(lev,ist,i)  + &
           & meanrad_up_k(i) 
          zdown_k(lev,ist,i)=                                      &
           & zdown_k(lev,ist,i)      + &
           & meanrad_down_k(i) 
          rad_tmp_k(i)=rad_tmp_k(i)+meanrad_down_k(i)

          rad_tmp_k(i)     = 0.0_JPRB
          meanrad_up_k     = 0.0_JPRB
          meanrad_down_k(i)= 0.0_JPRB

        else

          zup_k(lev,ist,i)=zup_k(lev,ist,i)  + &
           & meanrad_up_k(i) 
          zdown_k(lev,ist,i)=                                      &
           & zdown_k(lev,ist,i)      + &
           & meanrad_down_k(i) 
          rad_tmp_k(i)=rad_tmp_k(i)+meanrad_down_k(i)

          meanrad_down_k(i)=0._jprb

          IF (transmission % tau_surf(ist,i) > min_tau) THEN
            B1_k=rad_tmp_k(i)/(transmission % tau_layer(lev,ist,i)     * &
             & transmission % tau_surf(ist,i)) 
            B3_k=-rad_tmp_k(i)/(transmission % tau_layer(lev,ist,i)     * &
             & transmission % tau_surf(ist,i)) 
            B2_k=rad_tmp_k(i)/(transmission % tau_surf(ist,i))**2

            transmission_k % tau_surf(ist,i)=                                   &
             & transmission_k % tau_surf(ist,i)                                 - &
             & rad_tmp_k(i)*2*B2_3(ist,i)                         / &
             & (transmission % tau_surf(ist,i))**3 
            transmission_k % tau_layer(lev,ist,i)=                              &
             & transmission_k % tau_layer(lev,ist,i)                            - &
             & rad_tmp_k(i)                                                           * &
             & (B1_3(ist,i)-B3_3(ist,i))       / &
             & (transmission % tau_layer(lev,ist,i)**2                           * &
             & transmission % tau_surf(ist,i)) 

            transmission_k % tau_surf(ist,i)=                                   &
             & transmission_k % tau_surf(ist,i)                                 - &
             & rad_tmp_k(i)                                                           * &
             & (B1_3(ist,i)-B3_3(ist,i))       / &
             & (transmission % tau_layer(lev,ist,i)                              * &
             & transmission % tau_surf(ist,i)**2) 
            rad_tmp_k(i)=0._JPRB
          ELSE
            B1_k=0.0_JPRB
            B2_k=0.0_JPRB
            B3_k=0.0_JPRB
            rad_tmp_k(i)=0._JPRB
          ENDIF

          B1_k=B1_k+meanrad_up_k(i)
          B2_k=B2_k-meanrad_up_k(i)
          B3_k=B3_k+meanrad_up_k(i)

          meanrad_up_k(i)=0._JPRB

          transmission_k %od_sfrac (ist,i)=                                     &
           & transmission_k %od_sfrac (ist,i)                                 - &
           & B3_k*(auxrad%surfair(i)-auxrad%layer(lev,i))                           * &
           & (transmission % tau_layer(lev,ist,i)                              - &
           & transmission % tau_surf(ist,i))                                   / &
           & (transmission %od_sfrac (ist,i)**2) 

          transmission_k % tau_surf(ist,i)=                                     &
           & transmission_k % tau_surf(ist,i)-B3_k                           * &
           & (auxrad%surfair(i)-auxrad%layer(lev,i))                                 / &
           & (transmission %od_sfrac (ist,i)) 

          transmission_k % tau_layer(lev,ist,i)=                                &
           & transmission_k % tau_layer(lev,ist,i)                            + &
           & B3_k*(auxrad%surfair(i)-auxrad%layer(lev,i))                           / &
           & (transmission %od_sfrac (ist,i)) 

          rad_layer_k(lev,i)=rad_layer_k(lev,i)-B3_k                              * &
           & (transmission % tau_layer(lev,ist,i)                              - &
           & transmission % tau_surf(ist,i))                                   / &
           & (transmission %od_sfrac (ist,i)) 

          rad_surfair_k(i) =rad_surfair_k(i) +B3_k                                * &
           & (transmission % tau_layer(lev,ist,i)                              - &
           & transmission % tau_surf(ist,i))                                   / &
           & (transmission %od_sfrac (ist,i)) 

          !           B3_k=0._JPRB

          transmission_k % tau_surf(ist,i)=                                     &
           & transmission_k % tau_surf(ist,i)                                 + &
           & B2_k*(auxrad%surfair(i)-auxrad%layer(lev,i)) 

          rad_layer_k(lev,i)=rad_layer_k(lev,i)-B2_k                              * &
           & transmission % tau_surf(ist,i) 

          rad_surfair_k(i)=rad_surfair_k(i)+B2_k*transmission % tau_surf(ist,i)

          !           B2_k=0._JPRB

          transmission_k % tau_surf(ist,i)=                                     &
           & transmission_k % tau_surf(ist,i)                                 - &
           & B1_k*auxrad%layer(lev,i) 

          transmission_k % tau_layer(lev,ist,i)=                                &
           & transmission_k % tau_layer(lev,ist,i)                            + &
           & B1_k*auxrad%layer(lev,i) 

          rad_layer_k(lev,i)=rad_layer_k(lev,i)+B1_k                              * &
           & (transmission % tau_layer(lev,ist,i)                              - &
           & transmission % tau_surf(ist,i)) 

          !           B1_k=0._JPRB

        endif
      endif
    Else   ! -------------------Old code------------------------------------------------
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb

      zup_k(lev,ist,i)   = zup_k(lev,ist,i)  + &
       & meanrad_up_k(i) 
      zdown_k(lev,ist,i) =  meanrad_down_k(i) 
      rad_tmp_k(i)      = rad_tmp_k(i) + meanrad_down_k(i)                 ! cor 28/11

      !RWS meanrad_up_k(i)   = 0._JPRB
      meanrad_down_k(i) = 0._JPRB

      ! on peut creer une variable intermediaire pour gagner
      ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        tau_prod = transmission % tau_layer(lev,ist,i)                         * &
         & transmission % tau_surf(ist,i) 
        meanrad_up_k(i) = meanrad_up_k(i) + rad_tmp_k(i)                          / &
         & tau_prod 
        transmission_k % tau_layer(lev,ist,i) =                                 &
         & transmission_k % tau_layer(lev,ist,i) - rad_tmp_k(i)             * &
         & auxrad_stream%meanrad_up(ist,i)* transmission % tau_surf(ist,i)/(tau_prod*tau_prod) 
        transmission_k % tau_surf(ist,i)      =                                 &
         & transmission_k % tau_surf(ist,i)      - rad_tmp_k(i)             * &
         & auxrad_stream%meanrad_up(ist,i)*transmission%tau_layer(lev,ist,i)/(tau_prod*tau_prod) 
        rad_tmp_k(i) = 0.0_JPRB
      Else
        rad_tmp_k(i) = 0.0_JPRB
      End If

      ! on peut creer une variable intermediaire pour gagner
      ! du temps 0.5 * meanrad_up_k * ( auxrad%surfair(i) + auxrad%layer(lev,i)
      ! rad_surfair_k(i) est certainement inutile car plus utilise
      rad_surfair_k(i)   = rad_surfair_k(i)   + 0.5_JPRB * meanrad_up_k(i)        * &
       & ( transmission % tau_layer(lev,ist,i)                                - &
       & transmission % tau_surf(ist,i) ) 

      rad_layer_k(lev,i) = rad_layer_k(lev,i) + 0.5_JPRB * meanrad_up_k(i)        * &
       & ( transmission % tau_layer(lev,ist,i)                                - &
       & transmission % tau_surf(ist,i) ) 

      transmission_k % tau_layer(lev,ist,i) =                                   &
       & transmission_k % tau_layer(lev,ist,i) + 0.5_JPRB * meanrad_up_k(i) * &
       & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

      transmission_k % tau_surf(ist,i)      =                                   &
       & transmission_k % tau_surf(ist,i)     - 0.5_JPRB * meanrad_up_k(i)  * &
       & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

      meanrad_up_k(i) = 0._JPRB

    Endif  ! ---------------------------------------------------------------------------
  Enddo
End do
ist=0 
!-2.3 -----Near surface layer-------------------------------------------------------
!cdir nodep
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  If(coef % id_sensor == sensor_id_po)Then
    pol_id = coef % fastem_polar(chan) + 1_jpim
  Else
    pol_id = 0_jpim
  Endif
  !   aux     => aux_prof(    lprofiles(i) )
  !   aux_k   => aux_prof_k ( lprofiles(i) )

  ! add upward and downward parts
  lev = aux_prof(    lprofiles(i) ) % nearestlev_surf - 1
  If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB ) lev = lev + 1
  If ( aux_prof(    lprofiles(i) ) % pfraction_surf < 0.0_JPRB )then
    lev1=lev
  else
    lev1=lev+1
  endif
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
  if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
  meanrad_up_k(i) =meanrad_up_k(i)                                               + &
   & zup_k(aux_prof(    lprofiles(i) ) % nearestlev_surf,ist,i) 
    meanrad_down_k(i)    = meanrad_down_k(i)+  rad_k % reflclear(i)   * &
     & transmission % tau_surf(ist,i)*transmission % tau_surf(ist,i) * &
     & reflectivity(i) 

    transmission_k % tau_surf(ist,i)  =                                       &
     & transmission_k % tau_surf(ist,i)                                   + &
     & rad_k % reflclear(i)                                                     * &
     & 2._JPRB*transmission % tau_surf(ist,i)*reflectivity(i)              * &
     & auxrad_stream%meanrad_down(ist,i) 

    reflectivity_k(i) = reflectivity_k(i) + rad_k % reflclear(i)                * &
     & transmission % tau_surf(ist,i)*transmission % tau_surf(ist,i) * &
     & auxrad_stream%meanrad_down(ist,i) 

    meanrad_up_k(i) = meanrad_up_k(i) + rad_k % upclear(i)
    meanrad_up_k(i)      = meanrad_up_k(i)      + rad_k % clear(i)
    meanrad_down_k(i)    = meanrad_down_k(i)    + rad_k % clear(i)              * &
     & transmission % tau_surf(ist,i) * transmission % tau_surf(ist,i) * &
     & reflectivity(i) 

    transmission_k % tau_surf(ist,i) =                                        &
     & transmission_k % tau_surf(ist,i)                                     + &
     & rad_k % clear(i)                                                           * &
     & 2._JPRB * transmission % tau_surf(ist,i)*reflectivity(i)              * &
     & auxrad_stream%meanrad_down(ist,i) 

    reflectivity_k(i) = reflectivity_k(i) + rad_k % clear(i)                    * &
     & transmission % tau_surf(ist,i) * transmission % tau_surf(ist,i) * &
     & auxrad_stream%meanrad_down(ist,i) 

  If (.not. rt8_mode) Then !------------new code--------------------------------------------

    !-----Scattering of the solar beam----------------------------------------------------------

    if(addaerosl.or.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then

          !--------------Downward single scattering of the solar beam-------------------------------

          if(raytracing%pathsat(lev1,prof) /=                                      &
           & raytracing%pathsun(lev1,prof))then 

            if(auxrad_stream%meanrad_down_ref(ist,i)<0._jprb)then
              meanrad_down_k(i)=0._jprb
            endif
            if(transmission %tau_layer(lev,ist,i)>min_tau)then
             transmission_k %tau_surf(ist,i)=                                  &
             & transmission_k %tau_surf(ist,i)                               - &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)**2) 

            transmission_k %tau_layer(lev,ist,i)=                             &
             & transmission_k %tau_layer(lev,ist,i)                          - &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)**2                         * &
             & transmission %tau_surf(ist,i)) 

            transmission_k %tausun_layer(lev,ist,i)=                          &
             & transmission_k %tausun_layer(lev,ist,i)                       + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac4_3(ist,i)                                   = &
             & auxrad_stream_k%fac4_3(ist,i)                                 - &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac5_3(ist,i) =                                   &
             & auxrad_stream_k%fac5_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac7_3(ist,i)=                                    &
             & auxrad_stream_k%fac7_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac2_3(ist,i)=                                    &
             & auxrad_stream_k%fac2_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac6_3(ist,i)=                                    &
             & auxrad_stream_k%fac6_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac7_3(ist,i)                                    * &
             & (auxrad_stream%fac5_3(ist,i)                                   - &
             & auxrad_stream%fac4_3(ist,i))                                   * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            raytracing_k%pathsat(lev1,i)= raytracing_k%pathsat(lev1,i) + &
             & auxrad_stream_k%fac7_3(ist,i)                                 * &
             & raytracing%pathsun(lev1,prof)/ &
             & (raytracing%pathsun(lev1,prof)-raytracing%pathsat(lev1,prof))**2 

            raytracing_k%pathsun(lev1,i)=raytracing_k%pathsun(lev1,i)  - &
             & auxrad_stream_k%fac7_3(ist,i)                                 * &
             & raytracing%pathsat(lev1,prof)/ &
             & (raytracing%pathsun(lev1,prof)-raytracing%pathsat(lev1,prof))**2 
           endif
          else
            if(auxrad_stream%meanrad_down_ref(ist,i)<0._jprb)then
              meanrad_down_k(i)=0._jprb
            endif

            if(transmission %tau_layer(lev,ist,i)>min_tau)then
             transmission_k %tau_surf(ist,i)=                                  &
             & transmission_k %tau_surf(ist,i)                               - &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)**2) 

            transmission_k %tau_layer(lev,ist,i)=                             &
             & transmission_k %tau_layer(lev,ist,i)                          - &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)**2                         * &
             & transmission %tau_surf(ist,i)) 

            transmission_k % odsun_sfrac(ist,i)=                              &
             & transmission_k % odsun_sfrac(ist,i)                           + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            transmission_k %tausun_layer(lev,ist,i)=                          &
             & transmission_k %tausun_layer(lev,ist,i)                       + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac4_3(ist,i)=                                    &
             & auxrad_stream_k%fac4_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac2_3(ist,i)=                                    &
             & auxrad_stream_k%fac2_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac6_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 

            auxrad_stream_k%fac6_3(ist,i)                                   = &
             & auxrad_stream_k%fac6_3(ist,i)                                 + &
             & meanrad_down_k(i)                                                   * &
             & auxrad_stream%fac2_3(ist,i)                                    * &
             & auxrad_stream%fac4_3(ist,i)                                    * &
             & transmission %tausun_layer(lev,ist,i)                          * &
             & transmission % odsun_sfrac(ist,i)                              / &
             & (transmission %tau_layer(lev,ist,i)                            * &
             & transmission %tau_surf(ist,i)) 
           endif
          endif

          !--------------Upward single scattering of the solar beam---------------------------------
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
          transmission_k % tausun_layer(lev,ist,i)=                          &
           & transmission_k % tausun_layer(lev,ist,i)+meanrad_up_k(i)     * &
           & auxrad_stream%fac1_3(ist,i)                                    * &
           & auxrad_stream%fac2_3(ist,i)                                    * &
           & auxrad_stream%fac3_3(ist,i) 

          transmission_k % tausun_surf(ist,i)=                               &
           & transmission_k % tausun_surf(ist,i)     -meanrad_up_k(i)     * &
           & auxrad_stream%fac1_3(ist,i)                                    * &
           & auxrad_stream%fac2_3(ist,i)                                    * &
           & auxrad_stream%fac3_3(ist,i) 

          auxrad_stream_k%fac3_3(ist,i)=                                     &
           & auxrad_stream_k%fac3_3(ist,i)                                 + &
           & meanrad_up_k(i)                                                     * &
           & auxrad_stream%fac1_3(ist,i)                                    * &
           & auxrad_stream%fac2_3(ist,i)                                    * &
           & (transmission % tausun_layer(lev,ist,i)                        - &
           & transmission % tausun_surf(ist,i)) 

          auxrad_stream_k%fac2_3(ist,i)=                                     &
           & auxrad_stream_k%fac2_3(ist,i)                                 + &
           & meanrad_up_k(i)                                                     * &
           & auxrad_stream%fac1_3(ist,i)                                    * &
           & auxrad_stream%fac3_3(ist,i)                                    * &
           & (transmission % tausun_layer(lev,ist,i)                        - &
           & transmission % tausun_surf(ist,i)) 

          auxrad_stream_k%fac1_3(ist,i)=                                     &
           & auxrad_stream_k%fac1_3(ist,i)                                 + &
           & meanrad_up_k(i)                                                     * &
           & auxrad_stream%fac2_3(ist,i)                                    * &
           & auxrad_stream%fac3_3(ist,i)                                    * &
           & (transmission % tausun_layer(lev,ist,i)                        - &
           & transmission % tausun_surf(ist,i)) 


          transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)               = &
           & transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)            + &
           & auxrad_stream_k%fac6_3(ist,i)*coef % ss_solar_spectrum(chan)  * &
           & z4pi_r 

          transmission_k % od_sfrac(ist,i)                                 = &
           & transmission_k % od_sfrac(ist,i)                              - &
           & auxrad_stream_k%fac5_3(ist,i)                                 * &
           & exp(-transmission % od_sfrac(ist,i)) 

          transmission_k % odsun_sfrac(ist,i)=                               &
           & transmission_k % odsun_sfrac(ist,i)                           - &
           & auxrad_stream_k%fac4_3(ist,i)                                 * &
           & exp(-transmission % odsun_sfrac(ist,i)) 

          raytracing_k%pathsat(lev,i)                                        = &
           & raytracing_k%pathsat(lev,i)+auxrad_stream_k%fac3_3(ist,i)* &
           & raytracing%pathsun(lev,prof)                                        / &
           & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

          raytracing_k%pathsun(lev,i)                                        = &
           & raytracing_k%pathsun(lev,i)-auxrad_stream_k%fac3_3(ist,i)* &
           & raytracing%pathsat(lev,prof)                                        / &
           & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

          transmission_scatt_ir_stream_k%ssa(ist,i,lev)                    = &
           & transmission_scatt_ir_stream_k%ssa(ist,i,lev)                 + &
           & auxrad_stream_k%fac2_3(ist,i) 

          transmission_scatt_ir_stream_k%azphacup(ist,i,lev)               = &
           & transmission_scatt_ir_stream_k%azphacup(ist,i,lev)            + &
           & auxrad_stream_k%fac1_3(ist,i)*coef % ss_solar_spectrum(chan)  * &
           & z4pi_r
        endif
      endif
    endif

    if(transmission % tau_layer(lev,ist,i)>=0._JPRB.and.                       &
     & transmission % tau_surf(ist,i)<0._JPRB)then 
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
      zup_k(lev,ist,i)  =zup_k(lev,ist,i)  + &
       & meanrad_up_k(i) 
      zdown_k(lev,ist,i)=zdown_k(lev,ist,i)+ &
       & meanrad_down_k(i) 
      rad_tmp_k(i)      = rad_tmp_k(i) + meanrad_down_k(i)                  ! cor 28/11

      meanrad_down_k(i) = 0._JPRB

      ! on peut creer une variable intermediaire pour gagner
      ! du temps rad_tmp_k * auxrad_stream%meanrad_up / (tau_prod*tau_prod)

      If ( transmission % tau_surf(ist,i) > min_tau ) Then
        tau_prod = transmission % tau_layer(lev,ist,i)                       * &
         & transmission % tau_surf(ist,i) 
        meanrad_up_k(i) = meanrad_up_k(i) + rad_tmp_k(i)                        / &
         & tau_prod 
        transmission_k % tau_layer(lev,ist,i) =                               &
         & transmission_k % tau_layer(lev,ist,i) - rad_tmp_k(i)             * &
         & auxrad_stream%meanrad_up(ist,i) *  transmission % tau_surf(ist,i)                / &
         & (tau_prod*tau_prod) 
        transmission_k % tau_surf(ist,i)      =                               &
         & transmission_k % tau_surf(ist,i)      - rad_tmp_k(i)             * &
         & auxrad_stream%meanrad_up(ist,i)*transmission%tau_layer(lev,ist,i)                / &
         & (tau_prod*tau_prod) 
        rad_tmp_k(i) = 0.0_JPRB
      Else
        rad_tmp_k(i) = 0.0_JPRB
      End If

      ! on peut creer une variable intermediaire pour gagner
      ! du temps 0.5 * meanrad_up_k * ( auxrad%surfair(i) + auxrad%layer(lev,i)
      ! rad_surfair_k(i) est certainement inutile car plus utilise
      rad_surfair_k(i)   = rad_surfair_k(i)   + 0.5_JPRB * meanrad_up_k(i)      * &
       & ( transmission % tau_layer(lev,ist,i)                                - &
       & transmission % tau_surf(ist,i) ) 

      rad_layer_k(lev,i) = rad_layer_k(lev,i) + 0.5_JPRB * meanrad_up_k(i)      * &
       & ( transmission % tau_layer(lev,ist,i)                                - &
       & transmission % tau_surf(ist,i) ) 

      transmission_k % tau_layer(lev,ist,i) =                                 &
       & transmission_k % tau_layer(lev,ist,i) + 0.5_JPRB * meanrad_up_k(i) * &
       & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

      transmission_k % tau_surf(ist,i)      =                                 &
       & transmission_k % tau_surf(ist,i)     - 0.5_JPRB * meanrad_up_k(i)  * &
       & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

      meanrad_up_k(i) = 0._JPRB

    else
      if(transmission%od_sfrac(ist,i)<min_od.OR.                               &
       & ((transmission % tau_layer(lev,ist,i)                                 - &
       & transmission % tau_surf(ist,i))<min_od))THEN 
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
        zup_k(lev,ist,i)=zup_k(lev,ist,i)  + &
         & meanrad_up_k(i) 
        zdown_k(lev,ist,i)=                                      &
         & zdown_k(lev,ist,i)      + &
         & meanrad_down_k(i) 
        rad_tmp_k(i)=rad_tmp_k(i)+meanrad_down_k(i)

        rad_tmp_k(i)     = 0.0_JPRB
        meanrad_up_k(i)  = 0.0_JPRB
        meanrad_down_k(i)= 0.0_JPRB

      else
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb

        zup_k(lev,ist,i)=zup_k(lev,ist,i)  + &
         & meanrad_up_k(i) 
        zdown_k(lev,ist,i)=                                      &
         & zdown_k(lev,ist,i)      + &
         & meanrad_down_k(i) 
        rad_tmp_k(i)=rad_tmp_k(i)+meanrad_down_k(i)

        meanrad_down_k(i)=0._jprb

        IF (transmission % tau_surf(ist,i) > min_tau) THEN
          B1_k=rad_tmp_k(i)/(transmission % tau_layer(lev,ist,i)     * &
           & transmission % tau_surf(ist,i)) 
          B3_k=-rad_tmp_k(i)/(transmission % tau_layer(lev,ist,i)     * &
           & transmission % tau_surf(ist,i)) 
          B2_k=rad_tmp_k(i)/(transmission % tau_surf(ist,i))**2

          transmission_k % tau_surf(ist,i)=                                   &
           & transmission_k % tau_surf(ist,i)                                 - &
           & rad_tmp_k(i)*2*B2_3(ist,i)                         / &
           & (transmission % tau_surf(ist,i))**3 
          transmission_k % tau_layer(lev,ist,i)=                              &
           & transmission_k % tau_layer(lev,ist,i)                            - &
           & rad_tmp_k(i)                                                           * &
           & (B1_3(ist,i)-B3_3(ist,i))       / &
           & (transmission % tau_layer(lev,ist,i)**2                           * &
           & transmission % tau_surf(ist,i)) 

          transmission_k % tau_surf(ist,i)=                                   &
           & transmission_k % tau_surf(ist,i)                                 - &
           & rad_tmp_k(i)                                                           * &
           & (B1_3(ist,i)-B3_3(ist,i))       / &
           & (transmission % tau_layer(lev,ist,i)                              * &
           & transmission % tau_surf(ist,i)**2) 
          rad_tmp_k(i)=0._JPRB
        ELSE
          B1_k=0.0_JPRB
          B2_k=0.0_JPRB
          B3_k=0.0_JPRB
          rad_tmp_k(i)=0._JPRB
        ENDIF

        B1_k=B1_k+meanrad_up_k(i)
        B2_k=B2_k-meanrad_up_k(i)
        B3_k=B3_k+meanrad_up_k(i)

        meanrad_up_k(i)=0._JPRB

        transmission_k %od_sfrac (ist,i)=                                     &
         & transmission_k %od_sfrac (ist,i)                                 - &
         & B3_k*(auxrad%surfair(i)-auxrad%layer(lev,i))                           * &
         & (transmission % tau_layer(lev,ist,i)                              - &
         & transmission % tau_surf(ist,i))                                   / &
         & (transmission %od_sfrac (ist,i)**2) 

        transmission_k % tau_surf(ist,i)=                                     &
         & transmission_k % tau_surf(ist,i)-B3_k                           * &
         & (auxrad%surfair(i)-auxrad%layer(lev,i))                                 / &
         & (transmission %od_sfrac (ist,i)) 

        transmission_k % tau_layer(lev,ist,i)=                                &
         & transmission_k % tau_layer(lev,ist,i)                            + &
         & B3_k*(auxrad%surfair(i)-auxrad%layer(lev,i))                           / &
         & (transmission %od_sfrac (ist,i)) 

        rad_layer_k(lev,i)=rad_layer_k(lev,i)-B3_k                              * &
         & (transmission % tau_layer(lev,ist,i)                              - &
         & transmission % tau_surf(ist,i))                                   / &
         & (transmission %od_sfrac (ist,i)) 

        rad_surfair_k(i) =rad_surfair_k(i) +B3_k                                * &
         & (transmission % tau_layer(lev,ist,i)                              - &
         & transmission % tau_surf(ist,i))                                   / &
         & (transmission %od_sfrac (ist,i)) 

        !           B3_k=0._JPRB

        transmission_k % tau_surf(ist,i)=                                     &
         & transmission_k % tau_surf(ist,i)                                 + &
         & B2_k*(auxrad%surfair(i)-auxrad%layer(lev,i)) 

        rad_layer_k(lev,i)=rad_layer_k(lev,i)-B2_k                              * &
         & transmission % tau_surf(ist,i) 

        rad_surfair_k(i)=rad_surfair_k(i)+B2_k*transmission % tau_surf(ist,i)

        !           B2_k=0._JPRB

        transmission_k % tau_surf(ist,i)=                                     &
         & transmission_k % tau_surf(ist,i)                                 - &
         & B1_k*auxrad%layer(lev,i) 

        transmission_k % tau_layer(lev,ist,i)=                                &
         & transmission_k % tau_layer(lev,ist,i)                            + &
         & B1_k*auxrad%layer(lev,i) 

        rad_layer_k(lev,i)=rad_layer_k(lev,i)+B1_k                              * &
         & (transmission % tau_layer(lev,ist,i)                              - &
         & transmission % tau_surf(ist,i)) 

        !           B1_k=0._JPRB

      endif
    endif
  Else   ! -------------------Old code------------------------------------------------
! assume there is no atmospheric source term for 3rd/4th stokes vector elements
      if (pol_id >= 6_jpim ) meanrad_up_k(i) = 0.0_jprb
    zup_k(lev,ist,i)   = zup_k(lev,ist,i)  + &
     & meanrad_up_k(i) 
    zdown_k(lev,ist,i) =  meanrad_down_k(i) 
    rad_tmp_k(i)      = rad_tmp_k(i) + meanrad_down_k(i)                 ! cor 28/11

    !RWS meanrad_up_k(i)   = 0._JPRB
    meanrad_down_k(i) = 0._JPRB

    ! on peut creer une variable intermediaire pour gagner
    ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)

    If ( transmission % tau_surf(ist,i) > min_tau ) Then
      tau_prod = transmission % tau_layer(lev,ist,i)                         * &
       & transmission % tau_surf(ist,i) 
      meanrad_up_k(i) = meanrad_up_k(i) + rad_tmp_k(i)                          / &
       & tau_prod 
      transmission_k % tau_layer(lev,ist,i) =                                 &
       & transmission_k % tau_layer(lev,ist,i) - rad_tmp_k(i)             * &
       & auxrad_stream%meanrad_up(ist,i)* transmission % tau_surf(ist,i)/(tau_prod*tau_prod) 
      transmission_k % tau_surf(ist,i)      =                                 &
       & transmission_k % tau_surf(ist,i)      - rad_tmp_k(i)             * &
       & auxrad_stream%meanrad_up(ist,i)*transmission%tau_layer(lev,ist,i)/(tau_prod*tau_prod) 
      rad_tmp_k(i) = 0.0_JPRB
    Else
      rad_tmp_k(i) = 0.0_JPRB
    End If

    ! on peut creer une variable intermediaire pour gagner
    ! du temps 0.5 * meanrad_up_k * ( auxrad%surfair(i) + auxrad%layer(lev,i)
    ! rad_surfair_k(i) est certainement inutile car plus utilise
    rad_surfair_k(i)   = rad_surfair_k(i)   + 0.5_JPRB * meanrad_up_k(i)        * &
     & ( transmission % tau_layer(lev,ist,i)                                - &
     & transmission % tau_surf(ist,i) ) 

    rad_layer_k(lev,i) = rad_layer_k(lev,i) + 0.5_JPRB * meanrad_up_k(i)        * &
     & ( transmission % tau_layer(lev,ist,i)                                - &
     & transmission % tau_surf(ist,i) ) 

    transmission_k % tau_layer(lev,ist,i) =                                   &
     & transmission_k % tau_layer(lev,ist,i) + 0.5_JPRB * meanrad_up_k(i) * &
     & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

    transmission_k % tau_surf(ist,i)      =                                   &
     & transmission_k % tau_surf(ist,i)     - 0.5_JPRB * meanrad_up_k(i)  * &
     & ( auxrad%surfair(i) + auxrad%layer(lev,i) ) 

    meanrad_up_k(i) = 0._JPRB

  Endif  ! ---------------------------------------------------------------------------
End do
!---------------------------------------------------------------------------------------
!2.2 layers between standard pressure levels
!--------------------------------------------------------------------------------------
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  do ist=ircld%nstream(prof),1,-1
    Do lev = profiles(1) % nlevels, 2, -1
      rad_dwn_k_chan(i,lev,ist)    = 0._JPRB
      rad_up_k_chan(i,lev,ist) = 0._JPRB
      if(addaerosl.OR.addclouds)then
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            if(auxrad_stream%down_ref(ist,lev,i)<0._jprb)then
              zdown_k(lev,ist,i)=0._jprb
            endif
          endif
        endif
      endif
      zup_k(lev-1,ist,i) = zup_k(lev-1,ist,i)+&
       & zup_k(lev,ist,i)
      rad_up_k_chan(i,lev,ist) = rad_up_k_chan(i,lev,ist)+zup_k(lev,ist,i)
      zdown_k(lev-1,ist,i) = zdown_k(lev-1,ist,i)+&
       & zdown_k(lev,ist,i)
      rad_dwn_k_chan(i,lev,ist) = rad_dwn_k_chan(i,lev,ist)+zdown_k(lev,ist,i)
    enddo
  enddo

EndDo
ist=0
#ifdef RTTOV_ARCH_VECTOR
Do lev = profiles(1) % nlevels, 2, -1
  Do i = 1, nchannels
    prof=lprofiles(i)
    chan=channels(i)
#else
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  Do lev = profiles(1) % nlevels, 2, -1
#endif
    rad_dwn_k_chan(i,lev,ist)    = 0._JPRB
    rad_up_k_chan(i,lev,ist) = 0._JPRB
    if(addaerosl.OR.addclouds)then
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then
          if(auxrad_stream%down_ref(ist,lev,i)<0._jprb)then
            zdown_k(lev,ist,i)=0._jprb
          endif
        endif
      endif
    endif
    zup_k(lev-1,ist,i) = zup_k(lev-1,ist,i)+&
     & zup_k(lev,ist,i)
    rad_up_k_chan(i,lev,ist) = rad_up_k_chan(i,lev,ist)+zup_k(lev,ist,i)
    zdown_k(lev-1,ist,i) = zdown_k(lev-1,ist,i)+&
     & zdown_k(lev,ist,i)
    rad_dwn_k_chan(i,lev,ist) = rad_dwn_k_chan(i,lev,ist)+zdown_k(lev,ist,i)
  enddo

EndDo

Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  do ist=ircld%nstream(prof),1,-1
    Do lev = profiles(1) % nlevels, 2, -1
      If (.not. rt8_mode)then ! New code------------------------------------------------------
        !---Scattering of the solar beam----------------------------------------------------------

        if(addaerosl.OR.addclouds)then
          if(profiles(prof)%addsolar)then
            fac1_2_k = 0.0_jprb
            fac2_2_k = 0.0_jprb
            fac3_2_k = 0.0_jprb
            fac4_2_k = 0.0_jprb
            fac5_2_k = 0.0_jprb
            fac6_2_k = 0.0_jprb
            fac7_2_k = 0.0_jprb

            if(coef%ss_val_chn(chan)==1)then

              !-------------------Downward single scattering of the solar beam--------------------------

              if(raytracing%pathsat(lev,prof) /=                                     &
               & raytracing%pathsun(lev,prof))then 

              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                transmission_k %tau_layer(lev,ist,i)=                          &
                 & transmission_k %tau_layer(lev,ist,i)                       - &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                            * &
                 & auxrad_stream%fac7_2(lev,ist,i)                            * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                           - &
                 & auxrad_stream%fac4_2(lev,ist,i))                           * &
                 & transmission %tausun_layer(lev-1,ist,i)                    / &
                 & (transmission %tau_layer(lev-1,ist,i)                      * &
                 & transmission %tau_layer(lev,ist,i)**2) 

                transmission_k %tau_layer(lev-1,ist,i)=                        &
                 & transmission_k %tau_layer(lev-1,ist,i)                     - &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                            - &
                 & auxrad_stream%fac4_2(lev,ist,i))                            * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)**2                    * &
                 & transmission %tau_layer(lev,ist,i)) 

                transmission_k %tausun_layer(lev-1,ist,i)=                     &
                 & transmission_k %tausun_layer(lev-1,ist,i)                  + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                            - &
                 & auxrad_stream%fac4_2(lev,ist,i))                            / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac4_2_k=                             &
                 & fac4_2_k                          - &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac5_2_k=                             &
                 & fac5_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac7_2_k=                             &
                 & fac7_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                            - &
                 & auxrad_stream%fac4_2(lev,ist,i))                            * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac2_2_k=                             &
                 & fac2_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                            - &
                 & auxrad_stream%fac4_2(lev,ist,i))                            * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac6_2_k=                             &
                 & fac6_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac7_2(lev,ist,i)                             * &
                 & (auxrad_stream%fac5_2(lev,ist,i)                            - &
                 & auxrad_stream%fac4_2(lev,ist,i))                            * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 


                raytracing_k%pathsat(lev,i)=raytracing_k%pathsat(lev,i)   + &
                 & fac7_2_k                          * &
                 & raytracing%pathsun(lev,prof)                                     / &
                 & (raytracing%pathsun(lev,prof)-raytracing%pathsat(lev,prof))**2 

                raytracing_k%pathsun(lev,i)=raytracing_k%pathsun(lev,i)   - &
                 & fac7_2_k                          * &
                 & raytracing%pathsat(lev,prof)                                     / &
                 & (raytracing%pathsun(lev,prof)-raytracing%pathsat(lev,prof))**2 
               else
                  rad_dwn_k_chan(i,lev,ist) = 0._JPRB
               endif
              else
              if(transmission %tau_layer(lev,ist,i)>min_tau)then
                transmission_k %tau_layer(lev,ist,i)=                          &
                 & transmission_k %tau_layer(lev,ist,i)                       - &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)**2) 

                transmission_k %tau_layer(lev-1,ist,i)=                        &
                 & transmission_k %tau_layer(lev-1,ist,i)                     - &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)**2                    * &
                 & transmission %tau_layer(lev,ist,i)) 

                transmission_k % odsun_singlelayer(lev,ist,i)                = &
                 & transmission_k % odsun_singlelayer(lev,ist,i)              + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                transmission_k %tausun_layer(lev-1,ist,i)                    = &
                 & transmission_k %tausun_layer(lev-1,ist,i)                  + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac4_2_k                            = &
                 & fac4_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac2_2_k                            = &
                 & fac2_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac6_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 

                fac6_2_k                            = &
                 & fac6_2_k                          + &
                 & rad_dwn_k_chan(i,lev,ist)                            * &
                 & auxrad_stream%fac2_2(lev,ist,i)                             * &
                 & auxrad_stream%fac4_2(lev,ist,i)                             * &
                 & transmission %tausun_layer(lev-1,ist,i)                     * &
                 & transmission % odsun_singlelayer(lev,ist,i)                 / &
                 & (transmission %tau_layer(lev-1,ist,i)                       * &
                 & transmission %tau_layer(lev,ist,i)) 
                else
                  rad_dwn_k_chan(i,lev,ist) = 0._JPRB
                endif
              endif

              !----------------Upward single scattering of the solar beam-------------------------------

              fac1_2_k=                               &
               & fac1_2_k                      + &
               & rad_up_k_chan(i,lev,ist)                   * &
               & auxrad_stream%fac2_2(lev,ist,i)                         * &
               & auxrad_stream%fac3_2(lev,ist,i)                         * &
               & (transmission % tausun_layer(lev-1,ist,i)               - &
               & transmission % tausun_layer(lev,ist,i)) 

              fac2_2_k=                               &
               & fac2_2_k                      + &
               & rad_up_k_chan(i,lev,ist)                   * &
               & auxrad_stream%fac1_2(lev,ist,i)                         * &
               & auxrad_stream%fac3_2(lev,ist,i)                         * &
               & (transmission % tausun_layer(lev-1,ist,i)               - &
               & transmission % tausun_layer(lev,ist,i))  

              fac3_2_k=                               &
               & fac3_2_k                      + &
               & rad_up_k_chan(i,lev,ist)                   * &
               & auxrad_stream%fac1_2(lev,ist,i)                         * &
               & auxrad_stream%fac2_2(lev,ist,i)                         * &
               & (transmission % tausun_layer(lev-1,ist,i)               - &
               & transmission % tausun_layer(lev,ist,i)) 

              transmission_k % tausun_layer(lev-1,ist,i)=                      &
               & transmission_k % tausun_layer(lev-1,ist,i)             + &
               & rad_up_k_chan(i,lev,ist)                   * &
               & auxrad_stream%fac1_2(lev,ist,i)                         * &
               & auxrad_stream%fac2_2(lev,ist,i)                         * &
               & auxrad_stream%fac3_2(lev,ist,i) 

              transmission_k % tausun_layer(lev,ist,i)=                        &
               & transmission_k % tausun_layer(lev,ist,i)               - &
               & rad_up_k_chan(i,lev,ist)                   * &
               & auxrad_stream%fac1_2(lev,ist,i)                         * &
               & auxrad_stream%fac2_2(lev,ist,i)                         * &
               & auxrad_stream%fac3_2(lev,ist,i) 

              transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)             = &
               & transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)     + &
               & fac6_2_k                      * &
               & coef % ss_solar_spectrum(chan)*z4pi_r

              transmission_k % od_singlelayer(lev,ist,i)                     = &
               & transmission_k % od_singlelayer(lev,ist,i)             - &
               & fac5_2_k                      * &
               & exp(-transmission % od_singlelayer(lev,ist,i)) 


              transmission_k % odsun_singlelayer(lev,ist,i)                  = &
               & transmission_k % odsun_singlelayer(lev,ist,i)          - &
               & fac4_2_k                      * &
               & exp(-transmission % odsun_singlelayer(lev,ist,i)) 

              raytracing_k%pathsat(lev,i)                                      = &
               & raytracing_k%pathsat(lev,i)                              + &
               & fac3_2_k                      * &
               & raytracing%pathsun(lev,prof)                                 / &
               & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

              raytracing_k%pathsun(lev,i)=raytracing_k%pathsun(lev,i)     - &
               & fac3_2_k                      * &
               & raytracing%pathsat(lev,prof)                                 / &
               & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

              transmission_scatt_ir_stream_k%ssa(ist,i,lev)                  = &
               & transmission_scatt_ir_stream_k%ssa(ist,i,lev)          + &
               & fac2_2_k 

              transmission_scatt_ir_stream_k%azphacup(ist,i,lev)             = &
               & transmission_scatt_ir_stream_k%azphacup(ist,i,lev)     + &
               & fac1_2_k                      * &
               & coef % ss_solar_spectrum(chan)*z4pi_r 

            endiF
          endif
        endif

        if(transmission % tau_layer(lev-1,ist,i)>=0.and.                &
         & transmission % tau_layer(lev,ist,i)<0)then 

          auxrad_stream%meanrad_up_2d(ist,i,lev) = 0.5_JPRB * ( auxrad%layer(lev,i)   + &
           & auxrad%layer(lev-1,i) ) * &
           & ( transmission % tau_layer(lev-1,ist,i)                             - &
           & transmission % tau_layer(lev,ist,i)   ) 

          If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
            ! on peut creer une variable intermediaire pour gagner
            ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)
            tau_prod = transmission % tau_layer(lev,ist,i)                        * &
             & transmission % tau_layer(lev-1,ist,i) 
            rad_up_k_chan(i,lev,ist)  =rad_up_k_chan(i,lev,ist)   + rad_dwn_k_chan(i,lev,ist) / &
             & tau_prod 
            transmission_k % tau_layer(lev,ist,i)   =                              &
             & transmission_k % tau_layer(lev,ist,i)   - rad_dwn_k_chan(i,lev,ist)   * &
             & transmission % tau_layer(lev-1,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)      / &
             & (tau_prod*tau_prod) 
            transmission_k % tau_layer(lev-1,ist,i) =                              &
             & transmission_k % tau_layer(lev-1,ist,i) - rad_dwn_k_chan(i,lev,ist)   * &
             & transmission % tau_layer(lev,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)        / &
             & (tau_prod*tau_prod) 
            rad_dwn_k_chan(i,lev,ist) = 0._JPRB
          Else
            rad_dwn_k_chan(i,lev,ist) = 0._JPRB
          End If

          rad_layer_k(lev,i)   = rad_layer_k(lev,i)   +rad_up_k_chan(i,lev,ist)        * &
           & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
           & transmission % tau_layer(lev,ist,i) ) 
          rad_layer_k(lev-1,i) = rad_layer_k(lev-1,i) +rad_up_k_chan(i,lev,ist)        * &
           & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
           & transmission % tau_layer(lev,ist,i) ) 
          transmission_k % tau_layer(lev-1,ist,i) =                                 &
           & transmission_k % tau_layer(lev-1,ist,i) +rad_up_k_chan(i,lev,ist) * &
           & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 
          transmission_k % tau_layer(lev,ist,i)   =                                 &
           & transmission_k % tau_layer(lev,ist,i)  -rad_up_k_chan(i,lev,ist)  * &
           & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 

        else
          if(transmission %od_singlelayer(lev,ist,i)<min_od.OR.                      &
           & (transmission % tau_layer(lev-1,ist,i)                             - &
           & transmission % tau_layer(lev,ist,i))<min_od)THEN 

            rad_dwn_k_chan(i,lev,ist)= 0.0_JPRB

            rad_up_k_chan(i,lev,ist)= 0._JPRB
          else     

            If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then

              od_singlelayer_r=1.0_JPRB/transmission %od_singlelayer(lev,ist,i)
              dtau_lay = &
               & (transmission % tau_layer(lev-1,ist,i)               - &
               &  transmission % tau_layer(lev  ,ist,i) ) 
              daux_lay = (auxrad%layer(lev,i)-auxrad%layer(lev-1,i))
              B1_2 = auxrad%layer(lev-1,i)     * &
               & dtau_lay
              B2_2 =                             &
               & daux_lay * &
               & transmission % tau_layer(lev,ist,i) 

              B3_2 =                            &
               & daux_lay     * &
               & dtau_lay     * &
               & od_singlelayer_r
              transmission_k % tau_layer(lev-1,ist,i)=                              &
               & transmission_k % tau_layer(lev-1,ist,i)-rad_dwn_k_chan(i,lev,ist)      * &
               & (B1_2-B3_2)/ &
               & (transmission % tau_layer(lev,ist,i)                               * &
               & transmission % tau_layer(lev-1,ist,i)**2) 

              transmission_k % tau_layer(lev,ist,i)=                                &
               & transmission_k % tau_layer(lev,ist,i)-rad_dwn_k_chan(i,lev,ist)        * &
               & (B1_2-B3_2)/ &
               & (transmission % tau_layer(lev,ist,i)**2                            * &
               & transmission % tau_layer(lev-1,ist,i)) 

              transmission_k % tau_layer(lev,ist,i) =                               &
               & transmission_k % tau_layer(lev,ist,i) -rad_dwn_k_chan(i,lev,ist)*2     * &
               & B2_2                                      / &
               & (transmission % tau_layer(lev,ist,i)**3) 

              B2_k=rad_dwn_k_chan(i,lev,ist)/(transmission %tau_layer(lev,ist,i))**2
              B3_k=-rad_dwn_k_chan(i,lev,ist)                                           / &
               & (transmission % tau_layer(lev-1,ist,i)                           * &
               & transmission % tau_layer(lev,ist,i)) 
              B1_k=rad_dwn_k_chan(i,lev,ist)                                           / &
               & (transmission % tau_layer(lev-1,ist,i)                           * &
               & transmission % tau_layer(lev,ist,i)) 

              rad_dwn_k_chan(i,lev,ist) = 0._JPRB
            else
              B1_k=0.0_JPRB
              B2_k=0.0_JPRB
              B3_k=0.0_JPRB
              rad_dwn_k_chan(i,lev,ist) = 0._JPRB 
            endif

            B1_k=B1_k+rad_up_k_chan(i,lev,ist)
            B2_k=B2_k-rad_up_k_chan(i,lev,ist)
            B3_k=B3_k+rad_up_k_chan(i,lev,ist)

            rad_up_k_chan(i,lev,ist)=0._JPRB

            transmission_k %od_singlelayer(lev,ist,i)=                              &
             & transmission_k %od_singlelayer(lev,ist,i)                             - &
             & B3_k*(auxrad%layer(LEV,I)-auxrad%layer(LEV-1,i))                            * &
             & (transmission % tau_layer(lev-1,ist,i)                              - &
             & transmission % tau_layer(lev,ist,i))/ &
             & (transmission %od_singlelayer(lev,ist,i)**2) 

            transmission_k % tau_layer(lev,ist,i)=                                  &
             & transmission_k % tau_layer(lev,ist,i)-B3_k                       * &
             & (auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I))                              / &
             & (transmission %od_singlelayer(lev,ist,i)) 

            transmission_k % tau_layer(lev-1,ist,i)=                                &
             & transmission_k % tau_layer(lev-1,ist,i)+B3_k                     * &
             & (auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I))                             / &
             & (transmission %od_singlelayer(lev,ist,i)) 

            rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)-B3_k                            * &
             & (transmission % tau_layer(lev-1,ist,i)                            - &
             & transmission % tau_layer(lev,ist,i))                              / &
             & (transmission %od_singlelayer(lev,ist,i)) 

            rad_layer_k(lev,i)=rad_layer_k(lev,i)+B3_k                                * &
             & (transmission % tau_layer(lev-1,ist,i)                            - &
             & transmission % tau_layer(lev,ist,i))                              / &
             & (transmission %od_singlelayer(lev,ist,i)) 
            !         B3_k=0._JPRB
            transmission_k % tau_layer(lev,ist,i)=                                  &
             & transmission_k % tau_layer(lev,ist,i)                + &
             & B2_k*(auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I)) 

            rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)                                  - &
             & B2_k*transmission % tau_layer(lev,ist,i) 

            rad_layer_k(lev,i)=rad_layer_k(lev,i)                                      + &
             & B2_k*transmission % tau_layer(lev,ist,i) 
            !         B2_k=0._JPRB

            transmission_k % tau_layer(lev,ist,i)=                                  &
             & transmission_k % tau_layer(lev,ist,i)-B1_k*auxrad%layer(LEV-1,I) 

            transmission_k % tau_layer(lev-1,ist,i)=                                &
             & transmission_k % tau_layer(lev-1,ist,i)+B1_k*auxrad%layer(LEV-1,I) 

            rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)                        + &
             & B1_k*(transmission % tau_layer(lev-1,ist,i)- &
             & transmission % tau_layer(lev,ist,i)) 
            !         B1_k=0._JPRB        
          endif
        endif
      Else  ! ---------------------------------OLD CODE-----------------------------------
        ! 
        auxrad_stream%meanrad_up_2d(ist,i,lev) = 0.5_JPRB * ( auxrad%layer(lev,i)              + &
         & auxrad%layer(lev-1,i) ) * &
         & ( transmission % tau_layer(lev-1,ist,i)                             - &
         & transmission % tau_layer(lev,ist,i)   ) 

        If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
          ! on peut creer une variable intermediaire pour gagner
          ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)
          tau_prod = transmission % tau_layer(lev,ist,i)                        * &
           & transmission % tau_layer(lev-1,ist,i) 
          rad_up_k_chan(i,lev,ist)  =rad_up_k_chan(i,lev,ist)   + rad_dwn_k_chan(i,lev,ist) / &
           & tau_prod 
          transmission_k % tau_layer(lev,ist,i)   =                              &
           & transmission_k % tau_layer(lev,ist,i)   - rad_dwn_k_chan(i,lev,ist)   * &
           & transmission % tau_layer(lev-1,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)      / &
           & (tau_prod*tau_prod) 
          transmission_k % tau_layer(lev-1,ist,i) =                              &
           & transmission_k % tau_layer(lev-1,ist,i) - rad_dwn_k_chan(i,lev,ist)   * &
           & transmission % tau_layer(lev,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)        / &
           & (tau_prod*tau_prod) 
          rad_dwn_k_chan(i,lev,ist) = 0._JPRB
        Else
          rad_dwn_k_chan(i,lev,ist) = 0._JPRB
        End If

        rad_layer_k(lev,i)   = rad_layer_k(lev,i)   +rad_up_k_chan(i,lev,ist)        * &
         & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
         & transmission % tau_layer(lev,ist,i) ) 
        rad_layer_k(lev-1,i) = rad_layer_k(lev-1,i) +rad_up_k_chan(i,lev,ist)        * &
         & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
         & transmission % tau_layer(lev,ist,i) ) 
        transmission_k % tau_layer(lev-1,ist,i) =                                 &
         & transmission_k % tau_layer(lev-1,ist,i) +rad_up_k_chan(i,lev,ist) * &
         & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 
        transmission_k % tau_layer(lev,ist,i)   =                                 &
         & transmission_k % tau_layer(lev,ist,i)  -rad_up_k_chan(i,lev,ist)  * &
         & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) )  
      ENDIF ! ----------------------------------------------------------------------------

    Enddo
  End Do
End Do
ist=0
#ifdef RTTOV_ARCH_VECTOR
Do lev = profiles(1) % nlevels, 2, -1
!cdir nodep
  Do i = 1, nchannels
    prof=lprofiles(i)
    chan=channels(i)
#else
Do i = 1, nchannels
  prof=lprofiles(i)
  chan=channels(i)
  Do lev = profiles(1) % nlevels, 2, -1
#endif
    If (.not. rt8_mode)then ! New code------------------------------------------------------
      !---Scattering of the solar beam----------------------------------------------------------

      if(addaerosl.OR.addclouds)then
        if(profiles(prof)%addsolar)then
          fac1_2_k = 0.0_jprb
          fac2_2_k = 0.0_jprb
          fac3_2_k = 0.0_jprb
          fac4_2_k = 0.0_jprb
          fac5_2_k = 0.0_jprb
          fac6_2_k = 0.0_jprb
          fac7_2_k = 0.0_jprb

          if(coef%ss_val_chn(chan)==1)then

            !-------------------Downward single scattering of the solar beam--------------------------

            if(raytracing%pathsat(lev,prof) /=                                     &
             & raytracing%pathsun(lev,prof))then 

             if(transmission %tau_layer(lev,ist,i)>min_tau)then
              transmission_k %tau_layer(lev,ist,i)=                          &
               & transmission_k %tau_layer(lev,ist,i)                       - &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                            * &
               & auxrad_stream%fac7_2(lev,ist,i)                            * &
               & (auxrad_stream%fac5_2(lev,ist,i)                           - &
               & auxrad_stream%fac4_2(lev,ist,i))                           * &
               & transmission %tausun_layer(lev-1,ist,i)                    / &
               & (transmission %tau_layer(lev-1,ist,i)                      * &
               & transmission %tau_layer(lev,ist,i)**2) 

              transmission_k %tau_layer(lev-1,ist,i)=                        &
               & transmission_k %tau_layer(lev-1,ist,i)                     - &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & (auxrad_stream%fac5_2(lev,ist,i)                            - &
               & auxrad_stream%fac4_2(lev,ist,i))                            * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)**2                    * &
               & transmission %tau_layer(lev,ist,i)) 

              transmission_k %tausun_layer(lev-1,ist,i)=                     &
               & transmission_k %tausun_layer(lev-1,ist,i)                  + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & (auxrad_stream%fac5_2(lev,ist,i)                            - &
               & auxrad_stream%fac4_2(lev,ist,i))                            / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac4_2_k=                             &
               & fac4_2_k                          - &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac5_2_k=                             &
               & fac5_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac7_2_k=                             &
               & fac7_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & (auxrad_stream%fac5_2(lev,ist,i)                            - &
               & auxrad_stream%fac4_2(lev,ist,i))                            * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac2_2_k=                             &
               & fac2_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & (auxrad_stream%fac5_2(lev,ist,i)                            - &
               & auxrad_stream%fac4_2(lev,ist,i))                            * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac6_2_k=                             &
               & fac6_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac7_2(lev,ist,i)                             * &
               & (auxrad_stream%fac5_2(lev,ist,i)                            - &
               & auxrad_stream%fac4_2(lev,ist,i))                            * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 


              raytracing_k%pathsat(lev,i)=raytracing_k%pathsat(lev,i)   + &
               & fac7_2_k                          * &
               & raytracing%pathsun(lev,prof)                                     / &
               & (raytracing%pathsun(lev,prof)-raytracing%pathsat(lev,prof))**2 

              raytracing_k%pathsun(lev,i)=raytracing_k%pathsun(lev,i)   - &
               & fac7_2_k                          * &
               & raytracing%pathsat(lev,prof)                                     / &
               & (raytracing%pathsun(lev,prof)-raytracing%pathsat(lev,prof))**2 
             else
                rad_dwn_k_chan(i,lev,ist) = 0._JPRB
             endif
            else
              if(transmission %tau_layer(lev,ist,i)>min_tau)then
              transmission_k %tau_layer(lev,ist,i)=                          &
               & transmission_k %tau_layer(lev,ist,i)                       - &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)**2) 

              transmission_k %tau_layer(lev-1,ist,i)=                        &
               & transmission_k %tau_layer(lev-1,ist,i)                     - &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)**2                    * &
               & transmission %tau_layer(lev,ist,i)) 

              transmission_k % odsun_singlelayer(lev,ist,i)                = &
               & transmission_k % odsun_singlelayer(lev,ist,i)              + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              transmission_k %tausun_layer(lev-1,ist,i)                    = &
               & transmission_k %tausun_layer(lev-1,ist,i)                  + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac4_2_k                            = &
               & fac4_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac2_2_k                            = &
               & fac2_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac6_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 

              fac6_2_k                            = &
               & fac6_2_k                          + &
               & rad_dwn_k_chan(i,lev,ist)                            * &
               & auxrad_stream%fac2_2(lev,ist,i)                             * &
               & auxrad_stream%fac4_2(lev,ist,i)                             * &
               & transmission %tausun_layer(lev-1,ist,i)                     * &
               & transmission % odsun_singlelayer(lev,ist,i)                 / &
               & (transmission %tau_layer(lev-1,ist,i)                       * &
               & transmission %tau_layer(lev,ist,i)) 
             else
                rad_dwn_k_chan(i,lev,ist) = 0._JPRB
             endif
            endif

            !----------------Upward single scattering of the solar beam-------------------------------

            fac1_2_k=                               &
             & fac1_2_k                      + &
             & rad_up_k_chan(i,lev,ist)                   * &
             & auxrad_stream%fac2_2(lev,ist,i)                         * &
             & auxrad_stream%fac3_2(lev,ist,i)                         * &
             & (transmission % tausun_layer(lev-1,ist,i)               - &
             & transmission % tausun_layer(lev,ist,i)) 

            fac2_2_k=                               &
             & fac2_2_k                      + &
             & rad_up_k_chan(i,lev,ist)                   * &
             & auxrad_stream%fac1_2(lev,ist,i)                         * &
             & auxrad_stream%fac3_2(lev,ist,i)                         * &
             & (transmission % tausun_layer(lev-1,ist,i)               - &
             & transmission % tausun_layer(lev,ist,i))  

            fac3_2_k=                               &
             & fac3_2_k                      + &
             & rad_up_k_chan(i,lev,ist)                   * &
             & auxrad_stream%fac1_2(lev,ist,i)                         * &
             & auxrad_stream%fac2_2(lev,ist,i)                         * &
             & (transmission % tausun_layer(lev-1,ist,i)               - &
             & transmission % tausun_layer(lev,ist,i)) 

            transmission_k % tausun_layer(lev-1,ist,i)=                      &
             & transmission_k % tausun_layer(lev-1,ist,i)             + &
             & rad_up_k_chan(i,lev,ist)                   * &
             & auxrad_stream%fac1_2(lev,ist,i)                         * &
             & auxrad_stream%fac2_2(lev,ist,i)                         * &
             & auxrad_stream%fac3_2(lev,ist,i) 

            transmission_k % tausun_layer(lev,ist,i)=                        &
             & transmission_k % tausun_layer(lev,ist,i)               - &
             & rad_up_k_chan(i,lev,ist)                   * &
             & auxrad_stream%fac1_2(lev,ist,i)                         * &
             & auxrad_stream%fac2_2(lev,ist,i)                         * &
             & auxrad_stream%fac3_2(lev,ist,i) 

            transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)             = &
             & transmission_scatt_ir_stream_k%azphacdo(ist,i,lev)     + &
             & fac6_2_k                      * &
             & coef % ss_solar_spectrum(chan)*z4pi_r

            transmission_k % od_singlelayer(lev,ist,i)                     = &
             & transmission_k % od_singlelayer(lev,ist,i)             - &
             & fac5_2_k                      * &
             & exp(-transmission % od_singlelayer(lev,ist,i)) 


            transmission_k % odsun_singlelayer(lev,ist,i)                  = &
             & transmission_k % odsun_singlelayer(lev,ist,i)          - &
             & fac4_2_k                      * &
             & exp(-transmission % odsun_singlelayer(lev,ist,i)) 

            raytracing_k%pathsat(lev,i)                                      = &
             & raytracing_k%pathsat(lev,i)                              + &
             & fac3_2_k                      * &
             & raytracing%pathsun(lev,prof)                                 / &
             & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

            raytracing_k%pathsun(lev,i)=raytracing_k%pathsun(lev,i)     - &
             & fac3_2_k                      * &
             & raytracing%pathsat(lev,prof)                                 / &
             & (raytracing%pathsat(lev,prof)+raytracing%pathsun(lev,prof))**2 

            transmission_scatt_ir_stream_k%ssa(ist,i,lev)                  = &
             & transmission_scatt_ir_stream_k%ssa(ist,i,lev)          + &
             & fac2_2_k 

            transmission_scatt_ir_stream_k%azphacup(ist,i,lev)             = &
             & transmission_scatt_ir_stream_k%azphacup(ist,i,lev)     + &
             & fac1_2_k                      * &
             & coef % ss_solar_spectrum(chan)*z4pi_r 

          endiF
        endif
      endif

      if(transmission % tau_layer(lev-1,ist,i)>=0.and.                &
       & transmission % tau_layer(lev,ist,i)<0)then 

        auxrad_stream%meanrad_up_2d(ist,i,lev) = 0.5_JPRB * ( auxrad%layer(lev,i)   + &
         & auxrad%layer(lev-1,i) ) * &
         & ( transmission % tau_layer(lev-1,ist,i)                             - &
         & transmission % tau_layer(lev,ist,i)   ) 

        If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
          ! on peut creer une variable intermediaire pour gagner
          ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)
          tau_prod = transmission % tau_layer(lev,ist,i)                        * &
           & transmission % tau_layer(lev-1,ist,i) 
          rad_up_k_chan(i,lev,ist)  =rad_up_k_chan(i,lev,ist)   + rad_dwn_k_chan(i,lev,ist) / &
           & tau_prod 
          transmission_k % tau_layer(lev,ist,i)   =                              &
           & transmission_k % tau_layer(lev,ist,i)   - rad_dwn_k_chan(i,lev,ist)   * &
           & transmission % tau_layer(lev-1,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)      / &
           & (tau_prod*tau_prod) 
          transmission_k % tau_layer(lev-1,ist,i) =                              &
           & transmission_k % tau_layer(lev-1,ist,i) - rad_dwn_k_chan(i,lev,ist)   * &
           & transmission % tau_layer(lev,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)        / &
           & (tau_prod*tau_prod) 
          rad_dwn_k_chan(i,lev,ist) = 0._JPRB
        Else
          rad_dwn_k_chan(i,lev,ist) = 0._JPRB
        End If

        rad_layer_k(lev,i)   = rad_layer_k(lev,i)   +rad_up_k_chan(i,lev,ist)        * &
         & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
         & transmission % tau_layer(lev,ist,i) ) 
        rad_layer_k(lev-1,i) = rad_layer_k(lev-1,i) +rad_up_k_chan(i,lev,ist)        * &
         & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
         & transmission % tau_layer(lev,ist,i) ) 
        transmission_k % tau_layer(lev-1,ist,i) =                                 &
         & transmission_k % tau_layer(lev-1,ist,i) +rad_up_k_chan(i,lev,ist) * &
         & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 
        transmission_k % tau_layer(lev,ist,i)   =                                 &
         & transmission_k % tau_layer(lev,ist,i)  -rad_up_k_chan(i,lev,ist)  * &
         & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 

      else
        if(transmission %od_singlelayer(lev,ist,i)<min_od.OR.                      &
         & (transmission % tau_layer(lev-1,ist,i)                             - &
         & transmission % tau_layer(lev,ist,i))<min_od)THEN 

          rad_dwn_k_chan(i,lev,ist)= 0.0_JPRB

          rad_up_k_chan(i,lev,ist)= 0._JPRB
        else     

          If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then

            od_singlelayer_r=1.0_JPRB/transmission %od_singlelayer(lev,ist,i)
            dtau_lay = &
             & (transmission % tau_layer(lev-1,ist,i)               - &
             &  transmission % tau_layer(lev  ,ist,i) ) 
            daux_lay = (auxrad%layer(lev,i)-auxrad%layer(lev-1,i))
            B1_2 = auxrad%layer(lev-1,i)     * &
             & dtau_lay
            B2_2 =                             &
             & daux_lay * &
             & transmission % tau_layer(lev,ist,i) 

            B3_2 =                            &
             & daux_lay     * &
             & dtau_lay     * &
             & od_singlelayer_r
            transmission_k % tau_layer(lev-1,ist,i)=                              &
             & transmission_k % tau_layer(lev-1,ist,i)-rad_dwn_k_chan(i,lev,ist)      * &
             & (B1_2-B3_2)/ &
             & (transmission % tau_layer(lev,ist,i)                               * &
             & transmission % tau_layer(lev-1,ist,i)**2) 

            transmission_k % tau_layer(lev,ist,i)=                                &
             & transmission_k % tau_layer(lev,ist,i)-rad_dwn_k_chan(i,lev,ist)        * &
             & (B1_2-B3_2)/ &
             & (transmission % tau_layer(lev,ist,i)**2                            * &
             & transmission % tau_layer(lev-1,ist,i)) 

            transmission_k % tau_layer(lev,ist,i) =                               &
             & transmission_k % tau_layer(lev,ist,i) -rad_dwn_k_chan(i,lev,ist)*2     * &
             & B2_2                                      / &
             & (transmission % tau_layer(lev,ist,i)**3) 

            B2_k=rad_dwn_k_chan(i,lev,ist)/(transmission %tau_layer(lev,ist,i))**2
            B3_k=-rad_dwn_k_chan(i,lev,ist)                                           / &
             & (transmission % tau_layer(lev-1,ist,i)                           * &
             & transmission % tau_layer(lev,ist,i)) 
            B1_k=rad_dwn_k_chan(i,lev,ist)                                           / &
             & (transmission % tau_layer(lev-1,ist,i)                           * &
             & transmission % tau_layer(lev,ist,i)) 

            rad_dwn_k_chan(i,lev,ist) = 0._JPRB
          else
            B1_k=0.0_JPRB
            B2_k=0.0_JPRB
            B3_k=0.0_JPRB
            rad_dwn_k_chan(i,lev,ist) = 0._JPRB 
          endif

          B1_k=B1_k+rad_up_k_chan(i,lev,ist)
          B2_k=B2_k-rad_up_k_chan(i,lev,ist)
          B3_k=B3_k+rad_up_k_chan(i,lev,ist)

          rad_up_k_chan(i,lev,ist)=0._JPRB

          transmission_k %od_singlelayer(lev,ist,i)=                              &
           & transmission_k %od_singlelayer(lev,ist,i)                             - &
           & B3_k*(auxrad%layer(LEV,I)-auxrad%layer(LEV-1,i))                            * &
           & (transmission % tau_layer(lev-1,ist,i)                              - &
           & transmission % tau_layer(lev,ist,i))/ &
           & (transmission %od_singlelayer(lev,ist,i)**2) 

          transmission_k % tau_layer(lev,ist,i)=                                  &
           & transmission_k % tau_layer(lev,ist,i)-B3_k                       * &
           & (auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I))                              / &
           & (transmission %od_singlelayer(lev,ist,i)) 

          transmission_k % tau_layer(lev-1,ist,i)=                                &
           & transmission_k % tau_layer(lev-1,ist,i)+B3_k                     * &
           & (auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I))                             / &
           & (transmission %od_singlelayer(lev,ist,i)) 

          rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)-B3_k                            * &
           & (transmission % tau_layer(lev-1,ist,i)                            - &
           & transmission % tau_layer(lev,ist,i))                              / &
           & (transmission %od_singlelayer(lev,ist,i)) 

          rad_layer_k(lev,i)=rad_layer_k(lev,i)+B3_k                                * &
           & (transmission % tau_layer(lev-1,ist,i)                            - &
           & transmission % tau_layer(lev,ist,i))                              / &
           & (transmission %od_singlelayer(lev,ist,i)) 
          !         B3_k=0._JPRB
          transmission_k % tau_layer(lev,ist,i)=                                  &
           & transmission_k % tau_layer(lev,ist,i)                + &
           & B2_k*(auxrad%layer(LEV,I)-auxrad%layer(LEV-1,I)) 

          rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)                                  - &
           & B2_k*transmission % tau_layer(lev,ist,i) 

          rad_layer_k(lev,i)=rad_layer_k(lev,i)                                      + &
           & B2_k*transmission % tau_layer(lev,ist,i) 
          !         B2_k=0._JPRB

          transmission_k % tau_layer(lev,ist,i)=                                  &
           & transmission_k % tau_layer(lev,ist,i)-B1_k*auxrad%layer(LEV-1,I) 

          transmission_k % tau_layer(lev-1,ist,i)=                                &
           & transmission_k % tau_layer(lev-1,ist,i)+B1_k*auxrad%layer(LEV-1,I) 

          rad_layer_k(lev-1,i)=rad_layer_k(lev-1,i)                        + &
           & B1_k*(transmission % tau_layer(lev-1,ist,i)- &
           & transmission % tau_layer(lev,ist,i)) 
          !         B1_k=0._JPRB        
        endif
      endif
    Else  ! ---------------------------------OLD CODE-----------------------------------
      ! 
      auxrad_stream%meanrad_up_2d(ist,i,lev) = 0.5_JPRB * ( auxrad%layer(lev,i)              + &
       & auxrad%layer(lev-1,i) ) * &
       & ( transmission % tau_layer(lev-1,ist,i)                             - &
       & transmission % tau_layer(lev,ist,i)   ) 

      If ( transmission % tau_layer(lev,ist,i) > min_tau ) Then
        ! on peut creer une variable intermediaire pour gagner
        ! du temps rad_tmp_k * meanrad_up / (tau_prod*tau_prod)
        tau_prod = transmission % tau_layer(lev,ist,i)                        * &
         & transmission % tau_layer(lev-1,ist,i) 
        rad_up_k_chan(i,lev,ist)  =rad_up_k_chan(i,lev,ist)   + rad_dwn_k_chan(i,lev,ist) / &
         & tau_prod 
        transmission_k % tau_layer(lev,ist,i)   =                              &
         & transmission_k % tau_layer(lev,ist,i)   - rad_dwn_k_chan(i,lev,ist)   * &
         & transmission % tau_layer(lev-1,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)      / &
         & (tau_prod*tau_prod) 
        transmission_k % tau_layer(lev-1,ist,i) =                              &
         & transmission_k % tau_layer(lev-1,ist,i) - rad_dwn_k_chan(i,lev,ist)   * &
         & transmission % tau_layer(lev,ist,i) * auxrad_stream%meanrad_up_2d(ist,i,lev)        / &
         & (tau_prod*tau_prod) 
        rad_dwn_k_chan(i,lev,ist) = 0._JPRB
      Else
        rad_dwn_k_chan(i,lev,ist) = 0._JPRB
      End If

      rad_layer_k(lev,i)   = rad_layer_k(lev,i)   +rad_up_k_chan(i,lev,ist)        * &
       & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
       & transmission % tau_layer(lev,ist,i) ) 
      rad_layer_k(lev-1,i) = rad_layer_k(lev-1,i) +rad_up_k_chan(i,lev,ist)        * &
       & 0.5_JPRB * ( transmission % tau_layer(lev-1,ist,i)                 - &
       & transmission % tau_layer(lev,ist,i) ) 
      transmission_k % tau_layer(lev-1,ist,i) =                                 &
       & transmission_k % tau_layer(lev-1,ist,i) +rad_up_k_chan(i,lev,ist) * &
       & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) ) 
      transmission_k % tau_layer(lev,ist,i)   =                                 &
       & transmission_k % tau_layer(lev,ist,i)  -rad_up_k_chan(i,lev,ist)  * &
       & 0.5_JPRB * ( auxrad%layer(lev,i) + auxrad%layer(lev-1,i) )  
    ENDIF ! ----------------------------------------------------------------------------

  Enddo
End Do

!2.1 layer above top pressure level
!----------------------------------
!
Do i=1,nchannels
  prof=lprofiles(i)
  chan=channels(i)
  do ist=ircld%nstream(prof),1,-1

    If(.not. rt8_mode)Then !-------------New code----------------------------------------------

      !---Scattering of the solar beam--------------------------------------------------
      if(addaerosl.or.addclouds)then
        if(profiles(prof)%addsolar)then
          fac1_1_k = 0.0_jprb
          fac2_1_k = 0.0_jprb
          fac3_1_k = 0.0_jprb
          fac4_1_k = 0.0_jprb
          fac5_1_k = 0.0_jprb
          fac6_1_k = 0.0_jprb
          fac7_1_k = 0.0_jprb

          if(coef%ss_val_chn(chan)==1)then

            !-------------Downward single scattering of the solar beam--------------------------------

            if(raytracing%pathsat(1,prof)/=raytracing%pathsun(1,prof))then

              if(auxrad_stream%down_ref(ist,1,i)<0._jprb)then
                zdown_k(1,ist,i)=0._jprb
              endif

              transmission_k %tau_layer(1,ist,i)                              = &
               & transmission_k %tau_layer(1,ist,i)                            - &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC7_1(IST,i)                                    * &
               & (auxrad_stream%fac5_1(ist,i)                                   - &
               & auxrad_stream%fac4_1(ist,i))                                   / &
               & transmission %tau_layer(1,ist,i)**2 

              fac4_1_k                                   = &
               & fac4_1_k                                 - &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC7_1(IST,i)                                    / &
               & transmission %tau_layer(1,ist,i) 

              fac5_1_k                                   = &
               & fac5_1_k                                 + &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC7_1(IST,i)                                    / &
               & transmission %tau_layer(1,ist,i) 

              fac7_1_k                                    = &
               & fac7_1_k                                  + &
               & zdown_k(1,ist,i)                                  * &
               & auxrad_stream%FAC6_1(IST,i)                                     * &
               & auxrad_stream%FAC2_1(IST,i)                                     * &
               & (auxrad_stream%fac5_1(ist,i)                                    - &
               & auxrad_stream%fac4_1(ist,i))                                    / &
               & transmission %tau_layer(1,ist,i) 

              fac2_1_k                                    = &
               & fac2_1_k                                  + &
               & zdown_k(1,ist,i)                                  * &
               & auxrad_stream%FAC6_1(IST,i)                                     * &
               & auxrad_stream%FAC7_1(IST,i)                                     * &
               & (auxrad_stream%fac5_1(ist,i)                                    - &
               & auxrad_stream%fac4_1(ist,i))                                    / &
               & transmission %tau_layer(1,ist,i) 

              fac6_1_k                                    = &
               & fac6_1_k                                  + &
               & zdown_k(1,ist,i)                                  * &
               & auxrad_stream%FAC2_1(IST,i)                                     * &
               & auxrad_stream%FAC7_1(IST,i)                                     * &
               & (auxrad_stream%fac5_1(ist,i)                                    - &
               & auxrad_stream%fac4_1(ist,i))                                    / &
               & transmission %tau_layer(1,ist,i) 

              raytracing_k%pathsat(1,i)=raytracing_k%pathsat(1,i)           + &
               & fac7_1_k*raytracing%pathsun(1,prof)      / &
               & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof))**2 

              raytracing_k%pathsun(1,i)=raytracing_k%pathsun(1,i)           - &
               & fac7_1_k*raytracing%pathsat(1,prof)      / &
               & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof))**2  

            else

              if(auxrad_stream%down_ref(ist,1,i)<0._jprb)then
                zdown_k(1,ist,i)=0._jprb
              endif

              transmission_k %tau_layer(1,ist,i)=                               &
               & transmission_k %tau_layer(1,ist,i)                            - &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC4_1(IST,i)                                    * &
               & transmission % odsun_singlelayer(1,ist,i)                      / &
               & transmission %tau_layer(1,ist,i)**2 

              transmission_k % odsun_singlelayer(1,ist,i)                     = &
               & transmission_k % odsun_singlelayer(1,ist,i)                   + &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC4_1(IST,i)                                    / &
               & transmission %tau_layer(1,ist,i) 

              fac4_1_k=                                    &
               & fac4_1_k                                 + &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & transmission % odsun_singlelayer(1,ist,i)                      / &
               & transmission %tau_layer(1,ist,i) 

              fac2_1_k=                                    &
               & fac2_1_k                                 + &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC6_1(IST,i)                                    * &
               & auxrad_stream%FAC4_1(IST,i)                                    * &
               & transmission % odsun_singlelayer(1,ist,i)                      / &
               & transmission %tau_layer(1,ist,i) 

              fac6_1_k=                                    &
               & fac6_1_k                                 + &
               & zdown_k(1,ist,i)                                 * &
               & auxrad_stream%FAC2_1(IST,i)                                    * &
               & auxrad_stream%FAC4_1(IST,i)                                    * &
               & transmission % odsun_singlelayer(1,ist,i)                      / &
               & transmission %tau_layer(1,ist,i)  

            endif

            !-------------Upward single scattering of the solar beam----------------------------------

            fac1_1_k=fac1_1_k+ &
             & zup_k(1,ist,i)                                   * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC3_1(IST,i)                                    * &
             & (1-transmission % tausun_layer(1,ist,i)) 

            fac2_1_k=fac2_1_k+ &
             & zup_k(1,ist,i)                                   * &
             & auxrad_stream%FAC1_1(IST,i)                                    * &
             & auxrad_stream%FAC3_1(IST,i)                                    * &
             & (1-transmission % tausun_layer(1,ist,i)) 

            fac3_1_k=fac3_1_k+ &
             & zup_k(1,ist,i)                                   * &
             & auxrad_stream%FAC1_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & (1-transmission % tausun_layer(1,ist,i)) 

            transmission_k % tausun_layer(1,ist,i)=                             &
             & transmission_k % tausun_layer(1,ist,i)                        - &
             & zup_k(1,ist,i)                                   * &
             & auxrad_stream%FAC1_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC3_1(IST,i) 

            transmission_scatt_ir_stream_k%azphacdo(ist,i,1)=                   &
             & transmission_scatt_ir_stream_k%azphacdo(ist,i,1)              + &
             & fac6_1_k                                 * &
             & coef % ss_solar_spectrum(chan)*z4pi_r

            transmission_k % od_singlelayer(1,ist,i)=                           &
             & transmission_k % od_singlelayer(1,ist,i)                      - &
             & fac5_1_k                                 * &
             & exp(-transmission % od_singlelayer(1,ist,i)) 

            transmission_k % odsun_singlelayer(1,ist,i)=                        &
             & transmission_k % odsun_singlelayer(1,ist,i)                   - &
             & fac4_1_k                                 * &
             & EXP(-transmission % odsun_singlelayer(1,ist,i)) 

            raytracing_k%pathsat(1,i)=raytracing_k%pathsat(1,i)            + &
             & fac3_1_k*raytracing%pathsun(1,prof)     / &
             & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof))**2 

            raytracing_k%pathsun(1,i)=raytracing_k%pathsun(1,i)            - &
             & fac3_1_k*raytracing%pathsat(1,prof)     / &
             & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof))**2 

            transmission_scatt_ir_stream_k%ssa(ist,i,1)=                        &
             & transmission_scatt_ir_stream_k%ssa(ist,i,1)                   + &
             & fac2_1_k 

            transmission_scatt_ir_stream_k%azphacup(ist,i,1)                  = &
             & transmission_scatt_ir_stream_k%azphacup(ist,i,1)               + &
             & fac1_1_k                                  * &
             & coef % ss_solar_spectrum(chan)*z4pi_r

          endif
        endif
      endif

      if(transmission %od_singlelayer(1,ist,i)<min_od.OR.                            &
       & ((1-transmission % tau_layer(1,ist,i))<min_od))then 
        zup_k(1,ist,i)   =0._JPRB
        zdown_k(1,ist,i) =0._JPRB
      else
        B1_1 = auxrad%layer(1,i)                   * &
         & (1.-transmission % tau_layer(1,ist,i)) 
        B2_1 = 0.
        B3_1 = 0.
        transmission_k % tau_layer(1,ist,i)=                                        &
         & transmission_k % tau_layer(1,ist,i)                           - &
         & zdown_k(1,ist,i)                                 * &
         & (B1_1+B2_1-B3_1)                                / &
         & transmission % tau_layer(1,ist,i)**2 
        B3_k=-zdown_k(1,ist,i)                                 / &
         & transmission % tau_layer(1,ist,i) 
        B2_k=zdown_k(1,ist,i)                                 / &
         & transmission % tau_layer(1,ist,i) 
        B1_k=zdown_k(1,ist,i)                                 / &
         & transmission % tau_layer(1,ist,i) 
        B1_k=B1_k+zup_k(1,ist,i)
        B2_k=B2_k-zup_k(1,ist,i)
        B3_k=B3_k+zup_k(1,ist,i)

!       B3_k=0._JPRB
!       B2_k=0._JPRB

        rad_layer_k(1,i)=rad_layer_k(1,i)+B1_k                                        * &
         & (1.-transmission % tau_layer(1,ist,i)) 

        transmission_k % tau_layer(1,ist,i)=                                        &
         & transmission_k % tau_layer(1,ist,i)                     - &
         & B1_k*auxrad%layer(1,i) 
!       B1_k=0._JPRB
        !      zup_k(1,ist,i)   =0._JPRB
        !      zdown_k(1,ist,i) =0._JPRB   
      endif

    Else  !------------Old Code------------------------------------------------------------

      zup_k(1,ist,i)    = zup_k(1,ist,i)         + &
       & zdown_k(1,ist,i)                                          / &
       & transmission % tau_layer(1,ist,i) 
      transmission_k % tau_layer(1,ist,i) =                                         &
       & transmission_k % tau_layer(1,ist,i)                                     - &
       & zdown_k(1,ist,i) *&
       & auxrad_stream%down(ist,1,i) / transmission % tau_layer(1,ist,i) 
      zdown_k(1,ist,i)  = 0._JPRB
      rad_layer_k(1,i) = rad_layer_k(1,i) + zup_k(1,ist,i)         * &
       & ( 1.0_JPRB-transmission % tau_layer(1,ist,i) ) 
      transmission_k % tau_layer(1,ist,i) =                                         &
       & transmission_k % tau_layer(1,ist,i)                                     - &
       & zup_k(1,ist,i) *auxrad%layer(1,i) 
      zup_k(1,ist,i)    = 0._JPRB

    Endif  !-------------------------------------------------------------------------------
  enddo
enddo
ist=0
!cdir nodep
Do i=1,nchannels
  prof=lprofiles(i)
  chan=channels(i)
  If(.not. rt8_mode)Then !-------------New code----------------------------------------------

    !---Scattering of the solar beam--------------------------------------------------
    if(addaerosl.or.addclouds)then
      if(profiles(prof)%addsolar)then
        fac1_1_k = 0.0_jprb
        fac2_1_k = 0.0_jprb
        fac3_1_k = 0.0_jprb
        fac4_1_k = 0.0_jprb
        fac5_1_k = 0.0_jprb
        fac6_1_k = 0.0_jprb
        fac7_1_k = 0.0_jprb

        if(coef%ss_val_chn(chan)==1)then

          !-------------Downward single scattering of the solar beam--------------------------------

          if(raytracing%pathsat(1,prof)/=raytracing%pathsun(1,prof))then

            if(auxrad_stream%down_ref(ist,1,i)<0._jprb)then
              zdown_k(1,ist,i)=0._jprb
            endif

            transmission_k %tau_layer(1,ist,i)                              = &
             & transmission_k %tau_layer(1,ist,i)                            - &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC7_1(IST,i)                                    * &
             & (auxrad_stream%fac5_1(ist,i)                                   - &
             & auxrad_stream%fac4_1(ist,i))                                   / &
             & transmission %tau_layer(1,ist,i)**2 

            fac4_1_k                                   = &
             & fac4_1_k                                 - &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC7_1(IST,i)                                    / &
             & transmission %tau_layer(1,ist,i) 

            fac5_1_k                                   = &
             & fac5_1_k                                 + &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC7_1(IST,i)                                    / &
             & transmission %tau_layer(1,ist,i) 

            fac7_1_k                                    = &
             & fac7_1_k                                  + &
             & zdown_k(1,ist,i)                                  * &
             & auxrad_stream%FAC6_1(IST,i)                                     * &
             & auxrad_stream%FAC2_1(IST,i)                                     * &
             & (auxrad_stream%fac5_1(ist,i)                                    - &
             & auxrad_stream%fac4_1(ist,i))                                    / &
             & transmission %tau_layer(1,ist,i) 

            fac2_1_k                                    = &
             & fac2_1_k                                  + &
             & zdown_k(1,ist,i)                                  * &
             & auxrad_stream%FAC6_1(IST,i)                                     * &
             & auxrad_stream%FAC7_1(IST,i)                                     * &
             & (auxrad_stream%fac5_1(ist,i)                                    - &
             & auxrad_stream%fac4_1(ist,i))                                    / &
             & transmission %tau_layer(1,ist,i) 

            fac6_1_k                                    = &
             & fac6_1_k                                  + &
             & zdown_k(1,ist,i)                                  * &
             & auxrad_stream%FAC2_1(IST,i)                                     * &
             & auxrad_stream%FAC7_1(IST,i)                                     * &
             & (auxrad_stream%fac5_1(ist,i)                                    - &
             & auxrad_stream%fac4_1(ist,i))                                    / &
             & transmission %tau_layer(1,ist,i) 

            raytracing_k%pathsat(1,i)=raytracing_k%pathsat(1,i)           + &
             & fac7_1_k*raytracing%pathsun(1,prof)      / &
             & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof))**2 

            raytracing_k%pathsun(1,i)=raytracing_k%pathsun(1,i)           - &
             & fac7_1_k*raytracing%pathsat(1,prof)      / &
             & (raytracing%pathsun(1,prof)-raytracing%pathsat(1,prof))**2  

          else

            if(auxrad_stream%down_ref(ist,1,i)<0._jprb)then
              zdown_k(1,ist,i)=0._jprb
            endif

            transmission_k %tau_layer(1,ist,i)=                               &
             & transmission_k %tau_layer(1,ist,i)                            - &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC4_1(IST,i)                                    * &
             & transmission % odsun_singlelayer(1,ist,i)                      / &
             & transmission %tau_layer(1,ist,i)**2 

            transmission_k % odsun_singlelayer(1,ist,i)                     = &
             & transmission_k % odsun_singlelayer(1,ist,i)                   + &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC4_1(IST,i)                                    / &
             & transmission %tau_layer(1,ist,i) 

            fac4_1_k=                                    &
             & fac4_1_k                                 + &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & transmission % odsun_singlelayer(1,ist,i)                      / &
             & transmission %tau_layer(1,ist,i) 

            fac2_1_k=                                    &
             & fac2_1_k                                 + &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC6_1(IST,i)                                    * &
             & auxrad_stream%FAC4_1(IST,i)                                    * &
             & transmission % odsun_singlelayer(1,ist,i)                      / &
             & transmission %tau_layer(1,ist,i) 

            fac6_1_k=                                    &
             & fac6_1_k                                 + &
             & zdown_k(1,ist,i)                                 * &
             & auxrad_stream%FAC2_1(IST,i)                                    * &
             & auxrad_stream%FAC4_1(IST,i)                                    * &
             & transmission % odsun_singlelayer(1,ist,i)                      / &
             & transmission %tau_layer(1,ist,i)  

          endif

          !-------------Upward single scattering of the solar beam----------------------------------

          fac1_1_k=fac1_1_k+ &
           & zup_k(1,ist,i)                                   * &
           & auxrad_stream%FAC2_1(IST,i)                                    * &
           & auxrad_stream%FAC3_1(IST,i)                                    * &
           & (1-transmission % tausun_layer(1,ist,i)) 

          fac2_1_k=fac2_1_k+ &
           & zup_k(1,ist,i)                                   * &
           & auxrad_stream%FAC1_1(IST,i)                                    * &
           & auxrad_stream%FAC3_1(IST,i)                                    * &
           & (1-transmission % tausun_layer(1,ist,i)) 

          fac3_1_k=fac3_1_k+ &
           & zup_k(1,ist,i)                                   * &
           & auxrad_stream%FAC1_1(IST,i)                                    * &
           & auxrad_stream%FAC2_1(IST,i)                                    * &
           & (1-transmission % tausun_layer(1,ist,i)) 

          transmission_k % tausun_layer(1,ist,i)=                             &
           & transmission_k % tausun_layer(1,ist,i)                        - &
           & zup_k(1,ist,i)                                   * &
           & auxrad_stream%FAC1_1(IST,i)                                    * &
           & auxrad_stream%FAC2_1(IST,i)                                    * &
           & auxrad_stream%FAC3_1(IST,i) 

          transmission_scatt_ir_stream_k%azphacdo(ist,i,1)=                   &
           & transmission_scatt_ir_stream_k%azphacdo(ist,i,1)              + &
           & fac6_1_k                                 * &
           & coef % ss_solar_spectrum(chan)*z4pi_r

          transmission_k % od_singlelayer(1,ist,i)=                           &
           & transmission_k % od_singlelayer(1,ist,i)                      - &
           & fac5_1_k                                 * &
           & exp(-transmission % od_singlelayer(1,ist,i)) 

          transmission_k % odsun_singlelayer(1,ist,i)=                        &
           & transmission_k % odsun_singlelayer(1,ist,i)                   - &
           & fac4_1_k                                 * &
           & EXP(-transmission % odsun_singlelayer(1,ist,i)) 

          raytracing_k%pathsat(1,i)=raytracing_k%pathsat(1,i)            + &
           & fac3_1_k*raytracing%pathsun(1,prof)     / &
           & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof))**2 

          raytracing_k%pathsun(1,i)=raytracing_k%pathsun(1,i)            - &
           & fac3_1_k*raytracing%pathsat(1,prof)     / &
           & (raytracing%pathsun(1,prof)+raytracing%pathsat(1,prof))**2 

          transmission_scatt_ir_stream_k%ssa(ist,i,1)=                        &
           & transmission_scatt_ir_stream_k%ssa(ist,i,1)                   + &
           & fac2_1_k 

          transmission_scatt_ir_stream_k%azphacup(ist,i,1)                  = &
           & transmission_scatt_ir_stream_k%azphacup(ist,i,1)               + &
           & fac1_1_k                                  * &
           & coef % ss_solar_spectrum(chan)*z4pi_r

        endif
      endif
    endif

    if(transmission %od_singlelayer(1,ist,i)<min_od.OR.                            &
     & ((1-transmission % tau_layer(1,ist,i))<min_od))then 
      zup_k(1,ist,i)   =0._JPRB
      zdown_k(1,ist,i) =0._JPRB
    else
      B1_1 = auxrad%layer(1,i)                   * &
       & (1.-transmission % tau_layer(1,ist,i)) 
      B2_1 = 0.
      B3_1 = 0.
      transmission_k % tau_layer(1,ist,i)=                                        &
       & transmission_k % tau_layer(1,ist,i)                           - &
       & zdown_k(1,ist,i)                                 * &
       & (B1_1+B2_1-B3_1)                                / &
       !                 & (B1_1)                                          / &
       & transmission % tau_layer(1,ist,i)**2 
      B3_k=-zdown_k(1,ist,i)                                 / &
       & transmission % tau_layer(1,ist,i) 
      B2_k=zdown_k(1,ist,i)                                 / &
       & transmission % tau_layer(1,ist,i) 
      B1_k=zdown_k(1,ist,i)                                 / &
       & transmission % tau_layer(1,ist,i) 
      B1_k=B1_k+zup_k(1,ist,i)
      B2_k=B2_k-zup_k(1,ist,i)
      B3_k=B3_k+zup_k(1,ist,i)

      !     B3_k=0._JPRB
      !     B2_k=0._JPRB

      rad_layer_k(1,i)=rad_layer_k(1,i)+B1_k                                        * &
       & (1.-transmission % tau_layer(1,ist,i)) 

      transmission_k % tau_layer(1,ist,i)=                                        &
       & transmission_k % tau_layer(1,ist,i)                     - &
       & B1_k*auxrad%layer(1,i) 
      !     B1_k=0._JPRB
      !      zup_k(1,ist,i)   =0._JPRB
      !      zdown_k(1,ist,i) =0._JPRB   
    endif

  Else  !------------Old Code------------------------------------------------------------

    zup_k(1,ist,i)    = zup_k(1,ist,i)         + &
     & zdown_k(1,ist,i)                                          / &
     & transmission % tau_layer(1,ist,i) 
    transmission_k % tau_layer(1,ist,i) =                                         &
     & transmission_k % tau_layer(1,ist,i)                                     - &
     & zdown_k(1,ist,i) *&
     & auxrad_stream%down(ist,1,i) / transmission % tau_layer(1,ist,i) 
    zdown_k(1,ist,i)  = 0._JPRB
    rad_layer_k(1,i) = rad_layer_k(1,i) + zup_k(1,ist,i)         * &
     & ( 1.0_JPRB-transmission % tau_layer(1,ist,i) ) 
    transmission_k % tau_layer(1,ist,i) =                                         &
     & transmission_k % tau_layer(1,ist,i)                                     - &
     & zup_k(1,ist,i) *auxrad%layer(1,i) 
    zup_k(1,ist,i)    = 0._JPRB

  Endif  !-------------------------------------------------------------------------------
enddo

rad_k%clear(:)=0._jprb
rad_k%cloudy(:)=0._jprb
auxrad_stream_k%cloudy=0._jprb
!
Call rttov_calcrad_k( &
 & nchannels,        &! in
 & nprofiles,        &! in
 & channels,         &! in
 & lprofiles,        &! in
 & profiles,         &! in
 & profiles_k,      &! inout
 & coef,             &! in
 & auxrad%skin,      &! in
 & auxrad%surfair,   &! in
 & auxrad%layer,     &! in
 & rad_skin_k,      &! in
 & rad_surfair_k,   &! in
 & rad_layer_k    )  ! in 
IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_K',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_integrate_k

