Subroutine rttov_opdep( &
      & nchannels,    &! in
      & nprofiles,    &! in
      & nlevels,      &! in
      & channels,     &! in
      & lprofiles,    &! in
      & predictors,   &! in
      & aux,          &! in
      & coef,         &! in
      & opdp_path,    &! inout
      & opdp_ref)      ! out
  !
  ! Description:
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  ! To interpolate optical depths on to levels of radiative transfer model
  ! (which, at present, entails only surface transmittance, as
  ! other optical depths are on *rt* levels) and to convert
  ! optical depths to transmittances.
  !
  ! Code based on OPDEP and RTTAU from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0    01/12/2002  New F90 code with structures (P Brunel A Smith) 
  !  1.1    29/01/2003  Add WV Continuum and CO2 capability (P Brunel)
  !  1.2    04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.3    26/09/2003  Modified to allow for multiple polarisations (S English)
  !  1.4    17/08/2004    Bug fixed in setting transmission to 1 (S English)
  !  1.5    28/02/2005 Improved vectorisation (D Dent)
  !  1.6    01/06/2005  Marco Matricardi (ECMWF):
  !            --       Computation of variable OD_SFRAC added.
  !  1.7    24/01/2007  Removed polarisation index (R Saunders) 
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

  ! Imported Parameters:
  Use rttov_const, Only: &
        & sensor_id_mw   ,&
        & max_optical_depth  
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
        & rttov_coef     ,&
        & predictors_Type,&
        & opdp_path_Type  ,&
        & profile_aux 

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),     Intent(in)    :: nchannels               ! Number of output radiances
  Integer(Kind=jpim),     Intent(in)    :: nprofiles               ! Number of profiles
  Integer(Kind=jpim),     Intent(in)    :: nlevels                 ! Number of pressure levels
  Integer(Kind=jpim),     Intent(in)    :: channels(nchannels)     ! Channel indices
  Integer(Kind=jpim),     Intent(in)    :: lprofiles(nchannels)    ! Profiles indices
  Type(predictors_Type),  Intent(in)    :: predictors ! Predictors
  Type(rttov_coef),       Intent(in)    :: coef                    ! Coefficients
  Type(profile_aux),      Intent(in)    :: aux( nprofiles )        ! auxillary profiles informations
  Type(opdp_path_Type),   Intent(inout) :: opdp_path               ! optical depths
  Real(Kind=jprb),        Intent(out)   :: opdp_ref(nlevels,nchannels)! layer optical depth 

!INTF

!local variables:
  Real(Kind=jprb) :: opticaldepth(nlevels,nchannels)       ! raw layer optical depth 

  Real(Kind=jprb),  Pointer :: debye_prof(:,:)        ! pointer on Debye profiles
  Integer(Kind=jpim) :: lev, chan, j, prof     ! loop variables

  ! cloud liquid water local variables
  Real(Kind=jprb) :: zf, zf_sq, z34_dif, z45_dif, z1_sq, z2_sq, z1_div, z2_div
  Real(Kind=jprb) :: z1_den, z2_den, zastar, z1_prod, z2_prod, z3_prod, z4_prod
  Real(Kind=jprb) :: zbstar, zbstar_sq, za2star, za2star_sq, zdiv, zgstar
  Real(Kind=jprb) :: z1f_sq_z1_sq, z2f_sq_z2_sq,t1,t2
  Integer(Kind=jpim) :: ii
  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  
  !- End of header --------------------------------------------------------
  IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP',0_jpim,ZHOOK_HANDLE)

  !-----------------------------------------
  !1. calculate layer gaseous optical depths
  !-----------------------------------------

  !--------------------------
  !1.1 start with mixed gases
  !--------------------------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
! unrolling reduces memory traffic and increases performance.

     if ( coef % nmixed == 10 ) then
        Do lev=1,nlevels
           t1= &
             & coef%mixedgas(lev,chan,1) * predictors % mixedgas(1,lev,prof) + &
             & coef%mixedgas(lev,chan,2) * predictors % mixedgas(2,lev,prof) + &
             & coef%mixedgas(lev,chan,3) * predictors % mixedgas(3,lev,prof) + &
             & coef%mixedgas(lev,chan,4) * predictors % mixedgas(4,lev,prof) + &
             & coef%mixedgas(lev,chan,5) * predictors % mixedgas(5,lev,prof)
           t2= &
             & coef%mixedgas(lev,chan,6) * predictors % mixedgas(6,lev,prof) + &
             & coef%mixedgas(lev,chan,7) * predictors % mixedgas(7,lev,prof) + &
             & coef%mixedgas(lev,chan,8) * predictors % mixedgas(8,lev,prof) + &
             & coef%mixedgas(lev,chan,9) * predictors % mixedgas(9,lev,prof) + &
             & coef%mixedgas(lev,chan,10) * predictors % mixedgas(10,lev,prof)
           opticaldepth(lev,j)= t1 + t2 
        End Do
     else
       opticaldepth(1:nlevels,j) = 0._JPRB
       Do ii=1,coef % nmixed
         Do lev=1,nlevels
          opticaldepth(lev,j)= opticaldepth(lev,j) + &
           & coef%mixedgas(lev,chan,ii) * predictors % mixedgas(ii,lev,prof)  
         End Do
       End Do
     endif
  End Do

  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
     if ( coef%nwater == 15 ) then
        Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors % watervapour(14,lev,prof) + &
             & coef%watervapour(lev,chan,15) * predictors % watervapour(15,lev,prof)
           opticaldepth(lev,j)= opticaldepth(lev,j) + t1 + t2
         End Do
      else
         Do ii=1,coef % nwater
           Do lev=1,nlevels
            opticaldepth(lev,j)= opticaldepth(lev,j) + &
             & coef%watervapour(lev,chan,ii) * predictors % watervapour(ii,lev,prof)  
           End Do
         End Do
      endif
 End Do

  !-------------
  !1.3 add ozone
  !-------------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
      if ( coef % nozone == 11 ) then
          Do lev=1,nlevels
           t1= &
           & coef%ozone(lev,chan,1) * predictors % ozone(1,lev,prof) + &
           & coef%ozone(lev,chan,2) * predictors % ozone(2,lev,prof) + &
           & coef%ozone(lev,chan,3) * predictors % ozone(3,lev,prof) + &
           & coef%ozone(lev,chan,4) * predictors % ozone(4,lev,prof) + &
           & coef%ozone(lev,chan,5) * predictors % ozone(5,lev,prof)
           t2= &
           & coef%ozone(lev,chan,6) * predictors % ozone(6,lev,prof) + &
           & coef%ozone(lev,chan,7) * predictors % ozone(7,lev,prof) + &
           & coef%ozone(lev,chan,8) * predictors % ozone(8,lev,prof) + &
           & coef%ozone(lev,chan,9) * predictors % ozone(9,lev,prof) + &
           & coef%ozone(lev,chan,10) * predictors % ozone(10,lev,prof) + &
           & coef%ozone(lev,chan,11) * predictors % ozone(11,lev,prof)
           opticaldepth(lev,j)= opticaldepth(lev,j) + t1 + t2
          End Do
      else
          Do ii=1,coef % nozone 
            Do lev=1,nlevels
             opticaldepth(lev,j)= opticaldepth(lev,j) + &
              & coef%ozone(lev,chan,ii) * predictors % ozone(ii,lev,prof)  
            End Do
          End Do
      endif
 End Do

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
     Do ii=1,coef % nwvcont
       Do lev=1,nlevels
        opticaldepth(lev,j)= opticaldepth(lev,j) + &
               & coef%wvcont(lev,chan,ii) * predictors % wvcont(ii,lev,prof)   
       End Do
     End Do
 End Do

  !-----------
  !1.5 add CO2
  !-----------
  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)

     Do ii=1,coef % nco2
       Do lev=1,nlevels
        opticaldepth(lev,j)= opticaldepth(lev,j) + &
              & coef%co2(lev,chan,ii) * predictors % co2(ii,lev,prof)   
       End Do
     End Do
 End Do

  !-----------
  !1.6 add N2O
  !-----------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
     Do ii=1,coef % nn2o
       Do lev=1,nlevels
         opticaldepth(lev,j)= opticaldepth(lev,j) + &
               & coef%n2o(lev,chan,ii) * predictors % n2o(ii,lev,prof) 
       End Do
     End Do
 End Do

  !-----------
  !1.7 add CO
  !-----------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
     Do ii=1,coef % nco
       Do lev=1,nlevels
        opticaldepth(lev,j)= opticaldepth(lev,j) + &
             & coef%co(lev,chan,ii) * predictors % co(ii,lev,prof) 
       End Do
     End Do
 End Do

  !-----------
  !1.8 add CH4
  !-----------

  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
     Do ii=1,coef % nch4
       Do lev=1,nlevels
        opticaldepth(lev,j)= opticaldepth(lev,j) + &
                 & coef%ch4(lev,chan,ii) * predictors % ch4(ii,lev,prof) 
       End Do
     End Do
 End Do

  !--------------------
  !1.9 add liquid water (MW only)
  !--------------------

     If ( coef % id_sensor == sensor_id_mw ) Then
  Do j = 1, nchannels
     chan = channels(j)
     prof = lprofiles(j)
        debye_prof => aux(prof) % debye_prof
        If( predictors % ncloud >= 1 ) Then
           Do lev = coef % mwcldtop, nlevels
              zf           = coef % frequency_ghz(chan)
              zf_sq        = zf*zf
              z1_sq        = debye_prof(1,lev) * debye_prof(1,lev)
              z2_sq        = debye_prof(2,lev) * debye_prof(2,lev)
              z34_dif      = debye_prof(3,lev) - debye_prof(4,lev)
              z45_dif      = debye_prof(4,lev) - debye_prof(5,lev)
              z1f_sq_z1_sq = zf_sq + z1_sq
              z2f_sq_z2_sq = zf_sq + z2_sq
              z1_div       = 1.0_JPRB / z1f_sq_z1_sq
              z2_div       = 1.0_JPRB / z2f_sq_z2_sq
              z1_den       = z34_dif * z1_div
              z2_den       = z45_dif * z2_div
              zastar       = debye_prof(3,lev) - zf_sq * (z1_den + z2_den)
              z1_prod      = z34_dif * debye_prof(1,lev)
              z2_prod      = z1_prod * z1_div
              z3_prod      = z45_dif * debye_prof(2,lev)
              z4_prod      = z3_prod * z2_div
              zbstar       = -zf * (z2_prod + z4_prod)
              zbstar_sq    = zbstar * zbstar
              za2star      = zastar + 2.0_JPRB
              za2star_sq   = za2star * za2star
              zdiv         = za2star_sq + zbstar_sq
              zgstar       = -3.0_JPRB * zbstar / zdiv
              opticaldepth(lev,j) = opticaldepth(lev,j) -&
                     & 1.5_JPRB * zf * zgstar * predictors % clw(lev,prof)  
           End Do
        Endif
 End Do
     End If


  !----------------------------------------
  !2. Assemble layer optical depths
  !----------------------------------------

  ! note that optical depth in the calculations is negative

  ! single layer
  ! store (neg) optical depth in reference array on COEF levs ...
  ! ... for TL, AD and K calculations
  opdp_ref(:,:) = opticaldepth(:,:)
  Where( opticaldepth(:,:) > 0.0_JPRB )
     opticaldepth = 0.0_JPRB
  End Where

  ! level to space
  opdp_path % atm_layer(1,:) = opticaldepth(1,:)
  Do lev = 2, nlevels
     opdp_path % atm_layer(lev,:) = opdp_path % atm_layer(lev-1,:) + opticaldepth(lev,:)
  End Do
  opdp_path%sun_layer = 0._jprb

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_opdep
