
Subroutine rttov_opdep_9_solar( &
      & nchannels,            &! in
      & nprofiles,            &! in
      & nlevels,              &! in
      & channels,             &! in
      & lprofiles,            &! in
      & profiles,             &! in    ! for solar switch 
      & predictors,           &! in
      & aux,                  &! in
      & coef,                 &! in
      & opdp_path,            &! inout
      & opdpsun_ref)           ! out
 !
  ! Description:
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  ! To interpolate optical depths on to levels of radiative transfer model
  ! (which, at present, entails only surface transmittance, as
  ! other optical depths are on *rt* levels) and to convert
  ! optical depths to transmittances.
  !
  ! Code based on OPDEP and RTTAU from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0    01/06/2005  Marco Matricardi (ECMWF):                        
  !            --       New routine based on rttov_transmit.F90.
  !                     Variable trace gases, CO2, N2O,CO and CH4
  !                     have been introduced for AIRS and IASI
  !  1.1    29/01/2007  Modified to remove polarisation (R Saunders)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

  ! Imported Parameters:
  Use rttov_const, Only: &
        & sensor_id_mw   ,&
        & max_optical_depth  
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :    &
        & rttov_coef         ,&
        & predictors_Type    ,&
        & opdp_path_Type  ,&
        & profile_aux        ,&
        & profile_Type  

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),     Intent(in)    :: nchannels                               ! Number of output radiances
  Integer(Kind=jpim),     Intent(in)    :: nprofiles                               ! Number of profiles
  Integer(Kind=jpim),     Intent(in)    :: nlevels                                 ! Number of pressure levels
  Integer(Kind=jpim),     Intent(in)    :: channels(nchannels)                     ! Channel indices
  Integer(Kind=jpim),     Intent(in)    :: lprofiles(nchannels)                    ! Profiles indices
  Type(predictors_Type),  Intent(in)    :: predictors                 ! Predictors
  Type(rttov_coef),       Intent(in)    :: coef                                    ! Coefficients
  Type(opdp_path_Type),   Intent(inout) :: opdp_path
  Type(profile_aux),      Intent(in)    :: aux( nprofiles )                        ! auxillary profiles informations
  Type(profile_Type),     Intent(in)    :: profiles(nprofiles)                     ! Atmospheric profiles
  Real(Kind=jprb),Intent(out)           :: opdpsun_ref(nlevels,nchannels)              ! layer optical depth 

!INTF

  !local variables:
  Real(Kind=jprb) :: opticaldepth(nlevels,nchannels)       ! raw layer optical depth 
  Real(Kind=jprb) :: chanx
  Integer(Kind=jpim) :: lev, chan,j, prof     ! loop variables

  ! cloud liquid water local variables
  Integer(Kind=jpim) :: ii,i
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  !-----------------------------------------
  !1. calculate layer gaseous optical depths
  !-----------------------------------------
  !--------------------------
  !1.1 start with mixed gases
  !--------------------------
IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_SOLAR',0_jpim,ZHOOK_HANDLE)

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)
     if(profiles(prof)%addsolar)then
       if(coef%ss_val_chn(chan)==1)then
         Do lev=1,nlevels
           opticaldepth(lev,j)=  &
           & coef%mixedgas(lev,chan,1) * predictors % mixedgas_sun(1,lev,prof)  
           Do ii=2,8
             opticaldepth(lev,j)= opticaldepth(lev,j) + &
             & coef%mixedgas(lev,chan,ii) * predictors % mixedgas_sun(ii,lev,prof)   
           End Do
             opticaldepth(lev,j)= opticaldepth(lev,j) + &
             & coef%mixedgas(lev,chan,9) * predictors % mixedgas_sun(9,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j) + &
             & coef%mixedgas(lev,chan,10) * predictors % mixedgas_sun(10,lev,prof) 
         End Do
       End if
     End if
  End Do
  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)

     if(profiles(prof)%addsolar)then
       if(coef%ss_val_chn(chan)==1)then

         if(chanx>=2000.and.chanx<=2295)then

           Do lev=1,nlevels
             Do ii=1,13
              opticaldepth(lev,j)= opticaldepth(lev,j)                              + &
              & coef%watervapour(lev,chan,ii) * predictors % watervapour_sun(ii,lev,prof) 
             End Do
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,14)*predictors % co2_sun(1,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,15)*predictors % co_sun(1,lev,prof) 
           End Do

         else if(chanx>=2295.25.and.chanx<=2359.75)then

           Do lev=1,nlevels
             Do ii=1,9
               opticaldepth(lev,j)= opticaldepth(lev,j)                             + &
               & coef%watervapour(lev,chan,ii) * predictors % watervapour_sun(ii,lev,prof) 
             End Do
              opticaldepth(lev,j)= opticaldepth(lev,j)                              + &
              & coef%watervapour(lev,chan,10) * predictors % watervapour_sun(14,lev,prof) 
              opticaldepth(lev,j)= opticaldepth(lev,j)                              + &
              & coef%watervapour(lev,chan,11) * predictors % watervapour_sun(11,lev,prof) 
              opticaldepth(lev,j)= opticaldepth(lev,j)                              + &
              & coef%watervapour(lev,chan,12) * predictors % watervapour_sun(12,lev,prof) 
              opticaldepth(lev,j)= opticaldepth(lev,j)                              + &
              & coef%watervapour(lev,chan,13) * predictors % watervapour_sun(13,lev,prof) 
           End Do

         else

           Do lev=1,nlevels
             Do ii=1,9
               opticaldepth(lev,j)= opticaldepth(lev,j)                             + &
               & coef%watervapour(lev,chan,ii) * predictors % watervapour_sun(ii,lev,prof) 
             End Do
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,10)*predictors % watervapour_sun(14,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,11)*predictors % watervapour_sun(11,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,12)*predictors % watervapour_sun(12,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,13)*predictors % watervapour_sun(15,lev,prof) 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,14)*predictors % ch4_sun(1,lev,prof)**1.25 
             opticaldepth(lev,j)= opticaldepth(lev,j)                               + &
             & coef%watervapour(lev,chan,15)*predictors % ch4_sun(1,lev,prof)**0.25* &
                                       & predictors % ch4_sun(3,lev,prof) 
           End Do
         endif
       endif
     endif
  End Do

  !-------------
  !1.3 add ozone
  !-------------

  If ( coef%nozone > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            Do lev=1,nlevels
              Do ii=1,13
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%ozone(lev,chan,ii) * predictors % ozone_sun(ii,lev,prof) 
              Enddo
            End Do
          End if
        End if
     End Do
  End If

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  If ( coef%nwvcont > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            Do lev=1,nlevels
              Do ii=1,coef % nwvcont
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                  & coef%wvcont(lev,chan,ii) * predictors % wvcont_sun(ii,lev,prof) 
              End Do
            End Do
          Endif
        Endif
     End Do
  End If

  !-----------
  !1.5 add CO2
  !-----------

  If ( coef%nco2 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx = coef%ff_cwn(chan)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            if(chanx>=2000.and.chanx<=2295)then
              Do lev=1,nlevels
                Do ii=1,14
                 opticaldepth(lev,j)= opticaldepth(lev,j) + &
                 & coef%co2(lev,chan,ii) * predictors % co2_sun(ii,lev,prof) 
                End Do
              End Do     
            else 
              Do lev=1,nlevels
                Do ii=1,7
                 opticaldepth(lev,j)= opticaldepth(lev,j) + &
                 & coef%co2(lev,chan,ii) * predictors % co2_sun(ii,lev,prof) 
                End Do
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,8) * predictors % co2_sun(9,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,9) * predictors % co2_sun(10,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,10) * predictors % co2_sun(11,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,11) * predictors % co2_sun(12,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,12) * predictors % co2_sun(13,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co2(lev,chan,13) * predictors % co2_sun(14,lev,prof) 
              End Do
            endif
         End if
       End if
     End Do
  End If

  !-----------
  !1.6 add N2O 
  !-----------

  If ( coef%nn2o > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            if(chanx>=2000.and.chanx<=2295)then
              Do lev=1,nlevels
                Do ii=1,10
                  opticaldepth(lev,j)= opticaldepth(lev,j)                           + &
                  & coef%n2o(lev,chan,ii) * predictors % n2o_sun(ii,lev,prof) 
                End Do
                opticaldepth(lev,j)= opticaldepth(lev,j)                           + &
                & coef%n2o(lev,chan,11) * predictors % co_sun(1,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j)                           + &
                & coef%n2o(lev,chan,12) * predictors % co_sun(1,lev,prof)**3       / &
                                       & predictors % co_sun(8,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j)                           + &
                & coef%n2o(lev,chan,13) * predictors % n2o_sun(11,lev,prof) 
                opticaldepth(lev,j)= opticaldepth(lev,j)                           + &
                & coef%n2o(lev,chan,14) * predictors % n2o_sun(12,lev,prof) 
              End Do
            else
              Do lev=1,nlevels
                Do ii=1,12
                  opticaldepth(lev,j)= opticaldepth(lev,j) + &
                  & coef%n2o(lev,chan,ii) * predictors % n2o_sun(ii,lev,prof) 
                End Do
              End Do
            endif
         Endif
       Endif
     End Do
  End If

  !-----------
  !1.7 add CO 
  !-----------

  If ( coef%nco > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            Do lev=1,nlevels
              Do ii=1,12
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%co(lev,chan,ii) * predictors % co_sun(ii,lev,prof) 
              End Do
            End Do
          End if
        End if
     End Do
  End If

  !-----------
  !1.8 add CH4
  !-----------

  If ( coef%nch4 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        if(profiles(prof)%addsolar)then
          if(coef%ss_val_chn(chan)==1)then
            Do lev=1,nlevels
              Do ii=1,coef % nch4
                opticaldepth(lev,j)= opticaldepth(lev,j) + &
                & coef%ch4(lev,chan,ii) * predictors % ch4_sun(ii,lev,prof) 
              End Do
            End Do
          End if
        End if
     End Do
  End If

  !----------------------------------------
  !2. Assemble layer optical depths
  !----------------------------------------

  ! note that optical depth in the calculations is negative
  ! gamma correction done in rttov_transmit_9_solar
  
  ! single layer
  ! put raw opticaldepth in reference array for TL, AD and K calculations
  ! then check if opticaldepth is sensible and constrain if necessary.
  do j=1, nchannels
    chan=channels(j)
    prof = lprofiles(j)
    if(profiles(prof)%addsolar)then
      if(coef%ss_val_chn(chan)==1)then
        do i=1,nlevels
          opdpsun_ref(i,j) = opticaldepth(i,j) 
          if( opticaldepth(i,j) > 0.0_JPRB )then
            opticaldepth(i,j) = 0.0_JPRB
          End if 
        enddo
      endif 
    endif
  enddo

  ! level to space
  opdp_path % sun_layer = 0.0_jprb
  do j=1, nchannels
    chan=channels(j)
    prof = lprofiles(j)
      if(profiles(prof)%addsolar)then
        if(coef%ss_val_chn(chan)==1)then
          opdp_path % sun_layer(1,j) = opticaldepth(1,j)
           Do lev = 2, nlevels
              opdp_path % sun_layer(lev,j) = opdp_path % sun_layer(lev-1,j) + opticaldepth(lev,j)
            End Do
        Endif
      Endif
  Enddo
 

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_SOLAR',1_jpim,ZHOOK_HANDLE)
End Subroutine rttov_opdep_9_solar
