Subroutine rttov_opdep_9_tl( &
      & nchannels,                   &! in
      & nprofiles,                   &! in
      & nlevels,                     &! in
      & channels,                    &! in
      & lprofiles,                   &! in
      & predictors,                  &! in
      & predictors_tl,               &! in
      & aux,                         &! in
      & aux_tl,                      &! in
      & coef,                        &! in
      & opdp_path,                   &! inout
      & opdp_path_tl,                &! inout
      & opdp_ref)                     ! inout
  !
  ! Description:
  ! Tangent linear of rttov_transmit
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  ! To interpolate optical depths on to levels of radiative transfer model
  ! (which, at present, entails only surface transmittance, as
  ! other optical depths are on *rt* levels) and to convert
  ! optical depths to transmittances.
  !
  ! Code based on OPDEPTL and RTTAUTL from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0    01/06/2005  Marco Matricardi (ECMWF):
  !            --       New routine based on rttov_transmit_tl.F90.
  !                     Variable trace gases, CO2, N2O,CO and CH4
  !                     have been introduced for AIRS and IASI
  !  1.1    29/01/2007  Removed polarisation R Saunders
  !  1.2    07/04/2008  Marco Matricardi (ECMWF):
  !                     modified the routine to be consistent
  !                     with the LBLRTM coefficients
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

  ! Imported Parameters:

  Use rttov_const, Only: &
        & sensor_id_mw   ,& 
        & min_tau        ,   &
        & min_od
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_types, Only : &
        & rttov_coef     ,&
        & predictors_Type,&
        & opdp_path_Type, &
        & profile_aux  

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)    :: nchannels
  Integer(Kind=jpim),    Intent(in)    :: nprofiles
  Integer(Kind=jpim),    Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),    Intent(in)    :: nlevels
  Integer(Kind=jpim),    Intent(in)    :: lprofiles(nchannels)
  Type(predictors_Type), Intent(in)    :: predictors
  Type(predictors_Type), Intent(in)    :: predictors_tl
  Type(rttov_coef),      Intent(in)    :: coef
  Type(profile_aux),     Intent(in)    :: aux( nprofiles )
  Type(profile_aux),     Intent(in)    :: aux_tl( nprofiles )
  Type(opdp_path_Type),  Intent(inout) :: opdp_path               ! optical depths
  Type(opdp_path_Type),  Intent(inout) :: opdp_path_tl            
  Real(Kind=jprb),       Intent(in)    :: opdp_ref(nlevels,nchannels)

!INTF

  !local variables:
  Real(Kind=jprb) :: opticaldepth_tl(nlevels,nchannels)
  Real(Kind=jprb) :: chanx

  Real(Kind=jprb),  Pointer :: debye_prof(:,:)
  Real(Kind=jprb),  Pointer :: debye_prof_tl(:,:)
  Integer(Kind=jpim) :: lev,chan,j
  Integer(Kind=jpim) :: prof

  ! cloud liquid water local variables
  Real(Kind=jprb) :: zf, zf_sq, z34_dif, z45_dif, z1_sq, z2_sq, z1_div, z2_div
  Real(Kind=jprb) :: z1_den, z2_den, zastar, z1_prod, z2_prod, z3_prod, z4_prod
  Real(Kind=jprb) :: zbstar, zbstar_sq, za2star, za2star_sq, zdiv, zgstar
  Real(Kind=jprb) :: z1f_sq_z1_sq, z2f_sq_z2_sq
  Real(Kind=jprb) :: z34_dif_tl, z45_dif_tl, z1_sq_tl, z2_sq_tl, z1_div_tl, z2_div_tl
  Real(Kind=jprb) :: z1_den_tl, z2_den_tl, zastar_tl, z1_prod_tl, z2_prod_tl, z3_prod_tl, z4_prod_tl
  Real(Kind=jprb) :: zbstar_tl, zbstar_sq_tl, za2star_tl, za2star_sq_tl, zdiv_tl, zgstar_tl
  Real(Kind=jprb) :: z1f_sq_z1_sq_tl, z2f_sq_z2_sq_tl
  Integer(Kind=jpim) :: ii
  Real(Kind=jprb) :: t1,t2
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!- End of header --------------------------------------------------------

  !-----------------------------------------
  !1. calculate layer gaseous optical depths
  !-----------------------------------------

  !--------------------------
  !1.1 start with mixed gases
  !--------------------------

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_TL',0_jpim,ZHOOK_HANDLE)

  Do j = 1,  nchannels
     chan = channels(j)
     prof = lprofiles(j)
     chanx= coef%ff_cwn(chan)

     if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

       Do lev=1,nlevels
           t1= &
             & coef%mixedgas(lev,chan,1) * predictors_tl % mixedgas(1,lev,prof) + &
             & coef%mixedgas(lev,chan,2) * predictors_tl % mixedgas(2,lev,prof) + &
             & coef%mixedgas(lev,chan,3) * predictors_tl % mixedgas(3,lev,prof) + &
             & coef%mixedgas(lev,chan,4) * predictors_tl % mixedgas(4,lev,prof) + &
             & coef%mixedgas(lev,chan,5) * predictors_tl % mixedgas(5,lev,prof)
           t2= &
             & coef%mixedgas(lev,chan,6) * predictors_tl % mixedgas(6,lev,prof) + &
             & coef%mixedgas(lev,chan,7) * predictors_tl % mixedgas(7,lev,prof) + &
             & coef%mixedgas(lev,chan,8) * predictors_tl % mixedgas(8,lev,prof) + &
             & coef%mixedgas(lev,chan,9) * predictors_tl % mixedgas(9,lev,prof) + &
             & coef%mixedgas(lev,chan,10)* predictors_tl % mixedgas(10,lev,prof)
           opticaldepth_tl(lev,j)= t1 + t2
       End Do

     else

       Do lev=1,nlevels
           t1= &
             & coef%mixedgas(lev,chan,1) * predictors_tl % mixedgas(1,lev,prof) + &
             & coef%mixedgas(lev,chan,2) * predictors_tl % mixedgas(2,lev,prof) + &
             & coef%mixedgas(lev,chan,3) * predictors_tl % mixedgas(3,lev,prof) + &
             & coef%mixedgas(lev,chan,4) * predictors_tl % mixedgas(4,lev,prof)
           t2= &
             & coef%mixedgas(lev,chan,5) * predictors_tl % mixedgas(5,lev,prof) + &
             & coef%mixedgas(lev,chan,6) * predictors_tl % mixedgas(6,lev,prof) + &
             & coef%mixedgas(lev,chan,7) * predictors_tl % mixedgas(7,lev,prof) + &
             & coef%mixedgas(lev,chan,8) * predictors_tl % mixedgas(8,lev,prof) 
           opticaldepth_tl(lev,j)= t1 + t2
       End Do
     endif
  End Do

  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1,  nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)

     if(chanx>=1095.25.and.chanx<=1750.00)then

       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % ch4(1,lev,prof)          + &
             & coef%watervapour(lev,chan,15) *  &
             & (predictors_tl % ch4(1,lev,prof) * predictors    % ch4(3,lev,prof)    + &
             &  predictors    % ch4(1,lev,prof) * predictors_tl % ch4(3,lev,prof))

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
       End Do

     else if(chanx>=1900.25.and.chanx<=1994.75)then

       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % co2(1,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

       End Do

     else if(chanx>=1995.00.and.chanx<=1999.75)then

       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % co2(1,lev,prof) + &
             & coef%watervapour(lev,chan,15) * predictors_tl % co(1,lev,prof) 

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
       End Do

     else if(chanx>=2000.and.chanx<=2249.75)then

       if(coef%ss_val_chn(chan)==1)then

         Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(14,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(15,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(16,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(17,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % co2(1,lev,prof) + &
             & coef%watervapour(lev,chan,15) * predictors_tl % co(1,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
         End Do

       else


         Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % co2(1,lev,prof) + &
             & coef%watervapour(lev,chan,15) * predictors_tl % co(1,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

         End Do

       End if

     else if(chanx>=2250.and.chanx<=2295)then

        Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) + &
             & coef%watervapour(lev,chan,14) * predictors_tl % co2(1,lev,prof) + &
             & coef%watervapour(lev,chan,15) * predictors_tl % co(1,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

       End Do

     else if(chanx>=2295.25.and.chanx<=2380)then

       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof) 

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2 
       End Do

     else if(chanx>=2380.25.and.chanx<=2760)then

       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(14,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(15,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(18,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(16,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(19,lev,prof) + &
             & coef%watervapour(lev,chan,14) * 1.25*predictors % ch4(1,lev,prof)**0.25 * &
             &                                  predictors_tl % ch4(1,lev,prof)         + &
             & coef%watervapour(lev,chan,15) *  &
             & (predictors_tl % ch4(1,lev,prof)*0.25*predictors % ch4(1,lev,prof)**(-0.75)* &
             &                                                  predictors % ch4(3,lev,prof) + &
             &  predictors % ch4(1,lev,prof)**0.25*predictors_tl % ch4(3,lev,prof))

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

       enddo

     else
       Do lev=1,nlevels
           t1= &
             & coef%watervapour(lev,chan,1) * predictors_tl % watervapour(1,lev,prof) + &
             & coef%watervapour(lev,chan,2) * predictors_tl % watervapour(2,lev,prof) + &
             & coef%watervapour(lev,chan,3) * predictors_tl % watervapour(3,lev,prof) + &
             & coef%watervapour(lev,chan,4) * predictors_tl % watervapour(4,lev,prof) + &
             & coef%watervapour(lev,chan,5) * predictors_tl % watervapour(5,lev,prof) + &
             & coef%watervapour(lev,chan,6) * predictors_tl % watervapour(6,lev,prof) + &
             & coef%watervapour(lev,chan,7) * predictors_tl % watervapour(7,lev,prof)
           t2= &
             & coef%watervapour(lev,chan,8) * predictors_tl % watervapour(8,lev,prof) + &
             & coef%watervapour(lev,chan,9) * predictors_tl % watervapour(9,lev,prof) + &
             & coef%watervapour(lev,chan,10) * predictors_tl % watervapour(10,lev,prof) + &
             & coef%watervapour(lev,chan,11) * predictors_tl % watervapour(11,lev,prof) + &
             & coef%watervapour(lev,chan,12) * predictors_tl % watervapour(12,lev,prof) + &
             & coef%watervapour(lev,chan,13) * predictors_tl % watervapour(13,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

       End Do
     endif
  End Do

  !-------------
  !1.3 add ozone
  !-------------

  If ( coef%nozone > 0 ) Then
     Do j = 1,  nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

          Do lev=1,nlevels
           t1= &
           & coef%ozone(lev,chan,1) * predictors_tl % ozone(1,lev,prof) + &
           & coef%ozone(lev,chan,2) * predictors_tl % ozone(2,lev,prof) + &
           & coef%ozone(lev,chan,3) * predictors_tl % ozone(12,lev,prof) + &
           & coef%ozone(lev,chan,4) * predictors_tl % ozone(4,lev,prof) + &
           & coef%ozone(lev,chan,5) * predictors_tl % ozone(5,lev,prof) + &
           & coef%ozone(lev,chan,6) * predictors_tl % ozone(6,lev,prof)
           t2= &
           & coef%ozone(lev,chan,7) * predictors_tl % ozone(7,lev,prof) + &
           & coef%ozone(lev,chan,8) * predictors_tl % ozone(13,lev,prof) + &
           & coef%ozone(lev,chan,9) * predictors_tl % ozone(9,lev,prof) + &
           & coef%ozone(lev,chan,10) * predictors_tl % ozone(10,lev,prof) + &
           & coef%ozone(lev,chan,11) * predictors_tl % ozone(11,lev,prof) + &
           & coef%ozone(lev,chan,12) * predictors_tl % ozone(14,lev,prof) +&
           & coef%ozone(lev,chan,13) * predictors_tl % ozone(15,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else

          Do lev=1,nlevels
           t1= &
           & coef%ozone(lev,chan,1) * predictors_tl % ozone(1,lev,prof) + &
           & coef%ozone(lev,chan,2) * predictors_tl % ozone(2,lev,prof) + &
           & coef%ozone(lev,chan,3) * predictors_tl % ozone(3,lev,prof) + &
           & coef%ozone(lev,chan,4) * predictors_tl % ozone(4,lev,prof) + &
           & coef%ozone(lev,chan,5) * predictors_tl % ozone(5,lev,prof)
           t2= &
           & coef%ozone(lev,chan,6) * predictors_tl % ozone(6,lev,prof) + &
           & coef%ozone(lev,chan,7) * predictors_tl % ozone(7,lev,prof) + &
           & coef%ozone(lev,chan,8) * predictors_tl % ozone(8,lev,prof) + &
           & coef%ozone(lev,chan,9) * predictors_tl % ozone(9,lev,prof) + &
           & coef%ozone(lev,chan,10) * predictors_tl % ozone(10,lev,prof) + &
           & coef%ozone(lev,chan,11) * predictors_tl % ozone(11,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        endif
     End Do
  End If

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  If ( coef%nwvcont > 0 ) Then
    If ( coef%nwvcont == 4) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=1,nlevels
         t1= &
           & coef%wvcont(lev,chan,1) * predictors_tl % wvcont(1,lev,prof) + &
           & coef%wvcont(lev,chan,2) * predictors_tl % wvcont(2,lev,prof)
         t2= &
           & coef%wvcont(lev,chan,3) * predictors_tl % wvcont(3,lev,prof) + &
           & coef%wvcont(lev,chan,4) * predictors_tl % wvcont(4,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
        End Do
     End Do
    Else
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=1,nlevels
          Do ii=1,coef % nwvcont
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + &
                  & coef%wvcont(lev,chan,ii) * predictors_tl % wvcont(ii,lev,prof)
          End Do
        End Do
     End Do
    End If
  End If

  !-----------
  !1.5 add CO2
  !-----------

  If ( coef%nco2 > 0 ) Then
     Do j = 1,  nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if(chanx>=2000.and.chanx<=2249.75)then

          Do lev=1,nlevels
            t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * predictors_tl % co2(3,lev,prof) + &
              & coef%co2(lev,chan,4) * predictors_tl % co2(4,lev,prof) + &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof) + &
              & coef%co2(lev,chan,6) * predictors_tl % co2(7,lev,prof) + &
              & coef%co2(lev,chan,7) * predictors_tl % co2(8,lev,prof)
            t2= &
              & coef%co2(lev,chan,8) * predictors_tl % co(1,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(10,lev,prof) + &
              & coef%co2(lev,chan,10) * predictors_tl % co2(11,lev,prof) + &
              & coef%co2(lev,chan,11) * predictors_tl % co2(12,lev,prof) + &
              & coef%co2(lev,chan,12) * predictors_tl % co2(13,lev,prof) + &
              & coef%co2(lev,chan,13) * predictors_tl % co2(14,lev,prof) + &
              & coef%co2(lev,chan,14) * predictors_tl % co2(15,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=2250.and.chanx<=2295.or.chanx>=1995.and.chanx<=1999.75)then

          Do lev=1,nlevels
           t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * predictors_tl % co2(3,lev,prof) + &
              & coef%co2(lev,chan,4) * predictors_tl % co2(4,lev,prof) + &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof)
            t2= &
              & coef%co2(lev,chan,6) * predictors_tl % co2(6,lev,prof) + &
              & coef%co2(lev,chan,7) * predictors_tl % co2(7,lev,prof) + &
              & coef%co2(lev,chan,8) * predictors_tl % co2(8,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(9,lev,prof) + &
              & coef%co2(lev,chan,10) * predictors_tl % co(1,lev,prof) + &
              & coef%co2(lev,chan,11) * predictors_tl % co2(10,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=2295.25.and.chanx<=2380)then

          Do lev=1,nlevels
           t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * predictors_tl % co2(3,lev,prof) + &
              & coef%co2(lev,chan,4) * predictors_tl % co2(4,lev,prof) + &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof)
            t2= &
              & coef%co2(lev,chan,6) * predictors_tl % co2(6,lev,prof) + &
              & coef%co2(lev,chan,7) * predictors_tl % co2(7,lev,prof) + &
              & coef%co2(lev,chan,8) * predictors_tl % co2(8,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(9,lev,prof) + &
              & coef%co2(lev,chan,10) * predictors_tl % co2(10,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=1,nlevels
           t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * predictors_tl % co2(3,lev,prof) + &
              & coef%co2(lev,chan,4) * predictors_tl % co2(4,lev,prof) + &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof) + &
              & coef%co2(lev,chan,6) * predictors_tl % co2(7,lev,prof)
           t2= &
              & coef%co2(lev,chan,7) * predictors_tl % co2(8,lev,prof) + &
              & coef%co2(lev,chan,8) * predictors_tl % co2(10,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(11,lev,prof) + &
              & coef%co2(lev,chan,10) * predictors_tl % co2(12,lev,prof) + &
              & coef%co2(lev,chan,11) * predictors_tl % co2(13,lev,prof) + &
              & coef%co2(lev,chan,12) * predictors_tl % co2(14,lev,prof) + &
              & coef%co2(lev,chan,13) * predictors_tl % co2(15,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=690.5.and.chanx<=1100)then

          Do lev=1,nlevels
           t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * (predictors_tl % co2(5,lev,prof)* &
                                       predictors % co2(6,lev,prof)**2 + &
                                       predictors_tl % co2(6,lev,prof)*2*&
                                       predictors % co2(6,lev,prof)     *&
                                       predictors % co2(5,lev,prof))    +&
              & coef%co2(lev,chan,4) * 2 * predictors % co2(3,lev,prof)* &
                                       predictors_tl % co2(3,lev,prof) 
           t2= &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof) + &
              & coef%co2(lev,chan,6) * 2 * predictors % co2(6,lev,prof)* &
                                       predictors_tl % co2(6,lev,prof) + &
              & coef%co2(lev,chan,7) * 2 * predictors % co2(7,lev,prof)* &
                                       predictors_tl % co2(7,lev,prof) + &
              & coef%co2(lev,chan,8) * predictors_tl % co2(8,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(9,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
         End Do


        else

          Do lev=1,nlevels
           t1= &
              & coef%co2(lev,chan,1) * predictors_tl % co2(1,lev,prof) + &
              & coef%co2(lev,chan,2) * predictors_tl % co2(2,lev,prof) + &
              & coef%co2(lev,chan,3) * predictors_tl % co2(3,lev,prof) + &
              & coef%co2(lev,chan,4) * predictors_tl % co2(4,lev,prof)
           t2= &
              & coef%co2(lev,chan,5) * predictors_tl % co2(5,lev,prof) + &
              & coef%co2(lev,chan,6) * predictors_tl % co2(6,lev,prof) + &
              & coef%co2(lev,chan,7) * predictors_tl % co2(7,lev,prof) + &
              & coef%co2(lev,chan,8) * predictors_tl % co2(8,lev,prof) + &
              & coef%co2(lev,chan,9) * predictors_tl % co2(9,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        endif
     End Do
  End If

  !-----------
  !1.6 add N2O
  !-----------

  If ( coef%nn2o > 0 ) Then
     Do j = 1,  nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if(chanx>=1050.and.chanx<=1350)then

          Do lev=1,nlevels
           t1= &
              & coef%n2o(lev,chan,1) * predictors_tl % n2o(1,lev,prof) + &
              & coef%n2o(lev,chan,2) * predictors_tl % n2o(2,lev,prof) + &
              & coef%n2o(lev,chan,3) * predictors_tl % n2o(3,lev,prof) + &
              & coef%n2o(lev,chan,4) * predictors_tl % n2o(4,lev,prof) + &
              & coef%n2o(lev,chan,5) * predictors_tl % n2o(5,lev,prof) + &
              & coef%n2o(lev,chan,6) * predictors_tl % n2o(6,lev,prof)
           t2= &
              & coef%n2o(lev,chan,7) * predictors_tl % n2o(7,lev,prof) + &
              & coef%n2o(lev,chan,8) * predictors_tl % n2o(8,lev,prof) + &
              & coef%n2o(lev,chan,9) * predictors_tl % n2o(9,lev,prof) + &
              & coef%n2o(lev,chan,10) * predictors_tl % n2o(10,lev,prof) + &
              & coef%n2o(lev,chan,11) * predictors_tl % ch4(1,lev,prof) + &
              & coef%n2o(lev,chan,12) * predictors_tl % ch4(10,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if((chanx>=1995.and.chanx<=1999.75).or.                              &
                & (chanx>=2250.and.chanx<=2295))then 

          Do lev=1,nlevels
           t1= &
              & coef%n2o(lev,chan,1) * predictors_tl % n2o(1,lev,prof) + &
              & coef%n2o(lev,chan,2) * predictors_tl % n2o(2,lev,prof) + &
              & coef%n2o(lev,chan,3) * predictors_tl % n2o(3,lev,prof) + &
              & coef%n2o(lev,chan,4) * predictors_tl % n2o(4,lev,prof) + &
              & coef%n2o(lev,chan,5) * predictors_tl % n2o(5,lev,prof) + &
              & coef%n2o(lev,chan,6) * predictors_tl % n2o(6,lev,prof)
           t2= &
              & coef%n2o(lev,chan,7) * predictors_tl % n2o(7,lev,prof) + &
              & coef%n2o(lev,chan,8) * predictors_tl % n2o(8,lev,prof) + &
              & coef%n2o(lev,chan,9) * predictors_tl % n2o(9,lev,prof) + &
              & coef%n2o(lev,chan,10) * predictors_tl % n2o(10,lev,prof) + &
              & coef%n2o(lev,chan,11) * predictors_tl % co(1,lev,prof)   + &
              & coef%n2o(lev,chan,12) *(                                 &
                 & 3*predictors % co(1,lev,prof)**2*predictors_tl % co(1,lev,prof)/ &
                 & predictors % co(8,lev,prof)                           - &
                 & predictors % co(1,lev,prof)**3*predictors_tl % co(8,lev,prof)  / &
                 & predictors % co(8,lev,prof)**2 )
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=2000.and.chanx<=2249.75)then
          Do lev=1,nlevels
           t1= &
              & coef%n2o(lev,chan,1) * predictors_tl % n2o(1,lev,prof) + &
              & coef%n2o(lev,chan,2) * predictors_tl % n2o(2,lev,prof) + &
              & coef%n2o(lev,chan,3) * predictors_tl % n2o(3,lev,prof) + &
              & coef%n2o(lev,chan,4) * predictors_tl % n2o(4,lev,prof) + &
              & coef%n2o(lev,chan,5) * predictors_tl % n2o(5,lev,prof) + &
              & coef%n2o(lev,chan,6) * predictors_tl % n2o(6,lev,prof) + &
              & coef%n2o(lev,chan,7) * predictors_tl % n2o(7,lev,prof)
           t2= &
              & coef%n2o(lev,chan,8) * predictors_tl % n2o(8,lev,prof) + &
              & coef%n2o(lev,chan,9) * predictors_tl % n2o(11,lev,prof) + &
              & coef%n2o(lev,chan,10) * predictors_tl % n2o(10,lev,prof) + &
              & coef%n2o(lev,chan,11) * predictors_tl % co(1,lev,prof) + &
              & coef%n2o(lev,chan,12) *(                                    &
                 & 3*predictors % co(1,lev,prof)**2*predictors_tl % co(1,lev,prof)/ &
                 & predictors % co(8,lev,prof)                           - &
                 & predictors % co(1,lev,prof)**3*predictors_tl % co(8,lev,prof)  / &
                 & predictors % co(8,lev,prof)**2 )                      + &
              & coef%n2o(lev,chan,13) * predictors_tl % n2o(12,lev,prof) + &
              & coef%n2o(lev,chan,14) * predictors_tl % n2o(13,lev,prof) 
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=1,nlevels
           t1= &
              & coef%n2o(lev,chan,1) * predictors_tl % n2o(1,lev,prof) + &
              & coef%n2o(lev,chan,2) * predictors_tl % n2o(2,lev,prof) + &
              & coef%n2o(lev,chan,3) * predictors_tl % n2o(3,lev,prof) + &
              & coef%n2o(lev,chan,4) * predictors_tl % n2o(4,lev,prof) + &
              & coef%n2o(lev,chan,5) * predictors_tl % n2o(5,lev,prof) + &
              & coef%n2o(lev,chan,6) * predictors_tl % n2o(6,lev,prof) + &
              & coef%n2o(lev,chan,7) * predictors_tl % n2o(7,lev,prof)
           t2= &
              & coef%n2o(lev,chan,8) * predictors_tl % n2o(8,lev,prof) + &
              & coef%n2o(lev,chan,9) * predictors_tl % n2o(11,lev,prof) + &
              & coef%n2o(lev,chan,10) * predictors_tl % n2o(10,lev,prof) + &
              & coef%n2o(lev,chan,11) * predictors_tl % n2o(12,lev,prof) + &
              & coef%n2o(lev,chan,12) * predictors_tl % n2o(13,lev,prof)
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        else

          Do lev=1,nlevels
           t1= &
              & coef%n2o(lev,chan,1) * predictors_tl % n2o(1,lev,prof) + &
              & coef%n2o(lev,chan,2) * predictors_tl % n2o(2,lev,prof) + &
              & coef%n2o(lev,chan,3) * predictors_tl % n2o(3,lev,prof) + &
              & coef%n2o(lev,chan,4) * predictors_tl % n2o(4,lev,prof) + &
              & coef%n2o(lev,chan,5) * predictors_tl % n2o(5,lev,prof)
           t2= &
              & coef%n2o(lev,chan,6) * predictors_tl % n2o(6,lev,prof) + &
              & coef%n2o(lev,chan,7) * predictors_tl % n2o(7,lev,prof) + &
              & coef%n2o(lev,chan,8) * predictors_tl % n2o(8,lev,prof) + &
              & coef%n2o(lev,chan,9) * predictors_tl % n2o(9,lev,prof) + &
              & coef%n2o(lev,chan,10) * predictors_tl % n2o(10,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2
          End Do

        endif
     End Do
  End If

  !-----------
  !1.7 add CO
  !-----------

  If ( coef%nco > 0 ) Then
     Do j = 1,  nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

          Do lev=1,nlevels
           t1= &
              & coef%co(lev,chan,1) * predictors_tl % co(1,lev,prof) + &
              & coef%co(lev,chan,2) * predictors_tl % co(2,lev,prof) + &
              & coef%co(lev,chan,3) * predictors_tl % co(3,lev,prof) + &
              & coef%co(lev,chan,4) * predictors_tl % co(4,lev,prof) + &
              & coef%co(lev,chan,5) * predictors_tl % co(5,lev,prof) + &
              & coef%co(lev,chan,6) * predictors_tl % co(6,lev,prof)
           t2= &
              & coef%co(lev,chan,7) * predictors_tl % co(7,lev,prof) + &
              & coef%co(lev,chan,8) * predictors_tl % co(8,lev,prof) + &
              & coef%co(lev,chan,9) * predictors_tl % co(9,lev,prof) + &
              & coef%co(lev,chan,10) * predictors_tl % co(10,lev,prof) + &
              & coef%co(lev,chan,11) * predictors_tl % co(12,lev,prof) + &
              & coef%co(lev,chan,12) * predictors_tl % co(13,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

          End do

		  else

        Do lev=1,nlevels
           t1= &
              & coef%co(lev,chan,1) * predictors_tl % co(1,lev,prof) + &
              & coef%co(lev,chan,2) * predictors_tl % co(2,lev,prof) + &
              & coef%co(lev,chan,3) * predictors_tl % co(3,lev,prof) + &
              & coef%co(lev,chan,4) * predictors_tl % co(4,lev,prof) + &
              & coef%co(lev,chan,5) * predictors_tl % co(5,lev,prof) + &
              & coef%co(lev,chan,6) * predictors_tl % co(6,lev,prof)
           t2= &
              & coef%co(lev,chan,7) * predictors_tl % co(7,lev,prof) + &
              & coef%co(lev,chan,8) * predictors_tl % co(8,lev,prof) + &
              & coef%co(lev,chan,9) * predictors_tl % co(9,lev,prof) + &
              & coef%co(lev,chan,10) * predictors_tl % co(10,lev,prof) + &
              & coef%co(lev,chan,11) * predictors_tl % co(11,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

        End Do

        Endif
     End Do
  End If

  !-----------
  !1.8 add CH4
  !-----------

  If ( coef%nch4 > 0 ) Then
    If ( coef%nch4 == 11 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=1,nlevels
           t1= &
             & coef%ch4(lev,chan,1) * predictors_tl % ch4(1,lev,prof) + &
             & coef%ch4(lev,chan,2) * predictors_tl % ch4(2,lev,prof) + &
             & coef%ch4(lev,chan,3) * predictors_tl % ch4(3,lev,prof) + &
             & coef%ch4(lev,chan,4) * predictors_tl % ch4(4,lev,prof) + &
             & coef%ch4(lev,chan,5) * predictors_tl % ch4(5,lev,prof)
           t2= &
             & coef%ch4(lev,chan,6) * predictors_tl % ch4(6,lev,prof) + &
             & coef%ch4(lev,chan,7) * predictors_tl % ch4(7,lev,prof) + &
             & coef%ch4(lev,chan,8) * predictors_tl % ch4(8,lev,prof) + &
             & coef%ch4(lev,chan,9) * predictors_tl % ch4(9,lev,prof) + &
             & coef%ch4(lev,chan,10) * predictors_tl % ch4(10,lev,prof) + &
             & coef%ch4(lev,chan,11) * predictors_tl % ch4(11,lev,prof)

           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + t1 + t2

        End Do
     End Do
    Else
     Do j = 1,  nchannels
        chan = channels(j)
        prof = lprofiles(j)
        Do lev=1,nlevels
          Do ii=1,coef % nch4
           opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) + &
                 & coef%ch4(lev,chan,ii) * predictors_tl % ch4(ii,lev,prof) 
          End Do
        End Do
     End Do
    End If
  End If
  !--------------------
  !1.9 add liquid water (MW only)
  !--------------------
  If ( coef % id_sensor == sensor_id_mw ) Then
     Do j = 1,  nchannels
        chan = channels(j)
        prof = lprofiles(j)
        debye_prof    => aux(prof) % debye_prof
        debye_prof_tl => aux_tl(prof) % debye_prof
        If( predictors % ncloud >= 1 ) Then
           Do lev = coef % mwcldtop, nlevels

              ! Repeat direct code
              zf           = coef % frequency_ghz(chan)
              zf_sq        = zf*zf
              z1_sq        = debye_prof(1,lev) * debye_prof(1,lev)
              z2_sq        = debye_prof(2,lev) * debye_prof(2,lev)
              z34_dif      = debye_prof(3,lev) - debye_prof(4,lev)
              z45_dif      = debye_prof(4,lev) - debye_prof(5,lev)
              z1f_sq_z1_sq = zf_sq + z1_sq
              z2f_sq_z2_sq = zf_sq + z2_sq
              z1_div       = 1.0_JPRB / z1f_sq_z1_sq
              z2_div       = 1.0_JPRB / z2f_sq_z2_sq
              z1_den       = z34_dif * z1_div
              z2_den       = z45_dif * z2_div
              zastar       = debye_prof(3,lev) - zf_sq * (z1_den + z2_den)
              z1_prod      = z34_dif * debye_prof(1,lev)
              z2_prod      = z1_prod * z1_div
              z3_prod      = z45_dif * debye_prof(2,lev)
              z4_prod      = z3_prod * z2_div
              zbstar       = -zf * (z2_prod + z4_prod)
              zbstar_sq    = zbstar * zbstar
              za2star      = zastar + 2.0_JPRB
              za2star_sq   = za2star * za2star
              zdiv         = za2star_sq + zbstar_sq
              zgstar       = -3.0_JPRB * zbstar / zdiv


              ! Now compute tangent-linear code
              !zf_tl    = 0
              !zf_sq_tl = 0
              z1_sq_tl       = 2.0_JPRB * debye_prof(1,lev) * debye_prof_tl(1,lev)
              z2_sq_tl       = 2.0_JPRB * debye_prof(2,lev) * debye_prof_tl(2,lev)
              z34_dif_tl     = debye_prof_tl(3,lev) - debye_prof_tl(4,lev)
              z45_dif_tl     = debye_prof_tl(4,lev) - debye_prof_tl(5,lev)
              z1f_sq_z1_sq_tl  = z1_sq_tl
              z2f_sq_z2_sq_tl  = z2_sq_tl
              z1_div_tl      = -z1f_sq_z1_sq_tl / (z1f_sq_z1_sq * z1f_sq_z1_sq)
              z2_div_tl      = -z2f_sq_z2_sq_tl / (z2f_sq_z2_sq * z2f_sq_z2_sq)
              z1_den_tl      = z34_dif * z1_div_tl + z34_dif_tl * z1_div
              z2_den_tl      = z45_dif * z2_div_tl + z45_dif_tl * z2_div
              zastar_tl      = debye_prof_tl(3,lev) - zf_sq * (z1_den_tl + z2_den_tl)

              z1_prod_tl     = z34_dif_tl * debye_prof(1,lev)&
                     & + z34_dif*debye_prof_tl(1,lev)  
              z2_prod_tl     = z1_prod_tl * z1_div + z1_prod * z1_div_tl
              z3_prod_tl     = z45_dif_tl * debye_prof(2,lev)&
                     & + z45_dif * debye_prof_tl(2,lev)  
              z4_prod_tl     = z3_prod_tl * z2_div + z3_prod * z2_div_tl
              zbstar_tl      = -zf * (z2_prod_tl + z4_prod_tl)
              zbstar_sq_tl   = 2.0_JPRB * zbstar * zbstar_tl
              za2star_tl     = zastar_tl
              za2star_sq_tl  = 2.0_JPRB * za2star * za2star_tl
              zdiv_tl        = za2star_sq_tl + zbstar_sq_tl
              zgstar_tl      = -3.0_JPRB*(zbstar_tl * zdiv - zbstar * zdiv_tl) / (zdiv * zdiv)

              opticaldepth_tl(lev,j)= opticaldepth_tl(lev,j) &
                     & - 1.5_JPRB * zf * ( zgstar_tl * predictors % clw(lev,prof) + &
                                    & zgstar    * predictors_tl % clw(lev,prof) )  

           End Do
        Endif
     End Do
  End If


  !----------------------------------------
  !2. Assemble layer optical depths
  !----------------------------------------
  ! note that optical depth in the calculations is negative

  ! single layer
  Where( opdp_ref(:,:) > 0.0_JPRB )
     opticaldepth_tl = 0.0_JPRB
  End Where

  opdp_path_tl % atm_layer(1,:) = opticaldepth_tl(1,:)
  Do lev = 2, nlevels
     opdp_path_tl % atm_layer(lev,:) = opdp_path_tl % atm_layer(lev-1,:) + opticaldepth_tl(lev,:)
  End Do

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_TL',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_opdep_9_tl
