  !
  ! Description:
  !    Parallel ( openmp ) version of RTTOV_DIRECT, _TL, _AD and _K.   
  !    This file contains the definition of RTTOV_PARALLEL_DIRECT, _TL, _AD and _K. 
  !    RTTOV_PARALLEL_DIRECT is activated when _RTTOV_PARALLEL_DIRECT is defined, etc...
  !    The interfaces of these parallel routines are identical to the original ones,
  !    plus the optional parameter nthreads.
  !
  ! Copyright:
  !
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          09/2007    Creation P. Marguinaud & P. Brunel
  !          10/2008    Fix for po sounders P. Marguinaud
  !

#ifdef _RTTOV_PARALLEL_INTERFACE
Interface
#endif

!Subroutine &
#if defined( _RTTOV_PARALLEL_DIRECT )
Subroutine  rttov_parallel_direct  &
#elif defined( _RTTOV_PARALLEL_TL )
Subroutine  rttov_parallel_tl      &
#elif defined( _RTTOV_PARALLEL_AD )
Subroutine  rttov_parallel_ad      &
#elif defined( _RTTOV_PARALLEL_K )
Subroutine  rttov_parallel_k       &
#else


#endif
      & ( errorstatus      &! out
      & , nprofiles        &! in
      & , nchannels        &! in
      & , channels         &! in
      & , lprofiles        &! in
      & , addinterp        &! in
      & , profiles         &! in 
#ifdef _RTTOV_PARALLEL_TL
      & , profiles_tl      &! in 
#endif      
#ifdef _RTTOV_PARALLEL_AD
      & , profiles_ad      &! in 
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , profiles_k       &! in 
#endif      
      & , coef             &! in
      & , coef_scatt_ir    &
      & , optp             &
      & , addsun           &! in
      & , addaerosl        &! in
      & , addclouds        &! in
#ifdef _RTTOV_PARALLEL_AD
      & , switchrad        &
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , switchrad        &
#endif      
      & , calcemis         &! in
      & , emissivity       &! inout
#ifdef _RTTOV_PARALLEL_TL
      & , emissivity_tl    &! inout
#endif      
#ifdef _RTTOV_PARALLEL_AD
      & , emissivity_ad    &! inout
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , emissivity_k     &! inout
#endif      
      & , transmission     &! inout
#ifdef _RTTOV_PARALLEL_TL
      & , transmission_tl  &
#endif      
#ifdef _RTTOV_PARALLEL_AD
      & , transmission_ad  &
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , transmission_k   &
#endif      
      & , radiancedata     &
#ifdef _RTTOV_PARALLEL_TL
      & , radiancedata_tl  &
#endif      
#ifdef _RTTOV_PARALLEL_AD
      & , radiancedata_ad  &
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , radiancedata_k   &
#endif      
      & , temp             &
#ifdef _RTTOV_PARALLEL_TL
      & , temp_tl          &
#endif      
#ifdef _RTTOV_PARALLEL_AD
      & , temp_ad          &
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , temp_k           &
#endif  
      &,  nthreads         &    
& )   


  Use rttov_const, Only :  &
       & errorstatus_success,&
       & errorstatus_warning,&
       & errorstatus_fatal,  &  
       & errorstatus_info,   &
       & sensor_id_po
  
! Imported Type Definitions:
  Use rttov_types, Only :            &
        & rttov_coef,                &
        & rttov_coef_scatt_ir,       &
        & rttov_optpar_ir,           &
        & profile_Type,              &
        & transmission_type,         &
        & radiance_Type,             &
        & rttov_temp
                      
  
  Use parkind1, Only : jpim,jprb
  Implicit None

#ifndef _RTTOV_PARALLEL_INTERFACE  

#ifdef _RTTOV_PARALLEL_DIRECT
include "rttov_direct.interface"
#endif
#ifdef _RTTOV_PARALLEL_TL
include "rttov_tl.interface"
#endif
#ifdef _RTTOV_PARALLEL_AD
include "rttov_ad.interface"
#endif
#ifdef _RTTOV_PARALLEL_K
include "rttov_k.interface"
#endif

include "rttov_alloc_prof.interface"
include "rttov_alloc_rad.interface"
include "rttov_errorreport.interface"

#endif

!subroutine arguments:
  Integer(Kind=jpim),            Intent(in)    :: nprofiles                    ! Number of profiles
  Integer(Kind=jpim),            Intent(out)   :: errorstatus( nprofiles ) ! return flag
  Integer(Kind=jpim),            Intent(in)    :: nchannels                    ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),            Intent(in)    :: channels( nchannels )        ! Channel indices
  Integer(Kind=jpim),            Intent(in)    :: lprofiles( nchannels )
  Logical,                       Intent(in)    :: addinterp  ! switch for User-Coef-User interpolations
  Type(profile_Type),            Intent(in)    :: profiles( nprofiles ) ! Atmospheric profiles
  Type(rttov_coef),              Intent(in)    :: coef                  ! RT Coefficients
  Type(rttov_coef_scatt_ir),     Intent(in)    :: coef_scatt_ir
  Type(rttov_optpar_ir),         Intent(in)    :: optp
  Logical,                       Intent(in)    :: addsun
  Logical,                       Intent(in)    :: addaerosl
  Logical,                       Intent(in)    :: addclouds  ! switch for cloud computations
#ifdef _RTTOV_PARALLEL_AD
  Logical,                       Intent(in)    :: switchrad  ! true if input is BT
#endif
#ifdef _RTTOV_PARALLEL_K
  Logical,                       Intent(in)    :: switchrad  ! true if input is BT
#endif
  Logical,                       Intent(in)    :: calcemis( nchannels )   ! switch for emmissivity calc.
  Real(Kind=jprb),               Intent(inout) :: emissivity( nchannels ) ! surface emmissivity
  Type(transmission_type),       Intent(inout) :: transmission   ! transmittances and layer optical depths
  Type(radiance_Type),           Intent(inout) :: radiancedata   ! radiances (mw/cm-1/ster/sq.m) and degK

#ifdef _RTTOV_PARALLEL_TL
  Type(profile_Type),            Intent(inout) :: profiles_tl( nprofiles ) 
  Real(Kind=jprb),               Intent(inout) :: emissivity_tl( nchannels ) 
  Type(transmission_type),       Intent(inout) :: transmission_tl
  Type(radiance_Type),           Intent(inout) :: radiancedata_tl 
#endif

#ifdef _RTTOV_PARALLEL_AD
  Type(profile_Type),            Intent(inout) :: profiles_ad( nprofiles ) 
  Real(Kind=jprb),               Intent(inout) :: emissivity_ad( nchannels ) 
  Type(transmission_type),       Intent(inout) :: transmission_ad
  Type(radiance_Type),           Intent(inout) :: radiancedata_ad 
#endif

#ifdef _RTTOV_PARALLEL_K
  Type(profile_Type),            Intent(inout) :: profiles_k( nchannels ) 
  Real(Kind=jprb),               Intent(inout) :: emissivity_k( nchannels ) 
  Type(transmission_type),       Intent(inout) :: transmission_k
  Type(radiance_Type), Optional, Intent(inout) :: radiancedata_k 
#endif
  
  Type(rttov_temp),    Optional, Intent(inout) :: temp
#ifdef _RTTOV_PARALLEL_TL
  Type(rttov_temp),    Optional, Intent(inout) :: temp_tl
#endif
#ifdef _RTTOV_PARALLEL_AD
  Type(rttov_temp),    Optional, Intent(inout) :: temp_ad
#endif
#ifdef _RTTOV_PARALLEL_K
  Type(rttov_temp),    Optional, Intent(inout) :: temp_k
#endif

  Integer(Kind=jpim),  Optional, Intent(in)    :: nthreads
!


#ifndef _RTTOV_PARALLEL_INTERFACE


  Character(len=*), Parameter :: NameOfRoutine = &
#ifdef _RTTOV_PARALLEL_DIRECT
    "rttov_parallel_direct"
#endif
#ifdef _RTTOV_PARALLEL_TL
    "rttov_parallel_tl"
#endif
#ifdef _RTTOV_PARALLEL_AD
    "rttov_parallel_ad"
#endif
#ifdef _RTTOV_PARALLEL_K
    "rttov_parallel_k"
#endif




  Integer(Kind=jpim) :: iband, nbands, ibandk
  Integer(Kind=jpim) :: ithread, mthreads
  Integer(Kind=jpim) :: nsplits
  Integer(Kind=jpim) :: ichannelk, iprofilek
  Integer(Kind=jpim) :: nprofiles12max
  Integer(Kind=jpim) :: nlevels
  Integer(Kind=jpim) :: nchannels_po
  Integer            :: mstat
  Integer(Kind=jpim), Allocatable :: errorstatus_x( :, : )            ! errorstatus/bands
  Integer(Kind=jpim), Allocatable :: iband1( : ), iband2( : )         ! band limits
  Integer(Kind=jpim), Allocatable :: ichannel1( : ), ichannel2( : )   ! channel limits
  Integer(Kind=jpim), Allocatable :: iprofile1( : ), iprofile2( : )   ! profile limits
  Integer(Kind=jpim), Allocatable :: lprofiles_x( :, : )              ! lprofiles/bands

  Type(transmission_type), Allocatable :: transmission_x( : )         ! transmission/bands ( pointer assoc )
  Type(radiance_Type),     Allocatable :: radiancedata_x( : )         ! radiancedata/bands ( pointer assoc )

#ifdef _RTTOV_PARALLEL_TL
  Type(transmission_type), Allocatable :: transmission_tl_x( : )  
  Type(radiance_Type),     Allocatable :: radiancedata_tl_x( : )  
#endif

#ifdef _RTTOV_PARALLEL_AD
  Integer(Kind=jpim)                   :: errorstatus_ad_x
  Type(transmission_type), Allocatable :: transmission_ad_x( : )  
  Type(radiance_Type),     Allocatable :: radiancedata_ad_x( : )  
  Type(profile_Type),      Allocatable :: profiles_ad_x( :, : ) 
#endif
  
#ifdef _RTTOV_PARALLEL_K
  Integer(Kind=jpim)                   :: errorstatus_k_x
  Type(transmission_type), Allocatable :: transmission_k_x( : )  
  Type(radiance_Type),     Allocatable :: radiancedata_k_x( : )  
#endif


! Activated if openmp
!  
!$  Integer, External :: omp_get_thread_num
!$  Integer, External :: omp_get_max_threads
!


  If( Present( temp ) ) Then
    Call Rttov_ErrorReport( errorstatus_warning, "temp argument will not be used !!", NameOfRoutine )
  EndIf
#ifdef _RTTOV_PARALLEL_TL
  If( Present( temp_tl ) ) Then
    Call Rttov_ErrorReport( errorstatus_warning, "temp_tl argument will not be used !!", NameOfRoutine )
  EndIf
#endif
#ifdef _RTTOV_PARALLEL_AD
  If( Present( temp_ad ) ) Then
    Call Rttov_ErrorReport( errorstatus_warning, "temp_ad argument will not be used !!", NameOfRoutine )
  EndIf
#endif
#ifdef _RTTOV_PARALLEL_K
  If( Present( temp_k ) ) Then
    Call Rttov_ErrorReport( errorstatus_warning, "temp_k argument will not be used !!", NameOfRoutine )
  EndIf
#endif

  nlevels = profiles( 1 ) % nlevels


  If( Present( nthreads ) ) Then
    mthreads = nthreads
  Else
! Default value for mthreads
    mthreads = 1_jpim
! Activated if openmp
!  
!$  mthreads = omp_get_max_threads()
!
  EndIf

! Check we do not have more bands than channels

  nsplits = 1_jpim
  nbands = mthreads * nsplits

  If( coef%id_sensor .eq. sensor_id_po ) Then
    If( nbands .gt. nprofiles ) nbands = nprofiles
  Else
    If( nbands .gt. nchannels ) nbands = nchannels
  EndIf
  
  Allocate( iband1( mthreads ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( iband2( mthreads ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  !
  ! we split the input array channels in bands
  !
  ibandk = 1_jpim
  Do ithread = 1, mthreads
    iband1( ithread ) = ibandk
    ibandk = ibandk + nbands / mthreads - 1_jpim
    If( ithread .eq. mthreads ) ibandk = nbands
    iband2( ithread ) = ibandk
    ibandk = ibandk + 1_jpim 
  EndDo


#ifdef _RTTOV_DEBUG  
  Write( *, '(A20," = ",I5)' ) "nprofiles", nprofiles
  Write( *, '(A20," = ",I5)' ) "nchannels", nchannels
  Write( *, '(A20," = ",I5)' ) "nbands", nbands
#endif  
  
  
  Allocate( ichannel1( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( ichannel2( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  Allocate( iprofile1( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( iprofile2( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  Allocate( transmission_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100

#ifdef _RTTOV_PARALLEL_TL
  Allocate( transmission_tl_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_tl_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
#endif

#ifdef _RTTOV_PARALLEL_AD
  Allocate( transmission_ad_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_ad_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
#endif
  
#ifdef _RTTOV_PARALLEL_K
  Allocate( transmission_k_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( radiancedata_k_x( nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
#endif
  
  Allocate( lprofiles_x( nchannels / nbands + nbands, nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Allocate( errorstatus_x( nchannels / nbands + nbands, nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  
  
  ichannel1( : ) = 0_jpim
  ichannel2( : ) = 0_jpim
  
  
  iprofile1( : ) = 0_jpim
  iprofile2( : ) = 0_jpim
  
  !
  ! Build the array of lower/upper bounds of bands
  !
  If( coef%id_sensor .eq. sensor_id_po ) Then
    !
    ! The approach is different for PO sensors
    ! because some channels cannot be separated in the
    ! computations; hence we must split the workload
    ! profile-wise
    !
    ichannelk = 1_jpim
    iprofilek = 1_jpim
    Do iband = 1, nbands
      nchannels_po = 0_jpim
      ichannel1( iband ) = ichannelk
      Do While( ( iprofilek .le. nprofiles ) &
          .and. ( ichannelk .le. nchannels ) )
        Do 
          If( ichannelk .gt. nchannels ) Exit
          ichannelk = ichannelk + 1_jpim
          nchannels_po = nchannels_po + 1_jpim
          If( lprofiles( ichannelk ) .ne. iprofilek ) Then
            iprofilek = iprofilek + 1_jpim
            Exit
          End If
        End Do
        If( nchannels_po .ge. nchannels / nbands ) Exit
      End Do
      ichannel2( iband ) = ichannelk - 1_jpim
    End Do
    ! What remains is given to the last band
    ichannel2( nbands ) = nchannels
  Else
    !
    ! For IR & MW sensors, we try to make bands
    ! with the same number of channels
    !
    ichannelk = 1_jpim
    Do iband = 1, nbands
      ichannel1( iband ) = ichannelk
      ichannelk = ichannelk + nchannels / nbands - 1_jpim
      If( iband .eq. nbands ) ichannelk = nchannels
      ichannel2( iband ) = ichannelk
      ichannelk = ichannelk + 1_jpim
    End Do
  End If


 
  lprofiles_x( :, : ) = 0_jpim
  
  Do iband = 1, nbands
    iprofile1( iband ) = lprofiles( ichannel1( iband ) )
    iprofile2( iband ) = lprofiles( ichannel2( iband ) )
    Do ichannelk = ichannel1( iband ), ichannel2( iband )
      lprofiles_x( ichannelk - ichannel1( iband ) + 1, iband ) &
        & = lprofiles( ichannelk ) - iprofile1( iband ) + 1
    EndDo
  EndDo 
  
#ifdef _RTTOV_PARALLEL_AD
  !
  ! for rttov_ad, we have to allocate a temporary array holding
  ! results from every band, and add the contribution of every band
  ! after the parallel loop
  !
  nprofiles12max = maxval( iprofile2( 1 : nbands ) - iprofile1( 1 : nbands ) + 1 )
  Allocate( profiles_ad_x( nprofiles12max, nbands ), stat = mstat )
  If( mstat .ne. 0 ) Goto 100
  Do iband = 1, nbands
    Call rttov_alloc_prof( errorstatus_ad_x, nprofiles12max, profiles_ad_x( :, iband ), &
      & nlevels, coef_scatt_ir, 1_jpim, addaerosl = addaerosl, addclouds = addclouds, init = .true. )
    If( errorstatus_ad_x .ne. 0 ) Goto 100
    Call NullifyProfiles( profiles_ad_x( :, iband ), nprofiles12max, addclouds, addaerosl )
  EndDo
#endif
  
  
#ifdef _RTTOV_DEBUG

  Do ithread = 1, mthreads
    Write( *, * )
    Write( *, '(A20," = ",I5)' ) "thread", ithread
    Write( *, '(A20," = ",I5)' ) "iband1", iband1( ithread )
    Write( *, '(A20," = ",100(I5))' ) "iband2", iband2( ithread )
    Write( *, * )
  EndDo

  Do iband = 1, nbands
    Write( *, * )
    Write( *, '(A20," = ",I5)' ) "band", iband
    Write( *, '(A20," = ",100(I5))' ) "channels", channels( ichannel1( iband ) : ichannel2( iband ) )
    Write( *, '(A20," = ",100(I5))' ) "lprofiles", lprofiles( ichannel1( iband ) : ichannel2( iband ) )
    Write( *, '(A20," = ",100(I5))' ) "lprofiles_x", lprofiles_x( :, iband )
    Write( *, * )
  EndDo

#endif  
  
  
  !
  ! we make the correct pointers associations between temporary transmissions
  ! and radiances and arrays passed as input
  !
  Do iband = 1, nbands
  
    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_x( iband ), transmission )
    Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
     & radiancedata_x( iband ), radiancedata )

#ifdef _RTTOV_PARALLEL_TL
    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_tl_x( iband ), transmission_tl )
    Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
     & radiancedata_tl_x( iband ), radiancedata_tl )
#endif
    
#ifdef _RTTOV_PARALLEL_AD
    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_ad_x( iband ), transmission_ad )
    Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
     & radiancedata_ad_x( iband ), radiancedata_ad )
#endif
    
#ifdef _RTTOV_PARALLEL_K
    Call AssociateTransmission( ichannel1( iband ), ichannel2( iband ), &
     & transmission_k_x( iband ), transmission_k )
    If( Present( radiancedata_k ) ) Then
      Call AssociateRadiance( ichannel1( iband ), ichannel2( iband ), &
       & radiancedata_k_x( iband ), radiancedata_k )
    Else
      !
      ! special case for the optional radiancedata_k of rttov_k
      ! we chose to provide the default
      !
      Call rttov_alloc_rad( errorstatus_k_x, Int(ichannel2( iband ) - ichannel1( iband ) + 1,jpim), &
      & radiancedata_k_x( iband ), nlevels, 1_jpim )
      If( errorstatus_k_x .ne. 0 ) Goto 100

      radiancedata_k_x( iband ) % clear( : )       = 0._jprb
      radiancedata_k_x( iband ) % cloudy( : )      = 0._jprb
      radiancedata_k_x( iband ) % bt_clear( : )    = 0._jprb
      radiancedata_k_x( iband ) % upclear( : )     = 0._jprb
      radiancedata_k_x( iband ) % reflclear( : )   = 0._jprb
      radiancedata_k_x( iband ) % overcast( :, : ) = 0._jprb
      radiancedata_k_x( iband ) % bt( : )          = 1._jprb
      radiancedata_k_x( iband ) % total( : )       = 1._jprb
      
    EndIf
#endif
    
    
  End Do

  
  
  
!$OMP PARALLEL DO PRIVATE( ithread, iband )
  Do iband = 1, nbands


!  Do ithread = 1, mthreads
!  Do iband = iband1( ithread ), iband2( ithread )

#ifdef _RTTOV_DEBUG
!    Print *, "================================================"
!    Print *, "THREAD = ", OMP_GET_THREAD_NUM(), " BAND = ", iband, &
!  & " ichannel1 = ", ichannel1( iband ), " ichannel2 = ", ichannel2( iband )

    Write( *, * )
#ifdef _RTTOV_PARALLEL_DIRECT
    Write( *, '(A)' ) "========================= rttov_direct ========================="
#endif
#ifdef _RTTOV_PARALLEL_TL
    Write( *, '(A)' ) "=========================== rttov_tl ==========================="
#endif
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A)' ) "=========================== rttov_ad ==========================="
#endif
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A)' ) "=========================== rttov_k ============================"
#endif
    Write( *, '(A20)' ) "errorstatus"
    Write( *, '(A20," = ",I5)' ) "nprofiles", iprofile2( iband ) - iprofile1( iband ) + 1
    Write( *, '(A20," = ",I5)' ) "nchannels", ichannel2( iband ) - ichannel1( iband ) + 1
    Write( *, '(A20," = ",100(I5))' ) "channels", channels( ichannel1( iband ) : ichannel2( iband ) )
    Write( *, '(A20," = ",100(I5))' ) "lprofiles", lprofiles_x( :, iband )
    Write( *, '(A20," = ",L5)' ) "addinterp", addinterp
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles",     &
    &"profiles", iprofile1( iband ), iprofile2( iband )
#ifdef _RTTOV_PARALLEL_TL
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles_tl",  &
    &"profiles_tl", iprofile1( iband ), iprofile2( iband )
#endif
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles_ad",  &
    &"profiles_ad", iprofile1( iband ), iprofile2( iband )
#endif
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "profiles_k",   &
    &"profiles_k", ichannel1( iband ), ichannel2( iband )
#endif
    Write( *, '(A20)' ) "coef"
    Write( *, '(A20)' ) "coef_scatt_ir"
    Write( *, '(A20)' ) "optp"
    Write( *, '(A20," = ",L5)' ) "addsun", addsun
    Write( *, '(A20," = ",L5)' ) "addaerosl", addaerosl
    Write( *, '(A20," = ",L5)' ) "addclouds", addclouds
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A20," = ",L5)' ) "switchrad", switchrad
#endif    
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A20," = ",L5)' ) "switchrad", switchrad
#endif    
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "calcemis",      &
    &"calcemis", ichannel1( iband ), ichannel2( iband )
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emissivity",    &
    &"emissivity", ichannel1( iband ), ichannel2( iband )
#ifdef _RTTOV_PARALLEL_TL
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emissivity_tl", &
    &"emissivity_tl", ichannel1( iband ), ichannel2( iband )
#endif
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emissivity_ad", &
    &"emissivity_ad", ichannel1( iband ), ichannel2( iband )
#endif
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A20," = ",A20," ( ",I5," : ",I5," ) ")' ) "emissivity_k",  &
    &"emissivity_k", ichannel1( iband ), ichannel2( iband )
#endif
    Write( *, '(A20)' ) "transmission"
#ifdef _RTTOV_PARALLEL_TL
    Write( *, '(A20)' ) "transmission_tl"
#endif
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A20)' ) "transmission_ad"
#endif
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A20)' ) "transmission_k"
#endif
    Write( *, '(A20)' ) "radiancedata"
#ifdef _RTTOV_PARALLEL_TL
    Write( *, '(A20)' ) "radiancedata_tl"
#endif
#ifdef _RTTOV_PARALLEL_AD
    Write( *, '(A20)' ) "radiancedata_ad"
#endif
#ifdef _RTTOV_PARALLEL_K
    Write( *, '(A20)' ) "radiancedata_k"
#endif
    Write( *, * )
    
#endif


    Call &
#ifdef _RTTOV_PARALLEL_DIRECT
      & rttov_direct                                               &
#endif
#ifdef _RTTOV_PARALLEL_TL
      & rttov_tl                                                   &
#endif
#ifdef _RTTOV_PARALLEL_AD
      & rttov_ad                                                   &
#endif
#ifdef _RTTOV_PARALLEL_K
      & rttov_k                                                    &
#endif
      & ( errorstatus_x( :, iband )                                &
      & , Int(iprofile2( iband ) - iprofile1( iband ) + 1,jpim)    &
      & , Int(ichannel2( iband ) - ichannel1( iband ) + 1,jpim)    &
      & , channels( ichannel1( iband ) : ichannel2( iband ) )      &
      & , lprofiles_x( :, iband )                                  &
      & , addinterp                                                &
      & , profiles( iprofile1( iband ) : iprofile2( iband ) )      &
#ifdef _RTTOV_PARALLEL_TL
      & , profiles_tl( iprofile1( iband ) : iprofile2( iband ) )   &
#endif
#ifdef _RTTOV_PARALLEL_AD
      & , profiles_ad_x( :, iband )                                &
#endif
#ifdef _RTTOV_PARALLEL_K
      & , profiles_k( ichannel1( iband ) : ichannel2( iband ) )    &
#endif
      & , coef                                                     &
      & , coef_scatt_ir                                            &
      & , optp                                                     &
      & , addsun                                                   &
      & , addaerosl                                                &
      & , addclouds                                                &
#ifdef _RTTOV_PARALLEL_AD
      & , switchrad                                                &
#endif      
#ifdef _RTTOV_PARALLEL_K
      & , switchrad                                                &
#endif      
      & , calcemis( ichannel1( iband ) : ichannel2( iband ) )      &
      & , emissivity( ichannel1( iband ) : ichannel2( iband ) )    &
#ifdef _RTTOV_PARALLEL_TL
      & , emissivity_tl( ichannel1( iband ) : ichannel2( iband ) ) &
#endif
#ifdef _RTTOV_PARALLEL_AD
      & , emissivity_ad( ichannel1( iband ) : ichannel2( iband ) ) &
#endif
#ifdef _RTTOV_PARALLEL_K
      & , emissivity_k( ichannel1( iband ) : ichannel2( iband ) )  &
#endif
      & , transmission_x( iband )                                  &
#ifdef _RTTOV_PARALLEL_TL
      & , transmission_tl_x( iband )                               &
#endif
#ifdef _RTTOV_PARALLEL_AD
      & , transmission_ad_x( iband )                               &
#endif
#ifdef _RTTOV_PARALLEL_K
      & , transmission_k_x( iband )                                &
#endif
      & , radiancedata_x( iband )                                  &
#ifdef _RTTOV_PARALLEL_TL
      & , radiancedata_tl_x( iband )                               &
#endif
#ifdef _RTTOV_PARALLEL_AD
      & , radiancedata_ad_x( iband )                               &
#endif
#ifdef _RTTOV_PARALLEL_K
      & , radiancedata_k_x( iband )                                &
#endif
      & ) 
   
!     Print *, iband, errorstatus_x( :, iband )
   
!  End Do
!  End Do

  End Do
!$OMP END PARALLEL DO


#ifdef _RTTOV_PARALLEL_AD
 !
 ! add every contribution in output profile_k array
 !
  Do iband = 1, nbands
    Do iprofilek = iprofile1( iband ), iprofile2( iband )
!      Print *, "profiles_ad( ", iprofilek, " ) = ", &
!      & "profiles_ad( ", iprofilek, " ) + profiles_ad_x( ", &
!      & iprofilek - iprofile1( iband ) + 1, ", ", iband, " )"
      Call AddProfiles( profiles_ad( iprofilek ), profiles_ad( iprofilek ), &
      & profiles_ad_x( iprofilek - iprofile1( iband ) + 1, iband ), addclouds, addaerosl )
    EndDo
  EndDo
#endif


#ifdef _RTTOV_PARALLEL_K
  If( .not. Present( radiancedata_k ) ) Then
    Do iband = 1, nbands

      Call rttov_alloc_rad( errorstatus_k_x, Int(ichannel2( iband ) - ichannel1( iband ) + 1,jpim), &
      & radiancedata_k_x( iband ), nlevels, 0_jpim )

    EndDo
  EndIf
#endif


!  Do iband = 1, nbands
!    Call CopyTransmission( ichannel1( iband ), ichannel2( iband ), transmission_x( iband ), transmission )
!  EndDo

  !
  ! we have to construct the global errorstatus array 
  ! from what we got from every thread
  !
  errorstatus( : ) = errorstatus_success 
  Do iband = 1, nbands
    Do iprofilek = iprofile1( iband ), iprofile2( iband )
!      Print *, iprofilek, " <= ", iband, iprofilek - iprofile1( iband ) + 1, &
!      & errorstatus_x( iprofilek - iprofile1( iband ) + 1, iband )
      Call SetMaxErrorStatus( &
       &  errorstatus( iprofilek ), &
       &  errorstatus_x( iprofilek - iprofile1( iband ) + 1, iband ) )
    EndDo
  EndDo

  
  DeAllocate( lprofiles_x )
  DeAllocate( errorstatus_x )

  DeAllocate( transmission_x )
  DeAllocate( radiancedata_x )

#ifdef _RTTOV_PARALLEL_TL
  DeAllocate( transmission_tl_x )
  DeAllocate( radiancedata_tl_x )
#endif
  
#ifdef _RTTOV_PARALLEL_AD
  DeAllocate( transmission_ad_x )
  DeAllocate( radiancedata_ad_x )
#endif

#ifdef _RTTOV_PARALLEL_K
  DeAllocate( transmission_k_x )
  DeAllocate( radiancedata_k_x )
#endif


#ifdef _RTTOV_PARALLEL_AD
  Do iband = 1, nbands
    Call rttov_alloc_prof( errorstatus_ad_x, nprofiles12max, profiles_ad_x( :, iband ), &
      & nlevels, coef_scatt_ir, 0_jpim, addaerosl = addaerosl, addclouds = addclouds )
  EndDo
  DeAllocate( profiles_ad_x )
#endif
  
  DeAllocate( iprofile2 )
  DeAllocate( iprofile1 )

  DeAllocate( ichannel2 )
  DeAllocate( ichannel1 )

  
  DeAllocate( iband2 )
  DeAllocate( iband1 )

  Return
  
100 Continue
    
    Call Rttov_ErrorReport( errorstatus_fatal, "Memory allocation failed", NameOfRoutine )

    Stop  
  
Contains

  ! ancillary routines

  Subroutine SetMaxErrorStatus( e1, e2 ) 
    Integer(Kind=jpim) :: e1, e2
    
    If( e1 .eq. errorstatus_fatal   ) Return
    If( e2 .eq. errorstatus_fatal   ) e1 = errorstatus_fatal
    If( e1 .eq. errorstatus_warning ) Return
    If( e2 .eq. errorstatus_warning ) e1 = errorstatus_warning
    If( e1 .eq. errorstatus_info    ) Return
    If( e2 .eq. errorstatus_info    ) e1 = errorstatus_info
    If( e1 .eq. errorstatus_success ) Return
    If( e2 .eq. errorstatus_success ) e1 = errorstatus_success
    
  End Subroutine
  
  Subroutine AssociateRadiance( ichannel1, ichannel2, radiancedata_x, radiancedata )
    Integer(Kind=jpim) :: ichannel1, ichannel2
    Type(radiance_Type) :: radiancedata_x, radiancedata
    
    radiancedata_x % lcloud &
      = radiancedata % lcloud

    radiancedata_x % clear &
      => radiancedata % clear( ichannel1 : ichannel2 )

    radiancedata_x % clear_out &
      => radiancedata % clear_out( ichannel1 : ichannel2 )

    radiancedata_x % cloudy &
      => radiancedata % cloudy( ichannel1 : ichannel2 )
    
    radiancedata_x % total &
      => radiancedata % total( ichannel1 : ichannel2 )
    
    radiancedata_x % total_out &
      => radiancedata % total_out( ichannel1 : ichannel2 )
    
    radiancedata_x % out &
      => radiancedata % out( ichannel1 : ichannel2 )
    
    radiancedata_x % out_clear &
      => radiancedata % out_clear( ichannel1 : ichannel2 )
    
    radiancedata_x % bt &
      => radiancedata % bt( ichannel1 : ichannel2 )
    
    radiancedata_x % bt_clear &
      => radiancedata % bt_clear( ichannel1 : ichannel2 )
    
    radiancedata_x % upclear &
      => radiancedata % upclear( ichannel1 : ichannel2 )
    
    radiancedata_x % dnclear &
      => radiancedata % dnclear( ichannel1 : ichannel2 )
    
    radiancedata_x % reflclear &
      => radiancedata % reflclear( ichannel1 : ichannel2 )
    
    radiancedata_x % overcast &
      => radiancedata % overcast( :, ichannel1 : ichannel2 )
  
    radiancedata_x % downcld &
      => radiancedata % downcld( :, ichannel1 : ichannel2 )
  
    radiancedata_x % up &
      => radiancedata % up( :, ichannel1 : ichannel2 )
  
    radiancedata_x % down &
      => radiancedata % down( :, ichannel1 : ichannel2 )
  
    radiancedata_x % surf &
      => radiancedata % surf( :, ichannel1 : ichannel2 )
  
  End Subroutine

  Subroutine AssociateTransmission( ichannel1, ichannel2, transmission_x, transmission )
    Integer(Kind=jpim) :: ichannel1, ichannel2
    Type(transmission_type) :: transmission_x, transmission
!    Integer(Kind=jpim) :: nstreams
    
!    nstreams = ubound( transmission % tau_layers, 1 )
!    Allocate( transmission_x % tau_layers( 0 : nstreams, ichannel2 - ichannel1 + 1 ) )
    
!    transmission_x % tau_layers( :, : ) = transmission % tau_layers( :, ichannel1 : ichannel2 )
      
    transmission_x % tau_layers & 
      => transmission % tau_layers( :, ichannel1 : ichannel2 )

    transmission_x % tau_total &
      => transmission % tau_total( ichannel1 : ichannel2 )
      
  End Subroutine
  
!  Subroutine CopyTransmission( ichannel1, ichannel2, transmission_x, transmission )
!    Integer(Kind=jpim) :: ichannel1, ichannel2
!    Type(transmission_type) :: transmission_x, transmission
!
!    transmission % tau_layers( :, ichannel1 : ichannel2 ) = transmission_x % tau_layers( :, : )
!    Deallocate( transmission_x % tau_layers )
!      
!  End Subroutine
  
  
  Subroutine NullifyProfiles( profiles, nprofiles, addclouds, addaerosl )
  Integer(Kind=jpim) :: nprofiles
  Type(profile_type) :: profiles( nprofiles )
  Integer :: j
  Logical :: addclouds, addaerosl
  
  Do j = 1, nprofiles
    profiles( j ) % p             = 0._jprb
    profiles( j ) % t             = 0._jprb
    profiles( j ) % q             = 0._jprb
    profiles( j ) % co2           = 0._jprb
    profiles( j ) % o3            = 0._jprb
    profiles( j ) % n2o           = 0._jprb
    profiles( j ) % co            = 0._jprb
    profiles( j ) % ch4           = 0._jprb
    profiles( j ) % clw           = 0._jprb
    if( addaerosl ) then
      profiles( j ) % aerosols      = 0._jprb
    endif
    if( addclouds ) then
      profiles( j ) % cloud         = 0._jprb
      profiles( j ) % cfrac         = 0._jprb
    endif
    profiles( j ) % zenangle      = 0._jprb
    profiles( j ) % azangle       = 0._jprb
    profiles( j ) % sunzenangle   = 0._jprb
    profiles( j ) % sunazangle    = 0._jprb
    profiles( j ) % elevation     = 0._jprb
    profiles( j ) % latitude      = 0._jprb
    profiles( j ) % ctp           = 0._jprb
    profiles( j ) % cfraction     = 0._jprb
    profiles( j ) % s2m % t       = 0._jprb 
    profiles( j ) % s2m % q       = 0._jprb       
    profiles( j ) % s2m % p       = 0._jprb 
    profiles( j ) % s2m % u       = 0._jprb 
    profiles( j ) % s2m % v       = 0._jprb 
    profiles( j ) % s2m % wfetc   = 0._jprb
    profiles( j ) % skin % t      = 0._jprb  
    profiles( j ) % skin % fastem = 0._jprb
  EndDo

  End Subroutine
  
  Subroutine AddProfiles( profileS, profileA, profileB, addclouds, addaerosl )
  Type(profile_type) :: profileS, profileA, profileB
  Logical :: addclouds, addaerosl



  profileS % t   = profileA % t   + profileB % t  
  profileS % q   = profileA % q   + profileB % q  
  profileS % co2 = profileA % co2 + profileB % co2
  profileS % o3  = profileA % o3  + profileB % o3 
  profileS % n2o = profileA % n2o + profileB % n2o
  profileS % co  = profileA % co  + profileB % co 
  profileS % ch4 = profileA % ch4 + profileB % ch4
  profileS % clw = profileA % clw + profileB % clw
  if( addaerosl ) then
    profileS % aerosols = profileA % aerosols + profileB % aerosols
  endif
  if( addclouds ) then
    profileS % cloud = profileA % cloud + profileB % cloud   
    profileS % cfrac = profileA % cfrac + profileB % cfrac 
  endif

! we do not add angles

  profileS % ctp              = profileA % ctp              + profileB % ctp             
  profileS % cfraction        = profileA % cfraction        + profileB % cfraction       

  profileS % s2m % t          = profileA % s2m % t          + profileB % s2m % t
  profileS % s2m % q          = profileA % s2m % q          + profileB % s2m % q
  profileS % s2m % p          = profileA % s2m % p          + profileB % s2m % p
  profileS % s2m % u          = profileA % s2m % u          + profileB % s2m % u
  profileS % s2m % v          = profileA % s2m % v          + profileB % s2m % v
  profileS % s2m % wfetc      = profileA % s2m % wfetc      + profileB % s2m % wfetc

  profileS % skin % t         = profileA % skin % t         + profileB % skin % t
  profileS % skin % fastem    = profileA % skin % fastem    + profileB % skin % fastem


  End Subroutine
  
#endif
  
End Subroutine

#ifdef _RTTOV_PARALLEL_INTERFACE
End Interface
#endif





