!
Subroutine rttov_readcoeffs_binary  (&
       & errorstatus,   &! out
       & coef,          &! inout
       & coef_scatt_ir, &! inout
       & optp ,         &! inout
       & file_lu,       &! in
       & channels,      &! in Optional
       & aerosl,        &! in Optional
       & clouds         )! in Optional
  ! Description:
  !
  ! Read an binary coefficient file and fills coeff structure
  !   arrays according to the optional list of channels.
  !
  ! The user can provide an optional list of channels in "channels" argument
  !  array to reduce the output coefficient structure to this list. This
  ! can be important for reducing the memory allocation required when running
  ! with advanced IR sounders (e.g. AIRS or IASI). If the user
  !  wants all channels the "channels" argument shall not be present.
  !
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       02/01/2003  A few comments added (R Saunders)
  !  1.2       24/01/2003  add tests on all read statements (P Brunel)
  !                        one record per channel for coefficients in binary format
  !                        New header to allow checking R4<->R8
  !  1.3       06/05/2003  Remove "optional" attribute of argument file_lu (P Brunel)
  !  1.4       02/06/2004  New format for FMV section with RTTOV8  (P. Brunel)
  !  1.5       15/06/2004  Corrected array dimension for coef % fmv_gas_pos (R Saunders)
  !  1.6       14/05/2007  Updated for RTTOV-9 ( P Brunel)
  !  1.7       12/12/2007  Option of additional layer at top flag read (R Saunders)
  !  1.8       27/06/2008  Introduced the case when no channels are present for the 
  !                        phase function in the solar range (M. Matricardi)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & rttov_magic_string  ,     rttov_magic_number  ,&
        & version_compatible_min  , version_compatible_max  ,&
        & sensor_id_hi        , sensor_id_mw        , sensor_id_ir   ,sensor_id_po   ,&
        & errorstatus_success , errorstatus_fatal   ,&
        & gas_id_mixed        , gas_id_watervapour  ,&
        & gas_id_ozone        , gas_id_wvcont       ,&
        & gas_id_co2          , gas_id_n2o          ,&
        & gas_id_co           , gas_id_ch4          ,&
        & sensor_name         , ngases_max          ,&
        & gas_name            , &
        & gas_unit_specconc  ,&
        & naer_max            ,&
        & nwcl_max            ,&
        & aer_name            ,&
        & wcl_name            , &
        & aer_id_inso         ,&
        & aer_id_waso         ,&
        & aer_id_soot         ,&
        & aer_id_ssam         ,&
        & aer_id_sscm         ,&
        & aer_id_minm         ,&
        & aer_id_miam         ,&
        & aer_id_micm         ,&
        & aer_id_mitr         ,&
        & aer_id_suso         ,&
        & aer_id_vola         ,&
        & wcl_id_stco         ,&
        & wcl_id_stma         ,&
        & wcl_id_cucc         ,&
        & wcl_id_cucp         ,&
        & wcl_id_cuma         ,&
        & deg2rad
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK


  ! Imported Type Definitions:
  Use rttov_types, Only :  &
        & rttov_coef      ,&
        & rttov_optpar_ir ,&
        & rttov_coef_scatt_ir 


  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None


  ! subroutine arguments
  ! scalar arguments with intent(in):
  Integer(Kind=jpim), Intent (in)           :: file_lu     ! file logical unit number
  Integer(Kind=jpim), Optional, Intent (in) :: channels(:) ! list of channels to extract
  Logical,Optional,             Intent (in) :: aerosl
  Logical,Optional,             Intent (in) :: clouds

  ! scalar arguments with intent(inout):
  Type( rttov_coef ),        Intent (inout) :: coef    ! coefficients
  Type(rttov_optpar_ir),     Intent (inout) :: optp
  Type(rttov_coef_scatt_ir), Intent (inout) :: coef_scatt_ir

  ! scalar arguments with intent(out):
  Integer(Kind=jpim), Intent (out) :: errorstatus          ! return code

!INTF

#include "rttov_errorreport.interface"

  ! Local Scalars:
  Integer(Kind=jpim)           :: file_channels
  Logical                      :: all_channels
  Integer                      :: io_status
  Integer(Kind=jpim)           :: alloc_status(12) !pb
  Integer(Kind=jpim)           :: n
  Integer(Kind=jpim)           :: chn
  Integer(Kind=jpim)           :: i, is, k, icount, inctop
  Integer(Kind=jpim)           :: pha_channels
  Integer(Kind=jpim)           :: pha_1,pha_2
  Integer(Kind=jpim), Allocatable :: list_of_channels(:) 

  ! pointers for generic inputs
  Real(Kind=jprb),    Pointer :: values0(:)
  Real(Kind=jprb),    Pointer :: values1(:)
  Real(Kind=jprb),    Pointer :: values2(:)
  Real(Kind=jprb),    Pointer :: values3(:)
  Real(Kind=jprb),    Pointer :: values4(:)
  Integer(Kind=jpim), Pointer :: ivalues0(:)
  Integer(Kind=jpim), Pointer :: ivalues1(:)
  Complex (Kind=jprb),    Pointer :: values_c_0(:)
  Complex (Kind=jprb),    Pointer :: values_c_1(:)
  Real(Kind=jprb),    Pointer :: dvalues0(:,:)
  Real(Kind=jprb),    Pointer :: tvalues0(:,:,:)

  Character (len=16) :: bin_check_string
  Real(Kind=jprb)    :: bin_check_number
  Real(Kind=jprb)    :: bin_check_value
  Character (len=80) :: errMessage
  Character (len=23) :: NameOfRoutine = 'rttov_readcoeffs_binary'
  Logical            :: section_present
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  ! 0 Initialise variables
  !---------------------------------------------
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',0_jpim,ZHOOK_HANDLE)
  errorstatus     = errorstatus_success
  alloc_status(:) = 0

  ! test presence of channels argument
  If( Present ( channels ) ) Then
     all_channels = .False.
  Else
     all_channels = .True.
  Endif


  ! 3 Read binary file
  !-------------------
  ! Binary file
  Read(file_lu, iostat = io_status ) bin_check_string, bin_check_number
  If(io_status /= 0) Then
     errMessage = 'io status while reading header'
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  ! Verification of header string
  if ( bin_check_string /= rttov_magic_string ) Then
     errMessage = 'Wrong header string in file'
     errorstatus = errorstatus_fatal
     Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  End If


  ! Verification of single/double precision using a 5 digit number
  ! with exponent 12, which is always Ok for single precision
  bin_check_value = 1._JPRB - abs ( bin_check_number - rttov_magic_number )
  if ( bin_check_value > 1.01_JPRB .or. bin_check_value < 0.99_JPRB ) Then
     errMessage = 'File created with a different real precision (R4<->R8)'
     errorstatus = errorstatus_fatal
     Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  End If


! COEF structure (V7, V8, V9)
  If ( .not. (Present(aerosl) .or. Present(clouds))) Then
 



  errMessage = 'io status while reading IDENTIFICATION'
  Read(file_lu, iostat = io_status )&
         & coef % id_platform, &
         & coef % id_sat,      &
         & coef % id_inst,     &
         & coef % id_sensor 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif
  Read(file_lu, iostat = io_status )&
         & coef % id_comp_lvl,      &
         & coef % id_creation_date, &
         & coef % id_creation,      &
         & coef % id_Common_name 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  errMessage = 'io status while reading FAST_MODEL_VARIABLES'
  If( coef % id_comp_lvl <= 7 ) then
    Read(file_lu, iostat = io_status )&
        & coef % fmv_model_def, &
        & coef % fmv_chn,       &
        & coef % fmv_gas 
    coef % fmv_model_ver = 7
  Else
    Read(file_lu, iostat = io_status )&
        & coef % fmv_model_def, &
        & coef % fmv_model_ver, &
        & coef % fmv_chn,       &
        & coef % fmv_gas 
  Endif
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  coef % IncTop = .False.
  coef % IncZeeman =.False.
  If( coef % id_comp_lvl >= 9 ) then
    Read(file_lu, iostat = io_status )inctop
    If (inctop == 1)coef % IncTop = .True.
! for rttov-9 - temporary use of existing flag for Zeeman effect
    If(inctop == 2) coef % IncZeeman = .True.
  Endif
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  ! Error if the compatibility version of the coefficient file
  ! is not in the range defined by the constant module
  If(    coef % id_comp_lvl < version_compatible_min .Or.&
         & coef % id_comp_lvl > version_compatible_max ) Then 
     errorstatus = errorstatus_fatal
     Write( errMessage,&
            & '( "Version of coefficient file is incompatible with RTTOV library")' ) 
     Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  ! Take care of the user list of channels
  ! file_channels store the number of channels in the file
  ! coef % fmv_chn is the number of channels that the user requests
  file_channels = coef % fmv_chn
  If( .Not. all_channels ) Then
     coef % fmv_chn = Size( channels )
  Endif

  Allocate ( coef % fmv_gas_id ( coef % fmv_gas ), stat=alloc_status(1))
  Allocate ( coef % fmv_gas_pos( ngases_max ), stat=alloc_status(2))
  Allocate ( coef % fmv_var ( coef % fmv_gas ), stat=alloc_status(3))
  Allocate ( coef % fmv_coe ( coef % fmv_gas ), stat=alloc_status(12))       !pb
  Allocate ( coef % fmv_lvl ( coef % fmv_gas ), stat=alloc_status(4))
  Allocate ( coef % ff_ori_chn( coef % fmv_chn ), stat=alloc_status(5) )
  Allocate ( coef % ff_val_chn( coef % fmv_chn ), stat=alloc_status(6) )
  Allocate ( coef % ff_cwn( coef % fmv_chn ), stat=alloc_status(7) )
  Allocate ( coef % ff_bco( coef % fmv_chn ), stat=alloc_status(8) )
  Allocate ( coef % ff_bcs( coef % fmv_chn ), stat=alloc_status(9) )
  Allocate ( coef % ff_gam( coef % fmv_chn ), stat=alloc_status(10) )
  Allocate ( coef % gaz_units( coef % fmv_gas ), stat=alloc_status(11) )
  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "allocation of coefs arrays")' )
     Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  coef % fmv_gas_id(:) = 0_jpim
  coef % fmv_gas_pos(:) = 0_jpim
  coef % fmv_var(:) = 0_jpim
  coef % fmv_lvl(:) = 0_jpim
  coef % fmv_coe(:) = 0_jpim

  Read(file_lu, iostat = io_status )&
         & coef % fmv_gas_id,  &
         & coef % fmv_gas_pos, &
         & coef % fmv_var,     &
         & coef % fmv_lvl 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif
  
  If( coef % id_comp_lvl == 9 ) then        !pb
    Read(file_lu, iostat=io_status)&        !pb
           & coef % fmv_coe                 !pb
  Else                                      !pb
    coef % fmv_coe = coef % fmv_var         !pb
  End If                                    !pb
  Do n = 1, coef % fmv_gas
     Select Case( coef % fmv_gas_id(n) )
     Case( gas_id_mixed )
        coef % nmixed  = coef % fmv_var(n)
        coef % nlevels = coef % fmv_lvl(n)
        coef % ncmixed = coef % fmv_coe(n)    !pb
     Case( gas_id_watervapour )
        coef % nwater  = coef % fmv_var(n)
        coef % ncwater = coef % fmv_coe(n)    !pb
     Case( gas_id_ozone )
        coef % nozone  = coef % fmv_var(n)
        coef % ncozone = coef % fmv_coe(n)    !pb
     Case( gas_id_wvcont )
        coef % nwvcont  = coef % fmv_var(n)
        coef % ncwvcont = coef % fmv_coe(n)    !pb
     Case( gas_id_co2 )
        coef % nco2  = coef % fmv_var(n)
        coef % ncco2 = coef % fmv_coe(n)    !pb
     Case( gas_id_n2o )
        coef % nn2o  = coef % fmv_var(n)
        coef % ncn2o = coef % fmv_coe(n)    !pb
     Case( gas_id_co )
        coef % nco  = coef % fmv_var(n)
        coef % ncco = coef % fmv_coe(n)    !pb
     Case( gas_id_ch4 )
        coef % nch4  = coef % fmv_var(n)
        coef % ncch4 = coef % fmv_coe(n)    !pb
     End Select
  End Do

  if( coef % IncTop )Then
      coef % nlayers = coef % nlevels -1 
  else
      coef % nlayers = coef % nlevels  
  endif 

  errMessage = 'io status while reading GAZ_UNITS'
  Read(file_lu, iostat = io_status )&
         & coef % gaz_units 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif


  If( coef % id_comp_lvl == 9 ) then        !pb
     ! GAS_SPECTRAL_INTERVAL                  !pb
     Read(file_lu, iostat=io_status)&        !pb
           & coef%nintmixed  ,&                 !pb
           & coef%nintwater  ,&                 !pb
           & coef%nintozone  ,&                 !pb
           & coef%nintwvcont ,&                 !pb
           & coef%nintco2    ,&                 !pb
           & coef%nintn2o    ,&                 !pb
           & coef%nintco     ,&                 !pb
           & coef%nintch4                       !pb
     If(io_status /= 0) Then
       Call Rttov_ErrorReport (Int(io_status,jpim), errMessage, NameOfRoutine)
       errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     Endif

     If( coef%nintmixed > 0 ) Then       !pb
       allocate(coef%mixedgasint(2,coef%nintmixed),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of mixedgasint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%mixedgasint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintwater > 0 ) Then       !pb
       allocate(coef%watervapourint(2,coef%nintwater),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of watervapourint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%watervapourint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintozone > 0 ) Then       !pb
       allocate(coef%ozoneint(2,coef%nintozone),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of ozoneint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%ozoneint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintwvcont > 0 ) Then       !pb
       allocate(coef%wvcontint(2,coef%nintwvcont),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of wvcontint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%wvcontint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintco2 > 0 ) Then       !pb
       allocate(coef%co2int(2,coef%nintco2),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of co2int array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%co2int    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintn2o > 0 )  Then       !pb
       allocate(coef%n2oint(2,coef%nintn2o),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of n2oint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%n2oint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintco > 0 )  Then       !pb
       allocate(coef%coint(2,coef%nintco),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of coint array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%coint    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

     If( coef%nintch4 > 0 )  Then       !pb
       allocate(coef%ch4int(2,coef%nintch4),stat=alloc_status(1))
       If( alloc_status(1) /= 0 ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of ch4int array")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read(file_lu, iostat=io_status) coef%ch4int    !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     End If

  End If                                                      !pb


  If( coef % id_comp_lvl == 9 ) then         !pb

    Read( file_lu ) section_present
     
    If( section_present ) Then
     !TRANSMITTANCE_TRESHOLD                  !pb
      Allocate ( coef % tt_chn      ( coef % fmv_chn ), stat=alloc_status(1) )
      Allocate ( coef % tt_val_chn  ( coef % fmv_chn ), stat=alloc_status(2) )
      Allocate ( coef % tt_cwn      ( coef % fmv_chn ), stat=alloc_status(3) )
      Allocate ( coef % tt_a0       ( coef % fmv_chn ), stat=alloc_status(4) )
      Allocate ( coef % tt_a1       ( coef % fmv_chn ), stat=alloc_status(5) )
      If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of transmittance treshold coefs arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
      End If

      If( all_channels ) Then
        Read(file_lu, iostat=io_status)&        !pb
           & coef % tt_chn     ,&               !pb
           & coef % tt_val_chn ,&               !pb
           & coef % tt_cwn     ,&               !pb
           & coef % tt_a0      ,&               !pb
           & coef % tt_a1                       !pb
        If(io_status /= 0) Then
          Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
          errorstatus = errorstatus_fatal
          IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
          Return
        Endif

      Else
        Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
        Allocate ( ivalues1( file_channels ), stat=alloc_status(2) )
        Allocate (  values0( file_channels ), stat=alloc_status(3) )
        Allocate (  values1( file_channels ), stat=alloc_status(4) )
        Allocate (  values2( file_channels ), stat=alloc_status(5) )
        If( Any(alloc_status /= 0) ) Then
          errorstatus = errorstatus_fatal
          Write( errMessage, '( "allocation of transmittance treshold coefs arrays")' )
          Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
          IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
          Return
        End If
        Read(file_lu, iostat=io_status)&        !pb
           & ivalues0     ,&               !pb
           & ivalues1 ,&               !pb
           & values0   ,&               !pb
           & values1   ,&               !pb
           & values2                     !pb
        If(io_status /= 0) Then
          Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
          errorstatus = errorstatus_fatal
          IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
          Return
        Endif
        coef % tt_chn(:)        = ivalues0 ( channels(:) )
        coef % tt_val_chn(:)    = ivalues1 ( channels(:) )
        coef % tt_cwn(:)        =  values0 ( channels(:) )
        coef % tt_a0(:)         =  values1 ( channels(:) )
        coef % tt_a1(:)         =  values2 ( channels(:) )
        Deallocate ( ivalues0, stat=alloc_status(1) )
        Deallocate ( ivalues1, stat=alloc_status(2) )
        Deallocate (  values0, stat=alloc_status(3) )
        Deallocate (  values1, stat=alloc_status(4) )
        Deallocate (  values2, stat=alloc_status(5) )
        If( Any(alloc_status /= 0) ) Then
          errorstatus = errorstatus_fatal
          Write( errMessage, '( "deallocation of transmittance treshold coefs arrays")' )
          Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
          IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
          Return
        End If

      End If
    End If                                     !pb
  End If                                     !pb

    
   If( coef % id_comp_lvl == 9 ) then          !pb

     Read( file_lu ) section_present
     
     If( section_present ) Then
       !SOLAR_SPECTRUM                          !pb
       Allocate ( coef % ss_chn            ( coef % fmv_chn ), stat=alloc_status(1) )
       Allocate ( coef % ss_val_chn        ( coef % fmv_chn ), stat=alloc_status(2) )
       Allocate ( coef % ss_cwn            ( coef % fmv_chn ), stat=alloc_status(3) )
       Allocate ( coef % ss_solar_spectrum ( coef % fmv_chn ), stat=alloc_status(4) )
       If( Any(alloc_status /= 0) ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of solar spectrum coefs arrays")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
         IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If

       If( all_channels ) Then
         Read(file_lu, iostat=io_status)&        !pb
             & coef % ss_chn     ,&               !pb
             & coef % ss_val_chn ,&               !pb
             & coef % ss_cwn     ,&               !pb
             & coef % ss_solar_spectrum           !pb
         If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
         Endif
       Else
         Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
         Allocate ( ivalues1( file_channels ), stat=alloc_status(2) )
         Allocate (  values0( file_channels ), stat=alloc_status(3) )
         Allocate (  values1( file_channels ), stat=alloc_status(4) )
         If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "allocation of solar spectrum coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
         End If
         Read( file_lu, iostat = io_status )&
                   & ivalues0, &
                   & ivalues1, &
                   & values0,  &
                   & values1 
         If(io_status /= 0) Then
           errorstatus = errorstatus_fatal
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
         Endif
         coef % ss_chn(:)            = ivalues0 ( channels(:) )
         coef % ss_val_chn(:)        = ivalues1 ( channels(:) )
         coef % ss_cwn(:)            =  values0 ( channels(:) )
         coef % ss_solar_spectrum(:) =  values1 ( channels(:) )
         Deallocate ( ivalues0, stat=alloc_status(1) )
         Deallocate ( ivalues1, stat=alloc_status(2) )
         Deallocate (  values0, stat=alloc_status(3) )
         Deallocate (  values1, stat=alloc_status(4) )
         If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "deallocation of solar spectrum coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
         End If
       End If
     EndIf
   End If                                     !pb


  If( coef % id_comp_lvl == 9 ) then         !pb

    Read( file_lu ) section_present
     
    If( section_present ) Then
     !WATER_OPTICAL_CONSTANT                 !pb
     Allocate ( coef % woc_chn            ( coef % fmv_chn ), stat=alloc_status(1) )
     Allocate ( coef % woc_cwn            ( coef % fmv_chn ), stat=alloc_status(2) )
     Allocate ( coef % woc_waopc_ow      ( coef % fmv_chn ), stat=alloc_status(3) )
     Allocate ( coef % woc_waopc_fw      ( coef % fmv_chn ), stat=alloc_status(4) )
     If( Any(alloc_status /= 0) ) Then
       errorstatus = errorstatus_fatal
       Write( errMessage, '( "allocation of water optical constants coefs arrays")' )
       Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     End If
     If( all_channels ) Then
       Read(file_lu, iostat=io_status)&       !pb
           & coef % woc_chn      ,&            !pb
           & coef % woc_cwn      ,&            !pb
           & coef % woc_waopc_ow ,&            !pb
           & coef % woc_waopc_fw               !pb
       If(io_status /= 0) Then
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
         IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
     Else
       Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
       Allocate ( values1( file_channels ), stat=alloc_status(2) )
       Allocate ( values_c_0( file_channels ), stat=alloc_status(3) )
       Allocate ( values_c_1( file_channels ), stat=alloc_status(4) )
       If( Any(alloc_status /= 0) ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of solar spectrum coefs arrays")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
         IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
       Read( file_lu, iostat = io_status )&
                      & ivalues0,&
                      & values1 ,&
                      & values_c_0 ,&
                      & values_c_1 
       If(io_status /= 0) Then
         errorstatus = errorstatus_fatal
         Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
         errorstatus = errorstatus_fatal
         IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       Endif
       coef % woc_chn(:)              = ivalues0 ( channels(:) )
       coef % woc_cwn(:)              = values1 ( channels(:) )
       coef % woc_waopc_ow(:)         = values_c_0 ( channels(:) )
       coef % woc_waopc_fw(:)         = values_c_1 ( channels(:) )
       Deallocate ( ivalues0, stat=alloc_status(1) )
       Deallocate ( values1, stat=alloc_status(2) )
       Deallocate ( values_c_0, stat=alloc_status(3) )
       Deallocate ( values_c_1, stat=alloc_status(4) )
       If( Any(alloc_status /= 0) ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "deallocation of solar spectrum coefs arrays")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
         IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
       End If
     End If 
   End If
  End If
  
                                     !pb

  If( coef % id_comp_lvl == 9 ) then         !pb
     
    Read( file_lu ) section_present

    If( section_present ) Then
     !WAVE_SPECTRUM                          !pb
     Read(file_lu, iostat=io_status)&       !pb
           & coef % ws_nomega                  !pb
     If(io_status /= 0) Then
       Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
       errorstatus = errorstatus_fatal
       IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     Endif
     Allocate ( coef % ws_k_omega ( coef % ws_nomega ), stat=alloc_status(1) )
     Allocate ( coef % ws_npoint  ( coef % ws_nomega ), stat=alloc_status(2) )
     If( Any(alloc_status /= 0) ) Then
       errorstatus = errorstatus_fatal
       Write( errMessage, '( "allocation of solar spectrum coefs arrays")' )
       Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     End If

     Read(file_lu, iostat=io_status)&       !pb
           & coef % ws_k_omega   ,&            !pb
           & coef % ws_npoint                  !pb
     If(io_status /= 0) Then
       Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
       errorstatus = errorstatus_fatal
       IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     Endif
   End If                                    !pb
  End If                                    !pb



  errMessage = 'io status while reading FILTER_FUNCTIONS'
  If( all_channels ) Then
     Read(file_lu, iostat = io_status )&
            & coef % ff_ori_chn, &
            & coef % ff_val_chn, &
            & coef % ff_cwn,     &
            & coef % ff_bco,     &
            & coef % ff_bcs,     &
            & coef % ff_gam 
     If(io_status /= 0) Then
        Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
        errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     Endif
  Else
     Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
     Allocate ( ivalues1( file_channels ), stat=alloc_status(2) )
     Allocate (  values0( file_channels ), stat=alloc_status(3) )
     Allocate (  values1( file_channels ), stat=alloc_status(4) )
     Allocate (  values2( file_channels ), stat=alloc_status(5) )
     Allocate (  values3( file_channels ), stat=alloc_status(6) )
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of ff coefs arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     Read(file_lu, iostat = io_status )&
            & ivalues0, &
            & ivalues1, &
            & values0,  &
            & values1,  &
            & values2,  &
            & values3 
     If(io_status /= 0) Then
        Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
        errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     Endif
     coef % ff_ori_chn(:) = ivalues0 ( channels(:) )
     coef % ff_val_chn(:) = ivalues1 ( channels(:) )
     coef % ff_cwn(:)     =  values0 ( channels(:) )
     coef % ff_bco(:)     =  values1 ( channels(:) )
     coef % ff_bcs(:)     =  values2 ( channels(:) )
     coef % ff_gam(:)     =  values3 ( channels(:) )
     Deallocate ( ivalues0, stat=alloc_status(1) )
     Deallocate ( ivalues1, stat=alloc_status(2) )
     Deallocate (  values0, stat=alloc_status(3) )
     Deallocate (  values1, stat=alloc_status(4) )
     Deallocate (  values2, stat=alloc_status(5) )
     Deallocate (  values3, stat=alloc_status(6) )
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "deallocation of ff coefs arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If
  Endif

  errMessage = 'io status while reading FUNDAMENTAL_CONSTANTS'
  Read(file_lu, iostat = io_status )&
         & coef % fc_speedl,    &
         & coef % fc_planck_c1, &
         & coef % fc_planck_c2, &
         & coef % fc_sat_height 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  errMessage = 'io status while reading EMISSIVITY model versions'
  Read(file_lu, iostat = io_status )&
         & coef % fastem_ver, &
         & coef % ssirem_ver 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  errMessage = 'io status while reading FASTEM'
  If( coef % fastem_ver >= 1 ) Then
     Read(file_lu, iostat = io_status ) coef % fastem_coef_nb
     If(io_status /= 0) Then
        Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
        errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     Endif
     Allocate ( coef % fastem_coef  ( coef % fastem_coef_nb ), stat=alloc_status(1) )
     Allocate ( coef % fastem_polar ( coef % fmv_chn ), stat=alloc_status(2) )
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of fastem coefs arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     If( all_channels ) Then
        Read(file_lu, iostat = io_status )&
               & coef % fastem_coef ,&
               & coef % fastem_polar 
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     Else
        Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
        If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "allocation of fastem coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        End If
        Read(file_lu, iostat = io_status )&
               & coef % fastem_coef,&
               & ivalues0 
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
        coef % fastem_polar(:) = ivalues0( channels (:) )
        Deallocate ( ivalues0, stat=alloc_status(1) )
        If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "deallocation of fastem coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Endif
  Endif

  errMessage = 'io status while reading SSIREM'
  If( coef % ssirem_ver >= 1 ) Then
     Allocate ( coef % ssirem_chn ( coef % fmv_chn ), stat=alloc_status(1) )
     Allocate ( coef % ssirem_a0  ( coef % fmv_chn ), stat=alloc_status(2) )
     Allocate ( coef % ssirem_a1  ( coef % fmv_chn ), stat=alloc_status(3) )
     Allocate ( coef % ssirem_a2  ( coef % fmv_chn ), stat=alloc_status(4) )
     Allocate ( coef % ssirem_xzn1( coef % fmv_chn ), stat=alloc_status(5) )
     Allocate ( coef % ssirem_xzn2( coef % fmv_chn ), stat=alloc_status(6) )
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of ssirem coefs arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     If( all_channels ) Then
        Read(file_lu, iostat = io_status )&
               & coef % ssirem_chn, &
               & coef % ssirem_a0,  &
               & coef % ssirem_a1,  &
               & coef % ssirem_a2,  &
               & coef % ssirem_xzn1,&
               & coef % ssirem_xzn2 
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     Else
        Allocate ( ivalues0( file_channels ), stat=alloc_status(1) )
        Allocate (  values0( file_channels ), stat=alloc_status(2) )
        Allocate (  values1( file_channels ), stat=alloc_status(3) )
        Allocate (  values2( file_channels ), stat=alloc_status(4) )
        Allocate (  values3( file_channels ), stat=alloc_status(5) )
        Allocate (  values4( file_channels ), stat=alloc_status(6) )
        If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "allocation of ssirem coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        End If
        Read(file_lu, iostat = io_status )&
               & ivalues0, &
               & values0,  &
               & values1,  &
               & values2,  &
               & values3,  &
               & values4 
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
        coef % ssirem_chn(:)    = ivalues0 ( channels(:) )
        coef % ssirem_a0(:)     =  values0 ( channels(:) )
        coef % ssirem_a1(:)     =  values1 ( channels(:) )
        coef % ssirem_a2(:)     =  values2 ( channels(:) )
        coef % ssirem_xzn1(:)   =  values3 ( channels(:) )
        coef % ssirem_xzn2(:)   =  values4 ( channels(:) )
        Deallocate ( ivalues0, stat=alloc_status(1) )
        Deallocate (  values0, stat=alloc_status(2) )
        Deallocate (  values1, stat=alloc_status(3) )
        Deallocate (  values2, stat=alloc_status(4) )
        Deallocate (  values3, stat=alloc_status(5) )
        Deallocate (  values4, stat=alloc_status(6) )
        If( Any(alloc_status /= 0) ) Then
           errorstatus = errorstatus_fatal
           Write( errMessage, '( "deallocation of ssirem coefs arrays")' )
           Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Endif
  Endif

  Allocate ( coef % ref_prfl_p ( coef % fmv_lvl(gas_id_mixed) ), stat=alloc_status(1) )
  Allocate ( coef % ref_prfl_t ( coef % fmv_lvl(gas_id_mixed), coef % fmv_gas ), stat=alloc_status(2) )
  Allocate ( coef % ref_prfl_mr( coef % fmv_lvl(gas_id_mixed), coef % fmv_gas ), stat=alloc_status(3) )

  Allocate ( coef % lim_prfl_p( coef % fmv_lvl(gas_id_mixed) ), stat=alloc_status(4) )
  Allocate ( coef % lim_prfl_tmax( coef % fmv_lvl(gas_id_mixed) ), stat=alloc_status(5) )
  Allocate ( coef % lim_prfl_tmin( coef % fmv_lvl(gas_id_mixed) ), stat=alloc_status(6) )
  Allocate ( coef % lim_prfl_gmin( coef % fmv_lvl(gas_id_mixed), coef % fmv_gas ), stat=alloc_status(7) )
  Allocate ( coef % lim_prfl_gmax( coef % fmv_lvl(gas_id_mixed), coef % fmv_gas ), stat=alloc_status(8) )
  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "allocation of fmv coefs arrays")' )
     Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  errMessage = 'io status while reading REFERENCE PROFILE'
  Read(file_lu, iostat = io_status )&
         & coef % ref_prfl_p, &
         & coef % ref_prfl_t, &
         & coef % ref_prfl_mr 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  errMessage = 'io status while reading PROFILE LIMITS'
  Read(file_lu, iostat = io_status )&
         & coef % lim_prfl_p,    &
         & coef % lim_prfl_tmax, &
         & coef % lim_prfl_tmin, &
         & coef % lim_prfl_gmax, &
         & coef % lim_prfl_gmin 
  If(io_status /= 0) Then
     Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
     errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
     Return
  Endif

  ! FAST COEFFICIENT section
  errMessage = 'io status while reading Mixed gases coefs'
  If ( coef % ncmixed > 0 ) Then
     Allocate (                &
            & coef % mixedgas (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncmixed  ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of mixed gaz coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % mixedgas(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % mixedgas(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do

  Endif


  errMessage = 'io status while reading Water vapour coefs'
  If ( coef % ncwater > 0 ) Then
     Allocate (                &
            & coef % watervapour (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncwater  ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of water vapour coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % watervapour(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % watervapour(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do

  Endif

  errMessage = 'io status while reading Ozone coefs'
  If ( coef % ncozone > 0 ) Then
     Allocate (                &
            & coef % ozone (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncozone  ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of ozone coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % ozone(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % ozone(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif

  errMessage = 'io status while reading WV continuum coefs'
  If ( coef % ncwvcont > 0 ) Then
     Allocate (                &
            & coef % wvcont (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncwvcont ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of WV continuum coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % wvcont(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % wvcont(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif

  errMessage = 'io status while reading CO2 coefs'
  If ( coef % ncco2 > 0 ) Then
     Allocate (                &
            & coef % co2      (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncco2    ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of CO2 coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % co2(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % co2(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif

  errMessage = 'io status while reading N2O coefs'
  If ( coef % ncn2o > 0 ) Then
     Allocate (                &
            & coef % n2o      (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncn2o    ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of N2O coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % n2o(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % n2o(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif

  errMessage = 'io status while reading CO coefs'
  If ( coef % ncco > 0 ) Then
     Allocate (                &
            & coef % co       (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncco     ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of CO coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % co(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % co(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif

  errMessage = 'io status while reading CH4 coefs'
  If ( coef % ncch4 > 0 ) Then
     Allocate (                &
            & coef % ch4      (  &
            & coef % nlevels  ,  &
            & coef % fmv_chn  ,  &
            & coef % ncch4    ), &
            & stat= alloc_status(1)) 
     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of CH4 coefs array")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     i = 1
     Do chn = 1, file_channels
        If( all_channels ) Then
           Read(file_lu, iostat = io_status ) coef % ch4(: , chn , :)
        Else If( chn == channels( i ) ) Then
           Read(file_lu, iostat = io_status ) coef % ch4(: , i , :)
           If( i < coef % fmv_chn ) Then
              i = i + 1
           End If
        Else
           Read(file_lu, iostat = io_status )
        End If
        If(io_status /= 0) Then
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage,NameOfRoutine)
           errorstatus = errorstatus_fatal
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
           Return
        Endif
     End Do
  Endif



! COEF_SCATT_IR aerosols structure (V9)
  Else If ( Present(aerosl) .and. coef % id_comp_lvl >= 9 ) Then
     If( .Not. all_channels ) Then
       Allocate(list_of_channels(size(channels)))
       list_of_channels = channels
     Else
       Allocate(list_of_channels(coef % fmv_chn))
       list_of_channels = (/ (i, i=1,coef % fmv_chn) /)
     End If

     !AEROSOLS_COMPONENTS
     read(file_lu, iostat=io_status) &
       & coef_scatt_ir % fmv_aer_chn, &
       & coef_scatt_ir % fmv_aer_pha_chn, &
       & coef_scatt_ir % fmv_aer_sun_chn, &
       & coef_scatt_ir % fmv_aer_comp, &
       & coef_scatt_ir % fmv_aer_ph

     ! Take care of the user list of channels
     ! file_channels store the number of channels in the file
     ! coef % fmv_aer_chn is the number of channels that the user requests
     file_channels = coef_scatt_ir % fmv_aer_chn
     If( .Not. all_channels ) Then
        coef_scatt_ir % fmv_aer_chn = Size( channels )
     Endif

     Allocate ( coef_scatt_ir % fmv_aer_ph_val ( coef_scatt_ir % fmv_aer_ph ), stat=alloc_status(1))
     Allocate ( coef_scatt_ir % fmv_aer_ph_val_cos ( coef_scatt_ir % fmv_aer_ph ), stat=alloc_status(2))

     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir %fmv_aer_ph_val

     Do i=1,coef_scatt_ir %fmv_aer_ph
       coef_scatt_ir % fmv_aer_ph_val_cos(i)=cos(coef_scatt_ir % fmv_aer_ph_val(i)*deg2rad)
     End Do

     coef_scatt_ir % fmv_aer_ph_val_min=99999.0_JPRB
     do i=1,coef_scatt_ir %fmv_aer_ph-1
       if(coef_scatt_ir % fmv_aer_ph_val_min > coef_scatt_ir % fmv_aer_ph_val(i+1)-coef_scatt_ir % fmv_aer_ph_val(i))then
         coef_scatt_ir % fmv_aer_ph_val_min=coef_scatt_ir % fmv_aer_ph_val(i+1)-coef_scatt_ir % fmv_aer_ph_val(i)
       endif
     enddo
     coef_scatt_ir % fmv_aer_ph_val_min=coef_scatt_ir % fmv_aer_ph_val_min/2.0_JPRB

     icount=coef_scatt_ir % fmv_aer_ph_val(coef_scatt_ir %fmv_aer_ph)/coef_scatt_ir % fmv_aer_ph_val_min
     Allocate ( coef_scatt_ir % ifmv_aer_ph_val(icount))

     k=1
     do i=1,icount
       if(coef_scatt_ir % fmv_aer_ph_val(k) >= (i+1)*coef_scatt_ir % fmv_aer_ph_val_min)then
         coef_scatt_ir % ifmv_aer_ph_val(i)=k
       else
         k=k+1
         coef_scatt_ir % ifmv_aer_ph_val(i)=k
       endif
     enddo

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_aer_sun_chn)then
         is=is+1
       endif
     enddo

     pha_channels=coef_scatt_ir % fmv_aer_pha_chn
     if(is/=coef_scatt_ir % fmv_aer_pha_chn)then
       coef_scatt_ir % fmv_aer_pha_chn=is
     endif

     ! allocate arrays of FAST_MODEL_VARIABLES section
     Allocate ( coef_scatt_ir % fmv_aer_rh ( coef_scatt_ir % fmv_aer_comp ), stat=alloc_status(3))
     Allocate ( coef_scatt_ir % channels_solar ( is ), stat=alloc_status(4))
     Allocate ( optp%optpaer (coef_scatt_ir % fmv_aer_comp ), stat=alloc_status(5))

     If( Any(alloc_status /= 0) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "allocation of inso optp arrays")' )
        Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     Do n = 1, coef_scatt_ir % fmv_aer_comp
        optp % optpaer(n) % dim  = 0
        optp % optpaer(n) % fmv_aer_chn = 0
        optp % optpaer(n) % fmv_wcl_chn = 0
        optp % optpaer(n) % fmv_icl_chn = 0
        optp % optpaer(n) % fmv_aer_pha_chn = 0
        optp % optpaer(n) % fmv_wcl_pha_chn = 0
        optp % optpaer(n) % fmv_icl_pha_chn = 0
        optp % optpaer(n) % fmv_aer_sun_chn = 0
        optp % optpaer(n) % fmv_wcl_sun_chn = 0
        optp % optpaer(n) % fmv_icl_sun_chn = 0
        optp % optpaer(n) % fmv_aer_comp = 0
        optp % optpaer(n) % fmv_wcl_comp = 0
        optp % optpaer(n) % fmv_icl_comp = 0
        optp % optpaer(n) % fmv_icl_ishp = 0
        optp % optpaer(n) % fmv_aer_pha_ioff = 0
        optp % optpaer(n) % fmv_wcl_pha_ioff = 0
        optp % optpaer(n) % fmv_icl_pha_ioff = 0
        optp % optpaer(n) % fmv_aer_ph = 0
        optp % optpaer(n) % fmv_wcl_ph = 0
        optp % optpaer(n) % fmv_icl_ph = 0
        optp % optpaer(n) % icl_nabs = 0
        optp % optpaer(n) % icl_nsca = 0
        optp % optpaer(n) % icl_nbpr = 0
        Nullify( optp % optpaer(n) % fmv_aer_rh )
        Nullify( optp % optpaer(n) % fmv_wcl_rh )
        Nullify( optp % optpaer(n) % fmv_aer_rh_val )
        Nullify( optp % optpaer(n) % fmv_wcl_rh_val )
        Nullify( optp % optpaer(n) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpaer(n) % ifmv_wcl_ph_val )
        Nullify( optp % optpaer(n) % fmv_aer_ph_val )
        Nullify( optp % optpaer(n) % fmv_aer_ph_val_cos )
        Nullify( optp % optpaer(n) % ifmv_aer_ph_val )
        Nullify( optp % optpaer(n) % fmv_wcl_ph_val )
        Nullify( optp % optpaer(n) % fmv_icl_ph_val )
        Nullify( optp % optpaer(n) % fmv_icl_ph_val_cos )
        Nullify( optp % optpaer(n) % ifmv_icl_ph_val )
        Nullify( optp % optpaer(n) % fmv_icl_dg )
        Nullify( optp % optpaer(n) % channels_solar )
        Nullify( optp % optpaer(n) % abs )
        Nullify( optp % optpaer(n) % sca )
        Nullify( optp % optpaer(n) % bpr )
        Nullify( optp % optpaer(n) % pha )
        Nullify( optp % optpaer(n) % confac )
     enddo

     is=0
     coef_scatt_ir%fmv_aer_pha_ioff=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_aer_sun_chn)then
         coef_scatt_ir%fmv_aer_pha_ioff=i
         exit
       endif
     enddo

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_aer_sun_chn)then
         is=is+1
         coef_scatt_ir % channels_solar ( is )=list_of_channels(i)
       endif
     enddo


     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir % fmv_aer_rh

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Allocate ( optp%optpaer(n)% fmv_aer_rh_val ( coef_scatt_ir % fmv_aer_rh(n) ), stat=alloc_status(1))
       Read(file_lu, iostat=io_status) &
        & optp%optpaer(n)%fmv_aer_rh_val 
     End Do


     !'AEROSOLS_PARAMETERS'
     pha_1=coef_scatt_ir % fmv_aer_sun_chn
     pha_2=coef_scatt_ir % fmv_aer_sun_chn+pha_channels-1

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Allocate ( optp%optpaer(n) % abs(coef_scatt_ir % fmv_aer_chn, &
          & coef_scatt_ir % fmv_aer_rh(n)), stat= alloc_status(1)) 
       If ( all_channels ) Then
         Read(file_lu, iostat=io_status) &
         & optp%optpaer(n) % abs
       Else
         Allocate ( dvalues0( file_channels, coef_scatt_ir % fmv_aer_rh(n)), stat=alloc_status(2) )
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpaer(n) % abs(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Allocate ( optp%optpaer(n) % sca(coef_scatt_ir % fmv_aer_chn, &
          & coef_scatt_ir % fmv_aer_rh(n)), stat= alloc_status(1)) 
       If ( all_channels ) Then
         Read(file_lu, iostat=io_status) &
         & optp%optpaer(n) % sca
       Else
         Allocate ( dvalues0( file_channels, coef_scatt_ir % fmv_aer_rh(n)), stat=alloc_status(2) )
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpaer(n) % sca(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Allocate ( optp%optpaer(n) % bpr(coef_scatt_ir % fmv_aer_chn, &
          & coef_scatt_ir % fmv_aer_rh(n)), stat= alloc_status(1)) 
       If ( all_channels ) Then
         Read(file_lu, iostat=io_status) &
         & optp%optpaer(n) % bpr
       Else
         Allocate ( dvalues0( file_channels, coef_scatt_ir % fmv_aer_rh(n)), stat=alloc_status(2) )
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpaer(n) % bpr(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Allocate ( optp%optpaer(n) % pha(coef_scatt_ir % fmv_aer_pha_chn, &
          & coef_scatt_ir % fmv_aer_rh(n),coef_scatt_ir % fmv_aer_ph), stat= alloc_status(1)) 
       if(pha_channels>0)then
         If ( all_channels ) Then
           Read(file_lu, iostat=io_status) &
           & optp%optpaer(n) % pha
         Else
           Allocate ( tvalues0( & 
           & pha_1:pha_2, coef_scatt_ir%fmv_aer_rh(n), coef_scatt_ir%fmv_aer_ph), &
           & stat=alloc_status(2) )
           Read(file_lu, iostat=io_status) tvalues0
           optp%optpaer(n) % pha(:,:,:) = tvalues0(coef_scatt_ir%channels_solar(:),:,:)
           Deallocate ( tvalues0 )
         End If
       End If
     End Do

     Deallocate( list_of_channels )


! OPTP clouds structure (V9)
  Else If ( Present(clouds) .and. coef % id_comp_lvl >= 9 ) Then
     If( .Not. all_channels ) Then
       Allocate(list_of_channels(size(channels)))
       list_of_channels = channels
     Else
       Allocate(list_of_channels(coef % fmv_chn))
       list_of_channels = (/ (i, i=1,coef % fmv_chn) /)
     End If

     !WATERCLOUD_TYPES
     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir % fmv_wcl_chn, &
       & coef_scatt_ir % fmv_wcl_pha_chn, &
       & coef_scatt_ir % fmv_wcl_sun_chn, &
       & coef_scatt_ir % fmv_wcl_comp, &
       & coef_scatt_ir % fmv_wcl_ph

     ! Take care of the user list of channels
     ! file_channels store the number of channels in the file
     ! coef % fmv_chn is the number of channels that the user requests
     file_channels = coef_scatt_ir % fmv_wcl_chn
     If( .Not. all_channels ) Then
        coef_scatt_ir % fmv_wcl_chn = Size( channels )
     Endif

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_wcl_sun_chn)then
         is=is+1
       endif
     enddo

     pha_channels=coef_scatt_ir % fmv_wcl_pha_chn
     if(is/=coef_scatt_ir % fmv_wcl_pha_chn)then
       coef_scatt_ir % fmv_wcl_pha_chn=is
     endif

     Allocate ( coef_scatt_ir % fmv_wcl_ph_val ( coef_scatt_ir % fmv_wcl_ph ), stat=alloc_status(1))
     Allocate ( coef_scatt_ir % fmv_wcl_ph_val_cos ( coef_scatt_ir % fmv_wcl_ph ), stat=alloc_status(1))

     coef_scatt_ir % fmv_wcl_ph_val=0.

     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir %fmv_wcl_ph_val
     Do i=1,coef_scatt_ir %fmv_wcl_ph
       coef_scatt_ir % fmv_wcl_ph_val_cos(i)=cos(coef_scatt_ir % fmv_wcl_ph_val(i)*deg2rad)
     End Do
     coef_scatt_ir % fmv_wcl_ph_val_min=99999.0_JPRB
     do i=1,coef_scatt_ir %fmv_wcl_ph-1
       if(coef_scatt_ir % fmv_wcl_ph_val_min > coef_scatt_ir % fmv_wcl_ph_val(i+1)-coef_scatt_ir % fmv_wcl_ph_val(i))then
         coef_scatt_ir % fmv_wcl_ph_val_min=coef_scatt_ir % fmv_wcl_ph_val(i+1)-coef_scatt_ir % fmv_wcl_ph_val(i)
       endif
     enddo
     coef_scatt_ir % fmv_wcl_ph_val_min=coef_scatt_ir % fmv_wcl_ph_val_min/2.0_JPRB

     icount=coef_scatt_ir % fmv_wcl_ph_val(coef_scatt_ir %fmv_wcl_ph)/coef_scatt_ir % fmv_wcl_ph_val_min
     Allocate ( coef_scatt_ir % ifmv_wcl_ph_val(icount))

     k=1
     do i=1,icount
       if(coef_scatt_ir % fmv_wcl_ph_val(k) >= (i+1)*coef_scatt_ir % fmv_wcl_ph_val_min)then
         coef_scatt_ir % ifmv_wcl_ph_val(i)=k
       else
         k=k+1
         coef_scatt_ir % ifmv_wcl_ph_val(i)=k
       endif
     enddo

     Allocate ( coef_scatt_ir % fmv_wcl_rh ( coef_scatt_ir % fmv_wcl_comp ), stat=alloc_status(1))
     Allocate ( coef_scatt_ir % channels_solar ( is ), stat=alloc_status(2))
     Allocate ( coef_scatt_ir % confac ( coef_scatt_ir % fmv_wcl_comp ), stat=alloc_status(3))
     Allocate ( optp%optpwcl (coef_scatt_ir % fmv_wcl_comp ), stat=alloc_status(4))
     If( Any(alloc_status /= 0) ) Then
         errorstatus = errorstatus_fatal
         Write( errMessage, '( "allocation of inso optp arrays")' )
         Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
         Return
     End If

     Do n = 1, coef_scatt_ir % fmv_wcl_comp

        optp % optpwcl(n) % dim  = 0
        optp % optpwcl(n) % fmv_aer_chn = 0
        optp % optpwcl(n) % fmv_wcl_chn = 0
        optp % optpwcl(n) % fmv_icl_chn = 0
        optp % optpwcl(n) % fmv_aer_pha_chn = 0
        optp % optpwcl(n) % fmv_wcl_pha_chn = 0
        optp % optpwcl(n) % fmv_icl_pha_chn = 0
        optp % optpwcl(n) % fmv_aer_sun_chn = 0
        optp % optpwcl(n) % fmv_wcl_sun_chn = 0
        optp % optpwcl(n) % fmv_icl_sun_chn = 0
        optp % optpwcl(n) % fmv_aer_comp = 0
        optp % optpwcl(n) % fmv_wcl_comp = 0
        optp % optpwcl(n) % fmv_icl_comp = 0
        optp % optpwcl(n) % fmv_icl_ishp = 0
        optp % optpwcl(n) % fmv_aer_pha_ioff = 0
        optp % optpwcl(n) % fmv_wcl_pha_ioff = 0
        optp % optpwcl(n) % fmv_icl_pha_ioff = 0
        optp % optpwcl(n) % fmv_aer_ph = 0
        optp % optpwcl(n) % fmv_wcl_ph = 0
        optp % optpwcl(n) % fmv_icl_ph = 0
        optp % optpwcl(n) % icl_nabs = 0
        optp % optpwcl(n) % icl_nsca = 0
        optp % optpwcl(n) % icl_nbpr = 0
        Nullify( optp % optpwcl(n) % fmv_aer_rh )
        Nullify( optp % optpwcl(n) % fmv_wcl_rh )
        Nullify( optp % optpwcl(n) % fmv_aer_rh_val )
        Nullify( optp % optpwcl(n) % fmv_wcl_rh_val )
        Nullify( optp % optpwcl(n) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpwcl(n) % ifmv_wcl_ph_val )
        Nullify( optp % optpwcl(n) % fmv_aer_ph_val )
        Nullify( optp % optpwcl(n) % fmv_aer_ph_val_cos )
        Nullify( optp % optpwcl(n) % ifmv_aer_ph_val )
        Nullify( optp % optpwcl(n) % fmv_wcl_ph_val )
        Nullify( optp % optpwcl(n) % fmv_icl_ph_val )
        Nullify( optp % optpwcl(n) % fmv_icl_ph_val_cos )
        Nullify( optp % optpwcl(n) % ifmv_icl_ph_val )
        Nullify( optp % optpwcl(n) % fmv_icl_dg )
        Nullify( optp % optpwcl(n) % channels_solar )
        Nullify( optp % optpwcl(n) % abs )
        Nullify( optp % optpwcl(n) % sca )
        Nullify( optp % optpwcl(n) % bpr )
        Nullify( optp % optpwcl(n) % pha )
        Nullify( optp % optpwcl(n) % confac )
     enddo

     is=0
     coef_scatt_ir%fmv_wcl_pha_ioff=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_wcl_sun_chn)then
         coef_scatt_ir%fmv_wcl_pha_ioff=i
         exit
       endif
     enddo

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_wcl_sun_chn)then
         is=is+1
         coef_scatt_ir % channels_solar ( is )=list_of_channels(i)
       endif
     enddo

     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir % fmv_wcl_rh
     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir%confac

     Do n = 1, coef_scatt_ir % fmv_wcl_comp 
       Allocate ( optp%optpwcl(n)% fmv_wcl_rh_val ( & 
          & coef_scatt_ir % fmv_wcl_rh(n) ), stat=alloc_status(1))
       Read(file_lu, iostat=io_status) &
         & optp%optpwcl(n)%fmv_wcl_rh_val
     End Do

     !WATERCLOUD_PARAMETERS
     pha_1=coef_scatt_ir % fmv_wcl_sun_chn
     pha_2=coef_scatt_ir % fmv_wcl_sun_chn+pha_channels-1

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Allocate ( optp%optpwcl(n) % abs(coef_scatt_ir % fmv_wcl_chn, &
          & coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(1)) 
       If( all_channels ) Then 
         Read(file_lu, iostat=io_status) &
           & optp%optpwcl(n) % abs
       Else
         Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(2))
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpwcl(n) % abs(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Allocate ( optp%optpwcl(n) % sca(coef_scatt_ir % fmv_wcl_chn, &
          & coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(1)) 
       If( all_channels ) Then 
         Read(file_lu, iostat=io_status) &
           & optp%optpwcl(n) % sca
       Else
         Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(2))
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpwcl(n) % sca(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Allocate ( optp%optpwcl(n) % bpr(coef_scatt_ir % fmv_wcl_chn, &
          & coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(1)) 
       If( all_channels ) Then 
         Read(file_lu, iostat=io_status) &
           & optp%optpwcl(n) % bpr
       Else
         Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % fmv_wcl_rh(n)), stat= alloc_status(2))
         Read(file_lu, iostat=io_status) dvalues0
         optp%optpwcl(n) % bpr(:,:) = dvalues0(channels(:),:)
         Deallocate ( dvalues0 )
       End If
     End Do

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Allocate ( optp%optpwcl(n) % pha(     &
          & coef_scatt_ir % fmv_wcl_pha_chn, &
          & coef_scatt_ir % fmv_wcl_rh(n),   &
          & coef_scatt_ir % fmv_wcl_ph), stat= alloc_status(1)) 
       if(pha_channels>0)then
         If( all_channels ) Then 
           Read(file_lu, iostat=io_status) &
             & optp%optpwcl(n) % pha
         Else
           Allocate( tvalues0( &
           & pha_1:pha_2,      &
           & coef_scatt_ir % fmv_wcl_rh(n), &
           & coef_scatt_ir % fmv_wcl_ph), stat= alloc_status(2))
           Read(file_lu, iostat=io_status) tvalues0
           optp%optpwcl(n) % pha(:,:,:) = &
           & tvalues0(coef_scatt_ir%channels_solar(:),:,:)
           Deallocate ( tvalues0 )
         End If
       End If
     End Do


     !ICECLOUD_TYPES
     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir % fmv_icl_chn, &
       & coef_scatt_ir % fmv_icl_pha_chn, &
       & coef_scatt_ir % fmv_icl_sun_chn, &
       & coef_scatt_ir % icl_nabs, &
       & coef_scatt_ir % icl_nsca, &
       & coef_scatt_ir % icl_nbpr, &
       & coef_scatt_ir % fmv_icl_comp, &
       & coef_scatt_ir % fmv_icl_ishp, &
       & coef_scatt_ir % fmv_icl_ph

     ! Take care of the user list of channels
     ! file_channels store the number of channels in the file
     ! coef % fmv_chn is the number of channels that the user requests
     file_channels = coef_scatt_ir % fmv_icl_chn
     If( .Not. all_channels ) Then
        coef_scatt_ir % fmv_icl_chn = Size( channels )
     Endif

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_icl_sun_chn)then
         is=is+1
       endif
     enddo

     pha_channels=coef_scatt_ir % fmv_icl_pha_chn
     if(is/=coef_scatt_ir % fmv_icl_pha_chn)then
       coef_scatt_ir % fmv_icl_pha_chn=is
     endif 

     Allocate ( coef_scatt_ir % fmv_icl_dg ( coef_scatt_ir % fmv_icl_comp,            &
              & coef_scatt_ir % fmv_icl_ishp ), stat=alloc_status(1)) 
     Allocate ( coef_scatt_ir % fmv_icl_ph_val ( coef_scatt_ir % fmv_icl_ph ), stat=alloc_status(2))
     Allocate ( coef_scatt_ir % fmv_icl_ph_val_cos ( coef_scatt_ir % fmv_icl_ph ), stat=alloc_status(3))
     Allocate ( optp%optpicl (coef_scatt_ir % fmv_icl_ishp ), stat=alloc_status(4))

     If( Any(alloc_status /= 0) ) Then
       errorstatus = errorstatus_fatal
       Write( errMessage, '( "allocation of effective diameter arrays")' )
       Call Rttov_ErrorReport (errorstatus, errMessage,NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)
       Return
     End If

     Do n = 1, coef_scatt_ir % fmv_icl_ishp
        optp % optpicl(n) % dim  = 0
        optp % optpicl(n) % fmv_aer_chn = 0
        optp % optpicl(n) % fmv_wcl_chn = 0
        optp % optpicl(n) % fmv_icl_chn = 0
        optp % optpicl(n) % fmv_aer_pha_chn = 0
        optp % optpicl(n) % fmv_wcl_pha_chn = 0
        optp % optpicl(n) % fmv_icl_pha_chn = 0
        optp % optpicl(n) % fmv_aer_sun_chn = 0
        optp % optpicl(n) % fmv_wcl_sun_chn = 0
        optp % optpicl(n) % fmv_icl_sun_chn = 0
        optp % optpicl(n) % fmv_aer_comp = 0
        optp % optpicl(n) % fmv_wcl_comp = 0
        optp % optpicl(n) % fmv_icl_comp = 0
        optp % optpicl(n) % fmv_icl_ishp = 0
        optp % optpicl(n) % fmv_aer_pha_ioff = 0
        optp % optpicl(n) % fmv_wcl_pha_ioff = 0
        optp % optpicl(n) % fmv_icl_pha_ioff = 0
        optp % optpicl(n) % fmv_aer_ph = 0
        optp % optpicl(n) % fmv_wcl_ph = 0
        optp % optpicl(n) % fmv_icl_ph = 0
        optp % optpicl(n) % icl_nabs = 0
        optp % optpicl(n) % icl_nsca = 0
        optp % optpicl(n) % icl_nbpr = 0
        Nullify( optp % optpicl(n) % fmv_aer_rh )
        Nullify( optp % optpicl(n) % fmv_wcl_rh )
        Nullify( optp % optpicl(n) % fmv_aer_rh_val )
        Nullify( optp % optpicl(n) % fmv_wcl_rh_val )
        Nullify( optp % optpicl(n) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpicl(n) % ifmv_wcl_ph_val )
        Nullify( optp % optpicl(n) % fmv_aer_ph_val )
        Nullify( optp % optpicl(n) % fmv_aer_ph_val_cos )
        Nullify( optp % optpicl(n) % ifmv_aer_ph_val )
        Nullify( optp % optpicl(n) % fmv_wcl_ph_val )
        Nullify( optp % optpicl(n) % fmv_icl_ph_val )
        Nullify( optp % optpicl(n) % fmv_icl_ph_val_cos )
        Nullify( optp % optpicl(n) % ifmv_icl_ph_val )
        Nullify( optp % optpicl(n) % fmv_icl_dg )
        Nullify( optp % optpicl(n) % channels_solar )
        Nullify( optp % optpicl(n) % abs )
        Nullify( optp % optpicl(n) % sca )
        Nullify( optp % optpicl(n) % bpr )
        Nullify( optp % optpicl(n) % pha )
        Nullify( optp % optpicl(n) % confac )
     End Do


     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_ph_val
     Do i=1,coef_scatt_ir %fmv_icl_ph
          coef_scatt_ir % fmv_icl_ph_val_cos(i)=cos(coef_scatt_ir % fmv_icl_ph_val(i)*deg2rad)
     End Do

     coef_scatt_ir % fmv_icl_ph_val_min=99999.0_JPRB
     do i=1,coef_scatt_ir %fmv_icl_ph-1
       if(coef_scatt_ir % fmv_icl_ph_val_min > coef_scatt_ir % fmv_icl_ph_val(i+1)-coef_scatt_ir % fmv_icl_ph_val(i))then
          coef_scatt_ir % fmv_icl_ph_val_min=coef_scatt_ir % fmv_icl_ph_val(i+1)-coef_scatt_ir % fmv_icl_ph_val(i)
       endif
     enddo
     coef_scatt_ir % fmv_icl_ph_val_min=coef_scatt_ir % fmv_icl_ph_val_min/2.0_JPRB

     icount=coef_scatt_ir % fmv_icl_ph_val(coef_scatt_ir %fmv_icl_ph)/coef_scatt_ir % fmv_icl_ph_val_min
     Allocate ( coef_scatt_ir % ifmv_icl_ph_val(icount))

     k=1
     do i=1,icount
       if(coef_scatt_ir % fmv_icl_ph_val(k) >= (i+1)*coef_scatt_ir % fmv_icl_ph_val_min)then
         coef_scatt_ir % ifmv_icl_ph_val(i)=k
       else
         k=k+1
         coef_scatt_ir % ifmv_icl_ph_val(i)=k
       endif
     enddo

     Allocate ( coef_scatt_ir % channels_solar ( is ), stat=alloc_status(2))

     is=0
     coef_scatt_ir%fmv_icl_pha_ioff=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_icl_sun_chn)then
         coef_scatt_ir%fmv_icl_pha_ioff=i
         exit
       endif
     enddo

     is=0
     do i=1,size(list_of_channels)
       if(list_of_channels(i)>=coef_scatt_ir % fmv_icl_sun_chn)then
         is=is+1
         coef_scatt_ir % channels_solar ( is )=list_of_channels(i)
       endif
     enddo

     !HEXAGONAL_PARAMETERS

     pha_1=coef_scatt_ir % fmv_icl_sun_chn
     pha_2=coef_scatt_ir % fmv_icl_sun_chn+pha_channels-1

     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_dg(:,1)


     Allocate ( optp%optpicl(1) % abs(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nabs), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(1) % abs
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nabs), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(1) % abs(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(1) % sca(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nsca), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(1) % sca
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nsca), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(1) % sca(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(1) % bpr(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nbpr), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(1) % bpr
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nbpr), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(1) % bpr(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(1) % pha( &
          & coef_scatt_ir % fmv_icl_pha_chn, &
          & coef_scatt_ir % fmv_icl_comp,    &
          & coef_scatt_ir % fmv_icl_ph), stat= alloc_status(1)) 
     if(pha_channels>0)then
       If( all_channels ) Then 
         Read(file_lu, iostat=io_status) &
           & optp%optpicl(1) % pha
       Else
         Allocate( tvalues0( &
           & pha_1:pha_2,  &
           & coef_scatt_ir % fmv_icl_comp,    &
           & coef_scatt_ir % fmv_icl_ph), stat= alloc_status(2)) 
         Read(file_lu, iostat=io_status) tvalues0
         optp%optpicl(1) % pha(:,:,:) = tvalues0(coef_scatt_ir%channels_solar(:),:,:)
         Deallocate ( tvalues0 )
       End If
     End If


     !AGGREGATE_PARAMETERS
     pha_1=coef_scatt_ir % fmv_icl_sun_chn
     pha_2=coef_scatt_ir % fmv_icl_sun_chn+pha_channels-1

     Read(file_lu, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_dg(:,2)


     Allocate ( optp%optpicl(2) % abs(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nabs), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(2) % abs
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nabs), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(2) % abs(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(2) % sca(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nsca), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(2) % sca
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nsca), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(2) % sca(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(2) % bpr(coef_scatt_ir % fmv_icl_chn, &
          & coef_scatt_ir % icl_nbpr), stat= alloc_status(1))
     If( all_channels ) Then 
       Read(file_lu, iostat=io_status) &
         & optp%optpicl(2) % bpr
     Else
       Allocate( dvalues0( &
         & file_channels, coef_scatt_ir % icl_nbpr), stat= alloc_status(2))
       Read(file_lu, iostat=io_status) dvalues0
       optp%optpicl(2) % bpr(:,:) = dvalues0(channels(:),:)
       Deallocate ( dvalues0 )
     End If

     Allocate ( optp%optpicl(2) % pha( &
          & coef_scatt_ir % fmv_icl_pha_chn, &
          & coef_scatt_ir % fmv_icl_comp,    &
          & coef_scatt_ir % fmv_icl_ph), stat= alloc_status(1)) 
     if(pha_channels>0)then
       If( all_channels ) Then 
         Read(file_lu, iostat=io_status) &
           & optp%optpicl(2) % pha
       Else
         Allocate( tvalues0( &
           & pha_1:pha_2,  &
           & coef_scatt_ir % fmv_icl_comp,    &
           & coef_scatt_ir % fmv_icl_ph), stat= alloc_status(2)) 
         Read(file_lu, iostat=io_status) tvalues0
         optp%optpicl(2) % pha(:,:,:) = tvalues0(coef_scatt_ir%channels_solar(:),:,:)
         Deallocate ( tvalues0 )
       End If
     End If

     Deallocate( list_of_channels )

  End If

IF (LHOOK) CALL DR_HOOK('RTTOV_READCOEFFS_BINARY',1_jpim,ZHOOK_HANDLE)

  !
  ! Here add reading of new sections for binary format in order to keep compatibility with
  ! previous versions
  !



End Subroutine rttov_readcoeffs_binary

