!
Subroutine rttov_scatt(   &
      & errorstatus,       &! out
      & nlevels,           &! in
      & nchannels,         &! in
      & nprofiles,         &! in
      & channels,          &! in
      & frequencies,       &! in
      & lprofiles,         &! in
      & profiles,          &! in
      & cld_profiles,      &! in
      & coef_rttov,        &! in
      & coef_scatt,        &! in
      & calcemiss,         &! in
      & emissivity_in,     &! inout
      & radiance,          &! inout
      & cfrac,             &! out  (diagnostic only)
      & lnewcld)            ! in, optional
  ! Description:
  ! to compute microwave multi-channel radiances and brightness
  ! temperatures for many profiles per call in a cloudy and/or rainy sky.
  !
  ! Copyright:
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method / Validation :
  ! --------------------
  ! - Bauer, P., 2002: Microwave radiative transfer modeling in clouds and precipitation.
  !     Part I: Model description.
  !     NWP SAF Report No. NWPSAF-EC-TR-005, 21 pp.
  ! - Moreau, E., P. Bauer and F. Chevallier, 2002: Microwave radiative transfer modeling in clouds and precipitation.
  !     Part II: Model evaluation.
  !     NWP SAF Report No. NWPSAF-EC-TR-006, 27 pp.
  ! - Chevallier, F., and P. Bauer, 2003:
  !     Model rain and clouds over oceans:comparison with SSM/I observations. Mon. Wea. Rev., 131, 1240-1255.
  ! - Smith, E. A., P. Bauer, F. S. Marzano, C. D. Kummerow, D. McKague, A. Mugnai, G. Panegrossi, 2002:
  !     Intercomparison of microwave radiative transfer models for precipitating clouds.
  !     IEEE Trans. Geosci. Remote Sens. 40, 541-549.
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !   1.0    09/2002      Initial version     (F. Chevallier)
  !   1.1    05/2003      RTTOV7.3 compatible (F. Chevallier)
  !   1.2    03/2004      Added polarimetry   (R. Saunders)
  !   1.3    08/2004      Polarimetry fixes   (U. O'Keeffe)
  !   1.4    11/2004      Clean-up            (P. Bauer)
  !   1.5    07/2005      Polarimetry fixes after re-write (U O'Keeffe)
  !   1.6    11/2005      Add errorstatus to iniscatt arguments and use a temporary
  !                       radiance type for the calcpolarisation call (J Cameron)
  !   1.7    11/2007      RTTOV9 version      (A. Geer)
  !   1.8    07/2008      Clear sky speed-ups (A. Geer)
  !   1.9    10/2008      Revised cloud partitioning (A. Geer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !   Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  
  USE YOMHOOK, ONLY: LHOOK , DR_HOOK
  
  Use rttov_const, Only :   &
        & sensor_id_mw        ,&
        & errorstatus_success ,&
        & errorstatus_fatal   ,&
        & ccthres  

  Use rttov_types, Only :    &
        & rttov_coef           ,&
        & rttov_scatt_coef     ,&
        & rttov_coef_scatt_ir  ,&
        & rttov_optpar_ir      ,&
        & geometry_Type        ,&
        & profile_Type         ,&
        & profile_cloud_Type   ,&
        & profile_scatt_aux    ,&
        & transmission_Type    ,&
        & radiance_Type        

  Use parkind1, Only : jpim     ,jprb

  Implicit None


!* Subroutine arguments:
  Integer (Kind=jpim), Intent (in)  :: nlevels ! Number of levels
  Integer (Kind=jpim), Intent (in)  :: nprofiles  ! Number of profiles
  Integer (Kind=jpim), Intent (in)  :: nchannels  ! Number of channels*profiles=radiances
  Integer (Kind=jpim), Intent (in)  :: channels    (nchannels) ! Channel indices
  Integer (Kind=jpim), Intent (in)  :: frequencies (nchannels) ! Frequency indices
  Integer (Kind=jpim), Intent (in)  :: lprofiles   (nchannels) ! Profile indices
  Integer (Kind=jpim), Intent (out) :: errorstatus (nprofiles) ! Error return flag

  Logical,             Intent (in)  :: calcemiss     (nchannels)  ! Switch for emmissivity calculation
  Real    (Kind=jprb), Intent (in)  :: emissivity_in (nchannels)  ! Surface emmissivity 

  Type (profile_Type),        Intent (in)    :: profiles     (nprofiles)  
  Type (rttov_coef),          Intent (in)    :: coef_rttov               ! RTTOV Coefficients
  Type (rttov_scatt_coef),    Intent (in)    :: coef_scatt               ! RTTOV_SCATT Coefficients
  Type (profile_cloud_Type),  Intent (in)    :: cld_profiles (nprofiles) ! Cloud profiles 
  Type (radiance_Type),       Intent (inout) :: radiance                 ! Radiances
 
  Real (Kind=jprb), Intent (out)  :: cfrac (nprofiles)  ! Diagnostic 2-IC cloud fraction

  Logical, optional, Intent(in)   :: lnewcld ! Switch on revised cloud/rain treatment

!INTF

#include "rttov_direct.interface"
#include "rttov_iniscatt.interface"
#include "rttov_eddington.interface"
#include "rttov_errorreport.interface"
 
  Integer (Kind=jpim), target :: sa__mclayer (nchannels)
 
  Real (Kind=jprb), target :: t__tau_total  (nchannels)
  Real (Kind=jprb), target :: t__tau_layers (nlevels,nchannels)

  Real (Kind=jprb), target :: sa__cfrac   (nprofiles)  
  Real (Kind=jprb), target :: sa__ems_bnd (nchannels)
  Real (Kind=jprb), target :: sa__ref_bnd (nchannels)
  Real (Kind=jprb), target :: sa__ems_cld (nchannels)
  Real (Kind=jprb), target :: sa__ref_cld (nchannels)
  
  Real (Kind=jprb), target :: sa__tbd (nprofiles,nlevels+1)
  
  Real (Kind=jprb), target :: sa__delta  (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__tau    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__ext    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__ssa    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__asm    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__lambda (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__h      (nchannels,nlevels)
  
  Real (Kind=jprb), target :: sa__b0     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__b1     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__bn     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__dz     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__clw    (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__ciw    (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__totalice (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__rain   (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__sp     (nprofiles,nlevels)

!* Local variables:
  Logical             :: addinterp, addcloud, addsun, addaerosol, switchrad
  Logical             :: usenewcld
  Integer (Kind=jpim) :: iprof, ichan   
  Real    (Kind=jprb) :: emissivity (nchannels)           
  Real    (Kind=jprb) :: cld_bt     (nchannels)           
  
  Type (transmission_Type) :: transmission  
  Type (geometry_Type)     :: angles (nprofiles)
  Type (profile_scatt_aux) :: scatt_aux  
  Type (rttov_optpar_ir)   :: optp   
  Type (rttov_coef_scatt_ir) :: coef_scatt_ir                             

  Character (len=80) :: errMessage
  Character (len=15) :: NameOfRoutine = 'rttov_scatt '
  
  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  
  !- End of header --------------------------------------------------------

  if (lhook) call dr_hook('RTTOV_SCATT',0_jpim,zhook_handle)

  usenewcld = .false.
  if(present(lnewcld)) then
    if(lnewcld) usenewcld = .true.
  endif
  
  errorstatus(:)  = errorstatus_success
  
  ! We do not make use of IR aerosols or scattering and in that case, the 
  ! following coefficients are not used by RTTOV_DIRECT, except for initialising
  ! an unused target variable. Ideally I would make optp and coef_scatt_ir
  ! optional.
  coef_scatt_ir % fmv_aer_comp = 1
  coef_scatt_ir % fmv_wcl_comp = 1

  transmission % tau_total  => t__tau_total
  transmission % tau_layers => t__tau_layers
 
  scatt_aux % cfrac    => sa__cfrac
  scatt_aux % ems_bnd  => sa__ems_bnd
  scatt_aux % ref_bnd  => sa__ref_bnd
  scatt_aux % ems_cld  => sa__ems_cld
  scatt_aux % ref_cld  => sa__ref_cld
  scatt_aux % tbd      => sa__tbd
  scatt_aux % mclayer  => sa__mclayer
  scatt_aux % delta    => sa__delta
  scatt_aux % tau      => sa__tau
  scatt_aux % ext      => sa__ext
  scatt_aux % ssa      => sa__ssa
  scatt_aux % asm      => sa__asm
  scatt_aux % lambda   => sa__lambda
  scatt_aux % h        => sa__h
  scatt_aux % b0       => sa__b0
  scatt_aux % b1       => sa__b1
  scatt_aux % bn       => sa__bn
  scatt_aux % dz       => sa__dz
  scatt_aux % clw      => sa__clw
  scatt_aux % ciw      => sa__ciw
  scatt_aux % totalice => sa__totalice
  scatt_aux % rain     => sa__rain
  scatt_aux % sp       => sa__sp
       
!*         1.   Gas absorption
  switchrad = .true.   ! input to RTTOV is BT  

  ! Profiles will be interpolated from model/RTTOV-SCATT levels to 
  ! RTTOV coefficient levels within RTTOV itself.
  addinterp  = .true.  
  addcloud   = .false.
  addsun     = .false.
  addaerosol = .false.
  
  emissivity (:) = emissivity_in (:)
  Call rttov_direct(          & ! 
         & errorstatus,        &! out
         & nprofiles,          &! in
         & nchannels,          &! in
         & channels,           &! in
         & lprofiles,          &! in
         & addinterp,          &! in
         & profiles,           &! in
         & coef_rttov,         &! in
         & coef_scatt_ir,      &! in (not used here)
         & optp,               &! in (not used here)
         & addsun,             &! in
         & addaerosol,         &! in
         & addcloud,           &! in
         & calcemiss,          &! in
         & emissivity,         &! inout
         & transmission,       &! inout
         & radiance     )       ! inout 


  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_direct")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  scatt_aux % ems_cld (:) = emissivity_in (:)      
  scatt_aux % ref_cld (:) = 1.0_JPRB - emissivity_in (:)      
  
!*  2.   Initialisations for Eddington
  Call rttov_iniscatt(       &
         & errorstatus,       &! out
         & nlevels,           &! in
         & nchannels,         &! in        
         & nprofiles,         &! in
         & channels,          &! in
         & frequencies,       &! in
         & lprofiles,         &! in
         & profiles,          &! in  
         & cld_profiles,      &! in
         & coef_rttov,        &! in  
         & coef_scatt,        &! in  
         & transmission,      &! in
         & calcemiss,         &! in
         & usenewcld,         &! in
         & angles,            &! out
         & scatt_aux)          ! inout   

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_iniscatt")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT',1_jpim,ZHOOK_HANDLE)
     Return
  End If

!* 3.   Eddington (in temperature space)
  Call rttov_eddington(  &
        & nlevels,        &! in
        & nchannels,         &! in
        & nprofiles,         &! in
        & lprofiles,         &! in
        & angles,            &! in
        & profiles,          &! in  
        & scatt_aux,         &! in
        & cld_bt)             ! out   

!*  4.   Combine clear and cloudy parts
  Do ichan = 1, nchannels

    iprof = lprofiles (ichan)
    if (scatt_aux % cfrac (iprof) > ccthres ) then

      radiance % bt (ichan) = cld_bt (ichan) * scatt_aux % cfrac (iprof) & 
        & + radiance % bt_clear (ichan) * (1.0_JPRB - scatt_aux % cfrac (iprof))

    else

      radiance % bt (ichan) = radiance % bt_clear (ichan)

    endif
  End Do
  cfrac (:) = scatt_aux % cfrac (:) ! Forward model diagnostic only  
     
  if (lhook) call dr_hook('RTTOV_SCATT',1_jpim,zhook_handle)

End Subroutine rttov_scatt
