!
Subroutine rttov_scatt_ad( & 
     & errorstatus,        &! out
     & nlevels,            &! in
     & nchannels,          &! in
     & nprofiles,          &! in
     & nprofilesad,        &! in
     & channels,           &! in
     & frequencies,        &! in
     & lprofiles,          &! in
     & profiles,           &! in 
     & cld_profiles,       &! in
     & coef_rttov,         &! in
     & coef_scatt,         &! in
     & calcemiss,          &! in
     & emissivity_in,      &! in
     & profiles_ad,        &! inout
     & cld_profiles_ad,    &! inout
     & emissivity_in_ad,   &! inout
     & radiance,           &! inout
     & radiance_ad,        &! inout 
     & lnewcld)             ! in, optional
     
  ! Description:
  ! AD of subroutine
  ! to compute microwave multi-channel radiances and brightness
  ! temperatures for many profiles per call in a cloudy and/or rainy sky.
  !
  ! According to the argument switchrad the main input total or bt is used
  ! switchrad == true    bt is the input, brightness temperature
  ! switchrad == false   total is the input, radiance
  !
  ! NOTE ON ADJOINT / K: in RTTOV_SCATT the adjoint and "K" routines are 
  ! combined. Normal adjoint behaviour is that the number of profiles in 
  ! the adjoint variables ("nprofilesad") is the same as in the direct 
  ! variables ("nprofiles") but to get "K" behaviour, the adjoint increments 
  ! for each channel are kept separate and "nprofilesad" should be the same 
  ! as the number of channels, "nchannels"       
  !
  ! Copyright:
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! - Bauer, P., 2002: Microwave radiative transfer modeling in clouds and 
  !     precipitation. Part I: Model description.
  !     NWP SAF Report No. NWPSAF-EC-TR-005, 21 pp.
  ! - Moreau, E., P. Bauer and F. Chevallier, 2002: Microwave radiative 
  !     transfer modeling in clouds and precipitation. Part II: Model evaluation.
  !     NWP SAF Report No. NWPSAF-EC-TR-006, 27 pp.
  ! - Chevallier, F., and P. Bauer, 2003:
  !     Model rain and clouds over oceans:comparison with SSM/I observations. 
  !     Mon. Wea. Rev., 131, 1240-1255.
  ! - Smith, E. A., P. Bauer, F. S. Marzano, C. D. Kummerow, D. McKague, 
  !     A. Mugnai, G. Panegrossi, 2002:
  !     Intercomparison of microwave radiative transfer models for precipitating clouds.
  !     IEEE Trans. Geosci. Remote Sens. 40, 541-549.
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !   1.0    09/2002      Initial version     (F. Chevallier)
  !   1.1    05/2003      RTTOV7.3 compatible (F. Chevallier)
  !   1.2    03/2004      Added polarimetry   (R. Saunders)
  !   1.3    08/2004      Polarimetry fixes   (U. O'Keeffe)
  !   1.4    11/2004      Clean-up            (P. Bauer)
  !   1.5    07/2005      Polarimetry fixes   (U. O'Keeffe)
  !   1.6    11/2005      Add errorstatus to iniscatt arguments and use a temporary
  !                       radiance type for the calcpolarisation call (J Cameron)
  !   1.7    11/2007      RTTOV9 version      (A. Geer)
  !   1.8    07/2008      Clear sky speed-ups (A. Geer)
  !   1.9    10/2008      Revised cloud partitioning (A. Geer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !   Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  
  USE YOMHOOK, ONLY: LHOOK , DR_HOOK
  
  Use rttov_const, Only :   &
       & sensor_id_mw,      &
       & errorstatus_success, &
       & errorstatus_fatal,   &  
       & adk_adjoint,         &
       & adk_k,               &
       & ccthres

  Use rttov_types, Only :    &
       & rttov_coef           ,&
       & rttov_scatt_coef     ,&
       & rttov_coef_scatt_ir  ,&
       & rttov_optpar_ir      ,&
       & geometry_Type        ,&
       & profile_Type         ,&
       & profile_cloud_Type   ,&
       & profile_scatt_aux    ,&
       & transmission_Type    ,&
       & radiance_Type        

  Use parkind1, Only : jpim     ,jprb
  
  Implicit None


!* Subroutine arguments:
  Integer (Kind=jpim), Intent (in)  :: nlevels ! Number of levels
  Integer (Kind=jpim), Intent (in)  :: nprofiles  ! Number of profiles
  Integer (Kind=jpim), Intent (in)  :: nprofilesad ! Number of profiles in adjoint
  Integer (Kind=jpim), Intent (in)  :: nchannels  ! Number of channels*profiles=radiances
  Integer (Kind=jpim), Intent (in)  :: channels    (nchannels) ! Channel indices
  Integer (Kind=jpim), Intent (in)  :: frequencies (nchannels) ! Frequency indices
  Integer (Kind=jpim), Intent (in)  :: lprofiles   (nchannels) ! Profile indices
  Integer (Kind=jpim), Intent (out) :: errorstatus (nprofiles) ! Error return flag

  Logical,             Intent (in)    :: calcemiss        (nchannels)          ! Switch for emmissivity calculation
  Real    (Kind=jprb), Intent (in)    :: emissivity_in    (nchannels)          ! Surface emmissivity 
  Real    (Kind=jprb), Intent (inout) :: emissivity_in_ad (nchannels)          ! Surface emmissivity 
  
  Type (profile_Type),        Intent (in)    :: profiles        (nprofiles)    ! Atmospheric profiles
  Type (profile_Type),        Intent (inout) :: profiles_ad     (nprofilesad)    
  Type (rttov_coef),          Intent (in)    :: coef_rttov                     ! RTTOV Coefficients
  Type (rttov_scatt_coef),    Intent (in)    :: coef_scatt                     ! RTTOV_SCATT Coefficients
  Type (profile_cloud_Type),  Intent (in)    :: cld_profiles    (nprofiles)    ! Cloud profiles 
  Type (profile_cloud_Type),  Intent (inout) :: cld_profiles_ad (nprofilesad)   
  Type (radiance_Type),       Intent (inout) :: radiance         ! Radiances
  Type (radiance_Type),       Intent (inout) :: radiance_ad          

  Logical, optional, Intent(in)   :: lnewcld ! Switch on revised cloud/rain treatment

!INTF

#include "rttov_direct.interface"
#include "rttov_iniscatt.interface"
#include "rttov_eddington.interface"
#include "rttov_ad.interface"
#include "rttov_k.interface"
#include "rttov_iniscatt_ad.interface"
#include "rttov_eddington_ad.interface"
#include "rttov_errorreport.interface"
   
  Integer (Kind=jpim), target :: sa__mclayer    (nchannels)
  Integer (Kind=jpim), target :: sa_ad__mclayer (nchannels)
      
  Real (Kind=jprb), target :: t__tau_total     (nchannels)
  Real (Kind=jprb), target :: t__tau_layers    (nlevels,nchannels)
  Real (Kind=jprb), target :: t_ad__tau_total  (nchannels)
  Real (Kind=jprb), target :: t_ad__tau_layers (nlevels,nchannels)
  
  Real (Kind=jprb), target :: sa__cfrac   (nprofiles)  
  Real (Kind=jprb), target :: sa__ems_bnd (nchannels)
  Real (Kind=jprb), target :: sa__ref_bnd (nchannels)
  Real (Kind=jprb), target :: sa__ems_cld (nchannels)
  Real (Kind=jprb), target :: sa__ref_cld (nchannels)
  
  Real (Kind=jprb), target :: sa__tbd (nprofiles,nlevels+1)
  
  Real (Kind=jprb), target :: sa__delta  (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__tau    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__ext    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__ssa    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__asm    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__lambda (nchannels,nlevels)
  Real (Kind=jprb), target :: sa__h      (nchannels,nlevels)
  
  Real (Kind=jprb), target :: sa__b0     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__b1     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__bn     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__dz     (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__clw    (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__ciw    (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__totalice (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__rain   (nprofiles,nlevels)
  Real (Kind=jprb), target :: sa__sp     (nprofiles,nlevels)

  Real (Kind=jprb), target :: sa_ad__cfrac   (nprofilesad)
  Real (Kind=jprb), target :: sa_ad__ems_bnd (nchannels)
  Real (Kind=jprb), target :: sa_ad__ref_bnd (nchannels)
  Real (Kind=jprb), target :: sa_ad__ems_cld (nchannels)
  Real (Kind=jprb), target :: sa_ad__ref_cld (nchannels)
  
  Real (Kind=jprb), target :: sa_ad__tbd (nprofilesad,nlevels+1)
  
  Real (Kind=jprb), target :: sa_ad__delta  (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__tau    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__ext    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__ssa    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__asm    (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__lambda (nchannels,nlevels)
  Real (Kind=jprb), target :: sa_ad__h      (nchannels,nlevels)
  
  Real (Kind=jprb), target :: sa_ad__b0     (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__b1     (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__bn     (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__dz     (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__clw    (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__ciw    (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__totalice (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__rain   (nprofilesad,nlevels)
  Real (Kind=jprb), target :: sa_ad__sp     (nprofilesad,nlevels)

!* Local variables:
  Logical             :: addinterp, addcloud, addsun, addaerosol, switchrad
  Logical             :: usenewcld
  Integer (Kind=jpim) :: iprof, ichan   
  Integer (Kind=jpim) :: iprofad, adk  
  Real    (Kind=jprb) :: emissivity    (nchannels)
  Real    (Kind=jprb) :: emissivity_ad (nchannels)

  Real (Kind=jprb), Dimension (nchannels) :: cld_bt, cld_bt_ad
       
  Type (transmission_Type)   :: transmission, transmission_ad
  Type (geometry_Type)       :: angles (nprofiles)
  Type (profile_scatt_aux)   :: scatt_aux, scatt_aux_ad   
  Type (rttov_optpar_ir)     :: optp   
  Type (rttov_coef_scatt_ir) :: coef_scatt_ir                             
  
  Character (len=80) :: errMessage
  Character (len=15) :: NameOfRoutine = 'rttov_scatt_ad '

  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  
  !- End of header --------------------------------------------------------
  
  if (lhook) call dr_hook('RTTOV_SCATT_AD',0_jpim,zhook_handle)        

  usenewcld = .false.
  if(present(lnewcld)) then
    if(lnewcld) usenewcld = .true.
  endif
  
  errorstatus(:)  = errorstatus_success

  if (nprofilesad == nprofiles) then 
     adk = adk_adjoint   ! Adjoint mode
  else if (nprofilesad == nchannels) then
     adk = adk_k         ! K mode
  else
     Write( errMessage, '( "incorrect number of profiles in adjoint/K")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT_AD',1_jpim,ZHOOK_HANDLE)
     Return
  endif 

  ! We do not make use of IR aerosols or scattering and in that case, the 
  ! following coefficients are not used by RTTOV_DIRECT, except for initialising
  ! an unused target variable. Ideally I would make optp and coef_scatt_ir
  ! optional.
  coef_scatt_ir % fmv_aer_comp = 1
  coef_scatt_ir % fmv_wcl_comp = 1

  transmission % tau_total  => t__tau_total
  transmission % tau_layers => t__tau_layers

  transmission_ad % tau_total  => t_ad__tau_total
  transmission_ad % tau_layers => t_ad__tau_layers

  scatt_aux % cfrac    => sa__cfrac
  scatt_aux % ems_bnd  => sa__ems_bnd
  scatt_aux % ref_bnd  => sa__ref_bnd
  scatt_aux % ems_cld  => sa__ems_cld
  scatt_aux % ref_cld  => sa__ref_cld
  scatt_aux % tbd      => sa__tbd
  scatt_aux % mclayer  => sa__mclayer
  scatt_aux % delta    => sa__delta
  scatt_aux % tau      => sa__tau
  scatt_aux % ext      => sa__ext
  scatt_aux % ssa      => sa__ssa
  scatt_aux % asm      => sa__asm
  scatt_aux % lambda   => sa__lambda
  scatt_aux % h        => sa__h
  scatt_aux % b0       => sa__b0
  scatt_aux % b1       => sa__b1
  scatt_aux % bn       => sa__bn
  scatt_aux % dz       => sa__dz
  scatt_aux % clw      => sa__clw
  scatt_aux % ciw      => sa__ciw
  scatt_aux % totalice => sa__totalice
  scatt_aux % rain     => sa__rain
  scatt_aux % sp       => sa__sp

  scatt_aux_ad % cfrac    => sa_ad__cfrac
  scatt_aux_ad % ems_bnd  => sa_ad__ems_bnd
  scatt_aux_ad % ref_bnd  => sa_ad__ref_bnd
  scatt_aux_ad % ems_cld  => sa_ad__ems_cld
  scatt_aux_ad % ref_cld  => sa_ad__ref_cld
  scatt_aux_ad % tbd      => sa_ad__tbd
  scatt_aux_ad % mclayer  => sa_ad__mclayer
  scatt_aux_ad % delta    => sa_ad__delta
  scatt_aux_ad % tau      => sa_ad__tau
  scatt_aux_ad % ext      => sa_ad__ext
  scatt_aux_ad % ssa      => sa_ad__ssa
  scatt_aux_ad % asm      => sa_ad__asm
  scatt_aux_ad % lambda   => sa_ad__lambda
  scatt_aux_ad % h        => sa_ad__h
  scatt_aux_ad % b0       => sa_ad__b0
  scatt_aux_ad % b1       => sa_ad__b1
  scatt_aux_ad % bn       => sa_ad__bn
  scatt_aux_ad % dz       => sa_ad__dz
  scatt_aux_ad % clw      => sa_ad__clw
  scatt_aux_ad % ciw      => sa_ad__ciw
  scatt_aux_ad % totalice => sa_ad__totalice
  scatt_aux_ad % rain     => sa_ad__rain
  scatt_aux_ad % sp       => sa_ad__sp

!*         1.   Gas absorption
  switchrad = .true.   ! input to RTTOV is BT  

  ! Profiles will be interpolated from model/RTTOV-SCATT levels to 
  ! RTTOV coefficient levels within RTTOV itself.
  addinterp  = .true.  
  addsun     = .false.
  addaerosol = .false.
  addcloud   = .false.
  
  emissivity (:) = emissivity_in (:) 

  Call rttov_direct(          &   
        & errorstatus,        &! out
        & nprofiles,          &! in
        & nchannels,          &! in
        & channels,           &! in
        & lprofiles,          &! in
        & addinterp,          &! in
        & profiles,           &! in
        & coef_rttov,         &! in
        & coef_scatt_ir,      &! in (not used here)
        & optp,               &! in (not used here)
        & addsun,             &! in
        & addaerosol,         &! in
        & addcloud,           &! in
        & calcemiss,          &! in
        & emissivity,         &! inout
        & transmission,       &! inout
        & radiance     )       ! inout 

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_direct")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  scatt_aux % ems_cld (:) = emissivity_in (:)      
  scatt_aux % ref_cld (:) = 1.0_JPRB - emissivity_in (:)      
  
!*  2.   Initialisations for Eddington
  Call rttov_iniscatt(       &
        & errorstatus,       &! out
        & nlevels,           &! in
        & nchannels,         &! in
        & nprofiles,         &! in
        & channels,          &! in
        & frequencies,       &! in
        & lprofiles,         &! in
        & profiles,          &! in  
        & cld_profiles,      &! in
        & coef_rttov,        &! in  
        & coef_scatt,        &! in  
        & transmission,      &! in
        & calcemiss,         &! in
        & usenewcld,         &! in
        & angles,            &! out
        & scatt_aux   )       ! inout   

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_iniscatt")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If

!* 3.   Eddington (in temperature space)
   Call rttov_eddington(     &
        & nlevels,           &! in
        & nchannels,         &! in
        & nprofiles,         &! in
        & lprofiles,         &! in
        & angles,            &! in
        & profiles,          &! in  
        & scatt_aux,         &! in
        & cld_bt)             ! out   

!*  4.   Combine clear and cloudy parts
  Do ichan = 1, nchannels

    iprof = lprofiles (ichan)
    if (scatt_aux % cfrac (iprof) > ccthres ) then

      radiance % bt (ichan) = cld_bt (ichan) * scatt_aux % cfrac (iprof) & 
        & + radiance % bt_clear (ichan) * (1.0_JPRB - scatt_aux % cfrac (iprof))

    else

      radiance % bt (ichan) = radiance % bt_clear (ichan)

    endif
  End Do
    
!* ADJOINT PART
  
  scatt_aux_ad % cfrac   (:)   = 0.0_JPRB
  scatt_aux_ad % ems_bnd (:)   = 0.0_JPRB
  scatt_aux_ad % ref_bnd (:)   = 0.0_JPRB
  scatt_aux_ad % ems_cld (:)   = 0.0_JPRB
  scatt_aux_ad % ref_cld (:)   = 0.0_JPRB
  scatt_aux_ad % tbd     (:,:) = 0.0_JPRB
  scatt_aux_ad % delta   (:,:) = 0.0_JPRB
  scatt_aux_ad % tau     (:,:) = 0.0_JPRB
  scatt_aux_ad % ext     (:,:) = 0.0_JPRB
  scatt_aux_ad % ssa     (:,:) = 0.0_JPRB
  scatt_aux_ad % asm     (:,:) = 0.0_JPRB
  scatt_aux_ad % lambda  (:,:) = 0.0_JPRB
  scatt_aux_ad % h       (:,:) = 0.0_JPRB
  scatt_aux_ad % b0      (:,:) = 0.0_JPRB
  scatt_aux_ad % b1      (:,:) = 0.0_JPRB
  scatt_aux_ad % bn      (:,:) = 0.0_JPRB
  scatt_aux_ad % dz      (:,:) = 0.0_JPRB
  scatt_aux_ad % clw     (:,:) = 0.0_JPRB
  scatt_aux_ad % ciw     (:,:) = 0.0_JPRB
  scatt_aux_ad % totalice (:,:) = 0.0_JPRB
  scatt_aux_ad % rain    (:,:) = 0.0_JPRB
  scatt_aux_ad % sp      (:,:) = 0.0_JPRB

  transmission_ad % tau_total       (:)   = 0.0_JPRB
  transmission_ad % tau_layers      (:,:) = 0.0_JPRB 

  cld_bt_ad (:) = 0.0_JPRB

!* 4.   Combine clear and cloudy parts  
  Do ichan = 1, nchannels
    iprof = lprofiles (ichan)

    if (adk == adk_adjoint) then
      iprofad = iprof
    else if (adk == adk_k) then
      iprofad = ichan
    endif

    if (scatt_aux % cfrac (iprof) > ccthres ) then

      cld_bt_ad (ichan) = cld_bt_ad (ichan) + &
       & scatt_aux % cfrac (iprof) * radiance_ad % bt (ichan) 
      
      scatt_aux_ad % cfrac (iprofad) = scatt_aux_ad % cfrac (iprofad) + &
       & (cld_bt (ichan) - radiance % bt_clear (ichan)) * radiance_ad % bt (ichan) 

      radiance_ad  % bt_clear (ichan) = radiance_ad % bt_clear (ichan) + &
       & (1.0_JPRB - scatt_aux % cfrac (iprof)) * radiance_ad % bt (ichan)

      radiance_ad % bt (ichan) = 0.0_JPRB

    else

      radiance_ad % bt_clear (ichan) = radiance_ad % bt_clear (ichan) + &
       & radiance_ad % bt (ichan)

      radiance_ad % bt (ichan) = 0.0_JPRB

    endif

  End Do

!* 3.   Eddington (in temperature space)
  Call rttov_eddington_ad(   &
        & nlevels,           &! in
        & nchannels,         &! in
        & nprofiles,         &! in
        & nprofilesad,       &! in
        & lprofiles,         &! in
        & angles,            &! in
        & profiles,          &! in  
        & profiles_ad,       &! inout  
        & scatt_aux,         &! in
        & scatt_aux_ad,      &! inout
        & cld_bt,            &! out  
        & cld_bt_ad)          ! inout   

!*  2.   Initialisations for Eddington  
  Call rttov_iniscatt_ad(    &
        & errorstatus,       &! out
        & nlevels,           &! in
        & nchannels,         &! in
        & nprofiles,         &! in
        & nprofilesad,       &! in
        & channels,          &! in
        & frequencies,       &! in
        & lprofiles,         &! in
        & profiles,          &! in 
        & profiles_ad,       &! inout
        & cld_profiles,      &! in
        & cld_profiles_ad,   &! inout
        & coef_rttov,        &! in 
        & coef_scatt,        &! in  
        & transmission,      &! in
        & transmission_ad,   &! inout
        & calcemiss,         &! in
        & usenewcld,         &! in
        & angles,            &! out
        & scatt_aux,         &! inout
        & scatt_aux_ad)       ! inout   

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_iniscatt_ad")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If
        
  emissivity_in_ad (:) = emissivity_in_ad (:) - scatt_aux_ad % ref_cld (:)
  scatt_aux_ad % ref_cld (:) = 0.0_JPRB      
  emissivity_in_ad (:) = emissivity_in_ad (:) + scatt_aux_ad % ems_cld (:)  
  scatt_aux_ad % ems_cld (:) = 0.0_JPRB   
  
!*         1.   Gas absorption
  emissivity_ad (:) = 0.0_JPRB
 
  if (adk == adk_adjoint) then  
    Call rttov_ad(        &
       & errorstatus,     &! out
       & nprofiles,       &! in
       & nchannels,       &! in
       & channels,        &! in
       & lprofiles,       &! in
       & addinterp,       &! in
       & profiles,        &! in
       & profiles_ad,     &! inout
       & coef_rttov,      &! in
       & coef_scatt_ir,   &! in (not used here)
       & optp,            &! in (not used here)
       & addsun,          &! in
       & addaerosol,      &! in
       & addcloud,        &! in
       & switchrad,       &! in
       & calcemiss,       &! in
       & emissivity,      &! inout
       & emissivity_ad,   &! inout
       & transmission,    &! inout
       & transmission_ad, &! inout
       & radiance,        &! inout
       & radiance_ad     ) ! inout 
  else if (adk == adk_k) then 
    Call rttov_k(        &
       & errorstatus,     &! out
       & nprofiles,       &! in
       & nchannels,       &! in
       & channels,        &! in
       & lprofiles,       &! in
       & addinterp,       &! in
       & profiles,        &! in
       & profiles_ad,     &! inout
       & coef_rttov,      &! in
       & coef_scatt_ir,   &! in (not used here)
       & optp,            &! in (not used here)
       & addsun,          &! in
       & addaerosol,      &! in
       & addcloud,        &! in
       & switchrad,       &! in
       & calcemiss,       &! in
       & emissivity,      &! inout
       & emissivity_ad,   &! inout
       & transmission,    &! inout
       & transmission_ad, &! inout
       & radiance,        &! inout
       & radiance_ad     ) ! inout 
  endif

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Write( errMessage, '( "error in rttov_ad")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_SCATT_AD',1_jpim,ZHOOK_HANDLE)
     Return
  End If
  
  emissivity_in_ad (:) = emissivity_in_ad (:) + emissivity_ad (:)
  emissivity_ad    (:) = 0.0_JPRB

!*  4b.   Combine clear and cloudy parts - again - because rttov_ad/k has
!*        overwritten the bt array with clear-sky values.
  Do ichan = 1, nchannels
    iprof = lprofiles (ichan)
    if (scatt_aux % cfrac (iprof) > ccthres ) then

      radiance % bt (ichan) = cld_bt (ichan) * scatt_aux % cfrac (iprof) &
       & + radiance % bt_clear (ichan) * (1.0_JPRB - scatt_aux % cfrac (iprof))

    else

      radiance % bt (ichan) = radiance % bt_clear (ichan)

    endif
  End Do

  if (lhook) call dr_hook('RTTOV_SCATT_AD',1_jpim,zhook_handle)

End Subroutine rttov_scatt_ad
