!
Subroutine rttov_setpredictors_7_tl( &
      & nprofiles,  &!
      & prof,        &! in
      & prof_tl,     &! in
      & geom,        &! in
      & coef,        &! in
      & aux,         &! in
      & predictors,  &! in
      & predictors_tl, &! inout
      & raytracing,   &
      & raytracing_tl ) ! inout  
  ! 
  ! Description
  ! RTTOV-7 Model
  ! TL of rttov_setpredictors
  ! To calculate and store the profile variables (predictors) required
  ! in subsequent transmittance calculations.
  ! Code based on PRFTAU from previous versions of RTTOV
  ! Only one profile per call
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! see RTTOV7 science and validation report pages 18/19
  ! variable names are close to the documentation
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
  !  1.2       29/03/2005  Add end of header comment (J. Cameron)
  !  1.3       08/12/2005  Add surface humidity (R Saunders)
  !  1.4       03/03/2006  Marco Matricardi (ECMWF):
  !               --       Altitude dependent local zenith angle introduced.
  !  1.5       12/08/2008  Zeeman effect based on Yong Han (P. Rayer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & gravity        ,   &
        & use_q2m 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
        & rttov_coef     ,&
        & profile_Type   ,&
        & geometry_Type  ,&
        & profile_aux    ,&
        & predictors_Type,&
        & raytracing_type  

  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)  :: nprofiles
  Type(profile_Type),    Intent(in)  :: prof(nprofiles)
  Type(profile_Type),    Intent(in)  :: prof_tl(nprofiles)
  Type(rttov_coef),      Intent(in)  :: coef
  Type(geometry_Type),   Intent(in)  :: geom(nprofiles)
  Type(predictors_Type), Intent(in)  :: predictors
  Type(predictors_Type), Intent(inout) :: predictors_tl ! in because of mem allocation
  Type(raytracing_type), Intent(in)  :: raytracing
  Type(raytracing_type), Intent(in)  :: raytracing_tl
  Type(profile_aux) ,    Intent(in)    :: aux(nprofiles)           ! auxillary profiles info.

!INTF

  !local variables:
  Integer(Kind=jpim) :: level
  Integer(Kind=jpim) :: iv2, iv3

  ! user profile
  Real(Kind=jprb) :: t(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: w(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: o(nprofiles,prof(1) % nlevels)

  ! reference profile
  Real(Kind=jprb) :: tr(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: wr(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: or(nprofiles,prof(1) % nlevels)

  ! user - reference
  Real(Kind=jprb) :: dt(nprofiles,prof(1) % nlevels)

  ! pressure weighted
  Real(Kind=jprb) :: tw(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ww(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ow(nprofiles,prof(1) % nlevels)

  ! intermediate variables
  Real(Kind=jprb) :: sum1(nprofiles),sum2(nprofiles)
  Real(Kind=jprb) :: deltac(prof(1) % nlevels)
  Real(Kind=jprb) :: sec_wr(prof(1) % nlevels)
  Real(Kind=jprb) :: sec_or(prof(1) % nlevels)

  ! TL variables
  Real(Kind=jprb) :: t_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: w_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: o_tl(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: tr_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: wr_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: or_tl(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: dt_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: dto_tl(nprofiles,prof(1) % nlevels)

  Real(Kind=jprb) :: tw_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ww_tl(nprofiles,prof(1) % nlevels)
  Real(Kind=jprb) :: ow_tl(nprofiles,prof(1) % nlevels)


  Real(Kind=jprb) :: sec_or_tl(prof(1) % nlevels)
  Real(Kind=jprb) :: sec_wr_tl(prof(1) % nlevels)

  Real(Kind=jprb) :: zsqrt, zrecip
  Integer(Kind=jpim) :: i
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  ! profile layer quantities
  ! Direct variables
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_TL',0_jpim,ZHOOK_HANDLE)
  Do i=1,nprofiles
    t(i,1) = prof(i) % t(1)
    w(i,1) = prof(i) % q(1)
  Enddo

!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      t(i,level) = ( prof(i) % t(level-1) +  prof(i) % t(level ) ) *0.5_JPRB
      w(i,level) = ( prof(i) % q(level-1) +  prof(i) % q(level ) ) *0.5_JPRB
    Enddo
  Enddo

  If ( use_q2m )Then
    Do i=1,nprofiles
  ! include surface humidity
      iv3 = aux(i) % nearestlev_surf - 1
      iv2 = aux(i) % nearestlev_surf
      If ( iv2 <= coef % nlevels) Then
        w(i,iv2) = (prof(i) % s2m % q  + prof(i) % q(iv3)) *0.5_JPRB
      Endif
    Enddo
  Endif

  Do i=1,nprofiles
    If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
      o(i,1) = prof(i) % o3(1)
    Endif
  EndDo
!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        o(i,level ) = ( prof(i) % o3(level-1) + prof(i) % o3(level  ) ) *0.5_JPRB
      Endif
    EndDo
  EndDo

  ! TL variables

  Do i=1,nprofiles
    t_tl(i,1) = prof_tl(i) % t(1)
    w_tl(i,1) = prof_tl(i) % q(1)
  Enddo
!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      t_tl(i,level) = ( prof_tl(i) % t(level-1) +  prof_tl(i) % t(level ) ) *0.5_JPRB
      w_tl(i,level) = ( prof_tl(i) % q(level-1) +  prof_tl(i) % q(level ) ) *0.5_JPRB
    Enddo
  Enddo

  ! include tl surface humidity
  If ( use_q2m )Then
    Do i=1,nprofiles
      iv3 = aux(i) % nearestlev_surf - 1
      iv2 = aux(i) % nearestlev_surf
      If ( iv2 <= coef % nlevels) Then
        w_tl(i,iv2) = (prof_tl(i) % s2m % q  + prof_tl(i) % q(iv3)) *0.5_JPRB
      Endif
    Enddo
  Endif

  Do i=1,nprofiles
    If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
      o_tl(i,1) = prof_tl(i) % o3(1)
    Endif
  EndDo
!CDIR NOLOOPCHG
  Do level=2,prof(1)%nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        o_tl(i,level ) = ( prof_tl(i) % o3(level-1) + prof_tl(i) % o3(level  ) ) *0.5_JPRB
      Endif
    EndDo
  EndDo

  ! calculate deviations from reference profile (layers)
  ! if no input O3 profile, set to reference value (dto =0)
  ! Direct variables & TL variables
!CDIR NOLOOPCHG
  Do level=1,prof(1)%nlevels
    Do i=1,nprofiles
      dt(i,level)  = t(i,level) - coef % tstar(level)
      dt_tl(i,level)  = t_tl(i,level)
  ! Direct value of dto NOT needed for TL
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
       dto_tl(i,level) = t_tl(i,level)
      Else
       dto_tl(i,level) = 0._JPRB
      Endif

  ! calculate (profile / reference profile) ratios; tr wr or
  ! if no input O3 profile, set to reference value (or =1)
  ! Direct variables
      tr(i,level) = t(i,level) / coef % tstar(level)
      wr(i,level) = w(i,level) / coef % wstar(level)
  ! TL variables
      tr_tl(i,level) = t_tl(i,level) / coef % tstar(level)
      wr_tl(i,level) = w_tl(i,level) / coef % wstar(level)
  ! if no input O3 profile, set to reference value (or =1)
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        or(i,level) = o(i,level) / coef % ostar(level)
        or_tl(i,level) = o_tl(i,level) / coef % ostar(level)
      Else
        or(i,level) = 1._JPRB
        or_tl(i,level) = 0._JPRB
      Endif
    EndDo
  EndDo

  ! calculate profile / reference profile sums: tw ww ow
  ! if no input O3 profile, set to reference value (ow =1)
  ! Direct variables
  Do i=1,nprofiles
    tw(i,1) = 0._JPRB
    tw_tl(i,1) = 0._JPRB
  Enddo
  Do level = 2 , prof(1) % nlevels
    Do i=1,nprofiles
      tw(i, level ) = tw(i, level-1 ) + coef % dpp( level ) * tr (i, level -1 )
      tw_tl(i, level ) = tw_tl(i, level-1 ) + coef % dpp( level ) * tr_tl (i, level -1 )
    End Do
  End Do


  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof_tl(1) % nlevels
    Do i=1,nprofiles
      sum1(i) = sum1(i) + coef % dpp( level ) *      w       (i, level )
      sum2(i) = sum2(i) + coef % dpp( level ) * coef % wstar ( level )
      ww (i, level ) = sum1(i) / sum2(i)
    End Do
  End Do
  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof_tl(1) % nlevels
    Do i=1,nprofiles
      sum1(i) = sum1(i) + coef % dpp( level ) *      w_tl    (i, level )
      sum2(i) = sum2(i) + coef % dpp( level ) * coef % wstar ( level )
      ww_tl (i, level ) = sum1(i) / sum2(i)
    End Do
  End Do


  ! if no input O3 profile, set to reference value (ow =1)
  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof_tl(1) % nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + coef % dpp( level ) *      o       (i, level )
        sum2(i) = sum2(i) + coef % dpp( level ) * coef % ostar ( level )
        ow (i, level ) = sum1(i) / sum2(i)
      Else
        ow(i,level) = 1._JPRB
      Endif
    End Do
  End Do

  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, prof_tl(1) % nlevels
    Do i=1,nprofiles
      If ( prof(i) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + coef % dpp( level ) *      o_tl       (i, level )
        sum2(i) = sum2(i) + coef % dpp( level ) * coef % ostar ( level )
        ow_tl (i, level ) = sum1(i) / sum2(i)
      Else
        ow_tl(i,level) = 0._JPRB
      Endif
    End Do
  End Do




  ! ATTENTION
  !  w_tl(:) = prof_tl % q(:)


  !5) set predictors
  !--

  !5.1 mixed gases
  !---
 If(coef % id_inst /= 36)Then  

   Do level = 1, prof_tl(1) % nlevels
     Do i=1,nprofiles
  ! No Zeeman effect
       predictors_tl % mixedgas(1,level,i) = raytracing_tl % pathsat(level,i) 
       predictors_tl % mixedgas(2,level,i) = raytracing_tl % pathsat(level,i)*2    * &
                       & raytracing % pathsat(level,i) 
       predictors_tl % mixedgas(3,level,i) = tr_tl(i,level)                        * &
                       & raytracing % pathsat(level,i)                             + &
                       & raytracing_tl % pathsat(level,i)*tr(i,level)  
       predictors_tl % mixedgas(4,level,i) = 2._JPRB * tr_tl(i,level)              * &
                       & predictors % mixedgas(3,level,i)                          + &
                       & raytracing_tl % pathsat(level,i)                          * &
                       & predictors % mixedgas(5,level,i)**2 
       predictors_tl % mixedgas(5,level,i) = tr_tl(i,level)
       predictors_tl % mixedgas(6,level,i) = 2._JPRB * tr_tl(i,level) * tr(i,level)
       predictors_tl % mixedgas(7,level,i) = raytracing_tl % pathsat(level,i)      * &
                       & tw(i,level)                                               + &
                       & tw_tl(i,level)*raytracing % pathsat(level,i) 
       predictors_tl % mixedgas(8,level,i) = raytracing_tl % pathsat(level,i)      * &
                       & tw(i,level)  /tr(i,level)                                   + &
                       & tw_tl(i,level)*raytracing % pathsat(level,i)/tr(i,level)      - &
                       & tr_tl(i,level)*tw(i,level)*raytracing % pathsat(level,i)      / &
                       & tr(i,level)**2._JPRB 
       predictors_tl % mixedgas(9,level,i) = raytracing_tl % pathsat(level,i)      * &
                       & 0.5_JPRB/predictors % mixedgas(9,level,i) 
       if(level==1)then
         predictors_tl % mixedgas(10,level,i)=0
       else
         predictors_tl % mixedgas(10,level,i)= raytracing_tl % pathsat(level,i)    * &
                       & 0.5_JPRB *tw(i,level)**0.25_JPRB                          / &
                       & predictors % mixedgas(9,level,i)                          + &
                       & tw_tl(i,level)*0.25_JPRB*predictors % mixedgas(9,level,i)   / &
                       & tw(i,level)**0.75 
       endif

        If(coef % id_inst == 3 .and. coef % IncZeeman)Then  
  ! AMSUA ch14 with Zeeman effect (coef % nmixed = 14, otherwise = 10)
  ! predictors 1 - 4 are overlapped with other channels      
         predictors_tl % mixedgas(11,level,i) = prof(i) % cosbk**2 * raytracing_tl % pathsat(level,i) 
         predictors_tl % mixedgas(12,level,i) = 2.0_JPRB * prof(i) % Be * raytracing % pathsat(level,i)* &
                                              & raytracing_tl % pathsat(level,i)
         predictors_tl % mixedgas(13,level,i) = prof(i) % Be**3 * raytracing_tl % pathsat(level,i)
         predictors_tl % mixedgas(14,level,i) = 2.0_JPRB * (prof(i) % cosbk * prof(i) % Be)**2 * &
                                              & raytracing % pathsat(level,i) * raytracing_tl % pathsat(level,i)
        Endif           
 
     enddo
   enddo

  Else ! ssmisz predictors (Zeeman-affected channels of SSMIS allocated to sensor SSMISZ))

    Do level = 1, prof_tl(1) % nlevels
      Do i=1,nprofiles
        Predictors_tl % mixedgas(1, level,i) = raytracing_tl % pathsat(level,i)       
        Predictors_tl % mixedgas(2, level,i) = -(Predictors % mixedgas(2,level,i) / t(i, level)) * t_tl(i,level) + &
                                             (300.0_JPRB / t(i, level)) * raytracing_tl % pathsat(level,i)
        Predictors_tl % mixedgas(3, level,i) = prof(i) % cosbk**2 * raytracing_tl % pathsat(level,i)    
        Predictors_tl % mixedgas(4, level,i) = Predictors_tl % mixedgas(2,level,i) / prof(i) % Be     
        Predictors_tl % mixedgas(5, level,i) = Predictors_tl % mixedgas(2,level,i) * prof(i) % cosbk**2
        Predictors_tl % mixedgas(6, level,i) = raytracing_tl % pathsat(level,i) / prof(i) % Be 
        Predictors_tl % mixedgas(7, level,i) = Predictors_tl % mixedgas(6,level,i) / prof(i) % Be
        Predictors_tl % mixedgas(8, level,i) = prof(i) % Be * raytracing_tl % pathsat(level,i)       
        Predictors_tl % mixedgas(9, level,i) = prof(i) % Be**3 * raytracing_tl % pathsat(level,i)       
        Predictors_tl % mixedgas(10,level,i) = Predictors_tl % mixedgas(3,level,i) * prof(i) % Be    
        Predictors_tl % mixedgas(11,level,i) = Predictors_tl % mixedgas(10,level,i) * prof(i) % Be
      Enddo 
    Enddo

  Endif

  !5.2 water vapour  ( numbers in right hand are predictor numbers
  ! in the reference document for RTTOV7 (science and validation report)
  !----------------

  Do level = 1, prof_tl(1) % nlevels
    Do i=1,nprofiles
      sec_wr(level)    =raytracing%pathsat(level,i)*wr(i,level)
      sec_wr_tl(level) =raytracing_tl%pathsat(level,i)*wr(i,level)                 + &
                        & raytracing%pathsat(level,i)*wr_tl(i,level)  
      predictors_tl % watervapour(1,level,i)  = sec_wr_tl(level)                     !  7
      predictors_tl % watervapour(2,level,i)  = 0.5_JPRB * sec_wr_tl(level)      / &
                         & predictors % watervapour(2,level,i)                         !  5 
      zrecip=1.0_JPRB/ predictors % watervapour(1,level,i)
      predictors_tl % watervapour(3,level,i) = sec_wr_tl(level)*wr(i,level)        /  &! 12
                         & ww(i,level)                                             + &
                         & wr_tl(i,level)*sec_wr(level)/ ww(i,level)                 - &
                         & ww_tl(i,level)*wr(i,level)*sec_wr(level)/ww(i,level)**2    
      predictors_tl % watervapour(4,level,i)  =                                      &! 4
                         & sec_wr_tl(level) * predictors % watervapour(4,level,i)  * &
                         & zrecip                                                +  &
                         & predictors % watervapour(1,level,i) * dt_tl(i,level) 
      predictors_tl % watervapour(5,level,i)  =                                      &! 1
                         & 2 * predictors % watervapour(1,level,i) * sec_wr_tl(level)   
      predictors_tl % watervapour(6,level,i)  =                                      &! 11
                         & predictors % watervapour(2,level,i) * dt_tl(i,level)      +  &
                         & 0.5_JPRB * sec_wr_tl(level)                           * &
                          & predictors % watervapour(6,level,i) *zrecip 
      predictors_tl % watervapour(7,level,i)  =                                      &! 6
                         & 0.25_JPRB * sec_wr_tl(level)                          / &
                         & predictors % watervapour(7,level,i)**3 
      predictors_tl % watervapour(8,level,i)  =                                      &! 13
                         & predictors_tl % watervapour(2,level,i)                  * &
                         & wr(i,level) / ww(i,level)                                 + &
                         & wr_tl(i,level)*predictors % watervapour(2,level,i)        / &
                         & ww(i,level)                                             - &
                         & ww_tl(i,level)*predictors % watervapour(2,level,i)        * &
                         & wr(i,level) /ww(i,level)**2 
      predictors_tl % watervapour(9,level,i)  =                                      &!  8
                         & 3 * sec_wr_tl(level) * predictors % watervapour(5,level,i) 
      predictors_tl % watervapour(10,level,i)  =                                     &! 9
                         & 4 * sec_wr_tl(level) * predictors % watervapour(9,level,i) 
      predictors_tl % watervapour(11,level,i)  =                                     &! 10
                         & Abs(dt(i,level))                                        * &
                         & (sec_wr_tl(level) * dt(i,level)                         + &
                         & 2 * sec_wr(level) * dt_tl(i,level) )  
      zsqrt=Sqrt(predictors % watervapour(13,level,i))
      predictors_tl % watervapour(12,level,i)  =                                     &!  3 
                         & 4 * raytracing%pathsat(level,i) * ww_tl(i,level)          * &
                         & predictors % watervapour(12,level,i)/zsqrt              + &
                         & 4 * raytracing_tl%pathsat(level,i) * ww(i,level)          * &
                         & predictors % watervapour(12,level,i)/ zsqrt 
      predictors_tl % watervapour(13,level,i)  =                                     &!  4
                         & 2 * raytracing%pathsat(level,i) * ww_tl(i,level) * zsqrt  + &
                         & 2 * raytracing_tl%pathsat(level,i) * ww(i,level) * zsqrt 
      predictors_tl % watervapour(14,level,i) = sec_wr_tl(level)*wr(i,level)       /   &!  14
                        & tr(i,level)                                              + &
                        & wr_tl(i,level)*sec_wr(level) / tr(i,level)                 - &
                        & tr_tl(i,level)*sec_wr(level) * wr(i,level) / tr(i,level)**2 
       predictors_tl % watervapour(15,level,i)  =sec_wr_tl(level) * wr(i,level)    /    &!  15
                        & tr(i,level)**4                                           + &
                        & wr_tl(i,level)*sec_wr(level)/ tr(i,level)**4               - &
                        & tr_tl(i,level)*4*sec_wr(level) * wr(i,level) / tr(i,level)**5 
    End Do
  End Do


  !5.3 ozone
  !---------

  If ( coef % nozone > 0 ) Then
    Do level = 1, prof_tl(1) % nlevels
      Do i=1,nprofiles
        sec_or   (level) = raytracing%pathsat(level,i)    * or(i,level)
        sec_or_tl(level) = raytracing_tl%pathsat(level,i) * or(i,level)            + &
                        & raytracing%pathsat(level,i)    * or_tl(i,level) 
  
        predictors_tl % ozone(1,level,i)  =sec_or_tl(level)
  
        predictors_tl % ozone(2,level,i)  =                                        &
                        & 0.5_JPRB * sec_or_tl(level) / predictors % ozone(2,level,i)    
  
        predictors_tl % ozone(3,level,i)  =                                        &
                        & sec_or_tl(level) * predictors % ozone(3,level,i)         / &
                        & predictors % ozone(1,level,i)                            + &
                        & predictors % ozone(1,level,i) * dto_tl(i,level)  
  
        predictors_tl % ozone(4,level,i)  =                                        &
                        & 2 * sec_or_tl(level) * predictors % ozone(1,level,i)   
  
        predictors_tl % ozone(5,level,i)  =                                        &
                        & 0.5_JPRB*sec_or_tl(level) * predictors % ozone(3,level,i)/ &
                        & (predictors % ozone(1,level,i)*predictors%ozone(2,level,i))+ &
                        & predictors % ozone(2,level,i) * dto_tl(i,level)  
  
        predictors_tl % ozone(6,level,i)  = sec_or_tl(level)*or(i,level)*ow (i,level)+ &
                        & or_tl(i,level)*sec_or(level)* ow (i,level)                 + &
                        & ow_tl(i,level)*sec_or(level) * or(i,level) 
  
        predictors_tl % ozone(7,level,i)  =predictors_tl % ozone(2,level,i)        * &
                        & or(i,level) / ow(i,level)                                  + &
                        & or_tl(i,level)*predictors % ozone(2,level,i) /ow(i,level)    - &
                        & ow_tl(i,level)*predictors % ozone(2,level,i) * or(i,level)   / &
                        & ow(i,level)**2 
  
        predictors_tl % ozone(8,level,i)  = sec_or_tl(level) * ow(i,level)         + &
                        & sec_or(level) * ow_tl(i,level) 
  
        zsqrt=Sqrt(predictors % ozone(10,level,i))
  
        predictors_tl % ozone(9,level,i)= raytracing%pathsat(level,i)*or_tl(i,level) * &
                        & zsqrt                                                  + &
                        & predictors % ozone(1,level,i)*0.5                        * &
                        & raytracing%pathsat(level,i)*ow_tl(i,level)                 / &
                        & zsqrt                                                  + &
                        & 1.5*raytracing_tl%pathsat(level,i)                       * &
                        & or(i,level)*zsqrt  
  
        predictors_tl % ozone(10,level,i)= raytracing_tl%pathsat(level,i)*ow(i,level)+ &
                        & raytracing%pathsat(level,i)*ow_tl(i,level)  
   
        predictors_tl % ozone(11,level,i)=2*ow_tl(i,level)*raytracing%pathsat(level,i)*&
                        & predictors % ozone(10,level,i)                           + &
                        & 2*raytracing_tl%pathsat(level,i)*ow(i,level)               * &
                        & predictors % ozone(10,level,i) 
      End Do
    End Do
  Endif

  !5.4 cloud
  !---------
!CDIR NOLOOPCHG
   Do level = 1, prof(1) % nlevels
     deltac(level) = 0.1820_JPRB * 100.0_JPRB * coef % dp(level) / (4.3429_JPRB * gravity)
     Do i=1,nprofiles
       If ( prof(i) % clw_Data ) Then
         predictors_tl % clw(level,i) = deltac(level) * prof_tl(i)%clw(level) * geom(i)%seczen
       Endif
     Enddo
   EndDo
!CDIR NOLOOPCHG
     Do level = 2, prof_tl(1) % nlevels
       Do i=1,nprofiles
         If ( prof (i)% clw_Data ) Then
           predictors_tl % clw(level,i) = 0.5_JPRB *                                &
            & ( predictors_tl % clw(level,i) + &
            & deltac(level) * prof_tl(i)%clw(level-1) * &
            & geom(i)%seczen ) 
         Endif
       End Do
     End Do
     Do i=1,nprofiles
       If ( .not. prof (i)% clw_Data ) Then
         predictors_tl % ncloud   = 0
       Endif
     End Do
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_TL',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_setpredictors_7_tl
