Subroutine rttov_writecoef  (&
       & errorstatus,   &! out
       & coef,          &! in
       & coef_scatt_ir, &! in
       & optp ,         &! in
       & file_id,       &! in
       & lbinary,       &! in Optional
       & aerosl,        &! in Optional
       & clouds         )! in Optional
  ! Description:
  ! write on unit file_id the coef structure.
  ! If lbinary is false or not present the file is assumed as
  ! an ASCII sequential formatted, in other case it is sequential unformatted.
  ! I/O write status are only tested at the end of the code
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       24/01/2003  insert I/O status (P Brunel)
  !                        one record per channel for coefficients in binary format
  !                        New header to allow checking R4<->R8
  !  1.2       02/06/2004  Update for RTTOV8 coefs (P. Brunel)
  !  1.3       02/08/2006  Change format for pressure levels f8.3 -> f9.4 (P. Brunel)
  !  1.4       14/05/2007  Updated for RTTOV-9 (P Brunel)
  !  1.5       19/02/2008  Another update for RTTOV-9 for inc_top (R Saunders)
  !  1.6       27/06/2008  Introduced the case where no channels are available for
  !                        the phase function in the solar range (M. Matricardi)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
       & rttov_magic_string  ,&
       & rttov_magic_number  ,&
       & errorstatus_success ,&
       & errorstatus_fatal   ,&
       & errorstatus_info    ,&
       & gas_id_mixed        ,&
       & gas_id_watervapour  ,&
       & gas_id_ozone        ,&
       & gas_id_wvcont       ,&
       & gas_id_co2          ,&
       & gas_id_n2o          ,&
       & gas_id_co           ,&
       & gas_id_ch4          ,&
       & gas_name            ,&
       & gas_unit_name

  ! Imported Type Definitions:
  Use rttov_types, Only :  &
        & rttov_coef      ,&
        & rttov_optpar_ir ,&
        & rttov_coef_scatt_ir 

  Use parkind1, Only : jpim     ,jprb
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Implicit None


  ! subroutine arguments
  ! scalar arguments with intent(in):
  Integer(Kind=jpim),        Intent (in) :: file_id      ! file logical unit number
  Type( rttov_coef ),        Intent (in) :: coef         ! coefficients
  Type(rttov_optpar_ir),     Intent (in) :: optp
  Type(rttov_coef_scatt_ir), Intent (in) :: coef_scatt_ir
  Logical, Optional, Intent (in) :: aerosl
  Logical, Optional, Intent (in) :: clouds
  Logical, Optional, Intent (in) :: lbinary      ! if binary file wanted
  ! scalar arguments with intent(in):
  Integer(Kind=jpim),            Intent (out) :: errorstatus  ! return code

!INTF

#include "rttov_errorreport.interface"


  ! local scalars
  Logical :: section_present
  Integer(Kind=jpim) :: i, j, l, k, n, nr, nrh
  Logical :: file_binary
  Logical :: laerosl, lclouds
  Integer            :: io_status
  Integer(Kind=jpim) :: id_comp_lvl
  Character (len=2) :: sensor
  Character (len=32) :: section

  Character (len=80) :: errMessage
  Character (len=16) :: NameOfRoutine = 'rttov_writecoef '
! Integer(Kind=jpim) :: Err_Unit         Logical error unit
! Integer(Kind=jpim) :: verbosity_level
  Integer(Kind=jpim) :: inctop         ! Flag to include top level
  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  !- End of header --------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_WRITECOEF',0_jpim,ZHOOK_HANDLE)

  errorstatus     = errorstatus_success
  inctop = 0_jpim
!  Err_unit=-1_jpim
!  verbosity_level=3_jpim
!  Call rttov_errorhandling (Err_unit, verbosity_level)

  ! Consider lbinary option to create the option
  If(Present(lbinary)) Then
     file_binary = lbinary
  Else
     file_binary = .False.
  Endif
  If( Present(aerosl)) Then
     laerosl = aerosl
  Else
     laerosl = .False.
  Endif
  If( Present(clouds)) Then
     lclouds = clouds
  Else
     lclouds = .False.
  Endif

ascii_binary:  If( file_binary ) Then

   ! Binary file
   Write( errMessage, '( "write coefficient to file_id ", i2, " in binary format")' ) &
           & file_id
   Call Rttov_ErrorReport (errorstatus_info, errMessage, NameOfRoutine)

   ! Write a string that could be displayed
   ! Write a real number to be able to check single/double precision
   Write(file_id, iostat=io_status) rttov_magic_string, rttov_magic_number

   ! COEF structure (V7, V8, V9)
   If ( .not. (laerosl .or. lclouds)) Then

     Write(file_id, iostat=io_status)&
           & coef % id_platform,&
           & coef % id_sat,     &
           & coef % id_inst,    &
           & coef % id_sensor
     If( coef % id_comp_lvl == 7 ) then
       Write( errMessage, '( "output will be version 8 format compatible")' )
       Call Rttov_ErrorReport (errorstatus_info, errMessage, NameOfRoutine)
       ! even if input coef file is version 7 format compatible
       ! the output will be version 8 format compatible
       ! The variable coef % fmv_model_ver contains the fast model 
       ! version for the predictors and should never be modified
       id_comp_lvl = 8
     Else
       id_comp_lvl = coef % id_comp_lvl
     End If
     Write(file_id, iostat=io_status) &
           & id_comp_lvl,             &
           & coef % id_creation_date, &
           & coef % id_creation,      &
           & coef % id_Common_name
     Write(file_id, iostat=io_status)&
             & coef % fmv_model_def,&
             & coef % fmv_model_ver,&
             & coef % fmv_chn,      &
             & coef % fmv_gas
     
     If( coef % id_comp_lvl >= 9 )Then
         If(coef % IncTop)inctop = 1_jpim                   ! RWS  
         If(coef % IncZeeman)inctop = 2_jpim                   ! RWS  
         Write(file_id, iostat=io_status)inctop             ! RWS
     Endif

     Write(file_id, iostat=io_status)&
           & coef % fmv_gas_id,  &
           & coef % fmv_gas_pos, &
           & coef % fmv_var,     &
           & coef % fmv_lvl

     If( coef % id_comp_lvl == 9 ) then         !pb
       Write(file_id, iostat=io_status)&        !pb
           & coef % fmv_coe                     !pb
     End If                                     !pb

     Write(file_id, iostat= io_status )&
         & coef % gaz_units 

     If( coef % id_comp_lvl == 9 ) then         !pb
       ! GAS_SPECTRAL_INTERVAL                  !pb
       Write(file_id, iostat=io_status)&        !pb
           & coef%nintmixed  ,&                 !pb
           & coef%nintwater  ,&                 !pb
           & coef%nintozone  ,&                 !pb
           & coef%nintwvcont ,&                 !pb
           & coef%nintco2    ,&                 !pb
           & coef%nintn2o    ,&                 !pb
           & coef%nintco     ,&                 !pb
           & coef%nintch4                       !pb
        If( coef%nintmixed > 0 ) &              !pb
          & Write(file_id, iostat=io_status) coef%mixedgasint    !pb
        If( coef%nintwater > 0 ) &                               !pb
          & Write(file_id, iostat=io_status) coef%watervapourint !pb
        If( coef%nintozone > 0 ) &                               !pb
          & Write(file_id, iostat=io_status) coef%ozoneint       !pb
        If( coef%nintwvcont > 0 ) &                              !pb
          & Write(file_id, iostat=io_status) coef%wvcontint      !pb
        If( coef%nintco2 > 0 ) &                                 !pb
          & Write(file_id, iostat=io_status) coef%co2int         !pb
        If( coef%nintn2o > 0 ) &                                 !pb
          & Write(file_id, iostat=io_status) coef%n2oint         !pb
        If( coef%nintco > 0 ) &                                  !pb
          & Write(file_id, iostat=io_status) coef%coint          !pb
        If( coef%nintch4 > 0 ) &                                 !pb
          & Write(file_id, iostat=io_status) coef%ch4int         !pb
     End If                                                      !pb

     If( coef % id_comp_lvl == 9 ) Then
       section_present = Associated( coef % tt_chn ) 
       !TRANSMITTANCE_TRESHOLD                  !pb
       Write( file_id ) section_present
       If( section_present ) &
       Write(file_id, iostat=io_status)&        !pb
           & coef % tt_chn     ,&               !pb
           & coef % tt_val_chn ,&               !pb
           & coef % tt_cwn     ,&               !pb
           & coef % tt_a0      ,&               !pb
           & coef % tt_a1                       !pb
     End If                                     !pb
 
    If( coef % id_comp_lvl == 9 ) then          !pb
       !SOLAR_SPECTRUM                          !pb
       section_present = Associated( coef % ss_chn ) 
       Write( file_id ) section_present
       If( section_present ) &
       Write(file_id, iostat=io_status)&        !pb
           & coef % ss_chn     ,&               !pb
           & coef % ss_val_chn ,&               !pb
           & coef % ss_cwn     ,&               !pb
           & coef % ss_solar_spectrum           !pb
     End If                                     !pb

     If( coef % id_comp_lvl == 9 ) then         !pb
       !WATER_OPTICAL_CONSTANT                 !pb
       section_present = Associated( coef % woc_chn ) 
       Write( file_id ) section_present
       If( section_present ) &
       Write(file_id, iostat=io_status)&       !pb
           & coef % woc_chn      ,&            !pb
           & coef % woc_cwn      ,&            !pb
           & coef % woc_waopc_ow ,&            !pb
           & coef % woc_waopc_fw               !pb
     End If                                    !pb

     If( coef % id_comp_lvl == 9 ) then         !pb
       !WAVE_SPECTRUM                          !pb
       section_present = Associated( coef % ws_k_omega ) 
       Write( file_id ) section_present
       If( section_present ) Then
         Write(file_id, iostat=io_status)&       !pb
             & coef % ws_nomega                  !pb
         Write(file_id, iostat=io_status)&       !pb
             & coef % ws_k_omega,        &       !pb
             & coef % ws_npoint                  !pb
       EndIf
     End If                                    !pb

     Write(file_id, iostat=io_status)&
           & coef % ff_ori_chn, &
           & coef % ff_val_chn, &
           & coef % ff_cwn,     &
           & coef % ff_bco,     &
           & coef % ff_bcs,     &
           & coef % ff_gam
     Write(file_id, iostat=io_status)&
           & coef % fc_speedl,    &
           & coef % fc_planck_c1, &
           & coef % fc_planck_c2, &
           & coef % fc_sat_height
     Write(file_id, iostat=io_status)&
           & coef % fastem_ver,&
           & coef % ssirem_ver
     If( coef % fastem_ver >= 1 ) Then
        Write(file_id, iostat=io_status)&
              & coef % fastem_coef_nb
        Write(file_id, iostat=io_status)&
              & coef % fastem_coef,&
              & coef % fastem_polar
     Endif
     If( coef % ssirem_ver >= 1 ) Then
        Write(file_id, iostat=io_status)&
              & coef % ssirem_chn, &
              & coef % ssirem_a0,  &
              & coef % ssirem_a1,  &
              & coef % ssirem_a2,  &
              & coef % ssirem_xzn1,&
              & coef % ssirem_xzn2
     Endif
     Write(file_id, iostat=io_status)&
           & coef % ref_prfl_p,&
           & coef % ref_prfl_t,&
           & coef % ref_prfl_mr
     Write(file_id, iostat=io_status)&
           & coef % lim_prfl_p,    &
           & coef % lim_prfl_tmax, &
           & coef % lim_prfl_tmin, &
           & coef % lim_prfl_gmax, &
           & coef % lim_prfl_gmin

     ! Write coefficients with ONE record per Channel
     If ( coef % nmixed > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % mixedgas( : , i, : )
        End Do
     Endif
     If ( coef % nwater > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % watervapour( : , i, : )
        End Do
     Endif
     If ( coef % nozone > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % ozone( : , i, : )
        End Do
     Endif
     If ( coef % nwvcont > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % wvcont( : , i, : )
        End Do
     Endif
     If ( coef % nco2 > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % co2( : , i, : )
        End Do
     Endif
     If ( coef % nn2o > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % n2o( : , i, : )
        End Do
     Endif
     If ( coef % nco > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % co( : , i, : )
        End Do
     Endif
     If ( coef % nch4 > 0 ) Then
        Do i = 1, coef % fmv_chn
           Write(file_id,iostat=io_status) coef % ch4( : , i, : )
        End Do
     Endif
     !

   ! COEF_SCATT_IR aerosols structure (V9)
   Else If ( laerosl .and. coef % id_comp_lvl >= 9 ) Then

     !AEROSOLS_COMPONENTS
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir % fmv_aer_chn, &
       & coef_scatt_ir % fmv_aer_pha_chn, &
!       & coef_scatt_ir % fmv_aer_sun_chn, &
       &  coef_scatt_ir%fmv_aer_pha_ioff, &
       & coef_scatt_ir % fmv_aer_comp, &
       & coef_scatt_ir % fmv_aer_ph


     Write(file_id, iostat=io_status) &
       & coef_scatt_ir %fmv_aer_ph_val

     Write(file_id, iostat=io_status) &
       & coef_scatt_ir % fmv_aer_rh

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Write(file_id, iostat=io_status) &
        & optp%optpaer(n)%fmv_aer_rh_val 
     End Do

     !AEROSOLS_PARAMETERS
     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Write(file_id, iostat=io_status) &
         & optp%optpaer(n) % abs
     End Do
     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Write(file_id, iostat=io_status) &
         & optp%optpaer(n) % sca
     End Do
     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Write(file_id, iostat=io_status) &
         & optp%optpaer(n) % bpr
     End Do
     if(coef_scatt_ir % fmv_aer_pha_chn>0)then
       Do n = 1, coef_scatt_ir % fmv_aer_comp
         Write(file_id, iostat=io_status) &
           & optp%optpaer(n) % pha
       End Do
     End if

   ! OPTP clouds structure (V9)
   Else If ( lclouds .and. coef % id_comp_lvl >= 9 ) Then

     !WATERCLOUD_TYPES
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir % fmv_wcl_chn, &
       & coef_scatt_ir % fmv_wcl_pha_chn, &
!       & coef_scatt_ir % fmv_wcl_sun_chn, &
       & coef_scatt_ir % fmv_wcl_pha_ioff, &
       & coef_scatt_ir % fmv_wcl_comp, &
       & coef_scatt_ir % fmv_wcl_ph

     Write(file_id, iostat=io_status) &
       & coef_scatt_ir %fmv_wcl_ph_val
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir % fmv_wcl_rh
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir%confac

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Write(file_id, iostat=io_status) &
         & optp%optpwcl(n)%fmv_wcl_rh_val
     End Do

     !WATERCLOUD_PARAMETERS
     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Write(file_id, iostat=io_status) &
         & optp%optpwcl(n) % abs
     End Do
     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Write(file_id, iostat=io_status) &
         & optp%optpwcl(n) % sca
     End Do
     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Write(file_id, iostat=io_status) &
         & optp%optpwcl(n) % bpr
     End Do
     if(coef_scatt_ir % fmv_wcl_pha_chn>0)then
       Do n = 1, coef_scatt_ir % fmv_wcl_comp
         Write(file_id, iostat=io_status) &
           & optp%optpwcl(n) % pha
       End Do
     End if


    !ICECLOUD_TYPES
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir % fmv_icl_chn, &
       & coef_scatt_ir % fmv_icl_pha_chn, &
!       & coef_scatt_ir % fmv_icl_sun_chn, &
       & coef_scatt_ir % fmv_icl_pha_ioff, &
       & coef_scatt_ir % icl_nabs, &
       & coef_scatt_ir % icl_nsca, &
       & coef_scatt_ir % icl_nbpr, &
       & coef_scatt_ir % fmv_icl_comp, &
       & coef_scatt_ir % fmv_icl_ishp, &
       & coef_scatt_ir % fmv_icl_ph
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_ph_val

     !HEXAGONAL_PARAMETERS
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_dg(:,1)
     Write(file_id, iostat=io_status) &
       & optp%optpicl(1) % abs
     Write(file_id, iostat=io_status) &
       & optp%optpicl(1) % sca
     Write(file_id, iostat=io_status) &
       & optp%optpicl(1) % bpr
     if(coef_scatt_ir % fmv_icl_pha_chn>0)then
       Write(file_id, iostat=io_status) &
        & optp%optpicl(1) % pha
     end if

     !AGGREGATE_PARAMETERS
     Write(file_id, iostat=io_status) &
       & coef_scatt_ir %fmv_icl_dg(:,2)
     Write(file_id, iostat=io_status) &
       & optp%optpicl(2) % abs
     Write(file_id, iostat=io_status) &
       & optp%optpicl(2) % sca
     Write(file_id, iostat=io_status) &
       & optp%optpicl(2) % bpr
     if(coef_scatt_ir % fmv_icl_pha_chn>0)then
       Write(file_id, iostat=io_status) &
         & optp%optpicl(2) % pha
     end if

   End If

     !

  Else

   !ASCII file
   Write( errMessage, '( "write coefficient to file_id ", i2, " in ASCII format")' ) &
           & file_id
   Call Rttov_ErrorReport (errorstatus_info, errMessage, NameOfRoutine)

   Write(file_id,'(a)') ' ! RTTOV coefficient file '//Trim(coef % id_Common_name)
   Write(file_id,'(a)') ' ! automatic creation by subroutine Rttov_writecoef '
   Write(file_id,'(a)') ' ! ------------------------------------------------------'

   ! COEF structure (V7, V8, V9)
   If ( .not. (laerosl .or. lclouds)) Then

     ! IDENTIFICATION
     section = 'IDENTIFICATION'
     Select Case (coef % id_sensor)
     Case (1_jpim)
        sensor = 'ir'
     Case (2_jpim)
        sensor = 'mw'
     Case (3_jpim)
        sensor = 'hi'
     Case (4_jpim)
        sensor = 'po'
     End Select

     If( coef % id_comp_lvl == 7 ) then
       Write( errMessage, '( "output will be version 8 format compatible")' )
       Call Rttov_ErrorReport (errorstatus_info, errMessage, NameOfRoutine)
       ! even if input coef file is version 7 format compatible
       ! the output will be version 8 format compatible
       ! The variable coef % fmv_model_ver contains the fast model 
       ! version for the predictors and should never be modified
       id_comp_lvl = 8
     Else
       id_comp_lvl = coef % id_comp_lvl
     End If

     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(3i3,T20,a)')&
           & coef % id_platform, coef % id_sat, coef % id_inst,'! platform sat_id instrument'
     Write(file_id,'(1x,a)')  TRIM(coef % id_Common_name)
     Write(file_id,'(1x,a,T20,a)')  sensor,'! sensor type [ir,mw,hi]'
     Write(file_id,'(1x,i2,T20,a)') id_comp_lvl,'! RTTOV coefficient file version number'
     Write(file_id,'(1x,a)') TRIM(coef % id_creation)
     Write(file_id,'(1x,i4,1x,i2.2,1x,i2.2,t20,a)') coef % id_creation_date,'! creation date'

     ! No LINE-BY-LINE section


     If( coef%line_by_line(1) .ne. 'xxxx' ) Then
       section = 'LINE-BY-LINE'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! '
       Do i = 1, 10
         If( coef%line_by_line(i) .eq. 'xxxx' ) Exit
         Write( file_id, '(a)' ) Trim(coef%line_by_line(i))
       EndDo
     EndIf

     ! FAST_MODEL_VARIABLES
     section = 'FAST_MODEL_VARIABLES'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' !'
     Write(file_id,'(1x,a,t20,a)')   coef % fmv_model_def,    '! fast model name'
     Write(file_id,'(1x,i4,t20,a)')  coef % fmv_model_ver,    '! fast model version compatibility level'
     Write(file_id,'(1x,i4,t20,a)')  coef % fmv_chn ,   '! Number of channels described in the coef file'
     Write(file_id,'(1x,i4,t20,a)')  coef % fmv_gas ,   '! Number of gases described in the coef file'
     If( coef % id_comp_lvl > 8 ) then
       If( coef % IncTop ) Then
         Write( file_id, * ) 1
       Else
         Write( file_id, * ) 0
       EndIf
     EndIf
     If( coef % id_comp_lvl == 9 ) then         !pb
       Do i = 1, coef % fmv_gas
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( coef % fmv_gas_id( i ) ) ),'! gas identification'
         Write(file_id,'(1x,3i4,t20,a)')  coef % fmv_var(i), coef % fmv_coe(i), &
          &      coef % fmv_lvl(i), '! variables/predictors  levels (pressure/absorber)'
       End Do
     Else
       Do i = 1, coef % fmv_gas
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( coef % fmv_gas_id( i ) ) ),'! gas identification'
         Write(file_id,'(1x,2i4,t20,a)')  coef % fmv_var(i), coef % fmv_lvl(i), '! variables/predictors  levels (pressure/absorber)'
       End Do
     End If

     ! GAZ_UNITS
     section = 'GAZ_UNITS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! Gaz concentrations can be expressed in '
     Write(file_id,'(a)') ' ! volume mixing ratio (ppmv)'
     Write(file_id,'(a)') ' ! specific concentration (kg/kg)'
     Write(file_id,'(a)') ' ! '
     Do i = 1, coef % fmv_gas
        Write(file_id,'(a)') ' !     '//gas_name( coef % fmv_gas_id( i ) )
        Write(file_id,'(1x,i4,t20,"! ",a)') &
                 & coef % gaz_units( i ), gas_unit_name( coef % gaz_units( i ) )
     End Do

     If( &
       ( coef % id_comp_lvl == 9 ) .and. &
       ( coef %nintmixed > 0 )     .and. &
       ( coef %nintwater > 0 )     .and. &
       ( coef %nintozone > 0 )     .and. &
       ( coef %nintwvcont > 0 )    .and. &
       ( coef %nintco2 > 0 )       .and. &
       ( coef %nintn2o > 0 )       .and. &
       ( coef %nintco > 0 )        .and. &
       ( coef %nintch4 > 0 )             &
     ) then         !pb
       section = 'GAS_SPECTRAL_INTERVAL'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! '
       If ( coef %nintmixed > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_mixed ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintmixed
         Do i = 1, coef %nintmixed
           Write(file_id,'(1x,2f10.3)')  coef%mixedgasint(1,i),coef%mixedgasint(2,i)
         End Do
       End If
       If ( coef %nintwater > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_watervapour ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintwater
         Do i = 1, coef %nintwater
           Write(file_id,'(1x,2f10.3)')  coef%watervapourint(1,i),coef%watervapourint(2,i)
         End Do
       End If
       If ( coef %nintozone > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_ozone ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintozone
         Do i = 1, coef %nintozone
           Write(file_id,'(1x,2f10.3)')  coef%ozoneint(1,i),coef%ozoneint(2,i)
         End Do
       End If
       If ( coef %nintwvcont > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_wvcont ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintwvcont
         Do i = 1, coef %nintwvcont
           Write(file_id,'(1x,2f10.3)')  coef%wvcontint(1,i),coef%wvcontint(2,i)
         End Do
       End If
       If ( coef %nintco2 > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_co2 ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintco2
         Do i = 1, coef %nintco2
           Write(file_id,'(1x,2f10.3)')  coef%co2int(1,i),coef%co2int(2,i)
         End Do
       End If
       If ( coef %nintn2o > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_n2o ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintn2o
         Do i = 1, coef %nintn2o
           Write(file_id,'(1x,2f10.3)')  coef%n2oint(1,i),coef%n2oint(2,i)
         End Do
       End If
       If ( coef %nintco > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_co ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintco
         Do i = 1, coef %nintco
           Write(file_id,'(1x,2f10.3)')  coef%coint(1,i),coef%coint(2,i)
         End Do
       End If
       If ( coef %nintch4 > 0 ) Then 
         Write(file_id,'(1x,a,t20,a)')  Trim(gas_name( gas_id_ch4 ) ),'! gas identification'
         Write(file_id,'(1x,i4)')  coef %nintch4
         Do i = 1, coef %nintch4
           Write(file_id,'(1x,2f10.3)')  coef%ch4int(1,i),coef%ch4int(2,i)
         End Do
       End If
     End If


     If( coef % id_comp_lvl == 9 .and. Associated( coef % tt_chn ) ) then         !pb
       section = 'TRANSMITTANCE_TRESHOLD'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! ' 
       Write(file_id,'(a)') ' ! chan number'
       Write(file_id,'(a)') ' ! validity of channel '
       Write(file_id,'(a)') ' ! central wave number'
       Write(file_id,'(a)') ' ! transmittance treshold'
       Write(file_id,'(a)') ' ! transmittance value   '         
       Do i = 1, coef % fmv_chn
         Write(file_id,'(2i5,3e19.10)',iostat=io_status)&
               & coef % tt_chn(i)    ,  &
               & coef % tt_val_chn(i),  &
               & coef % tt_cwn(i)    ,  &
               & coef % tt_a0(i)     ,  &
               & coef % tt_a1(i)          
         If(io_status /= 0) Then
           errMessage="write TRANSMITTANCE_TRESHOLD section"
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage, NameOfRoutine)
           errorstatus = errorstatus_fatal
           Return
         Endif
       End Do
     End If                                     !pb

     If( coef % id_comp_lvl == 9 .and. Associated( coef % ss_chn ) ) then         !pb
       section = 'SOLAR_SPECTRUM'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! ' 
       Write(file_id,'(a)') ' ! chan number'
       Write(file_id,'(a)') ' ! validity of channel'
       Write(file_id,'(a)') ' ! central wave number'
       Write(file_id,'(a)') ' ! solar spectrum'
       Do i = 1, coef % fmv_chn
         Write(file_id,'(2i5,2e19.10)',iostat=io_status)&
                 & coef % ss_chn(i)    ,  &
                 & coef % ss_val_chn(i),  &
                 & coef % ss_cwn(i)    ,  &
                 & coef % ss_solar_spectrum(i) 
         If(io_status /= 0) Then
           errMessage="write SOLAR_SPECTRUM section"
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage, NameOfRoutine)
           errorstatus = errorstatus_fatal
           Return
         Endif
       End Do
     End If                                     !pb

     If( coef % id_comp_lvl == 9 .and. associated( coef % woc_chn ) ) then         !pb
       section = 'WATER_OPTICAL_CONSTANT'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! ' 
       Write(file_id,'(a)') ' ! chan number'
       Write(file_id,'(a)') ' ! central wave number'
       Write(file_id,'(a)') ' ! ocean water optical constants(real and imaginary part)'
       Write(file_id,'(a)') ' ! fresh water optical constants(real and imaginary part)'
       Do i = 1, coef % fmv_chn
         Write(file_id,'(i5,e19.10,2(" (",e17.10,",",e17.10,")"))',iostat=io_status)&
               & coef % woc_chn(i)    ,   &
               & coef % woc_cwn(i)    ,   &
               & coef % woc_waopc_ow(i)  ,&
               & coef % woc_waopc_fw(i) 
         If(io_status /= 0) Then
           errMessage="write WATER_OPTICAL_CONSTANT section"
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage, NameOfRoutine)
           errorstatus = errorstatus_fatal
           Return
         Endif
       End Do
     End If                                     !pb

     If( coef % id_comp_lvl == 9 .and. Associated( coef%ws_npoint ) ) then         !pb
       section = 'WAVE_SPECTRUM'
       Write(file_id,'(a)') ' ! ------------------------------------------------------'
       Write(file_id,'(a)') Trim(section)
       Write(file_id,'(a)') ' ! '
       Write(file_id,'(a)') ' ! Number of points'
       Write(file_id,'(a)') ' ! Point number'
       Write(file_id,'(a)') ' ! WAve spectrum'
       Write(file_id,*,iostat=io_status) coef%ws_nomega
       Do i = 1, coef % ws_nomega
         Write(file_id,'(f10.3,f12.5)',iostat=io_status)coef%ws_npoint(i),coef%ws_k_omega(i)
         If(io_status /= 0) Then
           errMessage="write WAVE_SPECTRUM section"
           Call Rttov_ErrorReport (Int(io_status,jpim), errMessage, NameOfRoutine)
           errorstatus = errorstatus_fatal
           Return
         Endif
       End Do
     End If                                     !pb





     section = 'FILTER_FUNCTIONS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' ! Channel Number (from instrument original description)'
     Write(file_id,'(a)') ' ! Channel status '
     Write(file_id,'(a)') ' ! Central Wavenumber'
     Write(file_id,'(a)') ' ! Band Correction coefficients(Offset,Slope)'
     Write(file_id,'(a)') ' ! Gamma correction factor'

     Do i = 1, coef % fmv_chn
        Write(file_id,'(1x,i4,1x,i4,4(1x,e18.10))') &
              & coef % ff_ori_chn(i), coef % ff_val_chn(i), coef % ff_cwn(i),&
              & coef % ff_bco(i), coef % ff_bcs(i), coef % ff_gam(i)
     End Do


     section = 'FUNDAMENTAL_CONSTANTS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' ! units of constants for spectral radiance'
     Write(file_id,'(a)') ' ! first radiation constant(mW/(m2.sr.cm-4))'
     Write(file_id,'(a)') ' ! second radiation constant (cm.K)'
     Write(file_id,'(1x,f14.1,t30,a)') coef % fc_speedl,'! speed of light (cm/s)'
     Write(file_id,'(1x,1p,e15.8,0p,f10.6,t30,a)') coef % fc_planck_c1, coef % fc_planck_c2,'! Planck constants'
     Write(file_id,'(1x,f8.1,t30,a)') coef % fc_sat_height,'! nominal satellite height (km)'

     If( coef % fastem_ver >= 1 ) Then
        section = 'FASTEM'
        Write(file_id,'(a)') ' ! ------------------------------------------------------'
        Write(file_id,'(a)') Trim(section)
        Write(file_id,'(a)') ' ! '
        Write(file_id,'(a)') ' ! S. English fast generic millimetre wave ocean emissivity model'
        Write(file_id,'(a)') ' ! Polarisation of each channel', &
              & ' !       MPOL=0:  0.5_JPRB*(V+H)', &
              & ' !       MPOL=1: polarisation angle=90-incidence angle', &
              & ' !       MPOL=2: polarisation angle=incidence angle', &
              & ' !       MPOL=3: vertical polarisation', &
              & ' !       MPOL=4: horizontal polarisation'
        Write(file_id,'(1x,i2,a)') coef % fastem_ver,'   ! version number'
        Write(file_id,'(1x,i3,a)') coef % fastem_coef_nb,'  ! number of coefficients'
        Write(file_id,'(5e14.6)') coef % fastem_coef
        Write(file_id,'(20i3)') (coef % fastem_polar(i), i= 1, coef % fmv_chn)
     Endif

     If( coef % ssirem_ver >= 1 ) Then
        section = 'SSIREM'
        Write(file_id,'(a)') ' ! ------------------------------------------------------'
        Write(file_id,'(a)') Trim(section)
        Write(file_id,'(a)') ' ! '
        Write(file_id,'(a)') ' ! Channel Number (from instrument original description)'
        Write(file_id,'(a)') ' ! 5 coefficients for emissivity model ssirem'
        Write(file_id,'(1x,i2,a)') coef % ssirem_ver,'   ! version number'

        Do i = 1, coef % fmv_chn
           Write(file_id,'(1x,i4,3f12.7,2f4.1)') &
                 & coef % ssirem_chn(i) , coef % ssirem_a0(i),&
                 & coef % ssirem_a1(i)  , coef % ssirem_a2(i),&
                 & coef % ssirem_xzn1(i), coef % ssirem_xzn2(i)
        End Do
     Endif

     section = 'REFERENCE_PROFILE'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' ! Ref.pressure (hPa)'
     Write(file_id,'(a)') ' ! Ref.Temp (K) Ref.Volume Mixing Ratio [ppmv] for each gas'
     Write(file_id,'(a)') ' ! Note for MxG that mixing ratio is "missing"'

     Do i = 1, coef % fmv_gas
        Write(file_id,'(a)') ' !     '//gas_name( coef % fmv_gas_id( i ) )
        Do l = 1, coef % fmv_lvl(i)
           Write(file_id,'(1x,f9.4,2x,f7.3,1x,e13.6)')&
                 & coef % ref_prfl_p(l), coef % ref_prfl_t(l,i), coef % ref_prfl_mr(l,i)
!!$                & coef % ref_prfl_p(l), coef % ref_prfl_t(l,i), ref_mr(l,i)
        End Do
     End Do

     section = 'PROFILE_LIMITS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' ! Ref.pressure (hPa)'
     Write(file_id,'(a)') ' ! Temp Max (K) Temp Min (K)'
     Write(file_id,'(a)') ' ! Volume Mixing Ratio for  Max and Min [ppmv] for each gas'
     Write(file_id,'(a)') ' !      Temperature'
     Do l = 1, coef % fmv_lvl(1)
        Write(file_id,'(1x,f9.4,2(1x,f7.2))',iostat=io_status)&
              & coef % lim_prfl_p(l), coef % lim_prfl_tmax(l), coef % lim_prfl_tmin(l)
     End Do

     Do i = 1, coef % fmv_gas
        Write(file_id,'(a)') ' !     '//gas_name( coef % fmv_gas_id( i ) )
        Do l = 1, coef % fmv_lvl(i)
           Write(file_id,'(1x,f9.4,2x,e12.4,e12.4)',iostat=io_status)&
                 & coef % lim_prfl_p(l), coef % lim_prfl_gmax(l,i), coef % lim_prfl_gmin(l,i)
        End Do
     End Do


     section = 'FAST_COEFFICIENTS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' ! '
     Write(file_id,'(a)') ' ! transmission coefficients'
     Write(file_id,'(a)') ' ! Order of the gases:'
     Do i = 1, coef % fmv_gas
        Write(file_id,'(a)') ' !     '//gas_name( coef % fmv_gas_id ( i ) )
     End Do


     Do l = 1, coef % fmv_gas
        Write(file_id,'(a)') gas_name( coef % fmv_gas_id( l ) )

        Select Case( coef % fmv_gas_id(l) )

        Case(gas_id_mixed)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % mixedgas(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncmixed )
        Case(gas_id_watervapour)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % watervapour(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncwater )
        Case(gas_id_ozone)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % ozone(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncozone )
        Case(gas_id_wvcont)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % wvcont(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncwvcont )
        Case(gas_id_co2)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % co2(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncco2 )
        Case(gas_id_n2o)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % n2o(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncn2o )
        Case(gas_id_co)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % co(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncco )
        Case(gas_id_ch4)
           Write(file_id,'(5(1x,e15.8))',iostat=io_status)  &
                 & (((coef % ch4(i,j,k)   &
                  & ,i = 1, coef % fmv_lvl(l) ) &
                  & ,j = 1, coef % fmv_chn    ) &
                  & ,k = 1, coef % ncch4 )
        End Select
     End Do

     section = 'END'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)

   ! COEF_SCATT_IR aerosols structure (V9)
   Else If ( laerosl .and. coef % id_comp_lvl >= 9 ) Then

     !AEROSOLS_COMPONENTS
     section = 'AEROSOLS_COMPONENTS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'
     Write(file_id,'(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_aer_chn, &
       & '! number of channels for which optical parameters are stored',&
       & coef_scatt_ir % fmv_aer_pha_chn, &
       & '! Number of channels for which phase function values are stored',&
       & coef_scatt_ir%fmv_aer_pha_ioff, &
       & '! index of first channel for which phase function values are available',&
       & coef_scatt_ir % fmv_aer_comp, &
       & '! number of aerosols components',&
       & coef_scatt_ir % fmv_aer_ph, &
       & '! number of angles for phase function'

     Write(file_id, '(10f7.2)') &
       & coef_scatt_ir %fmv_aer_ph_val

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Write(file_id, '(a)') ' aerosol ! default name for rttov_writecoef'
       Write(file_id, '(1x,i8,t20,a)') &
         & coef_scatt_ir % fmv_aer_rh(n), &
         & '!RH values for which parameters are available'
       Write(file_id, '(10f7.2)') &
        & optp%optpaer(n)%fmv_aer_rh_val 
     End Do

     section = 'AEROSOLS_PARAMETERS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'

     Do n = 1, coef_scatt_ir % fmv_aer_comp
       Do nrh=1,coef_scatt_ir % fmv_aer_rh(n)
         Write( file_id,'(a)') ' ! ---------------------'
         Write( file_id,'(a)') ' aerosol ! default name for rttov_writecoef'
         Write( file_id, '(5e16.8)' ) optp%optpaer(n) % abs(:,nrh)
         Write( file_id, '(5e16.8)' ) optp%optpaer(n) % sca(:,nrh)
         Write( file_id, '(5e16.8)' ) optp%optpaer(n) % bpr(:,nrh)
         !Write( file_id, '(5e16.8)' ) ((optp%optpaer(n) % pha(i,nrh,j),&
         !& j = 1,coef_scatt_ir % fmv_aer_ph ),i=1,coef_scatt_ir % fmv_aer_pha_chn)
         If (coef_scatt_ir % fmv_aer_pha_chn>0)then
           Do i = 1, coef_scatt_ir % fmv_aer_pha_chn
             Write( file_id, '(5e16.8)' ) optp%optpaer(n) % pha(i,nrh,:)
           End Do
         end if
       End Do
     End Do


   ! OPTP clouds structure (V9)
   Else If ( lclouds .and. coef % id_comp_lvl >= 9 ) Then
     section = 'WATERCLOUD_TYPES'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_wcl_chn, &
       &'! number of channels for which optical parameters are stored'
     Write(file_id, '(1x,i8,t20,a)')     &
       & coef_scatt_ir % fmv_wcl_pha_chn, &
       &'! Number of channels for which phase function values are stored'
     Write(file_id, '(1x,i8,t20,a)')     &
!       & coef_scatt_ir % fmv_wcl_sun_chn, &
       & coef_scatt_ir % fmv_wcl_pha_ioff, &
       &'! index of channel for solar term'
     Write(file_id, '(1x,i8,t20,a)')  &
       & coef_scatt_ir % fmv_wcl_comp, &
       &'! number of water cloud types'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_wcl_ph,  &
       &'! number of angles for phase function for water cloud types'

     Write(file_id, '(10f7.2)') &
       & coef_scatt_ir %fmv_wcl_ph_val 

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Write( file_id,'(a)') ' cloud   ! default name for rttov_writecoef'
       Write(file_id, '(1x,i8,t20,a)') &
         & coef_scatt_ir % fmv_wcl_rh(n), &
         &'!RH values for which parameters are available'
       Write(file_id,  '(10f7.2)') &
         & optp%optpwcl(n)%fmv_wcl_rh_val
       Write(file_id, '(1x,f12.6,t20,a)') &
         & coef_scatt_ir%confac(n), &
         & '!Conversion from LWC to particle density'
     End Do


     section = 'WATERCLOUD_PARAMETERS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'

     Do n = 1, coef_scatt_ir % fmv_wcl_comp
       Do nrh=1,coef_scatt_ir % fmv_wcl_rh(n)
         Write( file_id,'(a)') ' cloud   ! default name for rttov_writecoef'
         Write(file_id, '(5e16.8)') & 
           & optp%optpwcl(n) % abs(:,nrh)
         Write(file_id, '(5e16.8)') & 
           &optp%optpwcl(n) % sca(:,nrh)
         Write(file_id, '(5e16.8)') & 
           & optp%optpwcl(n) % bpr(:,nrh)
         !Write(file_id, '(5e16.8)') & 
         !& ((optp%optpwcl(n) % pha(i,nrh,j),&
         !& j = 1, coef_scatt_ir % fmv_wcl_ph),i=1,coef_scatt_ir % fmv_wcl_pha_chn) 
         If(coef_scatt_ir % fmv_wcl_pha_chn>0)then
           Do i = 1, coef_scatt_ir % fmv_wcl_pha_chn
             Write(file_id, '(5e16.8)') & 
              & optp%optpwcl(n) % pha(i,nrh,:)
           End Do
         end if
       End Do
     End Do

     section = 'ICECLOUD_TYPES'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_icl_chn, &
       & '! number of channels for which regression coefficients are stored'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_icl_pha_chn, &
       & '! Number of channels for which phase function values are stored'
     Write(file_id, '(1x,i8,t20,a)') &
!       & coef_scatt_ir % fmv_icl_sun_chn, &
       & coef_scatt_ir % fmv_icl_pha_ioff, &
       & '! index of channel for solar term'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % icl_nabs, &
       & '! number of coefficients used in the regression for absorption optical depth'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % icl_nsca, &
       & '! number of coefficients used in the regression for scattering optical depth'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % icl_nbpr, &
       & '! number of coefficients used in the regression for backscattering parameter' 
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_icl_comp, &
       & '! number of size distributions used in the regression' 
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_icl_ishp, &
       & '! number of ice crystal shapes for which parameters are available'
     Write(file_id, '(1x,i8,t20,a)') &
       & coef_scatt_ir % fmv_icl_ph, &
       & '! number of angles for phase function for ice clouds' 
     Write(file_id, '(10f7.2)') &
       & coef_scatt_ir %fmv_icl_ph_val

     section = 'HEXAGONAL_PARAMETERS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'
     Write(file_id,'(a)') ' ! Effective diameter for each size distribution'
     Write(file_id,'(a)') ' !'

     Write(file_id, '(f10.4)') &
       & coef_scatt_ir %fmv_icl_dg(:,1)

     Write(file_id,'(a)') ' !'
     Write(file_id,'(a)') ' ! Regression coefficients for ice clouds'
     Write(file_id,'(a)') ' !'
     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4E16.8)') &
         & optp%optpicl(1) % abs(i,:)
     End Do
     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4E16.8)') &
         & optp%optpicl(1) % sca(i,:)
     End Do
     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4E16.8)') &
         & optp%optpicl(1) % bpr(i,:)
     End Do

     Do nr=1,coef_scatt_ir % fmv_icl_comp
       Write(file_id,'(a)') ' !'
       Write(file_id,'(a)') ' ! Phase function values'
       Write(file_id,'(a)') ' !'
       Write(file_id,'(a)') ' Hexagonal   ! default name for rttov_writecoef'
       !Write(file_id, '(5e16.8)') &
       !  & ((optp%optpicl(1) % pha(i,nr,j),&
       !  & j = 1, coef_scatt_ir % fmv_icl_ph),i=1,coef_scatt_ir % fmv_icl_pha_chn)
       If(coef_scatt_ir % fmv_icl_pha_chn>0)then
         Do i = 1, coef_scatt_ir % fmv_icl_pha_chn
           Write(file_id, '(5e16.8)') &
             & optp%optpicl(1) % pha(i,nr,:)
         End Do
       End if
     End Do


     section = 'AGGREGATE_PARAMETERS'
     Write(file_id,'(a)') ' ! ------------------------------------------------------'
     Write(file_id,'(a)') Trim(section)
     Write(file_id,'(a)') ' !'
     Write(file_id,'(a)') ' !'
     Write(file_id,'(a)') ' ! Effective diameter for each size distribution'
     Write(file_id,'(a)') ' !'
     Write(file_id, '(f10.4)') &
       & coef_scatt_ir %fmv_icl_dg(:,2)

     Write(file_id,'(a)') ' !'
     Write(file_id,'(a)') ' ! Regression coefficients for ice clouds'
     Write(file_id,'(a)') ' !'

     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4e16.8)') &
         & optp%optpicl(2) % abs(i,:)
     End Do
     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4e16.8)') &
         & optp%optpicl(2) % sca(i,:)
     End Do
     Do i=1,coef_scatt_ir % fmv_icl_chn
       Write(file_id, '(4e16.8)') &
         & optp%optpicl(2) % bpr(i,:)
     End Do

     Do nr=1,coef_scatt_ir % fmv_icl_comp
       Write(file_id,'(a)') ' !'
       Write(file_id,'(a)') ' ! Phase function values'
       Write(file_id,'(a)') ' !'
       Write(file_id,'(a)') ' aggregate   ! default name for rttov_writecoef'
       !Write(file_id, '(5e16.8)') &
       !  & ((optp%optpicl(2) % pha(i,nr,j),&
       !  & j = 1, coef_scatt_ir % fmv_icl_ph),i=1,coef_scatt_ir % fmv_icl_pha_chn)
       If(coef_scatt_ir % fmv_icl_pha_chn>0)then
         Do i = 1, coef_scatt_ir % fmv_icl_pha_chn
           Write(file_id, '(5e16.8)') &
             & optp%optpicl(2) % pha(i,nr,:)
         End Do
       End if
     End Do
   End If

  Endif ascii_binary

  If( io_status /= 0 ) Then
     Write( errMessage, '( "write IO error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Return
  End If

  Write( errMessage, '( "end of write coefficient")' ) 
  Call Rttov_ErrorReport (errorstatus_info, errMessage, NameOfRoutine)

  IF (LHOOK) CALL DR_HOOK('RTTOV_WRITECOEF',1_jpim,ZHOOK_HANDLE)
End Subroutine Rttov_writecoef

